"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
exports.__esModule = true;
var chalk = require("chalk");
var chokidar = require("chokidar");
var lodash_1 = require("lodash");
var path = require("path");
var ramda_1 = require("ramda");
var tableOfContents_1 = require("../../tableOfContents");
var types_1 = require("../../types");
var componentParsingUtils_1 = require("./utils/componentParsingUtils");
var errorUtil_1 = require("./utils/errorUtil");
var componentPathUtils_1 = require("./utils/componentPathUtils");
var fileUtils_1 = require("./utils/fileUtils");
var markdownHigherPath = path.join(__dirname, '../../..', 'src');
var componentOutputPath = path.join(__dirname, '../../../compiled-data', 'componentData.js');
var markdownOutputPath = path.join(__dirname, '../../../compiled-data', 'documentationData.js');
function buildMarkdownObj(markdownRelativePath, slug) {
    var markdownFilePath = path.join(markdownHigherPath, markdownRelativePath);
    var markdownFileNames = fileUtils_1.getFileNamesWithExt(markdownFilePath, 'md');
    var componentName = fileUtils_1.getFileNamesWithExt(markdownFilePath, 'tsx')[0];
    return {
        dirPath: markdownRelativePath,
        componentName: componentName
            ? componentName.replace('.tsx', '')
            : componentName,
        markdownFiles: markdownFileNames.map(function (markdownFileName) { return ({
            fileName: markdownFileName,
            contents: fileUtils_1.readMarkdownFile(markdownFilePath, markdownFileName, slug)
        }); })
    };
}
function buildComponentDataObj(componentPath, packageName) {
    var parsedComponentPathArray = ramda_1.flatten(types_1.packageDetails.map(function (packageDetail) {
        return fileUtils_1.parseComponentPaths(packageDetail.packagePath);
    }));
    var parsedComponentPath = ramda_1.find(ramda_1.propEq('componentPath', componentPath))(parsedComponentPathArray);
    var componentName = parsedComponentPath.componentName;
    var componentGroup = parsedComponentPath.componentGroup;
    var tsxFilePath = componentPathUtils_1.getComponentFilePath(componentPath, componentName);
    var content = fileUtils_1.readFile(tsxFilePath);
    var parsedComponentData = componentParsingUtils_1.getParsedComponentData(tsxFilePath, componentName)[0];
    if (parsedComponentData && lodash_1.size(parsedComponentData.props) === 0) {
        console.log(chalk.bgRed(' PROP '), 'No props found for', componentName);
    }
    var markdownFileName = fileUtils_1.getFileNamesWithExt(componentPath, 'md')[0];
    var markdownContent = fileUtils_1.readMarkdownFile(componentPath, markdownFileName, componentName);
    return {
        packageName: packageName,
        componentName: componentName,
        componentGroup: componentGroup,
        componentDescription: parsedComponentData && parsedComponentData.description,
        componentProps: parsedComponentData && parsedComponentData.props,
        componentCode: content,
        markdownContent: markdownContent,
        examples: componentParsingUtils_1.getParsedExampleData(componentPath, componentName)
    };
}
function generateComponentMetadata(allComponentPaths) {
    var componentData = [];
    allComponentPaths.forEach(function (componentPathInfo) {
        componentPathInfo.paths.forEach(function (componentPath) {
            try {
                componentData.push(buildComponentDataObj(componentPath, componentPathInfo.packageName));
            }
            catch (error) {
                errorUtil_1.addError(types_1.TemplateType.Component, "An error occurred while attempting to generate metadata for\n                    " + componentPath + ". " + error);
            }
        });
    });
    var errors = errorUtil_1.getErrorList(types_1.TemplateType.Component);
    return fileUtils_1.writeFile(componentOutputPath, "module.exports = " + JSON.stringify({
        data: ramda_1.flatten(componentData),
        errors: errors
    }));
}
function generateDocumentationMetadata(markdownObjPaths) {
    var documentationData = markdownObjPaths.map(function (_a) {
        var slug = _a.slug, markdownRelativePath = _a.markdownRelativePath;
        try {
            return buildMarkdownObj(markdownRelativePath, slug);
        }
        catch (error) {
            errorUtil_1.addError(types_1.TemplateType.Documentation, "An error occurred while attempting to generate documentation metadata for\n                  " + markdownRelativePath + ". " + error);
            return null;
        }
    }, {});
    var errors = errorUtil_1.getErrorList(types_1.TemplateType.Documentation);
    return fileUtils_1.writeFile(markdownOutputPath, "module.exports = " + JSON.stringify({
        data: documentationData,
        errors: errors
    }));
}
function parseMarkdownPaths(sections, basePath) {
    return sections.reduce(function (flatMap, section) {
        var currentPath = basePath
            ? basePath + "/" + section.slug
            : section.slug;
        flatMap.push({
            markdownRelativePath: currentPath,
            slug: section.slug
        });
        if (section.children) {
            var nestedPaths = parseMarkdownPaths(section.children, currentPath);
            flatMap.push.apply(flatMap, nestedPaths);
        }
        return flatMap;
    }, []);
}
//TODO: Fix this to optimize watching prop and/or metadata changes. It is currently disabled.
function initializeWatchMode() {
    var enableWatchMode = ramda_1.contains('--watch', process.argv);
    console.log(chalk.green('Build Starting...'));
    var timerText = chalk.bold('TOTAL TIME');
    console.time(timerText);
    var markdownObjPaths = parseMarkdownPaths(tableOfContents_1.tableOfContents);
    var markdownRelativePath = markdownObjPaths.map(function (markdownData) {
        return path.join(markdownHigherPath, markdownData.markdownRelativePath);
    });
    var allComponentPaths = types_1.packageDetails.map(function (packageDetail) { return ({
        paths: fileUtils_1.parseComponentPaths(packageDetail.packagePath).map(function (componentData) {
            return componentData.componentPath;
        }),
        packageName: packageDetail.packageName
    }); });
    if (enableWatchMode) {
        chokidar
            .watch(__spreadArrays(ramda_1.pluck('paths', allComponentPaths), markdownRelativePath))
            .on('change', function () {
            generateComponentMetadata(allComponentPaths);
            generateDocumentationMetadata(markdownObjPaths);
        });
    }
    generateComponentMetadata(allComponentPaths);
    generateDocumentationMetadata(markdownObjPaths);
    console.timeEnd(timerText);
}
initializeWatchMode();
errorUtil_1.errorLog();
