import * as chalk from 'chalk';
import * as chokidar from 'chokidar';
import { size } from 'lodash';
import * as path from 'path';
import { contains, find, flatten, pluck, propEq } from 'ramda';
import { ComponentDoc } from 'react-docgen-typescript/lib';
import { tableOfContents } from '../../tableOfContents';
import {
    IComponentData,
    IComponentPathInfo,
    IDocumentationFileGroup,
    IMarkdownFile,
    IMarkdownObjPaths,
    IParsedComponentPath,
    ITableOfContents,
    PackageDetail,
    packageDetails,
    TemplateType
} from '../../types';
import {
    getParsedComponentData,
    getParsedExampleData
} from './utils/componentParsingUtils';
import { addError, errorLog, getErrorList } from './utils/errorUtil';
import { getComponentFilePath } from './utils/componentPathUtils';
import {
    getFileNamesWithExt,
    parseComponentPaths,
    readFile,
    readMarkdownFile,
    writeFile
} from './utils/fileUtils';

const markdownHigherPath: string = path.join(__dirname, '../../..', 'src');
const componentOutputPath: string = path.join(
    __dirname,
    '../../../compiled-data',
    'componentData.js'
);
const markdownOutputPath: string = path.join(
    __dirname,
    '../../../compiled-data',
    'documentationData.js'
);

function buildMarkdownObj(
    markdownRelativePath: string,
    slug: string
): IDocumentationFileGroup {
    const markdownFilePath: string = path.join(
        markdownHigherPath,
        markdownRelativePath
    );
    const markdownFileNames: string[] = getFileNamesWithExt(
        markdownFilePath,
        'md'
    );
    const componentName: string = getFileNamesWithExt(
        markdownFilePath,
        'tsx'
    )[0];

    return {
        dirPath: markdownRelativePath,
        componentName: componentName
            ? componentName.replace('.tsx', '')
            : componentName,
        markdownFiles: markdownFileNames.map(
            (markdownFileName: string): IMarkdownFile => ({
                fileName: markdownFileName,
                contents: readMarkdownFile(
                    markdownFilePath,
                    markdownFileName,
                    slug
                )
            })
        )
    };
}

function buildComponentDataObj(
    componentPath: string,
    packageName: string
): IComponentData {
    const parsedComponentPathArray: IParsedComponentPath[] = flatten(
        packageDetails.map(
            (packageDetail: PackageDetail): IParsedComponentPath[] =>
                parseComponentPaths(packageDetail.packagePath)
        )
    );
    const parsedComponentPath: IComponentData = find(
        propEq('componentPath', componentPath)
    )(parsedComponentPathArray);
    const componentName: string | undefined = parsedComponentPath.componentName;
    const componentGroup: string | undefined =
        parsedComponentPath.componentGroup;
    const tsxFilePath: string = getComponentFilePath(
        componentPath,
        componentName as string
    );
    const content: string = readFile(tsxFilePath);
    const parsedComponentData: ComponentDoc = getParsedComponentData(
        tsxFilePath,
        componentName as string
    )[0];
    if (parsedComponentData && size(parsedComponentData.props) === 0) {
        console.log(chalk.bgRed(' PROP '), 'No props found for', componentName);
    }
    const markdownFileName: string = getFileNamesWithExt(
        componentPath,
        'md'
    )[0];
    const markdownContent: string = readMarkdownFile(
        componentPath,
        markdownFileName,
        componentName as string
    );

    return {
        packageName,
        componentName: componentName,
        componentGroup: componentGroup,
        componentDescription:
            parsedComponentData && parsedComponentData.description,
        componentProps: parsedComponentData && parsedComponentData.props,
        componentCode: content,
        markdownContent: markdownContent,
        examples: getParsedExampleData(componentPath, componentName)
    };
}

function generateComponentMetadata(
    allComponentPaths: IComponentPathInfo[]
): void {
    const componentData: IComponentData[] = [];
    allComponentPaths.forEach((componentPathInfo: IComponentPathInfo): void => {
        componentPathInfo.paths.forEach((componentPath: string): void => {
            try {
                componentData.push(
                    buildComponentDataObj(
                        componentPath,
                        componentPathInfo.packageName
                    )
                );
            } catch (error) {
                addError(
                    TemplateType.Component,
                    `An error occurred while attempting to generate metadata for
                    ${componentPath}. ${error}`
                );
            }
        });
    });

    const errors: string[] = getErrorList(TemplateType.Component);

    return writeFile(
        componentOutputPath,
        `module.exports = ${JSON.stringify({
            data: flatten(componentData),
            errors
        })}`
    );
}

function generateDocumentationMetadata(
    markdownObjPaths: IMarkdownObjPaths[]
): void {
    const documentationData: (IDocumentationFileGroup | null)[] = markdownObjPaths.map(
        ({
            slug,
            markdownRelativePath
        }: IMarkdownObjPaths): IDocumentationFileGroup | null => {
            try {
                return buildMarkdownObj(markdownRelativePath, slug);
            } catch (error) {
                addError(
                    TemplateType.Documentation,
                    `An error occurred while attempting to generate documentation metadata for
                  ${markdownRelativePath}. ${error}`
                );

                return null;
            }
        },
        {}
    );

    const errors: string[] = getErrorList(TemplateType.Documentation);

    return writeFile(
        markdownOutputPath,
        `module.exports = ${JSON.stringify({
            data: documentationData,
            errors
        })}`
    );
}

function parseMarkdownPaths(
    sections: ITableOfContents[],
    basePath?: string
): { markdownRelativePath: string; slug: string }[] {
    return sections.reduce(
        (
            flatMap: { markdownRelativePath: string; slug: string }[],
            section: ITableOfContents
        ): { markdownRelativePath: string; slug: string }[] => {
            const currentPath: string = basePath
                ? `${basePath}/${section.slug}`
                : section.slug;

            flatMap.push({
                markdownRelativePath: currentPath,
                slug: section.slug
            });
            if (section.children) {
                const nestedPaths: {
                    markdownRelativePath: string;
                    slug: string;
                }[] = parseMarkdownPaths(section.children, currentPath);
                flatMap.push(...nestedPaths);
            }

            return flatMap;
        },
        []
    );
}

//TODO: Fix this to optimize watching prop and/or metadata changes. It is currently disabled.
function initializeWatchMode(): void {
    const enableWatchMode: boolean = contains('--watch', process.argv);
    console.log(chalk.green('Build Starting...'));
    const timerText: string = chalk.bold('TOTAL TIME');
    console.time(timerText);
    const markdownObjPaths: IMarkdownObjPaths[] = parseMarkdownPaths(
        tableOfContents
    );
    const markdownRelativePath: string[] = markdownObjPaths.map(
        (markdownData: IMarkdownObjPaths): string =>
            path.join(markdownHigherPath, markdownData.markdownRelativePath)
    );

    const allComponentPaths: IComponentPathInfo[] = packageDetails.map(
        (packageDetail: PackageDetail): IComponentPathInfo => ({
            paths: parseComponentPaths(packageDetail.packagePath).map(
                (componentData: IParsedComponentPath): string =>
                    componentData.componentPath
            ),
            packageName: packageDetail.packageName
        })
    );

    if (enableWatchMode) {
        chokidar
            .watch([
                ...pluck('paths', allComponentPaths),
                ...markdownRelativePath
            ])
            .on('change', (): void => {
                generateComponentMetadata(allComponentPaths);
                generateDocumentationMetadata(markdownObjPaths);
            });
    }

    generateComponentMetadata(allComponentPaths);
    generateDocumentationMetadata(markdownObjPaths);
    console.timeEnd(timerText);
}

initializeWatchMode();
errorLog();
