import * as chalk from 'chalk';
import * as ts from 'typescript';

import {
    ComponentDoc,
    FileParser,
    withCompilerOptions
} from 'react-docgen-typescript';
import { IExampleData } from '../../../types';
import { absToRelative, getStatusLine } from './consoleUtil';
import { getExampleFileNames, readFile } from './fileUtils';
import { getProgram } from './serviceHost';
import { isEqual } from 'lodash';
import {
    getComponentExampleFilePath,
    getComponentPropertyFilePath
} from './componentPathUtils';

const parser: FileParser = withCompilerOptions({
    jsx: ts.JsxEmit.React,
    maxNodeModuleJsDepth: 1,
    module: ts.ModuleKind.CommonJS,
    target: ts.ScriptTarget.Latest
});

const getExampleName = (content): string => {
    return content.match(/export default (function )?(\w+)/m)[2];
};

export function getParsedComponentData(
    filePath: string[] | string,
    componentName: string
): ComponentDoc[] {
    return parser.parseWithProgramProvider(
        getComponentPropertyFilePath(filePath as string, componentName),
        getProgram
    );
}

export function getParsedExampleData(
    componentPath: string,
    componentName?: string
): IExampleData[] {
    const relativePath: string = absToRelative(componentPath);
    const timerOutput: string = getStatusLine({
        msg: `${relativePath}/${chalk.bold(`${componentName}.tsx`)}`,
        status: 'success',
        statusMsg: 'DONE'
    });
    console.time(timerOutput);
    const exampleFiles: string[] = getExampleFileNames(
        componentPath,
        componentName
    ).filter(
        (exampleFileName: string): boolean => exampleFileName !== 'index.ts'
    );
    const exampleBase: string = getComponentExampleFilePath(componentPath);
    const examplePaths: string[] = exampleFiles.map(
        (f: string): string => `${exampleBase}/${f}`
    );
    const parsedComponentData: ComponentDoc[] = getParsedComponentData(
        examplePaths,
        componentName as string
    );

    const componentExampleData: IExampleData[] = exampleFiles.map(
        (exampleFileName: string): IExampleData => {
            const content: string = readFile(
                `${exampleBase}/${exampleFileName}`
            );

            const componentData:
                | ComponentDoc
                | undefined = parsedComponentData.find(
                (component: ComponentDoc): boolean =>
                    isEqual(getExampleName(content), component.displayName)
            );

            return {
                exampleName: exampleFileName.replace('.tsx', ''),
                exampleDescription: componentData && componentData.description,
                exampleCode: content
            };
        }
    );
    console.timeEnd(timerOutput);

    return componentExampleData;
}
