import { getDirectories, getPathParts } from './fileUtils';
import * as path from 'path';
import { IParsedComponentPath } from '../../../types/component.types';
import { camelCase } from 'lodash';
import { EXTERNAL_COMPONENTS, isExternalLibrary } from '../externalLibraries';

export function getComponentNames(
    componentGroupPath: string,
    componentGroup: string
): string[] {
    if (isExternalLibrary(componentGroupPath)) {
        const componentNames = Object.keys(EXTERNAL_COMPONENTS).reduce(
            (acc: string[], key: string): string[] =>
                EXTERNAL_COMPONENTS[key].componentGroup === componentGroup
                    ? [...acc, key]
                    : acc,
            []
        );
        return componentNames.length ? componentNames : [componentGroup];
    }
    return getDirectories(path.join(componentGroupPath, componentGroup));
}

export function convertToParsedComponentPath(
    componentGroupPath: string,
    componentGroup: string,
    componentName: string
): IParsedComponentPath | null {
    if (isExternalLibrary(componentGroupPath)) {
        const [base]: string[] = getPathParts(componentGroupPath);
        if (!EXTERNAL_COMPONENTS[componentName]) {
            console.warn(
                'WARN',
                "External library component named '" +
                    componentName +
                    "' does not exist in settings (externalLibrary.ts)"
            );
            return null;
        }
        return {
            componentName,
            componentGroup: camelCase(componentGroup),
            componentPath: path.join(
                base,
                EXTERNAL_COMPONENTS[componentName].componentFolder
            )
        };
    }

    return {
        componentName,
        componentGroup,
        componentPath: path.join(
            componentGroupPath,
            componentGroup,
            componentName
        )
    };
}

export function getComponentFilePath(
    componentPath: string,
    componentName: string
): string {
    const [base]: string[] = getPathParts(componentPath);
    return isExternalLibrary(componentPath)
        ? path.join(
              base,
              EXTERNAL_COMPONENTS[componentName].componentFolder,
              EXTERNAL_COMPONENTS[componentName].componentFilename
          )
        : path.join(componentPath, `${componentName}.tsx`);
}

export function getComponentPropertyFilePath(
    filePath: string,
    componentName: string
): string {
    if (isExternalLibrary(filePath)) {
        const [base]: string[] = getPathParts(filePath);
        const externalComponent = EXTERNAL_COMPONENTS[componentName];
        return path.join(
            base,
            externalComponent.componentFolder,
            externalComponent.componentPropertyFilename
        );
    }
    return filePath;
}

export function getComponentExampleFilePath(componentPath: string): string {
    const [base, componentParts]: string[] = getPathParts(componentPath);
    if (isExternalLibrary(componentParts)) {
        const componentName = componentParts.slice(
            componentParts.lastIndexOf(path.sep) + 1
        );
        return path.join(
            base,
            EXTERNAL_COMPONENTS[componentName].exampleFolder
        );
    }

    return path.join(
        base,
        'packages',
        'documentation',
        'src',
        componentParts.replace('components', 'examples')
    );
}

export function getComponentMarkdownFilePath(
    markdownFilePath: string,
    markdownFileName: string,
    componentName: string
): string {
    if (isExternalLibrary(markdownFilePath)) {
        const [base]: string[] = getPathParts(markdownFilePath);
        const filename = EXTERNAL_COMPONENTS[componentName].markdownFilePath;
        if (filename) {
            return path.join(base, filename);
        }
    }
    return path.join(markdownFilePath, markdownFileName);
}
