import * as chalk from 'chalk';
import * as fs from 'fs';
import { flatten } from 'lodash';
import * as path from 'path';

import {
    IParsedComponentPath,
    PackageDetail,
    packageDetails
} from '../../../types';
import {
    convertToParsedComponentPath,
    getComponentExampleFilePath,
    getComponentMarkdownFilePath,
    getComponentNames
} from './componentPathUtils';
import { getStatusLine } from './consoleUtil';

export function getPathParts(componentPath: string): [string, string] {
    const [base, componentParts]: string[] = componentPath.split(
        `packages${path.sep}`
    );

    return [base, componentParts];
}

export function getFiles(filepath: string): string[] {
    return fs
        .readdirSync(filepath)
        .filter(
            (file: string): React.ReactNode =>
                fs.statSync(path.join(filepath, file)).isFile()
        );
}
export function getDirectories(filepath: string): string[] {
    return fs
        .readdirSync(filepath)
        .filter(
            (file: string): React.ReactNode =>
                fs.statSync(path.join(filepath, file)).isDirectory()
        );
}

export function writeFile(filepath: string, content: string): void {
    fs.writeFile(filepath, content, (err: NodeJS.ErrnoException): void => {
        if (err) {
            console.log(chalk.red(err));
        } else {
            filepath.match(/componentData/)
                ? console.log(chalk.green('Component data saved.'))
                : console.log(chalk.green('Documentation data saved.'));
        }
    });
}

export function readFile(filePath: string): string {
    return fs.readFileSync(filePath, 'utf-8');
}

export function getFileNamesWithExt(
    directoryPath: string,
    fileExt: string = 'md'
): string[] {
    let fileNames: string[] = [];
    const regex: RegExp = fileExt === 'md' ? /\.md$/ : /\.tsx$/;
    try {
        fileNames = getFiles(directoryPath).filter(
            (fileName: string): React.ReactNode => regex.test(fileName)
        );
    } catch (error) {
        console.log(chalk.yellow(`path ${directoryPath} does not exist`));
    }

    return fileNames;
}

export function parseComponentPaths(
    componentGroupPath: string
): IParsedComponentPath[] {
    const componentGroups: string[] = getDirectories(componentGroupPath);

    return componentGroups.reduce(
        (
            parsedComponentPaths: IParsedComponentPath[],
            componentGroup: string
        ): IParsedComponentPath[] => {
            const componentNames: string[] = getComponentNames(
                componentGroupPath,
                componentGroup
            );

            componentNames.forEach((componentName: string): void => {
                const parsedComponentPath = convertToParsedComponentPath(
                    componentGroupPath,
                    componentGroup,
                    componentName
                );
                if (parsedComponentPath) {
                    parsedComponentPaths.push(parsedComponentPath);
                }
            });

            return parsedComponentPaths;
        },
        []
    );
}

export function getExampleFileNames(
    componentPath: string,
    componentName?: string
): string[] {
    let exampleFiles: string[] = [];

    const examplePath: string = getComponentExampleFilePath(componentPath);

    try {
        exampleFiles = getFiles(examplePath);
    } catch (error) {
        console.log(
            getStatusLine({
                msg: chalk.yellow(
                    `No examples found for ${chalk.bold(
                        componentName || componentPath
                    )}.`
                ),
                status: 'warn',
                statusMsg: 'WARN'
            })
        );
    }

    return exampleFiles;
}

export function getAllComponentFiles(): string[] {
    return flatten(
        flatten(
            packageDetails.map((packageDetail: PackageDetail): string[][] =>
                parseComponentPaths(packageDetail.packagePath).map(
                    ({
                        componentPath,
                        componentName
                    }: IParsedComponentPath): string[] => {
                        const examplePath: string = getComponentExampleFilePath(
                            componentPath
                        );
                        return getExampleFileNames(componentPath)
                            .filter((f: string): boolean => f !== 'index.ts')
                            .map((f: string): string => `${examplePath}/${f}`)
                            .concat(`${componentPath}/${componentName}.tsx`);
                    }
                )
            )
        )
    );
}

export function readMarkdownFile(
    markdownFilePath: string,
    markdownFileName: string,
    sectionName: string
): string {
    try {
        return readFile(
            getComponentMarkdownFilePath(
                markdownFilePath,
                markdownFileName,
                sectionName
            )
        );
    } catch (error) {
        console.log(
            getStatusLine({
                msg: chalk.yellow(
                    `No markdown found for ${chalk.bold(sectionName)}.`
                ),
                status: 'warn',
                statusMsg: 'WARN'
            })
        );
    }

    return '';
}
