import cx from 'classnames';
import React from 'react';
import ReactMarkdown from 'react-markdown';
import {
    IComponentData,
    IExampleData,
    INavigationLink
} from '../../../../types';
import { componentData } from '../../../config/data';
import { Example, FloatingNavigation, PropsTable } from '../../components';
import * as styles from './ComponentPage.scss';

interface IComponentPageProps {
    componentName: string;
    componentPath: string;
    currentPath: string;
}

export class ComponentPage extends React.Component<IComponentPageProps> {
    getCurrentComponentData = (): IComponentData => {
        const currentPackageName = this.props.currentPath.split('/')[1];

        return componentData.find(
            ({ componentName, packageName }): boolean =>
                packageName === currentPackageName &&
                componentName === this.props.componentName
        );
    };

    render(): React.ReactNode {
        const currentComponentData: IComponentData = this.getCurrentComponentData();
        const {
            componentName,
            componentDescription,
            componentProps,
            componentGroup,
            examples,
            markdownContent,
            packageName
        } = currentComponentData;

        if (!examples) {
            return <div>No Example</div>;
        }

        const Examples: React.ReactNode = examples.map(
            (example: IExampleData): React.ReactNode => (
                <Example
                    componentGroup={componentGroup}
                    componentName={componentName}
                    example={example}
                    key={`${componentGroup}-${componentName}-${example.exampleName}`}
                    packageName={packageName}
                />
            )
        );

        const navItems: INavigationLink[] = [
            {
                to: `${this.props.componentPath}/overview`,
                currentPath: this.props.currentPath,
                scrollToSelector: '#overview',
                children: 'Overview'
            },
            {
                to: `${this.props.componentPath}/example`,
                currentPath: this.props.currentPath,
                scrollToSelector: '#example',
                children: 'Example'
            },
            {
                to: `${this.props.componentPath}/props`,
                currentPath: this.props.currentPath,
                scrollToSelector: '#props',
                children: 'Props'
            }
        ];

        return (
            <div className="va-grid va-columns-default">
                <div
                    className={cx(
                        'col-span-xl-10 col-span-12',
                        styles.pageContent
                    )}
                >
                    <div className={styles.typography}>
                        <h2 className="h2" id="overview">
                            {componentName}
                        </h2>
                        {componentDescription && <p>{componentDescription}</p>}
                        {!!markdownContent && (
                            <ReactMarkdown source={markdownContent} />
                        )}
                    </div>
                    <div className={styles.exampleContainer}>
                        <div className={styles.typography}>
                            <h3 className="h3" id="example">
                                Example
                                {examples.length > 1 && 's'}
                            </h3>
                        </div>
                        {examples.length > 0 ? Examples : 'No examples exist.'}
                    </div>
                    <div className={styles.typography}>
                        <h3 className="h3" id="props">
                            Props
                        </h3>
                        {componentProps ? (
                            <PropsTable componentProps={componentProps} />
                        ) : (
                            'This component accepts no props.'
                        )}
                    </div>
                </div>
                <FloatingNavigation
                    currentPath={this.props.currentPath}
                    navItems={navItems}
                />
            </div>
        );
    }
}
