import cx from 'classnames';
import { find, propEq } from 'ramda';
import React from 'react';
import { Redirect } from 'react-router-dom';
import {
    IMatchParams,
    INavigationLink,
    ITableOfContents
} from '../../../../types';
import { tableOfContents } from '../../../compiled-scripts/tableOfContents';
import { FloatingNavigation } from '../../components';
import { buildDocumentation, scrollToSelector } from '../../utils';
import * as styles from './DocumentationPage.scss';

interface IDocumentationPageProps {
    matchParams: IMatchParams;
    headerSlug: string;
    currentPath: string;
}

export class DocumentationPage extends React.Component<
    IDocumentationPageProps
> {
    getLastSlugWithSelector = (): string => {
        const {
            anchors,
            tertiaryGroup,
            secondaryGroup,
            primaryGroup
        } = this.props.matchParams;

        return [anchors, tertiaryGroup, secondaryGroup, primaryGroup].find(
            (slug: string): boolean =>
                !!slug &&
                !!document.querySelector(`section[data-scroll=${slug}]`)
        );
    };

    scroll = (): void => {
        const slug: string = this.getLastSlugWithSelector();
        if (!slug) {
            return;
        }
        scrollToSelector(`section[data-scroll=${slug}]`);
    };

    recurseDocumentationData = (
        section: ITableOfContents,
        dirPath: string
    ): React.ReactNode => (
        <section data-scroll={section.slug} key={dirPath}>
            {buildDocumentation(dirPath)}
            {section.children &&
                section.children.map(
                    (childSection: ITableOfContents): React.ReactNode =>
                        this.recurseDocumentationData(
                            childSection,
                            `${dirPath}/${childSection.slug}`
                        )
                )}
        </section>
    );

    createFloatingNavItems = (
        childrenOfPrimary: ITableOfContents[]
    ): INavigationLink[] => {
        const { matchParams, headerSlug, currentPath } = this.props;
        const navItems: INavigationLink[] = [];
        if (matchParams.secondaryGroup) {
            const currentPage: ITableOfContents = find(
                propEq('slug', matchParams.secondaryGroup)
            )(childrenOfPrimary);
            currentPage.children.forEach((child: ITableOfContents): void => {
                navItems.push({
                    to: `/${headerSlug}/${matchParams.primaryGroup}/${matchParams.secondaryGroup}/${child.slug}`,
                    currentPath: currentPath,
                    children: child.title
                });
            });
        }

        return navItems;
    };

    componentDidUpdate(): void {
        this.scroll();
    }

    componentDidMount(): void {
        this.scroll();
    }

    render(): React.ReactNode {
        const { matchParams, headerSlug } = this.props;
        const headerGroup: ITableOfContents = find(propEq('slug', headerSlug))(
            tableOfContents
        );
        const childrenOfHeader: ITableOfContents[] = headerGroup.children;
        const primaryGroup: ITableOfContents = find(
            propEq('slug', matchParams.primaryGroup)
        )(childrenOfHeader);
        const childrenOfPrimary: ITableOfContents[] = primaryGroup.children;

        const navItems: INavigationLink[] = this.createFloatingNavItems(
            childrenOfPrimary
        );

        return primaryGroup ? (
            <div className="va-grid va-columns-default">
                <div
                    className={cx(
                        'col-span-xl-10 col-span-12',
                        styles.pageContent,
                        styles.typography
                    )}
                >
                    {this.recurseDocumentationData(
                        primaryGroup,
                        `${headerSlug}/${matchParams.primaryGroup}`
                    )}
                </div>
                {!!navItems.length && (
                    <FloatingNavigation
                        currentPath={`${headerSlug}/${matchParams.primaryGroup}`}
                        navItems={navItems}
                    />
                )}
            </div>
        ) : (
            <Redirect to={`/${headerSlug}`} />
        );
    }
}
