import cx from 'classnames';
import { keys } from 'ramda';
import React from 'react';
import { Signal } from '@preamp/signal';

import {
    IComponentData,
    IComponentGroup,
    IComponentGroups,
    ITableOfContents
} from '../../../../types';
import { tableOfContents } from '../../../compiled-scripts/tableOfContents';
import { componentData } from '../../../config/data';
import { NavigationLink } from '../../components';
import * as styles from './Navigation.scss';

interface INavigationProps {
    currentPath: string;
}

const PreampLogo: React.ReactNode = (
    <svg viewBox="0 0 41 41">
        <g
            fill="none"
            fillRule="evenodd"
            id="preamp-logo"
            stroke="none"
            strokeWidth="1"
        >
            <path
                d="M25,0 L13,13.494873 L25,13.494873 L0,41 L0,3 L0.00509269341,2.82372721 C0.0929596233,1.30724616 1.30724616,0.0929596233 2.82372721,0.00509269341 L3,0 L25,0 Z M3.03609866,1.24947865 L2.8960326,1.25299971 C2.06493688,1.30115451 1.38815866,1.93187739 1.27006758,2.71192318 L1.25457134,2.85982587 L1.25,3 L1.25,37.766 L22.174,14.744 L10.2157426,14.744873 L22.215,1.25 L3.03609866,1.24947865 Z"
                fill="#000"
                fillRule="nonzero"
                id="Combined-Shape"
            />
            <path
                d="M29,0 L29.0004626,0.0824443105 C35.7501656,0.828821053 41,6.55132826 41,13.5 C41,20.4486717 35.7501656,26.1711789 29.0004626,26.9175557 L29,27 L0,27 L0,3 L0.00509269341,2.82372721 C0.0929596233,1.30724616 1.30724616,0.0929596233 2.82372721,0.00509269341 L3,0 L29,0 Z M3.03609866,1.24947865 L2.8960326,1.25299971 C2.06493688,1.30115451 1.38815866,1.93187739 1.27006758,2.71192318 L1.25457134,2.85982587 L1.25,3 L1.25,25.75 L28.182,25.75 L28.863076,25.6751287 C34.9314238,25.0040956 39.5949397,19.9287998 39.746211,13.8069491 L39.75,13.5 C39.75,7.34419684 35.1859652,2.17898677 29.1653083,1.36207767 L28.863076,1.32487134 L28.191,1.25 L3.03609866,1.24947865 Z"
                fill="#000"
                fillRule="nonzero"
                id="Combined-Shape"
            />
        </g>
    </svg>
);

export const Navigation: React.SFC<INavigationProps> = (
    props: INavigationProps
): React.ReactElement => {
    const [menuOpen, setMenuOpen] = React.useState(false);
    const groupedComponentData: IComponentGroups = componentData.reduce(
        (componentGroups: IComponentGroups, currComp: IComponentData): any => {
            componentGroups[currComp.componentGroup]
                ? componentGroups[currComp.componentGroup].push({
                      componentName: currComp.componentName,
                      packageName: currComp.packageName
                  })
                : (componentGroups[currComp.componentGroup] = [
                      {
                          componentName: currComp.componentName,
                          packageName: currComp.packageName
                      }
                  ]);

            return componentGroups;
        },
        {}
    );

    function menuToggle(): void {
        setMenuOpen(!menuOpen);
    }

    function componentNames(groupName: string): React.ReactNode {
        return groupedComponentData[groupName].map(
            (component: IComponentGroup): React.ReactNode => {
                const path = `/${component.packageName}/${groupName}/${component.componentName}`;

                return (
                    <li key={path}>
                        <NavigationLink
                            activeClassName={styles.navLinkActive}
                            className={cx(
                                styles.navLink,
                                styles.navLinkTertiary
                            )}
                            currentPath={props.currentPath}
                            to={path}
                        >
                            {component.componentName}
                        </NavigationLink>
                        <ul
                            className={cx(
                                styles.navList,
                                styles.navListAnchors
                            )}
                        >
                            <li>
                                <NavigationLink
                                    activeClassName={styles.navLinkActive}
                                    className={cx(
                                        styles.navLink,
                                        styles.navLinkAnchor
                                    )}
                                    currentPath={props.currentPath}
                                    scrollToSelector="#overview"
                                    to={`${path}/overview`}
                                >
                                    Overview
                                </NavigationLink>
                            </li>
                            <li>
                                <NavigationLink
                                    activeClassName={styles.navLinkActive}
                                    className={cx(
                                        styles.navLink,
                                        styles.navLinkAnchor
                                    )}
                                    currentPath={props.currentPath}
                                    scrollToSelector="#example"
                                    to={`${path}/example`}
                                >
                                    Example
                                </NavigationLink>
                            </li>
                            <li>
                                <NavigationLink
                                    activeClassName={styles.navLinkActive}
                                    className={cx(
                                        styles.navLink,
                                        styles.navLinkAnchor
                                    )}
                                    currentPath={props.currentPath}
                                    scrollToSelector="#props"
                                    to={`${path}/props`}
                                >
                                    Props
                                </NavigationLink>
                            </li>
                        </ul>
                    </li>
                );
            }
        );
    }

    const renderGroupsListItems: (packageName: string) => React.ReactNode = (
        packageName: string
    ): React.ReactNode =>
        keys(groupedComponentData).reduce(
            (itemArray: React.ReactNode[], groupName: string): any => {
                const currentComponentGroup: IComponentGroup[] =
                    groupedComponentData[groupName];
                const currentPackageName: string =
                    currentComponentGroup[0].packageName;

                //if the current component group is in the given slug, render those links
                if (currentPackageName === packageName) {
                    itemArray.push(
                        <li key={`${packageName}-${groupName}`}>
                            <NavigationLink
                                activeClassName={styles.navLinkActive}
                                className={cx(
                                    styles.navLink,
                                    styles.navLinkSecondary
                                )}
                                currentPath={props.currentPath}
                                to={`/${currentPackageName}/${groupName}`}
                            >
                                {groupName}
                            </NavigationLink>
                            <ul
                                className={cx(
                                    styles.navList,
                                    styles.navListTertiary
                                )}
                            >
                                {componentNames(groupName)}
                            </ul>
                        </li>
                    );
                }

                return itemArray;
            },
            []
        );

    function buildNav(
        sections: ITableOfContents[],
        baseLink: string = '',
        level: number = 0
    ): React.ReactNode {
        return sections.map(
            (section: ITableOfContents): React.ReactNode => {
                const path = `${baseLink}/${section.slug}`;
                let navLinkListClass: string;
                let navLinkClass: string;

                switch (level) {
                    case 0:
                        navLinkClass = styles.navLinkPrimary;
                        navLinkListClass = styles.navListSecondary;
                        break;
                    case 1:
                        navLinkClass = styles.navLinkSecondary;
                        navLinkListClass = styles.navListTertiary;
                        break;
                    case 2:
                        navLinkClass = styles.navLinkTertiary;
                        navLinkListClass = styles.navListAnchors;
                        break;
                    case 3:
                        navLinkClass = styles.navLinkAnchor;
                        navLinkListClass = '';
                        break;
                    default:
                        navLinkClass = '';
                        navLinkListClass = '';
                }

                const slugToGroupListMap: Record<string, React.ReactNode> = {
                    core: renderGroupsListItems('core'),
                    'data-vis': renderGroupsListItems('data-vis'),
                    datepicker: renderGroupsListItems('datepicker'),
                    'preamp-plus': renderGroupsListItems('preamp-plus'), // external
                    rev: renderGroupsListItems('rev'),
                    select: renderGroupsListItems('select'),
                    signal: renderGroupsListItems('signal'),
                    tables: renderGroupsListItems('tables'),
                    waveform: renderGroupsListItems('waveform')
                };

                return [
                    <li
                        className={cx({
                            [styles.navListItem]: level === 0
                        })}
                        key={path}
                    >
                        <NavigationLink
                            activeClassName={styles.navLinkActive}
                            className={cx(styles.navLink, navLinkClass)}
                            currentPath={props.currentPath}
                            to={path}
                        >
                            {section.title}
                            {section.subtitle && (
                                <span>{section.subtitle}</span>
                            )}
                        </NavigationLink>
                        {slugToGroupListMap[section.slug] ? (
                            <ul
                                className={cx(
                                    styles.navList,
                                    styles.navListSecondary
                                )}
                                key={`component-group-${path}`}
                            >
                                {slugToGroupListMap[section.slug]}
                            </ul>
                        ) : (
                            section.children && (
                                <ul
                                    className={cx(
                                        styles.navList,
                                        navLinkListClass
                                    )}
                                    key={`documentation-group-${path}`}
                                >
                                    {buildNav(
                                        section.children,
                                        path,
                                        level + 1
                                    )}
                                </ul>
                            )
                        )}
                    </li>
                ];
            }
        );
    }

    return (
        <nav
            className={cx(styles.navigation, {
                [styles.navigationOpen]: menuOpen
            })}
        >
            <div
                className={cx(styles.pageBlocker, {
                    [styles.pageBlockerVisible]: menuOpen
                })}
                onClick={menuToggle}
                role="button"
            />
            <div className={styles.navHeadline}>
                <NavigationLink
                    className={styles.preampLogo}
                    currentPath={props.currentPath}
                    to="/"
                >
                    <Signal icon={PreampLogo} />
                    <span className="va-sr-only">PreAmp</span>
                </NavigationLink>
                <span>PreAmp Design System</span>
                <button
                    className={cx(styles.menuButton, {
                        [styles.menuButtonOpen]: menuOpen
                    })}
                    onClick={menuToggle}
                >
                    <span />
                </button>
            </div>
            <div
                className={cx(styles.navListContainer, {
                    [styles.navListOpen]: menuOpen
                })}
            >
                <ul className={styles.navList}>
                    {buildNav(tableOfContents, '', 0)}
                </ul>
            </div>
        </nav>
    );
};
