import React from 'react';

import { findIndex } from 'lodash';

import { Button, ButtonTheme, IStep, Modal, Stepper } from '@preamp/core';

interface WizardStep extends IStep {
    content: string;
}

const steps: WizardStep[] = [
    {
        id: 1,
        label: 'First',
        title: 1,
        content:
            "Did he just throw my cat out of the window? Hey, you know how I'm, like, always trying to save the planet? Here's my chance. I gave it a cold? I gave it a virus. A computer virus. Is this my espresso machine? Wh-what is-h-how did you get my espresso machine?"
    },
    {
        id: 2,
        label: 'Second',
        title: 2,
        content:
            "Checkmate... Just my luck, no ice. So you two dig up, dig up dinosaurs? You're a very talented young man, with your own clever thoughts and ideas. Do you need a manager? What do they got in there? King Kong? You really think you can fly that thing? God help us, we're in the hands of engineers."
    },
    {
        id: 3,
        label: 'Third',
        title: 3,
        content:
            "This thing comes fully loaded. AM/FM radio, reclining bucket seats, and... power windows. Hey, take a look at the earthlings. Goodbye! What do they got in there? King Kong? We gotta burn the rain forest, dump toxic waste, pollute the air, and rip up the OZONE! 'Cause maybe if we screw up this planet enough, they won't want it anymore!"
    },
    {
        id: 4,
        label: 'Fourth',
        title: 4,
        content:
            "Forget the fat lady! You're obsessed with the fat lady! Drive us out of here! Jaguar shark! So tell me - does it really exist? You know what? It is beets. I've crashed into a beet truck."
    }
];

const defaultDisabledSteps: IStep[] = steps.filter(
    (_: IStep, index: number): any => index > 0
);

interface WizardModalState {
    isModalOpen: boolean;
    currentStep: WizardStep;
    disabledSteps: IStep[];
    numberOfCompletedSteps: number;
}

/**
 * An example of a Modal as a wizard using nested components. Clicking outside the modal will not close it because shouldCloseOnOverlayClick is set to false.
 */
class WizardModal extends React.Component<{}, WizardModalState> {
    state: WizardModalState = {
        isModalOpen: false,
        currentStep: steps[0],
        disabledSteps: defaultDisabledSteps,
        numberOfCompletedSteps: 0
    };

    onClickHandler = (step: WizardStep): void => {
        const { numberOfCompletedSteps } = this.state;
        const stepIndex: number = findIndex(steps, step);
        this.setState({
            currentStep: step,
            numberOfCompletedSteps:
                stepIndex > numberOfCompletedSteps
                    ? stepIndex
                    : numberOfCompletedSteps
        });
    };

    incrementClickHandler = (): void => {
        const { numberOfCompletedSteps, currentStep } = this.state;
        const nextStep: number =
            findIndex(steps, currentStep) === steps.length - 1
                ? steps.length - 1
                : (findIndex(steps, currentStep) as number) + 1;
        const completedSteps: number =
            nextStep > numberOfCompletedSteps
                ? nextStep
                : numberOfCompletedSteps;
        this.setState({
            currentStep: steps[nextStep],
            disabledSteps: steps.filter(
                (_: IStep, index: number): any => index > completedSteps
            ),
            numberOfCompletedSteps: completedSteps
        });
    };

    previousStep = (): void => {
        const { currentStep } = this.state;
        const prevStep: number = findIndex(steps, currentStep) - 1;
        this.setState({
            currentStep: steps[prevStep]
        });
    };

    resetHandler = (): void => {
        this.setState({
            currentStep: steps[0],
            disabledSteps: defaultDisabledSteps,
            numberOfCompletedSteps: 0
        });
    };

    toggleModal = (): void => {
        this.resetHandler();
        this.setState((prevState: WizardModalState): any => ({
            isModalOpen: !prevState.isModalOpen
        }));
    };

    renderFooter = (): React.ReactNode => (
        <>
            <Button onClick={this.toggleModal} theme={ButtonTheme.Link}>
                Cancel
            </Button>
            <Button
                disabled={this.state.currentStep.id === 1}
                onClick={this.previousStep}
                theme={ButtonTheme.Secondary}
            >
                Previous Step
            </Button>
            {this.state.currentStep.id < steps.length ? (
                <Button
                    dataUI="increment-step"
                    onClick={this.incrementClickHandler}
                    theme={ButtonTheme.Primary}
                >
                    Continue to {steps[this.state.currentStep.id].label}
                </Button>
            ) : (
                <Button
                    dataUI="increment-step"
                    onClick={this.resetHandler}
                    theme={ButtonTheme.Primary}
                >
                    Reset Steps
                </Button>
            )}
        </>
    );

    render(): React.ReactNode {
        const {
            currentStep,
            disabledSteps,
            numberOfCompletedSteps
        } = this.state;

        return (
            <>
                <Button onClick={this.toggleModal}>Wizard Modal</Button>
                <Modal
                    actions={this.renderFooter()}
                    isOpen={this.state.isModalOpen}
                    onHide={this.toggleModal}
                    shouldCloseOnOverlayClick={false}
                    title="Wizard Modal"
                >
                    <Stepper
                        currentStep={currentStep}
                        disabledSteps={disabledSteps}
                        minWidth={300}
                        numberOfCompletedSteps={numberOfCompletedSteps}
                        onClick={this.onClickHandler}
                        steps={steps}
                    />

                    <h3 className="va-wizard-title u-title-3">
                        {currentStep.label}: this is a step in the process.
                    </h3>
                    <div className="va-wizard-description u-body-1">
                        The description for {currentStep.label}
                    </div>
                    <div className="va-wizard-content">
                        <p className="w-1/2 text-center my-8 mx-auto">
                            {currentStep.content}
                        </p>
                    </div>
                </Modal>
            </>
        );
    }
}

export default WizardModal;
