import React, { ReactElement, useState } from 'react';

import {
    Button,
    ButtonTheme,
    Notification,
    NotificationTheme,
    TextField,
    TextFieldType
} from '@preamp/core';
import { Signal } from '@preamp/signal';

interface BasicNotificationState {
    notificationOpen: boolean;
    notificationThemeValue: NotificationTheme;
    notificationTitle: string;
    customNotificationIcon: React.ReactNode;
    durationTime: string;
}

/**
 * Basic notification component.
 */
function ExampleNotification(): ReactElement<{}> {
    const [state, setState] = useState<BasicNotificationState>({
        notificationOpen: false,
        notificationThemeValue: NotificationTheme.Default,
        notificationTitle: 'Something Happened',
        customNotificationIcon: null,
        durationTime: ''
    });

    const customSVG: React.ReactNode = (
        <svg viewBox="0 0 24 24">
            <defs>
                <path d="M24 24H0V0h24v24z" id="a" />
            </defs>
            <path d="M3 4V1h2v3h3v2H5v3H3V6H0V4h3zm3 6V7h3V4h7l1.83 2H21c1.1 0 2 .9 2 2v12c0 1.1-.9 2-2 2H5c-1.1 0-2-.9-2-2V10h3zm7 9c2.76 0 5-2.24 5-5s-2.24-5-5-5-5 2.24-5 5 2.24 5 5 5zm-3.2-5c0 1.77 1.43 3.2 3.2 3.2s3.2-1.43 3.2-3.2-1.43-3.2-3.2-3.2-3.2 1.43-3.2 3.2z" />
        </svg>
    );

    function openNotification(): void {
        setState({
            ...state,
            customNotificationIcon: null,
            notificationOpen: true,
            notificationThemeValue: NotificationTheme.Default,
            notificationTitle: 'Your Report is Ready for Review.'
        });
    }
    function openNotificationSuccess(): void {
        setState({
            ...state,
            customNotificationIcon: null,
            notificationOpen: true,
            notificationThemeValue: NotificationTheme.Success,
            notificationTitle: 'Success!'
        });
    }
    function openNotificationWarning(): void {
        setState({
            ...state,
            customNotificationIcon: null,
            notificationOpen: true,
            notificationThemeValue: NotificationTheme.Warning,
            notificationTitle: 'Your report is still processing.'
        });
    }
    function openNotificationError(): void {
        setState({
            ...state,
            customNotificationIcon: null,
            notificationOpen: true,
            notificationThemeValue: NotificationTheme.Error,
            notificationTitle: 'Something went wrong.'
        });
    }
    function openNotificationCustomIcon(): void {
        setState({
            ...state,
            customNotificationIcon: <Signal icon={customSVG} />,
            notificationOpen: true,
            notificationThemeValue: NotificationTheme.Default,
            notificationTitle: 'Custom Icon Notification'
        });
    }

    function notificationHide(): void {
        setState({
            ...state,
            notificationOpen: false
        });
    }
    function onDurationChange(
        event?: React.ChangeEvent<HTMLInputElement>
    ): void {
        setState({
            ...state,
            durationTime: event.currentTarget.value
        });
    }

    return (
        <>
            <h4 className="py-4">Click a button to open the notification.</h4>
            <Button onClick={openNotification} theme={ButtonTheme.Primary}>
                Default Notification
            </Button>
            <Button
                onClick={openNotificationWarning}
                theme={ButtonTheme.Secondary}
            >
                Warning Notification
            </Button>
            <Button
                onClick={openNotificationSuccess}
                theme={ButtonTheme.Success}
            >
                Success Notification
            </Button>
            <Button
                onClick={openNotificationError}
                theme={ButtonTheme.Destructive}
            >
                Error Notification
            </Button>
            <Button
                onClick={openNotificationCustomIcon}
                theme={ButtonTheme.Secondary}
            >
                Notification with a CustomIcon
            </Button>
            <div className="va-grid va-columns-default py-8">
                <div className="col-span-4">
                    <TextField
                        helpText="Enter a number to set a duration. Leave blank to require users to close the notification."
                        label="Duration (ms)"
                        onChange={onDurationChange}
                        type={TextFieldType.Number}
                        value={state.durationTime}
                    />
                </div>
            </div>
            <Notification
                customIcon={state.customNotificationIcon}
                duration={parseFloat(state.durationTime)}
                isOpen={state.notificationOpen}
                onHide={notificationHide}
                theme={state.notificationThemeValue}
                title={state.notificationTitle}
            >
                {state.notificationThemeValue === NotificationTheme.Default && (
                    <div>
                        This is the default notification{' '}
                        {state.customNotificationIcon && (
                            <span>with a custom icon</span>
                        )}
                    </div>
                )}
                {state.notificationThemeValue === NotificationTheme.Error && (
                    <div>
                        Your dashboard will be available once the report has
                        finished.
                    </div>
                )}
                {state.notificationThemeValue === NotificationTheme.Success && (
                    <div>You have successfully done a thing. Congrats!</div>
                )}
                {state.notificationThemeValue === NotificationTheme.Warning && (
                    <div>Just warning you of something.</div>
                )}
            </Notification>
        </>
    );
}

export default ExampleNotification;
