import React, { ReactElement, useState } from 'react';

import { Button, ButtonTheme, Card, CardNav, Checkbox } from '@preamp/core';
import { DropdownButton, OptionType, ValueType } from '@preamp/select';
import { KeyboardArrowDown, LiveTv, Signal } from '@preamp/signal';

export const exampleOptions: OptionType[] = [
    { label: 'Edit', value: '1' },
    { label: 'Duplicate', value: '2' },
    { label: 'Delete', value: '3' }
];

const labelItemStyle: any = {
    label: {
        fontSize: '0.625rem',
        letterSpacing: '0.1em'
    },
    item: {
        fontSize: '0.925rem'
    }
};

function ExampleCardNav(): ReactElement<{}> {
    const [selected, setSelected] = useState([]);
    const [childElements, setChildElements] = useState([]);

    function handleSelection(id: any): number[] {
        let cardNavSelected: number[] = [...selected];
        if (!selected.includes(id)) {
            cardNavSelected.push(id);
        } else {
            cardNavSelected = cardNavSelected.filter(
                (value: number): React.ReactNode => value !== id
            );
        }

        return cardNavSelected;
    }

    function dropdownChanged(value: ValueType<OptionType>): void {
        // eslint-disable-next-line no-console
        console.log('changed', value);
    }

    function onSelect(id?: any, checked?: boolean): void {
        // eslint-disable-next-line no-console
        console.log(id, checked);
        setSelected(handleSelection(id));
    }

    function preventEventDefaults(event?: React.MouseEvent): void {
        event.preventDefault();
        event.stopPropagation();
    }

    function buttonClick(event): void {
        preventEventDefaults(event);
        alert('button clicked');
    }

    function onClick(id?: any): void {
        setSelected(handleSelection(id));
    }

    function onActionClick(event?: React.MouseEvent): void {
        preventEventDefaults(event);
        const idx: number = childElements.length;
        const newChild: ReactElement = (
            <CardNav
                className="mb-3"
                id={`child-${idx}`}
                isExpandable
                key={idx}
                title={`Example CardNav inside #${idx}`}
            >
                Test inside #{idx}
            </CardNav>
        );
        setChildElements([...childElements, newChild]);
    }

    return (
        <>
            <CardNav
                action={
                    <Checkbox
                        checked={selected.includes('1')}
                        name="checkbox"
                        onChange={(
                            checked: boolean,
                            event: React.MouseEvent
                        ): void => {
                            preventEventDefaults(event);
                            onSelect('1', checked);
                        }}
                    />
                }
                id={'1'}
                isExpanded
                isSelected={selected.includes('1')}
                onClick={(): void => onClick('1')}
                title="Example #1 Default"
                utility={
                    <div className="u-position-top-right flex-col">
                        <div>$50,000</div>
                        <div className="flex items-center">
                            <Signal icon={LiveTv} size={1.5} /> 5
                        </div>
                    </div>
                }
            >
                <div className="flex flex-row">
                    <div className="mr-4">
                        <ul>
                            <li style={labelItemStyle.label}>LABEL</li>
                            <li style={labelItemStyle.item}>Value</li>
                        </ul>
                    </div>
                    <div className="mr-4">
                        <ul>
                            <li style={labelItemStyle.label}>LABEL</li>
                            <li style={labelItemStyle.item}>Value</li>
                        </ul>
                    </div>
                </div>
                <CardNav
                    className="mb-3"
                    id={'2'}
                    isExpandable
                    title="Example CardNav inside"
                >
                    Test inside 1
                </CardNav>
                <CardNav
                    action={
                        <Checkbox
                            checked={selected.includes('3')}
                            name="checkbox"
                            onChange={(
                                checked: boolean,
                                event: React.MouseEvent
                            ): void => {
                                preventEventDefaults(event);
                                onSelect('3', checked);
                            }}
                        />
                    }
                    className="mb-3"
                    id={'3'}
                    isSelected={selected.includes('3')}
                    onClick={(): void => onClick('3')}
                    title="Example CardNav inside with checkbox"
                >
                    Test inside 2
                </CardNav>
                {childElements.map((el: ReactElement): React.ReactNode => el)}
                <Button
                    isFullWidth
                    onClick={onActionClick}
                    theme={ButtonTheme.Secondary}
                >
                    + Add More
                </Button>
            </CardNav>
            <div className="m-4">Checked: {JSON.stringify(selected)}</div>
            <CardNav
                action={
                    <div onClick={preventEventDefaults} role="button">
                        <DropdownButton
                            dataUI="icon-dropdown"
                            icon={KeyboardArrowDown}
                            onChange={dropdownChanged}
                            options={exampleOptions}
                        />
                    </div>
                }
                isExpandable
                isSelected
                title="Card Nav # 2 Expandable"
                utility={
                    <div
                        className="flex-col justify-end"
                        style={{ fontWeight: 500, fontSize: '16px' }}
                    >
                        <div>$150,000</div>
                        <div className="flex items-center justify-end">
                            <Signal icon={LiveTv} size={1.5} /> 3
                        </div>
                    </div>
                }
            >
                <div className="items flex flex-row mb-2">
                    <div className="item flex flex-col mr-4">
                        <span style={labelItemStyle.label}>LABEL</span>
                        <span style={labelItemStyle.item}>Value</span>
                    </div>
                </div>
                <Card title="Example Card 2">
                    <p>test card inside</p>
                    <p>
                        <Button
                            onClick={buttonClick}
                            theme={ButtonTheme.Primary}
                        >
                            Click Me
                        </Button>
                    </p>
                </Card>
            </CardNav>

            <div className="m-4">Long title example:</div>
            <CardNav
                action={
                    <Checkbox
                        checked={selected.includes('5')}
                        name="checkbox"
                        onChange={(
                            checked: boolean,
                            event: React.MouseEvent
                        ): void => {
                            preventEventDefaults(event);
                            onSelect('5', checked);
                        }}
                    />
                }
                id={'5'}
                isExpanded
                isSelected={selected.includes('5')}
                onClick={(): void => onClick('5')}
                title="Example #3 Default with a hold lot of extra characters that make it an extremely long title for this particular card navigational item"
                utility={
                    <div className="flex-col justify-end">
                        <div>$1,450,000</div>
                        <div className="flex items-center justify-end">
                            <Signal icon={LiveTv} /> 5
                        </div>
                    </div>
                }
            >
                Card content
            </CardNav>
        </>
    );
}

export default ExampleCardNav;
