import React from 'react';

import {
    Button,
    ButtonTheme,
    Drawer,
    DrawerTab,
    DrawerTheme,
    RadioButton,
    RadioButtonGroup,
    Toggle
} from '@preamp/core';

interface ExampleDrawerState {
    isCollapsed: boolean;
    isLockedOpen: boolean;
    selectedTheme: string;
    isTabsToggled: boolean;
    selectedTabId: string;
}

/**
 * Default theme: The drawer is positioned to the right of the screen and should span the full
 * height of the screen and open onMouseEnter. To lock the drawer open, click
 * the top left arrow.
 * Floating theme: The drawer is positioned to the right of the screen and should float over the
 * screen. It is opened and closed programmatically, and can also be closed with the X in the top
 * right.
 */
class ExampleDrawer extends React.Component {
    state: ExampleDrawerState = {
        isCollapsed: true,
        isLockedOpen: false,
        selectedTheme: DrawerTheme.Default,
        isTabsToggled: false,
        selectedTabId: 'tab1'
    };

    tabs: DrawerTab[] = [
        {
            content: <div>Test content 1</div>,
            identifier: 'tab1',
            label: 'Tab 1 Title'
        },
        {
            content: <div>Test content 2</div>,
            identifier: 'tab2',
            label: 'Tab 2 Title'
        },
        {
            content: <div>Test content 3</div>,
            identifier: 'tab3',
            label: 'Tab 3 Title'
        }
    ];

    onMouseEnter = (): void => {
        if (this.state.selectedTheme === DrawerTheme.Floating) {
            console.log('onMouseEnter');
        } else {
            const { isLockedOpen } = this.state;
            if (!isLockedOpen) {
                this.setState({ isCollapsed: false });
            }
        }
    };

    onMouseLeave = (): void => {
        if (this.state.selectedTheme === DrawerTheme.Floating) {
            console.log('onMouseLeave');
        } else {
            const { isLockedOpen } = this.state;
            if (!isLockedOpen) {
                this.setState({ isCollapsed: true });
            }
        }
    };

    toggleOpen = (): void => {
        const { isLockedOpen } = this.state;

        this.setState({
            isCollapsed: false,
            isLockedOpen: !isLockedOpen
        });
    };

    onSetIsCollapsed = (isCollapsed: boolean): void => {
        this.setState({
            isCollapsed: isCollapsed
        });
    };

    onChangeThemeHandler = (selectedValue: string): void => {
        this.setState({ selectedTheme: selectedValue });
    };

    onToggleTabs = (checked: boolean): void => {
        this.setState({ isTabsToggled: checked });
    };

    onSetSelectedTabId = (selectedTabId: string): void => {
        this.setState({
            selectedTabId: selectedTabId
        });
    };

    render(): React.ReactNode {
        const { isCollapsed, isLockedOpen } = this.state;
        const { onMouseEnter, onMouseLeave, toggleOpen } = this;

        return (
            <>
                <RadioButtonGroup
                    isInline={false}
                    isOptional={true}
                    key={`form-element-1-group`}
                    name={'radioButtonGroup'}
                    onChange={this.onChangeThemeHandler}
                    required={false}
                    selectedValue={this.state.selectedTheme}
                    title={'Drawer Theme:'}
                >
                    <RadioButton
                        key={'radioButton1'}
                        label={'Default'}
                        name={'radioButton'}
                        value={DrawerTheme.Default}
                    />
                    <RadioButton
                        key={'radioButton2'}
                        label={'Floating'}
                        name={'radioButton'}
                        value={DrawerTheme.Floating}
                    />
                </RadioButtonGroup>
                <div className="ml-8">
                    <Button
                        dataUI="secondary"
                        disabled={
                            this.state.selectedTheme === DrawerTheme.Floating
                                ? false
                                : true
                        }
                        onClick={() => this.onSetIsCollapsed(false)}
                        theme={ButtonTheme.Secondary}
                    >
                        Show Drawer
                    </Button>
                    <div className="mt-2">
                        <Toggle
                            checked={this.state.isTabsToggled}
                            disabled={
                                this.state.selectedTheme ===
                                DrawerTheme.Floating
                                    ? false
                                    : true
                            }
                            label="Enable tabs header"
                            name="tabs-toggle"
                            onChange={this.onToggleTabs}
                        />
                    </div>
                </div>
                <Drawer
                    isCollapsed={isCollapsed}
                    isLockedOpen={isLockedOpen}
                    onClick={toggleOpen}
                    onMouseEnter={onMouseEnter}
                    onMouseLeave={onMouseLeave}
                    selectedTabId={this.state.selectedTabId}
                    setIsCollapsed={this.onSetIsCollapsed}
                    setSelectedTabId={this.onSetSelectedTabId}
                    tabs={this.state.isTabsToggled ? this.tabs : null}
                    theme={
                        this.state.selectedTheme === DrawerTheme.Floating
                            ? DrawerTheme.Floating
                            : DrawerTheme.Default
                    }
                    title="Drawer Title"
                >
                    {(this.state.selectedTheme === DrawerTheme.Floating ||
                        !isCollapsed) && (
                        <>
                            <p className="u-body-2">
                                Lorem ipsum dolor sit amet, consectetur
                                adipiscing elit, sed do eiusmod tempor
                                incididunt ut labore et dolore magna aliqua.
                            </p>
                            <p className="u-body-2 mt-4">
                                Lorem Ipsum is simply dummy text of the printing
                                and typesetting industry. Lorem Ipsum has been
                                the industry’s standard dummy text ever since
                                the 1500s, when an unknown printer took a galley
                                of type and scrambled it to make a type specimen
                                book.
                            </p>
                            <p className="u-body-2 mt-4">
                                It has survived not only five centuries, but
                                also the leap into electronic typesetting,
                                remaining essentially unchanged. It was
                                popularised in the 1960s with the release of
                                Letraset sheets containing Lorem Ipsum passages,
                                and more recently with desktop publishing
                                software like Aldus PageMaker including versions
                                of Lorem Ipsum.
                            </p>
                            <p className="u-body-2">
                                Lorem ipsum dolor sit amet, consectetur
                                adipiscing elit, sed do eiusmod tempor
                                incididunt ut labore et dolore magna aliqua.
                            </p>
                            <p className="u-body-2 mt-4">
                                Lorem Ipsum is simply dummy text of the printing
                                and typesetting industry. Lorem Ipsum has been
                                the industry’s standard dummy text ever since
                                the 1500s, when an unknown printer took a galley
                                of type and scrambled it to make a type specimen
                                book.
                            </p>
                            <p className="u-body-2 mt-4">
                                It has survived not only five centuries, but
                                also the leap into electronic typesetting,
                                remaining essentially unchanged. It was
                                popularised in the 1960s with the release of
                                Letraset sheets containing Lorem Ipsum passages,
                                and more recently with desktop publishing
                                software like Aldus PageMaker including versions
                                of Lorem Ipsum.
                            </p>
                        </>
                    )}
                </Drawer>
            </>
        );
    }
}

export default ExampleDrawer;
