import * as React from 'react';

import { filter, without } from 'lodash';

import { Drilldown, DrilldownItemType, Toggle } from '@preamp/core';

const sampleData: DrilldownItemType[] = [
    {
        id: 'age-and-gender',
        title: 'Age and Gender',
        items: [
            {
                id: 'item-1',
                title: 'Item 1',
                items: [
                    {
                        id: 'item-3',
                        title: 'Item 3',
                        subtitle: 'Lorem ipsum',
                        items: [
                            {
                                id: 'item-4',
                                title: 'Item 4',
                                subtitle: 'Lorem ipsum'
                            }
                        ]
                    }
                ]
            },
            { id: 'item-2', title: 'Item 2' }
        ]
    },
    {
        id: 'mri',
        title: 'MRI',
        items: [
            {
                id: 'auto',
                title: 'Auto',
                items: [
                    {
                        id: 'mri-child-1',
                        title:
                            'Business Purchases: Advertising/promotion [2015 Oct]',
                        subtitle: 'MRI - 34929 - 12,765,382'
                    }
                ]
            },
            {
                id: 'biz-purchases',
                title: 'Biz Purchases',
                items: [
                    {
                        id: 'mri-child-2',
                        title:
                            'Business Purchases: Advertising/promotion [2015 Oct]',
                        subtitle: 'MRI - 34929 - 12,765,382'
                    }
                ]
            }
        ]
    },
    {
        id: 'mri-custom',
        title: 'MRI Custom',
        items: [
            {
                id: 'item-6',
                title: 'Test 6',
                subtitle: 'Lorem ipsum'
            }
        ]
    },
    {
        id: 'non-selectable',
        disabled: true,
        title: 'Can not be selected'
    }
];

interface ExampleDrilldownState {
    activeItem?: DrilldownItemType;
    searchQuery?: string;
    filteredItems: DrilldownItemType[];
    isParentSelectable: boolean;
    lockChildrenOnSelect: boolean;
    selectedItemIds: string[];
    toolbarDisplay: boolean;
}

function reducer(
    state: ExampleDrilldownState,
    update: Partial<ExampleDrilldownState>
): ExampleDrilldownState {
    return { ...state, ...update };
}

/**
 *  A drilldown has two modes. The first only allows the selection of child-only elements. The second allows
 * selecting parent elements as well. This example allows toggling between both modes.
 */
const ExampleDrilldown: React.FC = (): React.ReactElement => {
    const [
        {
            activeItem,
            filteredItems,
            isParentSelectable,
            lockChildrenOnSelect,
            searchQuery,
            selectedItemIds,
            toolbarDisplay
        },
        updateState
    ] = React.useReducer(reducer, {
        activeItem: null,
        filteredItems: sampleData,
        isParentSelectable: false,
        lockChildrenOnSelect: false,
        searchQuery: '',
        selectedItemIds: [],
        toolbarDisplay: true
    });

    function onNavigate(newActiveItem: DrilldownItemType): void {
        updateState({ activeItem: newActiveItem, searchQuery: '' });
    }

    function onClear(): void {
        updateState({ filteredItems: sampleData, searchQuery: '' });
    }

    function onSearch(event: React.ChangeEvent<HTMLInputElement>): void {
        const newQuery: string = event.currentTarget.value;
        const currentItems: DrilldownItemType[] = activeItem
            ? activeItem.items
            : sampleData;

        if (newQuery.length > 0) {
            const searchedItems: DrilldownItemType[] = filter(
                currentItems,
                (item: DrilldownItemType): boolean =>
                    item.title.toLowerCase().includes(newQuery.toLowerCase())
            );
            updateState({
                filteredItems: searchedItems,
                searchQuery: newQuery
            });
        } else {
            updateState({
                filteredItems: sampleData,
                searchQuery: ''
            });
        }
    }

    function isChild(
        itemToSearch: DrilldownItemType,
        idToFind: string
    ): boolean {
        if (itemToSearch.items && itemToSearch.items.length > 0) {
            return itemToSearch.items.some(
                (childItem: DrilldownItemType): boolean => {
                    if (childItem.id && childItem.id === idToFind) {
                        return true;
                    } else if (childItem.items && childItem.items.length > 0) {
                        return isChild(childItem, idToFind);
                    } else {
                        return false;
                    }
                }
            );
        } else {
            return false;
        }
    }

    function addChildren(
        items: DrilldownItemType[],
        currentSelectedIds: string[]
    ): string[] {
        let newSelectedIds: string[] = [...currentSelectedIds];
        items.forEach((item: DrilldownItemType): void => {
            newSelectedIds = newSelectedIds.concat(item.id);
            if (item.items && item.items.length > 0) {
                newSelectedIds = addChildren(item.items, newSelectedIds);
            }
        });

        return newSelectedIds;
    }

    function removeChildren(
        items: DrilldownItemType[],
        currentSelectedIds: string[]
    ): string[] {
        let newSelectedIds: string[] = [...currentSelectedIds];
        items.forEach((item: DrilldownItemType): void => {
            newSelectedIds = without(newSelectedIds, item.id);
            if (item.items && item.items.length > 0) {
                newSelectedIds = removeChildren(item.items, newSelectedIds);
            }
        });

        return newSelectedIds;
    }

    function removeParentsAndChildren(
        selectedItems: string[],
        idToRemove: string,
        items: DrilldownItemType[]
    ): string[] {
        let selectedItemsUpdated: string[] = selectedItems;

        items.forEach((item: DrilldownItemType): void => {
            if (isChild(item, idToRemove)) {
                selectedItemsUpdated = without(selectedItemsUpdated, item.id);
                if (item.items && item.items.length > 0) {
                    selectedItemsUpdated = removeParentsAndChildren(
                        selectedItemsUpdated,
                        idToRemove,
                        item.items
                    );
                }
            } else if (
                item.id === idToRemove &&
                item.items &&
                item.items.length > 0
            ) {
                selectedItemsUpdated = removeChildren(
                    item.items,
                    selectedItemsUpdated
                );
            }
        });

        return selectedItemsUpdated;
    }

    function onSelect(selectedItem: DrilldownItemType): void {
        const isItemSelected: boolean = selectedItemIds.includes(
            selectedItem.id
        );
        let selectedIds: string[];
        if (isItemSelected) {
            selectedIds = without(selectedItemIds, selectedItem.id);
            if (isParentSelectable) {
                selectedIds = removeParentsAndChildren(
                    selectedIds,
                    selectedItem.id,
                    sampleData
                );
            }
        } else {
            selectedIds = selectedItemIds.concat(selectedItem.id);
            if (
                isParentSelectable &&
                selectedItem.items &&
                selectedItem.items.length > 0
            ) {
                selectedIds = addChildren(selectedItem.items, selectedIds);
            }
        }
        updateState({ selectedItemIds: selectedIds });
    }

    function toggleLockChildren(toggleState: boolean): void {
        updateState({
            lockChildrenOnSelect: toggleState,
            selectedItemIds: [],
            activeItem: null
        });
    }

    function toggleParentSelect(toggleState: boolean): void {
        updateState({
            isParentSelectable: toggleState,
            selectedItemIds: [],
            activeItem: null
        });
    }

    function toggleToolbarDisplay(toggleState: boolean): void {
        updateState({
            toolbarDisplay: toggleState
        });
    }

    return (
        <>
            <div className="flex justify-between my-3 bg-grey-lightest p-1">
                <Toggle
                    checked={isParentSelectable}
                    label="Parent selection enabled"
                    name="parent-selectable--ExampleDrilldown"
                    onChange={toggleParentSelect}
                />
                {isParentSelectable && (
                    <Toggle
                        checked={lockChildrenOnSelect}
                        label="Lock children"
                        name="lock-children--ExampleDrilldown"
                        onChange={toggleLockChildren}
                    />
                )}
                <Toggle
                    checked={toolbarDisplay}
                    label="Toolbar is visible"
                    name="toolbar-display--ExampleDrilldown"
                    onChange={toggleToolbarDisplay}
                />
            </div>
            <p className="mb-2 text-right">
                Selected Items: {selectedItemIds.length}
            </p>
            <div className="h-64">
                <Drilldown
                    activeItem={activeItem}
                    dataUI="drilldown-example--1"
                    isParentSelectable={isParentSelectable}
                    isToolbarVisible={toolbarDisplay}
                    items={filteredItems}
                    lockChildrenOnSelect={lockChildrenOnSelect}
                    onClear={onClear}
                    onNavigate={onNavigate}
                    onSearch={onSearch}
                    onSelect={onSelect}
                    searchQuery={searchQuery}
                    selectedItemIds={selectedItemIds}
                />
            </div>
        </>
    );
};

export default ExampleDrilldown;
