import * as React from 'react';

import { findIndex, uniq, without } from 'lodash';

import { Drilldown, DrilldownItemType } from '@preamp/core';

const sampleData: DrilldownItemType[] = [
    {
        id: 'age-and-gender',
        hasChildren: true,
        title: 'Age and Gender'
    },
    {
        id: 'mri',
        hasChildren: true,
        title: 'MRI'
    },
    {
        id: 'mri-custom',
        title: 'MRI Custom'
    },
    {
        id: 'mri-custom2',
        title: 'MRI Custom 2'
    },
    {
        id: 'mri-custom3',
        title: 'MRI Custom 3'
    },
    {
        id: 'mri-custom4',
        title: 'MRI Custom 4'
    }
];

const fakeChildren: DrilldownItemType[] = [
    {
        id: 'item-4',
        title: 'Example child'
    }
];

interface ExampleLazyLoadDrilldownState {
    activeItem: DrilldownItemType;
    isLoading: boolean;
    loadedItems: DrilldownItemType[];
    scrollCount: number;
    selectedItemIds: string[];
}

/**
 * Infinite scrolling Drilldown example where scrolling to the bottom of the first level item list triggers loading of more data.
 */
const ExampleLazyLoadDrilldown: React.FC = (): React.ReactElement => {
    const [state, setState] = React.useState<ExampleLazyLoadDrilldownState>({
        activeItem: null,
        isLoading: false,
        loadedItems: sampleData,
        scrollCount: 0,
        selectedItemIds: []
    });

    function updateData(count: number): void {
        setState(
            (prevState): ExampleLazyLoadDrilldownState => {
                const newData: DrilldownItemType[] = sampleData.map(
                    (item: DrilldownItemType): DrilldownItemType => ({
                        ...item,
                        id: `${item.id}-${count}`,
                        title: `${item.title} - # ${count}`
                    })
                );
                return {
                    ...prevState,
                    isLoading: false,
                    loadedItems: [...prevState.loadedItems, ...newData],
                    scrollCount: count
                };
            }
        );
    }

    function onNavigate(newActiveItem?: DrilldownItemType): void {
        setState(
            (prevState): ExampleLazyLoadDrilldownState => ({
                ...prevState,
                isLoading: true,
                scrollCount: 0
            })
        );

        if (newActiveItem && newActiveItem.items) {
            // children have already been loaded
            setState(
                (prevState): ExampleLazyLoadDrilldownState => ({
                    ...prevState,
                    activeItem: newActiveItem,
                    isLoading: false
                })
            );
        } else {
            // simulate load time
            setTimeout((): void => {
                setState(
                    (prevState): ExampleLazyLoadDrilldownState => {
                        if (newActiveItem) {
                            const parentIndex: number = findIndex(
                                prevState.loadedItems,
                                (item: DrilldownItemType): boolean =>
                                    newActiveItem.id === item.id
                            );
                            prevState.loadedItems[
                                parentIndex
                            ].items = fakeChildren;
                        }

                        return {
                            ...prevState,
                            activeItem: newActiveItem,
                            isLoading: false
                        };
                    }
                );
            }, 1000);
        }
    }

    function onSelect(selectedItem: DrilldownItemType): void {
        setState(
            (prevState): ExampleLazyLoadDrilldownState => {
                const isItemSelected: boolean = prevState.selectedItemIds.includes(
                    selectedItem.id
                );
                let selectedIds: string[];

                if (isItemSelected) {
                    selectedIds = without(
                        prevState.selectedItemIds,
                        selectedItem.id
                    );
                } else {
                    selectedIds = prevState.selectedItemIds.concat(
                        selectedItem.id
                    );
                }
                return {
                    ...prevState,
                    selectedItemIds: uniq(selectedIds)
                };
            }
        );
    }

    function infiniteScroll(): void {
        if (state.scrollCount < 3) {
            setState(
                (prevState): ExampleLazyLoadDrilldownState => ({
                    ...prevState,
                    isLoading: true
                })
            );

            setTimeout((): void => {
                updateData(state.scrollCount + 1);
            }, 2000);
        }
    }

    return (
        <div className="h-64">
            <Drilldown
                activeItem={state.activeItem}
                dataUI="drilldown-example--2"
                isLoading={state.isLoading}
                items={state.loadedItems}
                onNavigate={onNavigate}
                onScrollToBottom={infiniteScroll}
                onSelect={onSelect}
                selectedItemIds={state.selectedItemIds}
            />
        </div>
    );
};

export default ExampleLazyLoadDrilldown;
