import * as React from 'react';

import { filter, reduce } from 'lodash';

import {
    Button,
    ButtonTheme,
    Drilldown,
    DrilldownEditItem,
    DrilldownItemProps,
    DrilldownItemType,
    RightRail,
    RightRailTheme,
    TextField
} from '@preamp/core';

interface CustomItemsExampleState {
    activeItem?: DrilldownItemType;
    searchQuery?: string;
    selectedItemIds: string[];
    selectedItem?: DrilldownItemType;
    successItemId?: string;
    filteredItems: DrilldownItemType[];
}

// Individual Item function, applied to a single item
function itemFunction(props: DrilldownItemProps): React.ReactNode {
    return (
        <DrilldownEditItem {...props}>
            {`Custom - ${props.item.title} - no success state`}
        </DrilldownEditItem>
    );
}

const sampleData: DrilldownItemType[] = [
    {
        id: 'age-and-gender',
        items: [
            {
                id: 'item-4',
                items: [
                    {
                        id: 'item-5',
                        title: 'Test 5'
                    }
                ],
                title: 'Test 4',
                itemFunction
            }
        ],
        title: 'Age and Gender'
    },
    {
        id: 'mri',
        items: [
            {
                id: 'auto',
                items: [
                    {
                        id: 'mri-child-1',
                        title:
                            'Business Purchases: Advertising/promotion [2015 Oct]'
                    }
                ],
                title: 'Auto'
            },
            {
                id: 'biz-purchases',
                items: [
                    {
                        id: 'mri-child-2',
                        title:
                            'Business Purchases: Advertising/promotion [2015 Oct]'
                    }
                ],
                title: 'Biz Purchases'
            }
        ],
        title: 'MRI'
    },
    {
        id: 'mri-custom',
        items: [
            {
                id: 'item-6',
                title: 'Test 6'
            }
        ],
        title: 'MRI Custom'
    },
    {
        id: 'non-selectable',
        isSelectable: false,
        title: 'Can not be selected'
    }
];

/**
 *  Custom Drilldown Items can be supplied to replace the default. The custom component is used by both parent and child nodes.
 *  There are two methods for supplying Item components, globally through the itemFunction prop of the drilldown,
 *  or by providing an itemFunction on the item in the data.
 */
class CustomItemsExample extends React.Component<{}, CustomItemsExampleState> {
    state: CustomItemsExampleState = {
        filteredItems: sampleData,
        selectedItemIds: [sampleData[0].id],
        selectedItem: sampleData[0]
    };

    getAllItems(): DrilldownItemType[] {
        return reduce(
            sampleData,
            (
                output: DrilldownItemType[],
                item: DrilldownItemType
            ): DrilldownItemType[] => output.concat(item, item.items),
            []
        );
    }

    onNavigate = (activeItem: DrilldownItemType): void => {
        this.setState({ activeItem, searchQuery: null });
    };

    onSearch = (event: React.ChangeEvent<HTMLInputElement>): void => {
        const searchQuery: string = event.currentTarget.value;

        if (searchQuery.length > 0) {
            const filteredItems: DrilldownItemType[] = filter(
                this.getAllItems(),
                (item: DrilldownItemType): boolean =>
                    item.title.toLowerCase().includes(searchQuery.toLowerCase())
            );

            this.setState({
                filteredItems,
                searchQuery
            });
        } else {
            this.setState({
                filteredItems: sampleData,
                searchQuery: null
            });
        }
    };

    onButtonClick = (): void => {
        const { selectedItem } = this.state;
        this.setState({ successItemId: selectedItem.id });
    };

    onSuccessAnimationComplete = (): void => {
        this.setState({ successItemId: null });
    };

    onFieldChange = (event: React.ChangeEvent<HTMLInputElement>): void => {
        const { selectedItem } = this.state;
        this.setState({
            selectedItem: { ...selectedItem, title: event.currentTarget.value }
        });
    };

    onSelect = (selectedItem: DrilldownItemType): void => {
        if (selectedItem.id !== this.state.selectedItemIds[0]) {
            this.setState({ selectedItemIds: [selectedItem.id], selectedItem });
        }
    };

    // Globally specified item function, for all items in the drilldown
    itemFunction = (props: DrilldownItemProps): React.ReactNode => {
        const { onSuccessAnimationComplete } = this;
        const { successItemId } = this.state;

        return (
            <DrilldownEditItem
                {...props}
                isSuccess={props.item.id === successItemId}
                onSuccessComplete={onSuccessAnimationComplete}
            >
                {props.item.title}
            </DrilldownEditItem>
        );
    };

    render(): React.ReactNode {
        const {
            activeItem,
            filteredItems,
            searchQuery,
            selectedItemIds,
            selectedItem
        } = this.state;
        const { onButtonClick, onFieldChange, onNavigate, onSelect } = this;

        return (
            <>
                <div className="h-64 flex">
                    <div style={{ flex: '0 65%' }}>
                        <Drilldown
                            activeItem={activeItem}
                            dataUI="drilldown-example--3"
                            isParentSelectable
                            itemFunction={this.itemFunction}
                            items={filteredItems}
                            onNavigate={onNavigate}
                            onSelect={onSelect}
                            searchQuery={searchQuery}
                            selectedItemIds={selectedItemIds}
                        />
                    </div>
                    {selectedItem && (
                        <div
                            style={{
                                flex: '0 35%',
                                height: '100%',
                                marginLeft: 16
                            }}
                        >
                            <RightRail
                                className="h-full flex flex-col justify-between"
                                theme={RightRailTheme.Item}
                                title={`Editing ${selectedItem.title}`}
                            >
                                <div>
                                    <TextField
                                        label="name"
                                        onChange={onFieldChange}
                                        value={selectedItem.title}
                                    />
                                </div>
                                <div className="self-end py-2">
                                    <Button
                                        onClick={onButtonClick}
                                        theme={ButtonTheme.Secondary}
                                    >
                                        Cancel
                                    </Button>
                                    <Button
                                        onClick={onButtonClick}
                                        theme={ButtonTheme.Primary}
                                    >
                                        Save Changes
                                    </Button>
                                </div>
                            </RightRail>
                        </div>
                    )}
                </div>
            </>
        );
    }
}

export default CustomItemsExample;
