import * as React from 'react';

import { filter, reduce } from 'lodash';

import {
    Drilldown,
    DrilldownEditItem,
    DrilldownItemProps,
    DrilldownItemType
} from '@preamp/core';

interface EmptyMessagesExampleState {
    activeItem?: DrilldownItemType;
    searchQuery?: string;
    selectedItem?: DrilldownItemType;
    selectedItemIds: string[];
    filteredItems: DrilldownItemType[];
}

const sampleData: DrilldownItemType[] = [
    {
        id: 'top-level',
        items: [
            {
                hasChildren: true,
                id: 'empty-sub-parent',
                items: [],
                title: 'Empty Sub-Parent'
            }
        ],
        title: 'Top Level With Children'
    },
    {
        id: 'top-level-children',
        items: [
            {
                id: 'sub-level-children',
                items: [
                    {
                        id: 'leaf-child-1',
                        title: 'Leaf Child'
                    }
                ],
                title: 'Sub-Level With Children'
            },
            {
                id: 'sub-level-children-2',
                items: [
                    {
                        id: 'leaf-child-2',
                        title: 'Leaf Child'
                    }
                ],
                title: 'Sub-Level With Children 2'
            }
        ],
        title: 'Top Level With Sub-Parents'
    },
    {
        id: 'empty-top',
        hasChildren: true,
        items: [],
        title: 'Empty Top-Level'
    },
    {
        id: 'non-selectable',
        isSelectable: false,
        title: 'Can not be selected'
    }
];

function getAllItems(): DrilldownItemType[] {
    return reduce(
        sampleData,
        (
            output: DrilldownItemType[],
            item: DrilldownItemType
        ): DrilldownItemType[] => output.concat(item, item.items),
        []
    );
}

/**
 * Default Drilldown behavior prevents users from navigating into items that have no children.
 * Through extra configuration, the Drilldown has the ability to display an empty message when
 * drilling into an item with no children. To achieve this the data must be modeled in a specific
 * way: any item which has the hasChildren flag set to true, with an empty array of child items will
 * be able to be navigated into and will show an empty message.
 */
class EmptyMessagesExample extends React.Component<
    {},
    EmptyMessagesExampleState
> {
    state: EmptyMessagesExampleState = {
        filteredItems: sampleData,
        selectedItemIds: [sampleData[0].id],
        selectedItem: sampleData[0]
    };

    onNavigate = (activeItem: DrilldownItemType): void => {
        this.setState({ activeItem, searchQuery: null });
    };

    onSearch = (event: React.ChangeEvent<HTMLInputElement>): void => {
        const searchQuery: string = event.currentTarget.value;

        if (searchQuery.length > 0) {
            const filteredItems: DrilldownItemType[] = filter(
                getAllItems(),
                (item: DrilldownItemType): boolean =>
                    item.title.toLowerCase().includes(searchQuery.toLowerCase())
            );

            this.setState({
                filteredItems,
                searchQuery
            });
        } else {
            this.setState({
                filteredItems: sampleData,
                searchQuery: null
            });
        }
    };

    onFieldChange = (event: React.ChangeEvent<HTMLInputElement>): void => {
        const { selectedItem } = this.state;
        this.setState({
            selectedItem: { ...selectedItem, title: event.currentTarget.value }
        });
    };

    onSelect = (selectedItem: DrilldownItemType): void => {
        if (selectedItem.id !== this.state.selectedItemIds[0]) {
            this.setState({ selectedItemIds: [selectedItem.id], selectedItem });
        }
    };

    // Globally specified item function, for all items in the drilldown
    itemFunction = (props: DrilldownItemProps): React.ReactNode => {
        return (
            <DrilldownEditItem {...props}>{props.item.title}</DrilldownEditItem>
        );
    };

    render(): React.ReactNode {
        const {
            activeItem,
            filteredItems,
            searchQuery,
            selectedItemIds
        } = this.state;
        const { onNavigate, onSelect } = this;

        const emptyMessage: string =
            activeItem && `There are no ${activeItem.title}'s available`;

        return (
            <Drilldown
                activeItem={activeItem}
                dataUI="drilldown-example--4"
                emptyMessage={emptyMessage}
                isParentSelectable
                itemFunction={this.itemFunction}
                items={filteredItems}
                onNavigate={onNavigate}
                onSelect={onSelect}
                searchQuery={searchQuery}
                selectedItemIds={selectedItemIds}
            />
        );
    }
}

export default EmptyMessagesExample;
