import React, {
    ReactElement,
    ReactNode,
    createContext,
    useContext,
    useEffect,
    useCallback
} from 'react';

import { Drilldown, DrilldownItemType } from '@preamp/core';

import {
    DateCell,
    DateCellProps,
    IListTableColumn,
    ListTable,
    NameDescriptionCell,
    NameDescriptionCellProps,
    ListTableTheme
} from '@preamp/tables';

enum columnEnum {
    Name = 'name',
    Selected = 'selected',
    Created = 'created',
    LastUpdated = 'updated'
}

interface SelectableCheckboxListTableRow {
    identifier: number;
    [columnEnum.Name]: NameDescriptionCellProps;
    [columnEnum.Created]?: DateCellProps;
    [columnEnum.LastUpdated]?: DateCellProps;
}

const initialState = {
    activeItem: undefined,
    rowSelected: {
        selectedNode: []
    },
    filteredItems: [],
    selectedItemIds: [],
    selectedRows: []
};

const SelectedRowContext = createContext({ selectedRows: [] });

const ADD_ROW = 'ADD_ROW';
const SET_FILTERED_ITEMS = 'SET_FILTERED_ITEMS';
const SET_SELECTED_ITEM = 'SET_SELECTED_ITEM';
const SET_ACTIVE_ITEM = 'SET_ACTIVE_ITEM';

//eslint-disable-next-line
function reducer(state, action) {
    switch (action.type) {
        case ADD_ROW: {
            const { row } = action.payload;
            const isSelected = state.selectedRows.includes(row);

            const selectedRows = isSelected
                ? [
                      ...state.selectedRows.filter(
                          (selectedRow: number): React.ReactNode =>
                              selectedRow !== row
                      )
                  ]
                : state.selectedRows.concat(action.payload.row);

            return {
                ...state,
                selectedRows
            };
        }
        case SET_FILTERED_ITEMS: {
            return {
                ...state,
                filteredItems: action.payload.filteredItems
            };
        }
        case SET_SELECTED_ITEM: {
            return {
                ...state,
                selectedItemIds: [action.payload.selectedItemId]
            };
        }
        case SET_ACTIVE_ITEM: {
            return {
                ...state,
                activeItem: action.payload.activeItem
            };
        }
        default:
            return initialState;
    }
}

function NameDescriptionCellComponent({ value }: any): ReactNode {
    return <NameDescriptionCell name={value.name} />;
}

function CreatedDateCellComponent({ value }: any): ReactNode {
    return <DateCell date={value.date} description={value.description} />;
}

function UpdatedDateCellComponent({ value }: any): ReactNode {
    return <DateCell date={value.date} description={value.description} />;
}

const columns: IListTableColumn[] = [
    {
        identifier: columnEnum.Name,
        header: 'Name',
        minWidth: 150,
        CustomCell: NameDescriptionCellComponent
    },
    {
        identifier: columnEnum.Created,
        header: 'Created',
        minWidth: 100,
        CustomCell: CreatedDateCellComponent
    },
    {
        identifier: columnEnum.LastUpdated,
        header: 'Last Updated',
        minWidth: 100,
        CustomCell: UpdatedDateCellComponent
    }
];

const rows: SelectableCheckboxListTableRow[] = [
    {
        identifier: 1,
        [columnEnum.Name]: {
            name: 'Doug'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 1, 20)
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2018, 3, 20)
        }
    },
    {
        identifier: 2,
        [columnEnum.Name]: {
            name: 'Amy'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 1, 20)
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2018, 3, 20)
        }
    },
    {
        identifier: 3,
        [columnEnum.Name]: {
            name: 'Ben'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 1, 20)
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2018, 3, 20)
        }
    }
];

function ListTableNode({ addSelectedNode }: any): ReactElement {
    const selectedRows = useContext(SelectedRowContext);

    const onRowClick = useCallback(
        (rowIdentifier: number): void => {
            addSelectedNode(rowIdentifier);
        },
        [addSelectedNode]
    );

    return (
        <ListTable
            columns={columns}
            dataUI="checkmark-selectable-list-table"
            onRowClicked={onRowClick}
            rows={rows}
            selectedRows={selectedRows as any}
            showSelectableIcon
            theme={ListTableTheme.Drilldown}
        />
    );
}

const sampleData = (addSelectedNode): DrilldownItemType[] => {
    return [
        {
            id: 'level-1',
            title: 'Level 1',
            items: [
                {
                    id: 'level-2',
                    title: 'Level 2',
                    items: [
                        {
                            id: 'level-3',
                            title: 'Level 3',
                            // eslint-disable-next-line react/display-name
                            itemFunction: (): ReactNode => {
                                return (
                                    <ListTableNode
                                        addSelectedNode={addSelectedNode}
                                    />
                                );
                            }
                        }
                    ]
                }
            ]
        }
    ];
};

/**
 * Drilldown with ListTable being used as the Child Component. This example showcases the usage of
 * useCallback, useContext, and useReducer to pass around state
 */
export default function DrilldownWithListTableExample(): ReactNode {
    const [
        { activeItem, filteredItems, selectedItemIds, selectedRows },
        dispatch
    ] = React.useReducer(reducer, initialState);

    function addSelectedNode(rowIdentifier): void {
        dispatch({ type: ADD_ROW, payload: { row: rowIdentifier } });
    }

    useEffect((): void => {
        dispatch({
            type: SET_FILTERED_ITEMS,
            payload: {
                filteredItems: sampleData(addSelectedNode)
            }
        });
    }, [dispatch, selectedRows]);

    function onNavigate(activeItem: DrilldownItemType): void {
        dispatch({
            type: SET_ACTIVE_ITEM,
            payload: { activeItem }
        });
    }

    function onSelect(selectedItem: DrilldownItemType): void {
        if (selectedItem.id !== selectedItemIds[0]) {
            dispatch({
                type: SET_SELECTED_ITEM,
                payload: { selectedItemId: selectedItem.id }
            });
        }
    }

    return (
        <SelectedRowContext.Provider value={selectedRows}>
            <Drilldown
                activeItem={activeItem}
                breadcrumbRootTitle="Home"
                dataUI="drilldown-list-table-example"
                isBreadcrumbsVisibleInRoot
                items={filteredItems}
                onNavigate={onNavigate}
                onSelect={onSelect}
                selectedItemIds={selectedItemIds}
            />
            State of selected List Table Item: {selectedRows}
        </SelectedRowContext.Provider>
    );
}
