import React, { ReactNode, useState } from 'react';

import { Drilldown, DrilldownItemType } from '@preamp/core';
import { filter, without } from 'lodash';

const sampleData: DrilldownItemType[] = [
    {
        id: 'level-1',
        title: 'Level 1',
        items: [
            {
                id: 'level-1-2',
                title: 'Level 1-2',
                items: [
                    {
                        id: 'level-1-3',
                        title: 'Level 1-3'
                    }
                ]
            }
        ]
    },
    {
        id: 'level-2',
        title: 'Level 2'
    },
    {
        id: 'level-3',
        title: 'Level 3'
    },
    {
        id: 'level-4',
        title: 'Level 4'
    }
];

/**
 * Drilldown with Search prop passthrough: Placeholder and OnKeyPress
 */
export default function DrilldownWithSearchPassthroughExample(): ReactNode {
    const [activeItem, setActiveItem] = useState(null);
    const [filteredItems, setFilteredItems] = useState(sampleData);
    const [searchQuery, setSearchQuery] = useState('');
    const [selectedItemIds, setSelectedItemIds] = useState([]);

    function onNavigate(activeItem: DrilldownItemType): void {
        setActiveItem(activeItem);
    }

    function onSelect(selectedItem: DrilldownItemType): void {
        const isItemSelected: boolean = selectedItemIds.includes(
            selectedItem.id
        );

        let selectedIds: string[];

        if (isItemSelected) {
            selectedIds = without(selectedItemIds, selectedItem.id);
        } else {
            selectedIds = selectedItemIds.concat(selectedItem.id);
        }

        setSelectedItemIds(selectedIds);
    }

    function onSearch(event: React.ChangeEvent<HTMLInputElement>): void {
        const newQuery: string = event.currentTarget.value;

        if (newQuery.length > 0) {
            setSearchQuery(newQuery);
        } else {
            setFilteredItems(sampleData);
            setSearchQuery('');
        }
    }

    function onKeyPress(event): void {
        const newQuery: string = event.currentTarget.value;
        const currentItems: DrilldownItemType[] = activeItem
            ? activeItem.items
            : sampleData;

        if (event.charCode === 13) {
            const searchedItems: DrilldownItemType[] = filter(
                currentItems,
                (item: DrilldownItemType): boolean =>
                    item.title.toLowerCase().includes(newQuery.toLowerCase())
            );

            setFilteredItems(searchedItems);
        }
    }

    return (
        <>
            <Drilldown
                activeItem={activeItem}
                breadcrumbRootTitle="Home"
                dataUI="drilldown-list-table-example"
                isBreadcrumbsVisibleInRoot
                items={filteredItems}
                onKeyPress={onKeyPress}
                onNavigate={onNavigate}
                onSearch={onSearch}
                onSelect={onSelect}
                placeholder={'Press enter to search'}
                searchQuery={searchQuery}
                selectedItemIds={selectedItemIds}
            />
            <div className="mt-1">Selected Item: {selectedItemIds}</div>
        </>
    );
}
