import * as React from 'react';

import { Card, IconButton, TextField } from '@preamp/core';
import { OptionType, Select } from '@preamp/select';
import { Trash } from '@preamp/signal';
import {
    DateCell,
    DateCellProps,
    IListTableColumn,
    ListTable,
    ListTableTheme,
    NameDescriptionCell,
    NameDescriptionCellProps,
    NumberCell,
    NumberCellProps
} from '@preamp/tables';

/**
 * Various types of Card Headers
 */
interface ExampleCardFieldsState {
    firstName?: string;
    lastName?: string;
    address?: string;
}

function reducer(
    state: ExampleCardFieldsState,
    update: Partial<ExampleCardFieldsState>
): ExampleCardFieldsState {
    return { ...state, ...update };
}

const iconButtonStyles: any = {
    position: 'absolute',
    right: '0.5rem',
    top: '0.5rem'
};

const options: OptionType[] = [
    { label: 'Option 1', value: 'option_1' },
    { label: 'Option 2', value: 'option_2' }
];

enum columnEnum {
    Name = 'name',
    Created = 'created',
    LastUpdated = 'lastUpdated',
    Size = 'size'
}

interface BasicListTableRow {
    identifier: number;
    [columnEnum.Name]: NameDescriptionCellProps;
    [columnEnum.Created]: DateCellProps;
    [columnEnum.LastUpdated]: DateCellProps;
    [columnEnum.Size]: NumberCellProps;
}

function NameDescriptionCellComponent({ value }: any): React.ReactNode {
    return (
        <NameDescriptionCell
            description={value.description}
            descriptionTruncationLines={2}
            indicatorColor={value.indicatorColor}
            isIndicatorShown={value.isIndicatorShown}
            name={value.name}
            nameUrl={`https://www.videoamp.com/${value.name}`}
            status={value.status}
        />
    );
}
function CreatedDateCellComponent({ value }: any): React.ReactNode {
    return <DateCell date={value.date} description={value.description} />;
}
function UpdatedDateCellComponent({ value }: any): React.ReactNode {
    return <DateCell date={value.date} description={value.description} />;
}
function SizeNumberCellComponent({ value }: any): React.ReactNode {
    return <NumberCell description={value.description} value={value.value} />;
}

const columns: IListTableColumn[] = [
    {
        identifier: columnEnum.Name,
        header: 'Name',
        maxWidth: 335,
        CustomCell: NameDescriptionCellComponent
    },
    {
        identifier: columnEnum.Created,
        header: 'Created',
        maxWidth: 100,
        CustomCell: CreatedDateCellComponent
    },
    {
        identifier: columnEnum.LastUpdated,
        header: 'Last Updated',
        minWidth: 100,
        CustomCell: UpdatedDateCellComponent
    },
    {
        identifier: columnEnum.Size,
        header: 'Size',
        minWidth: 100,
        isMetric: true,
        CustomCell: SizeNumberCellComponent
    }
];

const rows: BasicListTableRow[] = [
    {
        identifier: 1,
        name: {
            name:
                'This is a really long name that should truncate and should show an ellipsis after set number of lines',
            description:
                "Is this my espresso machine? Wh-what is-h-how did you get my espresso machine? Must go faster. We gotta burn the rain forest, dump toxic waste, pollute the air, and rip up the OZONE! 'Cause maybe if we screw up this planet enough, they won't want it anymore!"
        },
        [columnEnum.Created]: {
            date: new Date(2017, 1, 20),
            description: 'John Doe'
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2018, 3, 20),
            description: 'John Doe'
        },
        [columnEnum.Size]: { value: 35189 }
    },
    {
        identifier: 2,
        [columnEnum.Name]: {
            name: 'Amy',
            description:
                'Donec id elit non mi porta gravida at eget metus. Sed posuere consectet'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 2, 20),
            description: 'John Doe'
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2019, 4, 20),
            description: 'John Doe'
        },
        [columnEnum.Size]: { value: 14138890 }
    },
    {
        identifier: 3,
        [columnEnum.Name]: {
            name: 'Carl',
            description:
                'Donec id elit non mi porta gravida at eget metus. Sed posuere consectet'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 2, 20),
            description: 'John Doe'
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2016, 5, 20),
            description: 'John Doe'
        },
        [columnEnum.Size]: { value: 145876 }
    },
    {
        identifier: 4,
        [columnEnum.Name]: {
            name: 'Doug',
            description:
                'Donec id elit non mi porta gravida at eget metus. Sed posuere consectet'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 3, 20),
            description: 'John Doe'
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2018, 6, 20),
            description: 'John Doe'
        },
        [columnEnum.Size]: { value: 75463 }
    },
    {
        identifier: 5,
        [columnEnum.Name]: {
            name: 'Emily',
            description:
                'Donec id elit non mi porta gravida at eget metus. Sed posuere consectet'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 4, 20),
            description: 'John Doe'
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2018, 7, 20),
            description: 'John Doe'
        },
        [columnEnum.Size]: { value: 984153 }
    },
    {
        identifier: 6,
        [columnEnum.Name]: {
            indicatorColor: 'blue',
            name: 'Lewis',
            description:
                'Donec id elit non mi porta gravida at eget metus. Sed posuere consectet'
        },
        [columnEnum.Created]: {
            date: new Date(2018, 3, 20),
            description: 'John Doe'
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2019, 6, 20),
            description: 'John Doe'
        },
        [columnEnum.Size]: { value: 75422 }
    }
];

const ExampleVariousCardHeaders: React.FC = (): React.ReactElement => {
    const [{ firstName, lastName, address }, setFieldValue] = React.useReducer(
        reducer,
        {
            firstName: '',
            lastName: '',
            address: ''
        }
    );
    const actionNode: React.ReactNode = (
        <div style={iconButtonStyles}>
            <IconButton dataUI="primary" icon={Trash} size="lg">
                Button
            </IconButton>
        </div>
    );

    function changeFirstName(e: React.ChangeEvent<HTMLInputElement>): void {
        setFieldValue({ firstName: e.target.value });
    }

    function changeLastName(e: React.ChangeEvent<HTMLInputElement>): void {
        setFieldValue({ lastName: e.target.value });
    }

    function changeAddress(e: React.ChangeEvent<HTMLInputElement>): void {
        setFieldValue({ firstName: e.target.value });
    }

    return (
        <div>
            <p className="my-4">Custom positioning of action:</p>
            <Card action={actionNode}>
                <div className="container">
                    <div className="flex mb-4">
                        <div className="w-1/2">
                            <div className="m-auto w-4/5">
                                <TextField
                                    label="First Name"
                                    onChange={changeFirstName}
                                    value={firstName}
                                />
                            </div>
                        </div>
                        <div className="w-1/2">
                            <div className="m-auto w-4/5">
                                <TextField
                                    label="Last Name"
                                    onChange={changeLastName}
                                    value={lastName}
                                />
                            </div>
                        </div>
                    </div>
                    <div className="flex">
                        <div className="w-full">
                            <div className="m-auto" style={{ width: '90%' }}>
                                <TextField
                                    label="Address"
                                    onChange={changeAddress}
                                    value={address}
                                />
                            </div>
                        </div>
                    </div>
                </div>
            </Card>

            <p className="my-4">Select field in Card action:</p>
            <Card
                action={<Select options={options} style={{ width: '8rem' }} />}
                title="Audiences"
            >
                <ListTable
                    columns={columns}
                    rows={rows}
                    theme={ListTableTheme.Simple}
                />
            </Card>
        </div>
    );
};

export default ExampleVariousCardHeaders;
