import * as React from 'react';

import {
    DividerTitle,
    PillButton,
    PillButtonGroup,
    PillTheme,
    TextAlign,
    TextField,
    TextFieldType,
    TextPosition
} from '@preamp/core';

interface ControlsProps {
    fontColor: string;
    fontSize: number;
    lineColor: string;
    lineSize: number;
    text: string;
    textAlign: TextAlign;
    textMaxWidth: number;
    textPosition: TextPosition;
}

const DEFAULT_CONTROLS: ControlsProps = {
    fontColor: 'var(--divider-title-color)',
    fontSize: 0.625,
    lineColor: 'var(--divider-title-line)',
    lineSize: 1,
    text: 'Default Title',
    textAlign: 'center',
    textMaxWidth: 100,
    textPosition: 'left'
};

interface ExampleControlsProps {
    controls: ControlsProps;
    setControl(property: string, value: string | number): void;
}

const ExampleControls: React.FC<ExampleControlsProps> = ({
    controls,
    setControl
}: ExampleControlsProps): React.ReactElement<HTMLElement> => {
    return (
        <>
            <div className="flex u-border-default p-6">
                <div className="w-1/6 va-form-label">Title</div>
                <div className="w-5/6">
                    <div className="flex">
                        <div className="flex-1 mr-6">
                            <TextField
                                label="Text"
                                onChange={(
                                    event: React.ChangeEvent<HTMLInputElement>
                                ): void =>
                                    setControl('text', event.target.value)
                                }
                                value={controls.text}
                            />
                        </div>
                        <div className="flex-1">
                            <div className="va-form-label">Position</div>
                            <PillButtonGroup
                                multiselect={false}
                                name="text-position-group"
                                onChange={([value]: TextPosition[]): void =>
                                    setControl('textPosition', value)
                                }
                                selectedValues={[controls.textPosition]}
                            >
                                {['left', 'center', 'right'].map(
                                    (key: string): React.ReactElement => (
                                        <PillButton
                                            key={key}
                                            label={key}
                                            name="text-position"
                                            theme={
                                                controls.textPosition === key
                                                    ? PillTheme.Active
                                                    : PillTheme.Default
                                            }
                                            value={key}
                                        />
                                    )
                                )}
                            </PillButtonGroup>
                        </div>
                    </div>
                    <div className="flex mt-3">
                        <div className="flex-1 mr-6">
                            <TextField
                                label="Font Size (rem)"
                                min="0"
                                onChange={(
                                    event: React.ChangeEvent<HTMLInputElement>
                                ): void =>
                                    setControl(
                                        'fontSize',
                                        Number(event.target.value)
                                    )
                                }
                                step={0.025}
                                textFieldType={TextFieldType.Number}
                                value={`${controls.fontSize}`}
                            />
                        </div>
                        <div className="flex-1">
                            <TextField
                                label="Font Color"
                                onChange={(
                                    event: React.ChangeEvent<HTMLInputElement>
                                ): void =>
                                    setControl('fontColor', event.target.value)
                                }
                                value={controls.fontColor}
                            />
                        </div>
                    </div>
                    <div className="flex mt-3">
                        <div className="flex-1 mr-6">
                            <TextField
                                label="Text Max Width"
                                min="0"
                                onChange={(
                                    event: React.ChangeEvent<HTMLInputElement>
                                ): void =>
                                    setControl(
                                        'textMaxWidth',
                                        Number(event.target.value)
                                    )
                                }
                                step={10}
                                textFieldType={TextFieldType.Number}
                                value={`${controls.textMaxWidth}`}
                            />
                        </div>
                        <div className="flex-1">
                            <div className="va-form-label">Text Alignment</div>
                            <PillButtonGroup
                                multiselect={false}
                                name="text-align-group"
                                onChange={([value]: TextAlign[]): void =>
                                    setControl('textAlign', value)
                                }
                                selectedValues={[controls.textAlign]}
                            >
                                {['left', 'center', 'right', 'justify'].map(
                                    (key: string): React.ReactElement => (
                                        <PillButton
                                            key={key}
                                            label={key}
                                            name="text-align"
                                            theme={
                                                controls.textAlign === key
                                                    ? PillTheme.Active
                                                    : PillTheme.Default
                                            }
                                            value={key}
                                        />
                                    )
                                )}
                            </PillButtonGroup>
                        </div>
                    </div>
                </div>
            </div>
            <div className="flex u-border-default p-6 mt-3">
                <div className="w-1/6 va-form-label">Divider</div>
                <div className="w-5/6 flex">
                    <div className="flex-1 mr-6">
                        <TextField
                            label="Size"
                            min="0"
                            onChange={(
                                event: React.ChangeEvent<HTMLInputElement>
                            ): void =>
                                setControl(
                                    'lineSize',
                                    Number(event.target.value)
                                )
                            }
                            step={1}
                            textFieldType={TextFieldType.Number}
                            value={`${controls.lineSize}`}
                        />
                    </div>
                    <div className="flex-1">
                        <TextField
                            label="Color"
                            onChange={(
                                event: React.ChangeEvent<HTMLInputElement>
                            ): void =>
                                setControl('lineColor', event.target.value)
                            }
                            value={controls.lineColor}
                        />
                    </div>
                </div>
            </div>
        </>
    );
};

export const DividerTitleExample: React.FC = (): React.ReactElement<HTMLElement> => {
    const [controls, setControls] = React.useState<ControlsProps>(
        DEFAULT_CONTROLS
    );

    function setControl(
        property: string,
        value: string | number | boolean
    ): void {
        setControls({ ...controls, [property]: value });
    }

    return (
        <section className="mt-5">
            <h4>Configuration Example</h4>
            <p className="mt-2 mb-5">Modify any property as desired.</p>

            <ExampleControls controls={controls} setControl={setControl} />

            <div className="my-12">
                <DividerTitle
                    fontColor={controls.fontColor}
                    fontSize={controls.fontSize}
                    lineColor={controls.lineColor}
                    lineSize={controls.lineSize}
                    text={controls.text}
                    textAlign={controls.textAlign}
                    textMaxWidth={controls.textMaxWidth}
                    textPosition={controls.textPosition}
                />
                <div className="u-body-2 mt-2">
                    Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed
                    do eiusmod tempor incididunt ut labore et dolore magna
                    aliqua. Ut enim ad minim veniam, quis knostrud exercitation
                    ullamco laboris nisi ut aliquip ex ea commodo consequat.
                    Duis aute irure dolor in reprehenderit in voluptate velit
                    esse cillum dolore eu fugiat nulla pariatur. Excepteur sint
                    occaecat cupidatat non proident, sunt in culpa qui officia
                    deserunt mollit anim id est laborum.
                </div>
            </div>
        </section>
    );
};

export default DividerTitleExample;
