import React, { ReactElement, useState } from 'react';

import {
    Button,
    ButtonTheme,
    ExpandableCard,
    IconButton,
    TextField,
    TextFieldType,
    TextFieldVariant
} from '@preamp/core';
import { DateRangeInput } from '@preamp/datepicker';
import { Clear } from '@preamp/signal';
import { DataTable, IDataTableColumn } from '@preamp/tables';

enum columnEnum {
    FirstName = 'firstName',
    LastName = 'lastName',
    Age = 'age',
    Location = 'location',
    Date = 'date'
}

interface BasicDataTableRow {
    identifier: number;
    [columnEnum.FirstName]: string;
    [columnEnum.LastName]: string;
    [columnEnum.Age]: number;
    [columnEnum.Location]: string;
    [columnEnum.Date]: string;
}

const columns: IDataTableColumn[] = [
    {
        identifier: columnEnum.FirstName,
        header: 'First Name'
    },
    {
        identifier: columnEnum.LastName,
        header: 'Last Name'
    },
    {
        identifier: columnEnum.Age,
        header: 'Age'
    },
    {
        identifier: columnEnum.Location,
        header: 'Location'
    },
    {
        identifier: columnEnum.Date,
        header: 'Date'
    }
];

const rows: BasicDataTableRow[] = [
    {
        identifier: 1,
        [columnEnum.FirstName]: 'Bob',
        [columnEnum.LastName]: 'Samuelson',
        [columnEnum.Age]: 42,
        [columnEnum.Location]: 'CA',
        [columnEnum.Date]: '11/17/1976'
    },
    {
        identifier: 2,
        [columnEnum.FirstName]: 'Amy',
        [columnEnum.LastName]: 'Gaudioso',
        [columnEnum.Age]: 26,
        [columnEnum.Location]: 'MA',
        [columnEnum.Date]: '10/20/1992'
    },
    {
        identifier: 3,
        [columnEnum.FirstName]: 'Juan',
        [columnEnum.LastName]: 'Giuffrida',
        [columnEnum.Age]: 32,
        [columnEnum.Location]: 'TX',
        [columnEnum.Date]: '9/15/1986'
    },
    {
        identifier: 4,
        [columnEnum.FirstName]: 'Doug',
        [columnEnum.LastName]: 'Ung',
        [columnEnum.Age]: 37,
        [columnEnum.Location]: 'NY',
        [columnEnum.Date]: '12/9/1981'
    },
    {
        identifier: 5,
        [columnEnum.FirstName]: 'Ben',
        [columnEnum.LastName]: 'Behrens',
        [columnEnum.Age]: 35,
        [columnEnum.Location]: 'NC',
        [columnEnum.Date]: '8/3/1983'
    }
];

/**
 * Example of the ExpandableCard component
 */
function BasicExpandableCard(): ReactElement<{}> {
    const [count, setCount] = useState(0);
    const [expanded, setExpanded] = useState(false);
    const [nameValue, setNameValue] = useState('Example Name');
    const [descriptionValue, setDescriptionValue] = useState(
        'This is the description'
    );

    function onNameChange(event?: React.ChangeEvent<HTMLInputElement>): void {
        setNameValue(event.currentTarget.value);
    }

    function onDescriptionChange(
        event?: React.ChangeEvent<HTMLInputElement>
    ): void {
        setDescriptionValue(event.currentTarget.value);
    }

    function clickButton(): void {
        setCount(count + 1);
    }

    return (
        <>
            <ExpandableCard
                action={
                    <IconButton
                        dataUI="primary"
                        icon={Clear}
                        onClick={clickButton}
                    >
                        Button
                    </IconButton>
                }
                expandableContent={
                    <>
                        <DataTable
                            columns={columns}
                            dataUI="basic-data-table"
                            limit={100}
                            offset={0}
                            onPageChange={null}
                            onResultsPerPageChange={null}
                            resultsPerPageOptions={[5, 10, 25, 50]}
                            rows={rows}
                            showPagination={true}
                            totalRows={5}
                        />

                        <Button
                            dataUI="secondary"
                            onClick={clickButton}
                            theme={ButtonTheme.Secondary}
                        >
                            Add More
                        </Button>
                    </>
                }
                isExpanded={expanded}
                onExpand={setExpanded}
            >
                <div className="va-grid va-columns-default">
                    <div className="col-span-6">
                        <div className="va-form-group">
                            <TextField
                                dataUI="name-example"
                                id="name-input"
                                label="Name"
                                name="fieldName"
                                onChange={onNameChange}
                                onFocus={null}
                                placeholder="Enter text..."
                                tabIndex={0}
                                textFieldType={TextFieldType.Text}
                                value={nameValue}
                                variant={TextFieldVariant.Jumbo}
                            />
                        </div>
                    </div>
                </div>
                <div className="va-grid va-columns-default">
                    <div className="col-span-4">
                        <TextField
                            dataUI="description-example"
                            id="description-input"
                            label="Description"
                            name="fieldDescription"
                            onChange={onDescriptionChange}
                            onFocus={null}
                            placeholder="Enter text..."
                            tabIndex={0}
                            textFieldType={TextFieldType.Text}
                            value={descriptionValue}
                        />
                    </div>
                    <div className="col-span-4">
                        <DateRangeInput
                            dateRangePickerProps={{
                                firstDayOfWeek: 1
                            }}
                            inputLabel="Enter Date Range"
                        />
                    </div>
                </div>
            </ExpandableCard>
            <div className="my-3">
                <Button
                    onClick={(): void => {
                        setExpanded(true);
                    }}
                    theme={ButtonTheme.Secondary}
                >
                    EXPAND
                </Button>
                <Button
                    onClick={(): void => {
                        setExpanded(false);
                    }}
                    theme={ButtonTheme.Secondary}
                >
                    UNEXPAND
                </Button>
            </div>
            <p className="my-3">Button Clicks: {count}</p>
            <p className="my-3">Is Expanded: {`${expanded}`}</p>
        </>
    );
}

export default BasicExpandableCard;
