import React from 'react';

import { RatioGauge } from '@preamp/core';
import {
    RadioButton,
    TextField,
    TextFieldType,
    Toggle,
    TooltipProps
} from '@preamp/core';
import { CheckCircle, GroupTile, Signal } from '@preamp/signal';
import { Button, ButtonTheme } from '@preamp/core';

interface ControlsProps {
    animationTime: number;
    circumferenceWidth: number;
    contentType: string;
    diameter: number;
    emptyColor: string;
    fillColor: string;
    fillContent: boolean;
    iconColor: string;
    iconSize: number;
    isAnimating: boolean;
    isLoading: boolean;
    markIconColor: string;
    markIconSize: number;
    markText: string;
    markTextColor: string;
    markType: string;
    ratio: number;
    showTooltip: boolean;
    text: string;
    textColor: string;
    tooltipProps: TooltipProps;
}

const DEFAULT_SETTINGS: ControlsProps = {
    animationTime: 350,
    circumferenceWidth: 10,
    contentType: 'icon',
    diameter: 100,
    emptyColor: 'var(--va-grey-300)',
    fillColor: 'var(--va-teal-350)',
    fillContent: true,
    iconColor: '',
    iconSize: 3,
    isAnimating: false,
    isLoading: true,
    markIconColor: '',
    markIconSize: 2,
    markText: 'This is a mark',
    markTextColor: '',
    markType: 'icon',
    ratio: 0.5,
    showTooltip: true,
    text: 'This is a content',
    textColor: '',
    tooltipProps: {
        content: 'This is a tooltip',
        hasArrow: true,
        placement: 'top'
    }
};
const LOADING_DURATION = 3000;

interface ExampleControlsProps {
    controls: ControlsProps;
    setControl: (
        property: string,
        value: string | number | boolean | object
    ) => void;
}

const ExampleControls = ({
    controls,
    setControl
}: ExampleControlsProps): React.ReactElement => {
    React.useEffect((): (() => void) => {
        if (controls.isLoading) {
            const timer: any = setTimeout((): void => {
                setControl('isAnimating', true);
                setControl('isLoading', false);
            }, LOADING_DURATION);

            return (): void => clearTimeout(timer);
        }
    }, [controls.isLoading, setControl]);

    React.useEffect((): (() => void) => {
        if (controls.isAnimating) {
            const timer: any = setTimeout((): void => {
                setControl('isAnimating', false);
            }, controls.animationTime);

            return (): void => clearTimeout(timer);
        }
    }, [controls.animationTime, controls.isAnimating, setControl]);

    return (
        <>
            <div className="flex">
                <div className="pr-3 flex-1">
                    <TextField
                        label="Ratio"
                        max="1"
                        min="0"
                        onChange={(
                            event: React.ChangeEvent<HTMLInputElement>
                        ): void =>
                            setControl('ratio', Number(event.target.value))
                        }
                        step={0.01}
                        textFieldType={TextFieldType.Number}
                        value={String(controls.ratio)}
                    />
                </div>
                <div className="pr-3 flex-1">
                    <TextField
                        label="Diameter"
                        min="0"
                        onChange={(
                            event: React.ChangeEvent<HTMLInputElement>
                        ): void =>
                            setControl('diameter', Number(event.target.value))
                        }
                        textFieldType={TextFieldType.Number}
                        value={String(controls.diameter)}
                    />
                </div>
                <div className="pr-3 flex-1">
                    <TextField
                        label="Circumference Width"
                        max={String(controls.diameter / 2)}
                        min="0"
                        onChange={(
                            event: React.ChangeEvent<HTMLInputElement>
                        ): void =>
                            setControl(
                                'circumferenceWidth',
                                Number(event.target.value)
                            )
                        }
                        textFieldType={TextFieldType.Number}
                        value={String(controls.circumferenceWidth)}
                    />
                </div>
                <div className="pr-3 flex-1">
                    <TextField
                        label="Ratio Color"
                        onChange={(
                            event: React.ChangeEvent<HTMLInputElement>
                        ): void => setControl('fillColor', event.target.value)}
                        value={controls.fillColor}
                    />
                </div>
                <div className="flex-1">
                    <TextField
                        label="Empty Color"
                        onChange={(
                            event: React.ChangeEvent<HTMLInputElement>
                        ): void => setControl('emptyColor', event.target.value)}
                        value={controls.emptyColor}
                    />
                </div>
            </div>
            <div className="flex p-6 mt-3 u-border-default">
                <div className="flex-1 pr-3">
                    <div className="va-form-label">Content</div>
                    <div className="pt-3">
                        <Toggle
                            checked={controls.fillContent}
                            disabled={controls.contentType === 'none'}
                            label="Fill content"
                            name="fill-content-toggle"
                            onChange={(checked: boolean): void => {
                                setControl('fillContent', checked);
                            }}
                        />
                    </div>
                </div>
                <div className="flex-1 pr-3">
                    <RadioButton
                        checked={controls.contentType === 'icon'}
                        label="Icon"
                        name="content-type"
                        onChange={(selected: boolean): void =>
                            selected ? setControl('contentType', 'icon') : null
                        }
                    />
                    <TextField
                        className="mt-3"
                        disabled={controls.contentType !== 'icon'}
                        helpText="Size (rem)"
                        min="0"
                        onChange={(
                            event: React.ChangeEvent<HTMLInputElement>
                        ): void =>
                            setControl('iconSize', Number(event.target.value))
                        }
                        step={0.25}
                        textFieldType={TextFieldType.Number}
                        value={String(controls.iconSize)}
                    />
                    <TextField
                        className="mt-3"
                        disabled={
                            controls.contentType !== 'icon' ||
                            controls.fillContent
                        }
                        helpText="Icon color"
                        onChange={(
                            event: React.ChangeEvent<HTMLInputElement>
                        ): void => setControl('iconColor', event.target.value)}
                        value={String(controls.iconColor)}
                    />
                </div>
                <div className="flex-1 pr-3">
                    <RadioButton
                        checked={controls.contentType === 'text'}
                        label="Text"
                        name="content-type"
                        onChange={(selected: boolean): void =>
                            selected ? setControl('contentType', 'text') : null
                        }
                    />
                    <TextField
                        className="mt-3"
                        disabled={controls.contentType !== 'text'}
                        helpText="Content Text"
                        onChange={(
                            event: React.ChangeEvent<HTMLInputElement>
                        ): void => setControl('text', event.target.value)}
                        value={controls.text}
                    />
                    <TextField
                        className="mt-3"
                        disabled={
                            controls.contentType !== 'text' ||
                            controls.fillContent
                        }
                        helpText="Text color"
                        onChange={(
                            event: React.ChangeEvent<HTMLInputElement>
                        ): void => setControl('textColor', event.target.value)}
                        value={String(controls.textColor)}
                    />
                </div>
                <div className="flex-1 pr-3">
                    <RadioButton
                        checked={controls.contentType === 'none'}
                        label="None"
                        name="content-type"
                        onChange={(selected: boolean): void =>
                            selected ? setControl('contentType', 'none') : null
                        }
                    />
                </div>
            </div>
            <div className="flex p-6 mt-3 u-border-default">
                <div className="flex-1 pr-3 va-form-label">Mark</div>
                <div className="flex-1 pr-3">
                    <RadioButton
                        checked={controls.markType === 'icon'}
                        label="Icon"
                        name="mark-type"
                        onChange={(selected: boolean): void =>
                            selected ? setControl('markType', 'icon') : null
                        }
                    />
                    <TextField
                        className="mt-3"
                        disabled={controls.markType !== 'icon'}
                        helpText="Size (rem)"
                        min="0"
                        onChange={(
                            event: React.ChangeEvent<HTMLInputElement>
                        ): void =>
                            setControl(
                                'markIconSize',
                                Number(event.target.value)
                            )
                        }
                        step={0.25}
                        textFieldType={TextFieldType.Number}
                        value={String(controls.markIconSize)}
                    />
                    <TextField
                        className="mt-3"
                        disabled={controls.markType !== 'icon'}
                        helpText="Icon color"
                        onChange={(
                            event: React.ChangeEvent<HTMLInputElement>
                        ): void =>
                            setControl('markIconColor', event.target.value)
                        }
                        value={String(controls.markIconColor)}
                    />
                </div>
                <div className="flex-1 pr-3">
                    <RadioButton
                        checked={controls.markType === 'text'}
                        label="Text"
                        name="mark-type"
                        onChange={(selected: boolean): void =>
                            selected ? setControl('markType', 'text') : null
                        }
                    />
                    <TextField
                        className="mt-3"
                        disabled={controls.markType !== 'text'}
                        helpText="Mark text"
                        onChange={(
                            event: React.ChangeEvent<HTMLInputElement>
                        ): void => setControl('markText', event.target.value)}
                        value={controls.markText}
                    />
                    <TextField
                        className="mt-3"
                        disabled={controls.markType !== 'text'}
                        helpText="Text color"
                        onChange={(
                            event: React.ChangeEvent<HTMLInputElement>
                        ): void =>
                            setControl('markTextColor', event.target.value)
                        }
                        value={String(controls.markTextColor)}
                    />
                </div>
                <div className="flex-1 pr-3">
                    <RadioButton
                        checked={controls.markType === 'none'}
                        label="None"
                        name="mark-type"
                        onChange={(selected: boolean): void =>
                            selected ? setControl('markType', 'none') : null
                        }
                    />
                </div>
            </div>
            <div className="flex p-6 mt-3 u-border-default">
                <div className="flex-1 pr-3">
                    <div className="va-form-label">Tooltip</div>
                    <p className="u-body-2">Basic tooltip configuration.</p>
                    <p className="u-body-2 mt-2">
                        See more props{' '}
                        <a href="#/core/presentation/Tooltip">here</a>.
                    </p>
                </div>
                <div className="flex-1 pr-3">
                    <Toggle
                        checked={controls.showTooltip}
                        label="Show tooltip"
                        name="show-tooltip-toggle"
                        onChange={(checked: boolean): void => {
                            setControl('showTooltip', checked);
                        }}
                    />
                </div>
                <div className="flex-1 pr-3">
                    <TextField
                        className="mt-3"
                        disabled={!controls.showTooltip}
                        label="Text"
                        onChange={(
                            event: React.ChangeEvent<HTMLInputElement>
                        ): void =>
                            setControl('tooltipProps', {
                                ...controls.tooltipProps,
                                content: event.target.value
                            })
                        }
                        value={String(controls.tooltipProps.content)}
                    />
                </div>
                <div className="flex-1 pr-3">
                    <label
                        className={`va-form-label ${(!controls.showTooltip &&
                            'va-label-disabled') ||
                            ''}`}
                    >
                        Placement
                    </label>
                    <div className="pl-6">
                        {['top', 'right', 'bottom', 'left'].map(
                            (placement: string): React.ReactElement => (
                                <div
                                    className="pb-2"
                                    key={`${placement}-tooltip`}
                                >
                                    <RadioButton
                                        checked={
                                            placement ===
                                            controls.tooltipProps.placement
                                        }
                                        disabled={!controls.showTooltip}
                                        label={placement}
                                        name="tooltip-placement"
                                        onChange={(selected: boolean): void =>
                                            setControl('tooltipProps', {
                                                ...controls.tooltipProps,
                                                placement: selected
                                                    ? placement
                                                    : undefined
                                            })
                                        }
                                    />
                                </div>
                            )
                        )}
                    </div>
                </div>
            </div>
            <div className="flex p-6 mt-3 u-border-default">
                <div className="flex-1 pr-3">
                    <div className="va-form-label">Loading</div>
                    <p className="u-body-2">
                        Duration to control how long the ratio takes to animate.
                        The prop duration receives the value in milliseconds.
                    </p>
                </div>
                <div className="flex-1 pr-3">
                    <TextField
                        label="MILLISECONDS"
                        min="0"
                        onChange={(
                            event: React.ChangeEvent<HTMLInputElement>
                        ): void =>
                            setControl(
                                'animationTime',
                                Number(event.target.value)
                            )
                        }
                        step={1}
                        textFieldType={TextFieldType.Number}
                        value={String(controls.animationTime)}
                    />
                </div>
                <div className="flex-1 pr-3 mt-5">
                    <Button
                        onClick={(): void => {
                            setControl('isLoading', true);
                        }}
                        theme={ButtonTheme.Primary}
                    >
                        Simulate
                    </Button>
                </div>
            </div>
        </>
    );
};

export const RatioGaugeConfigurationExample: React.FC = (): React.ReactElement => {
    const [controls, setControls] = React.useState<ControlsProps>(
        DEFAULT_SETTINGS
    );

    const setControl = React.useCallback(
        (property: string, value: string | number | boolean): void => {
            setControls(
                (prevState: ControlsProps): ControlsProps => ({
                    ...prevState,
                    [property]: value
                })
            );
        },
        []
    );

    const mark: React.ReactElement =
        controls.markType === 'icon' ? (
            <Signal
                color={controls.markIconColor}
                icon={CheckCircle}
                size={controls.markIconSize}
            />
        ) : controls.markType === 'text' ? (
            <div
                className="va-caption"
                style={{ color: controls.markTextColor }}
            >
                {controls.markText}
            </div>
        ) : null;
    const content: React.ReactElement =
        controls.contentType === 'icon' ? (
            <Signal
                color={!controls.fillContent && controls.iconColor}
                icon={GroupTile}
                size={controls.iconSize}
            />
        ) : controls.contentType === 'text' ? (
            <div style={{ color: !controls.fillContent && controls.textColor }}>
                {controls.text}
            </div>
        ) : null;

    return (
        <section className="mt-5">
            <h4>Configuration Example</h4>
            <p className="mt-2 mb-5">
                Modify any RatioGauge property as desired.
            </p>
            <ExampleControls controls={controls} setControl={setControl} />
            <div
                className="overflow-auto mt-6 text-center"
                style={{
                    height: controls.diameter + 120,
                    paddingTop: '3rem',
                    paddingBottom: '3rem'
                }}
            >
                <RatioGauge
                    circumferenceWidth={controls.circumferenceWidth}
                    diameter={controls.diameter}
                    duration={controls.animationTime}
                    emptyColor={controls.emptyColor}
                    fillColor={controls.fillColor}
                    fillContent={controls.fillContent}
                    isLoading={controls.isLoading}
                    mark={mark}
                    ratio={
                        controls.isLoading || controls.isAnimating
                            ? 0
                            : controls.ratio
                    }
                    tooltipProps={
                        controls.showTooltip ? controls.tooltipProps : {}
                    }
                >
                    {content}
                </RatioGauge>
            </div>
        </section>
    );
};

export default RatioGaugeConfigurationExample;
