import React from 'react';

import { formatToTimeZone } from 'date-fns-timezone';
import { listTimeZones } from 'timezone-support';

import { Button, Checkbox, TextField } from '@preamp/core';
import { DatePickerInput } from '@preamp/datepicker';

interface IInlineDatePickerInputState {
    checked: boolean;
    confirmedDay: Date;
    invalidTimeZone: boolean;
    selectedDay: Date;
    timezone: string;
    timezoneInput: string;
}

/**
 * Inline Date Picker Input Example with timezone option
 */
class InlineDatePickerInputTimezone extends React.Component<
    {},
    IInlineDatePickerInputState
> {
    state: IInlineDatePickerInputState = {
        checked: false,
        confirmedDay: null,
        invalidTimeZone: false,
        selectedDay: null,
        timezone: 'UTC',
        timezoneInput: ''
    };

    handleDayClick = (day: Date): void => {
        this.setState({
            selectedDay: day
        });
    };

    handleChange = (day: Date): void => {
        this.setState({
            selectedDay: day
        });
    };

    handleConfirmDay = (): void => {
        this.setState((prevState: IInlineDatePickerInputState): any => ({
            confirmedDay: prevState.selectedDay
        }));
    };

    handleToggleTimeZone = (): void => {
        this.setState((prevState: IInlineDatePickerInputState): any => ({
            checked: !prevState.checked
        }));
    };

    handleTimeZoneChange = (e: React.ChangeEvent<HTMLInputElement>): void => {
        e.preventDefault();

        const timezone: string = e.currentTarget.value;
        if (listTimeZones().includes(timezone)) {
            this.setState({
                timezone,
                timezoneInput: e.currentTarget.value,
                invalidTimeZone: false
            });
        } else {
            this.setState({
                invalidTimeZone: true,
                timezoneInput: e.currentTarget.value
            });
        }
    };

    render(): React.ReactNode {
        const {
            confirmedDay,
            selectedDay,
            timezone,
            checked,
            invalidTimeZone
        } = this.state;

        const formattedConfirmedDay: string = confirmedDay
            ? formatToTimeZone(confirmedDay, 'MM/DD/YYYY', {
                  timeZone: timezone
              })
            : '';

        const formattedConfirmedDayTimezone: string =
            confirmedDay && checked
                ? formatToTimeZone(confirmedDay, '(z)', { timeZone: timezone })
                : '';

        return (
            <>
                <div className="mt-4 mb-4">
                    <DatePickerInput
                        datePickerProps={{
                            initialMonth: new Date(2020, 4, 12)
                        }}
                        inline
                        onHandleChange={this.handleChange}
                        onHandleDayClick={this.handleDayClick}
                        selectedDay={confirmedDay}
                    />
                </div>

                <Button disabled={!selectedDay} onClick={this.handleConfirmDay}>
                    Confirm Date
                </Button>

                <div className="ml-2" style={{ display: 'inline' }}>
                    Confirmed Date: {formattedConfirmedDay}{' '}
                    {formattedConfirmedDayTimezone}
                </div>

                <div className="mt-8 flex">
                    <div className="mr-8">
                        <Checkbox
                            checked={checked}
                            label="Set timezone"
                            name="timezone"
                            onChange={this.handleToggleTimeZone}
                        />
                    </div>

                    {checked && (
                        <div className="flex-1">
                            <TextField
                                helpText={
                                    invalidTimeZone
                                        ? 'The timezone you entered is not valid.'
                                        : 'More information on timezone usage available in Datepicker'
                                }
                                label="Enter a Timezone (Europe/Berlin, UTC, etc...)"
                                name="timeZoneInput"
                                onChange={this.handleTimeZoneChange}
                            />
                        </div>
                    )}
                </div>
            </>
        );
    }
}

export default InlineDatePickerInputTimezone;
