### Getting Started

If you are using npm for dependency management, Mixer can be installed from the command line:

```bash
yarn add @preamp/mixer
```

Then import the styles into your project:

```css
import '@preamp/mixer/dist/mixer.css';
```

Otherwise, you can use the latest or specific version of the build via CDN:

Latest:

```
https://cdn.jsdelivr.net/npm/@preamp/mixer/dist/mixer.css
https://cdn.jsdelivr.net/npm/@preamp/mixer/dist/mixer.min.css
```

Versioned:

```
https://cdn.jsdelivr.net/npm/@preamp/mixer@[version]/dist/mixer.css
https://cdn.jsdelivr.net/npm/@preamp/mixer@[version]/dist/mixer.min.css
```

#### Optimization

If using the entire library of styles that mixer provides is overkill for your project, you can use the smaller, optimized, compiled CSS files.

-   Mixer Base
    -   `mixer-base.css` includes the basic helper utility classes (for example: `.block` for `display: block;`, `.p-2` for padding, `.m-2` for margins, etc.) A list of helper classes can be found on this [Cheatsheet](https://nerdcave.com/tailwind-cheat-sheet)
-   Mixer Colors
    -   `mixer-colors.css` includes the default color palette used by PreAmp.
-   Mixer Components
    -   `mixer-components.css` includes any styles used by PreAmp's components.
-   Mixer Grid
    -   `mixer-grid.css` includes styles used in Mixer's grid layout system.

Each package within PreAmp also has its CSS broken out to cut down on unused styles. In most cases if you use these package specific CSS files, it is also recommended that you use `mixer-base.css` and `mixer-colors.css` to ensure that everything will look as expected.

-   Core
    -   `core.css` includes styles used by the Core components.
-   Dates
    -   `dates.css` includes styles used by the date picker components.
-   Select
    -   `select.css` includes styles used by the select components.
-   Signal
    -   `signal.css` includes styles used by the Signal icon components.
-   Tables
    -   `tables.css` includes styles used by the Tables components.
-   Waveform
    -   `waveform.css` includes styles used in the Waveform data visualization package.

We recommend optimizing your CSS output with tools like [Purgecss](https://www.purgecss.com/) or [cssnano](https://cssnano.co/).

### Structure

Mixer is broken into the following partial groups:

-   Helpers
-   Base
-   Components
-   Utilities

_Helpers_ includes helper styles such as colors, variables, mixins, functions, themes, typography and layout.

_Base_ contains styles for base HTML elements.

_Components_ contain styles for the PreAmp Core component library.

_Utilities_ includes utility styles for animations, Tailwind utilities, etc.

### Functional Helper Classes

Mixer uses [Tailwind](https://tailwindcss.com/) to extend functional classes, making styling easy, clean and consistent.

Tailwind creates functional CSS classes that can be used for styling, however it is not recommended that they be used directly in HTML markup. Instead you can extend these classes in your SASS files:

```sass
.va-btn {
    @apply rounded py-2 px-4 font-normal text-va-button border-transparent border-2;
}
```

A cheat sheet for the default Tailwind helper classes can be found [here](http://nerdcave.com/tailwind-cheat-sheet).

Some common uses for functional helper classes include text color, background color, borders (color, width, style, radius), and spacing (padding, margin).

#### Color

Text color can be applied by using the helper class `text-[color name]`. For background colors, the class name would be `bg-[color name]`. The default color palette used in Mixer's `_colors.scss` partial can be use with both of these methods, so for example, if a colors from the palette should be applied to the text color and background color, this could be added to the class in your css:

```CSS
@apply text-va-indigo-500 bg-va-light-blue-200
```

More info can be found here: <a href="https://tailwindcss.com/docs/text-color" target="_blank">Text Color</a>, <a href="https://tailwindcss.com/docs/background-color" target="_blank">Background Color</a>

#### Border

Adding the class `border` would apply a 1px solid border to an element.

_Border color_ helper classes are similar to text color. Using `border-[color name]` applies the border color. Border style is also similar, `border-solid`, `border-dashed`, etc.

_Border width_ can be applied on the entire element or per side. Applying to all sides would be done using `border-[width]` so for example `border-2` would apply a 2px border-width. Per side is done by specifying the side in the class name, so for example `border-t-2` would apply a top border width of 2px. The sides are as you would expect: t (top), r (right), b (bottom), l(left).

_Border radius_ can be applied with the `rounded` class. It uses a similare method to apply radius to sides as the border width. The value of the radius is applied using the following additions to the `rounded` classname:

| className      | radius value |
| :------------- | ------------ |
| `rounded-none` | `0`          |
| `rounded-sm`   | `.125rem`    |
| `rounded`      | `.25rem`     |
| `rounded-lg`   | `.5rem`      |
| `rounded-full` | `999px`      |

#### Spacing

Padding and margin follow the same pattern of naming classes. Using `p-[size]` or `m-[size]` will apply the space to all sides while specifying a side such as `p-[side]-[size]` and `m-[side]-[size]`. Values for the sizes are as follows:

| -[size] | value     |
| :------ | --------- |
| `px`    | `1px`     |
| `0`     | `0`       |
| `1`     | `0.25rem` |
| `2`     | `0.5rem`  |
| `3`     | `0.75rem` |
| `4`     | `1rem`    |
| `5`     | `1.25rem` |
| `6`     | `1.5rem`  |
| `8`     | `2rem`    |
| `10`    | `2.5rem`  |
| `12`    | `3rem`    |
| `16`    | `4rem`    |
| `20`    | `5rem`    |
| `24`    | `6rem`    |
| `32`    | `8rem`    |
