import React, { FunctionComponent, ReactElement } from 'react';

import { times } from 'lodash';
import { addDays, format } from 'date-fns';
import numeral from 'numeral';

import {
    CellContext,
    ColumnDef,
    createColumnHelper,
    DataTable
} from '@videoamp-private/preamp-plus';

type ValueType = string | number | Date;

interface RowType {
    byteValue: number;
    currencyValue: number;
    dateValue: Date;
    numberValue: number;
    stringValue: string;
    unitValue: number;
}

const columnHelper = createColumnHelper<RowType>();

const COLUMNS: ColumnDef<RowType, ValueType>[] = [
    columnHelper.accessor(({ dateValue }: RowType): ValueType => dateValue, {
        cell: ({ getValue }: CellContext<RowType, ValueType>): string =>
            format(getValue() as Date, 'YYYY, MMMM DD'),
        header: 'Dates',
        size: 80
    }),
    columnHelper.accessor(
        ({ numberValue }: RowType): ValueType => numberValue,
        {
            cell: ({ getValue }: CellContext<RowType, ValueType>): string =>
                numeral(getValue() as number).format('0,0.000'),
            header: 'Numbers',
            size: 100
        }
    ),
    columnHelper.accessor(
        ({ stringValue }: RowType): ValueType => stringValue,
        {
            cell: ({ getValue }: CellContext<RowType, ValueType>): string =>
                getValue() as string,
            header: 'Strings (no format)',
            size: 75
        }
    ),
    columnHelper.accessor(({ byteValue }: RowType): ValueType => byteValue, {
        cell: ({ getValue }: CellContext<RowType, ValueType>): string =>
            numeral(getValue() as number).format('0.00b'),
        header: 'Bytes',
        size: 50
    }),
    columnHelper.accessor(
        ({ currencyValue }: RowType): ValueType => currencyValue,
        {
            cell: ({ getValue }: CellContext<RowType, ValueType>): string =>
                numeral(getValue() as number).format('$0,0.00'),
            header: 'Currencies',
            size: 100
        }
    ),
    columnHelper.accessor(({ unitValue }: RowType): ValueType => unitValue, {
        cell: ({ getValue }: CellContext<RowType, ValueType>): string =>
            `${numeral(getValue()).format('0,0')} units`,
        header: 'Other format (units)',
        size: 80
    })
];

const ROW_COUNT = 8;

const ROWS = times(ROW_COUNT).map(
    (x: number): RowType => ({
        byteValue: Math.random() * Math.pow(100, x),
        currencyValue: Math.random() * Math.pow(10, x),
        dateValue: addDays(new Date(), Math.round(Math.random() * 100)),
        numberValue: Math.random() * Math.pow(10, x),
        stringValue: `Row ${x}`,
        unitValue: Math.random() * 100
    })
);

/**
 * Value formatting
 */
const ValueFormattingExample: FunctionComponent = (): ReactElement<HTMLElement> => (
    <>
        <DataTable columns={COLUMNS} rows={ROWS} />

        <p className="mt-1 mb-6 text-right u-body-2">
            {COLUMNS.length} columns x {ROW_COUNT} rows ={' '}
            {COLUMNS.length * ROW_COUNT} cells
        </p>
    </>
);

export default ValueFormattingExample;
