import React, { FunctionComponent, ReactElement, useState } from 'react';

import { times } from 'lodash';

import {
    Button,
    Checkbox,
    DividerTitle,
    IconButton,
    Pill,
    TextField,
    TextFieldType,
    Tooltip
} from '@preamp/core';
import { Clear } from '@preamp/signal';

import {
    CellContext,
    ColumnDef,
    createColumnHelper,
    DataTable
} from '@videoamp-private/preamp-plus';

type RowType = Record<string, unknown>;

const columnHelper = createColumnHelper<RowType>();

function SelectCell({ name }: { name: string }): ReactElement {
    const [isChecked, setIsChecked] = useState<boolean>(false);
    return (
        <Checkbox
            checked={isChecked}
            name={name}
            onChange={(checked): void => setIsChecked(!!checked)}
        />
    );
}

function ColorCell({ color }: { color: string }): ReactElement {
    return (
        <div
            style={{
                backgroundColor: `${color}25`, // 25% opacity
                border: `5px solid ${color}`,
                borderRadius: 10,
                padding: 5,
                textAlign: 'center'
            }}
        >
            {color}
        </div>
    );
}

function ButtonWithTooltipCell({ label }: { label: string }): ReactElement {
    return (
        <Tooltip
            content={`Lorem ipsum dolor sit amet, consectetur adipiscing elit,
 sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.`}
            hasArrow
            placement="top"
            referenceClassName="inline-block"
        >
            <Button>{label}: Hover over me!</Button>
        </Tooltip>
    );
}

function PillCell({ value }: { value: string }): ReactElement {
    return <Pill append={<IconButton icon={Clear} />}>{value}</Pill>;
}

function DescriptionCell({ title }: { title: string }): ReactElement {
    return (
        <>
            <DividerTitle className="mb-3" text={title} />
            <p>
                Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do
                eiusmod tempor incididunt ut labore et dolore magna aliqua.
            </p>
        </>
    );
}

function EditCell({ value }: { value: string }): ReactElement {
    return <TextField placeholder={value} textFieldType={TextFieldType.Text} />;
}

const COLUMNS: ColumnDef<RowType>[] = [
    columnHelper.accessor('value', {
        id: 'checkbox-value',
        // eslint-disable-next-line react/display-name
        cell: ({
            getValue
        }: CellContext<RowType, unknown>): ReactElement<HTMLElement> => (
            <SelectCell name={getValue() as string} />
        ),
        header: 'Select'
    }),
    columnHelper.accessor('value', {
        id: 'string-value',
        cell: ({ getValue }: CellContext<RowType, unknown>): string =>
            getValue() as string,
        header: 'Strings'
    }),
    columnHelper.accessor('color', {
        // eslint-disable-next-line react/display-name
        cell: ({
            getValue
        }: CellContext<RowType, unknown>): ReactElement<HTMLElement> => (
            <ColorCell color={getValue() as string} />
        ),
        header: 'Random colors'
    }),
    columnHelper.accessor('link', {
        cell: ({
            getValue
        }: CellContext<RowType, unknown>): ReactElement<HTMLElement> => {
            const { label, url } =
                (getValue() as { label: string; url: string }) || {};
            return (
                <a
                    href={url}
                    referrerPolicy="no-referrer"
                    rel="noopener noreferrer"
                    target="_blank"
                >
                    {label}
                </a>
            );
        },
        header: 'Links'
    }),
    columnHelper.accessor('value', {
        id: 'tooltip-value',
        // eslint-disable-next-line react/display-name
        cell: ({
            getValue
        }: CellContext<RowType, unknown>): ReactElement<HTMLElement> => (
            <ButtonWithTooltipCell label={getValue() as string} />
        ),
        header: 'Buttons with Tooltip'
    }),
    columnHelper.accessor('value', {
        id: 'pill-value',
        // eslint-disable-next-line react/display-name
        cell: ({
            getValue
        }: CellContext<RowType, unknown>): ReactElement<HTMLElement> => (
            <PillCell value={getValue() as string} />
        ),
        header: 'Pills'
    }),
    columnHelper.accessor('value', {
        // eslint-disable-next-line react/display-name
        cell: ({
            getValue
        }: CellContext<RowType, unknown>): ReactElement<HTMLElement> => (
            <DescriptionCell title={getValue() as string} />
        ),
        header: 'Description'
    }),
    columnHelper.accessor('value', {
        id: 'edit-value',
        // eslint-disable-next-line react/display-name
        cell: ({
            getValue
        }: CellContext<RowType, unknown>): ReactElement<HTMLElement> => (
            <EditCell value={getValue() as string} />
        ),
        header: 'Edit'
    })
];

const ROW_COUNT = 3;

const ROWS = times(ROW_COUNT).map(
    (x: number): RowType => ({
        color:
            '#' +
            ('00000' + ((Math.random() * (1 << 24)) | 0).toString(16)).slice(
                -6
            ),
        link: { label: `Link ${x}`, url: 'https://www.google.com' },
        value: `Text ${x}`
    })
);

/**
 * Custom cells
 */
const CustomCellsExample: FunctionComponent = (): ReactElement<HTMLElement> => (
    <>
        <DataTable columns={COLUMNS} rows={ROWS} />

        <p className="mt-1 mb-6 text-right u-body-2">
            {COLUMNS.length} columns x {ROW_COUNT} rows ={' '}
            {COLUMNS.length * ROW_COUNT} cells
        </p>
    </>
);

export default CustomCellsExample;
