import React, { ReactElement, useState } from 'react';

import { Button } from '@preamp/core';
import {
    DataTable,
    CellContext,
    ColumnDef,
    createColumnHelper
} from '@videoamp-private/preamp-plus';
import { times } from 'lodash';

import { addDays, format } from 'date-fns';
import numeral from 'numeral';
/**
 *  Group cells example
 */

type ValueType = string | number | Date;

interface RowType {
    byteValue: number;
    currencyValue: number;
    dateValue: Date;
    numberValue: number;
    stringValue: string;
    unitValue: number;
}
const columnHelper = createColumnHelper<RowType>();

const COLUMNS: ColumnDef<RowType, ValueType>[] = [
    columnHelper.accessor(({ dateValue }: RowType): ValueType => dateValue, {
        cell: ({ getValue }: CellContext<RowType, ValueType>): string =>
            format(getValue() as Date, 'YYYY, MMMM DD'),
        header: 'Dates',
        size: 80
    }),
    columnHelper.accessor(
        ({ numberValue }: RowType): ValueType => numberValue,
        {
            cell: ({ getValue }: CellContext<RowType, ValueType>): string =>
                numeral(getValue() as number).format('0,0.000'),
            header: 'Numbers',
            size: 100
        }
    ),
    columnHelper.accessor(
        ({ stringValue }: RowType): ValueType => stringValue,
        {
            cell: ({ getValue }: CellContext<RowType, ValueType>): string =>
                getValue() as string,
            header: 'Strings (no format)',
            size: 75
        }
    ),
    columnHelper.accessor(({ byteValue }: RowType): ValueType => byteValue, {
        cell: ({ getValue }: CellContext<RowType, ValueType>): string =>
            numeral(getValue() as number).format('0.00b'),
        header: 'Bytes',
        size: 50
    }),
    columnHelper.accessor(
        ({ currencyValue }: RowType): ValueType => currencyValue,
        {
            cell: ({ getValue }: CellContext<RowType, ValueType>): string =>
                numeral(getValue() as number).format('$0,0.00'),
            header: 'Currencies',
            size: 100
        }
    ),
    columnHelper.accessor(({ unitValue }: RowType): ValueType => unitValue, {
        cell: ({ getValue }: CellContext<RowType, ValueType>): string =>
            numeral(getValue() as number).format('0,0') + 'units',
        header: 'Other format (units)',
        size: 80
    })
];

const ROW_COUNT = 6;

const rowGen = () => {
    return times(ROW_COUNT).map(
        (x: number): RowType => ({
            byteValue: Math.random() * Math.pow(100, x),
            currencyValue: Math.random() * Math.pow(10, x),
            dateValue: addDays(new Date(), Math.round(Math.random() * 1)),
            numberValue: Math.round(Math.random() * 1) + 5,
            stringValue: `Label 1`,
            unitValue: Math.random() * 100
        })
    );
};
const GroupCellsExample = (): ReactElement<HTMLElement> => {
    const [isLoading, setIsLoading] = useState<boolean>(false);

    const [rows, setRows] = useState<RowType[]>(rowGen());

    const randomizeRowData = (): void => {
        const rows: RowType[] = rowGen();
        setRows(rows);
    };
    const simulateLazyLoading = (): void => {
        setIsLoading(true);
        randomizeRowData();

        setTimeout((): void => {
            setIsLoading(false);
        }, 1000);
    };

    return (
        <>
            <div className="flex justify-between items-center mb-3">
                <p className="my-4">Group cells</p>
                <Button onClick={simulateLazyLoading}>
                    Simulate lazy loading
                </Button>
            </div>
            <p className="my-3">
                This table groups consecutive rows with the same column data.
            </p>
            <DataTable
                columns={COLUMNS}
                isLoading={isLoading}
                rows={rows}
                shouldGroupCells={true}
                sortingOptions={{
                    enableSorting: true,
                    initialSortingState: [
                        { id: 'Dates', desc: true },
                        { id: 'Numbers', desc: true },
                        { id: 'Strings (no format)', desc: true }
                    ],
                    maxMultiSortColCount: 3
                    // ... other sorting props
                }}
            />

            <p className="my-3">This table shows the raw data.</p>
            <DataTable
                columns={COLUMNS}
                isLoading={isLoading}
                rows={rows}
                shouldGroupCells={false}
                sortingOptions={{
                    enableSorting: true,
                    initialSortingState: [
                        { id: 'Dates', desc: true },
                        { id: 'Numbers', desc: true },
                        { id: 'Strings (no format)', desc: true }
                    ],
                    maxMultiSortColCount: 3
                    // ... other sorting props
                }}
            />
            <p className="mt-1 mb-6 text-right u-body-2">
                {COLUMNS.length} columns x {ROW_COUNT} rows ={' '}
                {COLUMNS.length * ROW_COUNT} cells
            </p>
        </>
    );
};

export default GroupCellsExample;
