import { useEffect, useMemo, useState } from 'react';

import { get, times } from 'lodash';

import { ColumnDef } from '@videoamp-private/preamp-plus';

const makeColumnData = (
    count: number,
    minColumnWidth: number,
    maxColumnWidth?: number
): ColumnDef<unknown, unknown>[] =>
    times(count).map(
        (k: number): ColumnDef<unknown, unknown> => ({
            accessorFn: (row: unknown): unknown => get(row, k),
            accessorKey: `column-${k}`,
            header: `COLUMN ${k}`,
            id: `column-${k}`,
            size: maxColumnWidth // i.e. variable column width
                ? minColumnWidth +
                  Math.round(Math.random() * (maxColumnWidth - minColumnWidth))
                : minColumnWidth
        })
    );

const makeRowData = (count: number, columnCount: number): number[][] =>
    times(count).map((): number[] =>
        times(columnCount).map((): number => Math.round(Math.random() * 10000))
    );

export const useMockData = ({
    columnCount,
    loadingDelayMs,
    maxColumnWidth,
    minColumnWidth,
    rowCount
}: {
    columnCount: number;
    loadingDelayMs?: number;
    maxColumnWidth?: number;
    minColumnWidth: number;
    rowCount: number;
}): {
    columns: ColumnDef<unknown>[];
    isLoading?: boolean;
    rows: number[][];
    summary: string;
} => {
    const summary = useMemo(
        (): string =>
            `${columnCount} columns x ${rowCount} rows = ${columnCount *
                rowCount} cells`,
        [columnCount, rowCount]
    );
    const columns = useMemo<ColumnDef<unknown, unknown>[]>(
        (): ColumnDef<unknown, unknown>[] =>
            makeColumnData(columnCount, minColumnWidth, maxColumnWidth),
        [columnCount, maxColumnWidth, minColumnWidth]
    );
    const [rows, setRows] = useState<number[][]>([]);
    const [isLoading, setIsLoading] = useState<boolean>();

    useEffect((): void => {
        if (loadingDelayMs) {
            setIsLoading(true);
            setTimeout((): void => {
                setRows(makeRowData(rowCount, columnCount));
                setIsLoading(false);
            }, loadingDelayMs);
        } else {
            setRows(makeRowData(rowCount, columnCount));
        }
    }, [columnCount, loadingDelayMs, rowCount]);

    return { columns, isLoading, rows, summary };
};
