import React, { ReactElement, useState } from 'react';

import {
    Autocomplete,
    Box,
    Check,
    InputLabel,
    MenuItem,
    TextField,
    ThemeProvider,
    Typography
} from '@preamp/rev';

type Option = {
    label: string;
    value: string;
};

const checkStyles = {
    height: 21,
    ml: -1,
    mr: 0.5,
    width: 16
};
const options: Option[] = [
    { label: 'Normal', value: 'normal' },
    { label: 'Fire', value: 'fire' },
    { label: 'Water', value: 'water' },
    { label: 'Grass', value: 'grass' },
    { label: 'Electric', value: 'electric' },
    { label: 'Ice', value: 'ice' },
    { label: 'Fighting', value: 'fighting' },
    { label: 'Poison', value: 'poison' },
    { label: 'Ground', value: 'ground' },
    { label: 'Flying', value: 'flying' },
    { label: 'Psychic', value: 'psychic' },
    { label: 'Bug', value: 'bug' },
    { label: 'Rock', value: 'rock' },
    { label: 'Ghost', value: 'ghost' },
    { label: 'Dark', value: 'dark' },
    { label: 'Dragon', value: 'dragon' },
    { label: 'Steel', value: 'steel' },
    { label: 'Fairy', value: 'fairy' }
];

const MultiSelectAllExample = (): ReactElement => {
    const [consolidatedValues, setConsolidatedValues] = useState<Option[]>([]);
    const [isConsolidated, setIsConsolidated] = useState<boolean>(false);
    const [multiValues, setMultiValues] = useState<Option[]>([]);

    return (
        <ThemeProvider>
            <Box component="section" sx={{ mb: 4 }}>
                <InputLabel htmlFor="multi-all">Multi Select All</InputLabel>
                <Autocomplete
                    disableCloseOnSelect
                    getOptionLabel={(option): string => option.label}
                    id="multi-all"
                    multiple
                    onChange={(_event, value: Option[]): void => {
                        if (value.some((obj): boolean => obj.value === 'all')) {
                            if (multiValues.length === options.length) {
                                setMultiValues([]);
                            } else {
                                setMultiValues(options);
                            }
                        } else {
                            setMultiValues(value);
                        }
                    }}
                    options={[
                        { label: 'Select All', value: 'all' },
                        ...options
                    ]}
                    renderInput={(params): ReactElement => (
                        <TextField {...params} placeholder="Select Type" />
                    )}
                    renderOption={(props, option): ReactElement => {
                        const hasCheck =
                            multiValues.includes(option) ||
                            multiValues.length === options.length;
                        return (
                            <MenuItem
                                {...props}
                                data-ui={`multi-all-${option.value}`}
                                sx={{
                                    backgroundColor: hasCheck
                                        ? 'var(--va-teal-100)'
                                        : 'initial',
                                    borderBottom:
                                        option.value === 'all'
                                            ? '1px solid'
                                            : 'none',
                                    borderColor: 'var(--va-grey-300)'
                                }}
                            >
                                {hasCheck && <Check sx={checkStyles} />}
                                <Typography sx={{ pl: hasCheck ? 0 : 1.5 }}>
                                    {option.label}
                                </Typography>
                            </MenuItem>
                        );
                    }}
                    size="small"
                    sx={{ width: 450 }}
                    value={multiValues}
                />
            </Box>
            <Box component="section" sx={{ mb: 4 }}>
                <InputLabel htmlFor="multi-consolidated">
                    Multi Select All Consolidated
                </InputLabel>
                <Autocomplete
                    disableClearable
                    disableCloseOnSelect
                    getOptionLabel={(option): string => option.label}
                    id="multi-consolidated"
                    isOptionEqualToValue={(option, value): boolean =>
                        option.value === value.value
                    }
                    multiple
                    onChange={(_event, value: Option[]): void => {
                        if (value.length > 0) {
                            const lastSelected = value[value.length - 1].value;
                            if (lastSelected === 'all') {
                                setConsolidatedValues(options);
                                setIsConsolidated(true);
                            } else {
                                if (!isConsolidated) {
                                    if (value.length === options.length) {
                                        setConsolidatedValues(options);
                                        setIsConsolidated(true);
                                    } else {
                                        setConsolidatedValues(value);
                                    }
                                } else {
                                    setConsolidatedValues(
                                        options.filter(
                                            (obj): boolean =>
                                                obj.value !== lastSelected
                                        )
                                    );
                                    setIsConsolidated(false);
                                }
                            }
                        } else {
                            setConsolidatedValues([]);
                            setIsConsolidated(false);
                        }
                    }}
                    options={[
                        { label: 'Select All', value: 'all' },
                        ...options
                    ]}
                    renderInput={(params): ReactElement => (
                        <TextField {...params} placeholder="Select Type" />
                    )}
                    renderOption={(props, option): ReactElement => {
                        const hasCheck =
                            consolidatedValues.includes(option) ||
                            isConsolidated;
                        return (
                            <MenuItem
                                {...props}
                                data-ui={`multi-consolidated-${option.value}`}
                                sx={{
                                    backgroundColor: hasCheck
                                        ? 'var(--va-teal-100)'
                                        : 'initial',
                                    borderBottom:
                                        option.value === 'all'
                                            ? '1px solid'
                                            : 'none',
                                    borderColor: 'var(--va-grey-300)'
                                }}
                            >
                                {hasCheck && <Check sx={checkStyles} />}
                                <Typography sx={{ pl: hasCheck ? 0 : 1.5 }}>
                                    {option.label}
                                </Typography>
                            </MenuItem>
                        );
                    }}
                    size="small"
                    sx={{ width: 450 }}
                    value={
                        isConsolidated
                            ? [
                                  {
                                      label: `All ( ${options.length} )`,
                                      value: 'all'
                                  }
                              ]
                            : consolidatedValues
                    }
                />
            </Box>
        </ThemeProvider>
    );
};

export default MultiSelectAllExample;
