import React, { DragEvent, ReactElement, useCallback, useState } from 'react';

import {
    DragIndicator,
    Table,
    TableBody,
    TableCell,
    TableContainer,
    TableHead,
    TableRow,
    ThemeProvider,
    Typography
} from '@preamp/rev';
import cx from 'classnames';

const DraggableListTableExample = (): ReactElement => {
    const [draggedElementIndex, setDraggedElementIndex] = useState<number>(-1);
    const [draggedElement, setDraggedElement] = useState<HTMLTableRowElement>(
        null
    );
    const [reordered, setReordered] = useState(false);
    const [rows, setRows] = useState([
        {
            name: 'Tony Stark',
            description: 'Some Description',
            createdAt: '01/03/2020 10:45 AM',
            status: 'Draft'
        },
        {
            name: 'Peter Parker',
            description: 'Some Description',
            createdAt: '12/04/2020 07:45 AM',
            status: 'Processing'
        },
        {
            name: 'Clark Kent',
            description: '',
            createdAt: '10/01/1959 2:00 AM',
            status: 'Processing'
        },
        {
            name: 'Gwen Stacy',
            description: 'Some Description',
            createdAt: '01/03/2020 10:45 AM',
            status: 'Draft'
        },
        {
            name: 'Barry Allen',
            description: 'Some Description',
            createdAt: '12/04/2020 07:45 AM',
            status: 'Processing'
        },
        {
            name: 'Diana Prince',
            description: '',
            createdAt: '10/01/1959 2:00 AM',
            status: 'Processing'
        }
    ]);
    const handleDragStart = useCallback(
        (e: DragEvent<HTMLTableRowElement>, index: number): void => {
            const draggedElement = e.currentTarget;

            setDraggedElement(draggedElement);
            setDraggedElementIndex(index);
            e.dataTransfer.effectAllowed = 'move';
            e.dataTransfer.dropEffect = 'move';
        },
        []
    );

    const handleDragEnter = useCallback(
        (e: DragEvent<HTMLTableRowElement>): void => {
            const draggedOverItem = e.currentTarget;

            if (draggedElement !== draggedOverItem) {
                setReordered(true);
                const el =
                    draggedElement.nextSibling === draggedOverItem
                        ? draggedOverItem.nextSibling
                        : draggedOverItem;

                (draggedElement.parentNode as HTMLTableSectionElement).insertBefore(
                    draggedElement,
                    el
                );
            }
        },
        [draggedElement]
    );
    const reset = (): void => {
        setDraggedElement(null);
        setDraggedElementIndex(-1);
        setReordered(false);
    };
    const handleDragEnd = (): void => {
        reset();
    };
    const handleDragOver = (e: DragEvent<HTMLTableRowElement>): void => {
        e.preventDefault();
    };
    const handleDrop = useCallback(
        (index: number): void => {
            if (index !== draggedElementIndex) {
                setRows((prevState): typeof rows => {
                    const list = prevState.slice();
                    const item = list[draggedElementIndex];

                    list.splice(draggedElementIndex, 1);
                    list.splice(
                        index > draggedElementIndex ? index + 1 : index,
                        0,
                        item
                    );

                    return list;
                });
            }
            reset();
        },
        [draggedElementIndex, rows]
    );

    return (
        <ThemeProvider>
            <Typography component="h2" gutterBottom variant="h5">
                Draggable List Table
            </Typography>
            <TableContainer>
                <Table size="small">
                    <TableHead>
                        <TableRow>
                            <TableCell sx={{ width: 32 }}></TableCell>
                            <TableCell sx={{ width: 300 }}>Name</TableCell>
                            <TableCell sx={{ width: 120 }}>Created</TableCell>
                            <TableCell sx={{ width: 120 }}>Status</TableCell>
                        </TableRow>
                    </TableHead>
                    <TableBody
                        className={cx({
                            disableHover: draggedElement !== null
                        })}
                    >
                        {rows.map(
                            (item, i): ReactElement => (
                                <TableRow
                                    className={cx({
                                        dragging:
                                            draggedElement !== null &&
                                            i === draggedElementIndex,
                                        reordered:
                                            reordered &&
                                            i === draggedElementIndex
                                    })}
                                    draggable
                                    key={i}
                                    onDragEnd={(): void => handleDragEnd()}
                                    onDragEnter={(
                                        e: DragEvent<HTMLTableRowElement>
                                    ): void => handleDragEnter(e)}
                                    onDragOver={(
                                        e: DragEvent<HTMLTableRowElement>
                                    ): void => handleDragOver(e)}
                                    onDragStart={(
                                        e: DragEvent<HTMLTableRowElement>
                                    ): void => handleDragStart(e, i)}
                                    onDrop={(): void => handleDrop(i)}
                                >
                                    <TableCell scope="row">
                                        <DragIndicator fontSize="small" />
                                    </TableCell>
                                    <TableCell component="th" scope="row">
                                        <Typography>{item.name}</Typography>
                                    </TableCell>
                                    <TableCell>
                                        <Typography
                                            color="inherit"
                                            component="time"
                                            variant="body2"
                                        >
                                            {item.createdAt}
                                        </Typography>
                                        <Typography
                                            color="inherit"
                                            sx={{
                                                fontSize: '.625rem'
                                            }}
                                            variant="body2"
                                        >
                                            {item.description}
                                        </Typography>
                                    </TableCell>
                                    <TableCell
                                        sx={{
                                            fontSize: '0.625rem',
                                            fontWeight: 'var(--font-bold)'
                                        }}
                                    >
                                        <Typography
                                            sx={{
                                                color: 'var(--va-grey-600)',
                                                fontWeight: 'var(--font-bold)'
                                            }}
                                            variant="subtitle1"
                                        >
                                            {item.status}
                                        </Typography>
                                    </TableCell>
                                </TableRow>
                            )
                        )}
                    </TableBody>
                </Table>
            </TableContainer>
        </ThemeProvider>
    );
};
export default DraggableListTableExample;
