import React from 'react';

import dateFnsFormat from 'date-fns/format';

import { Search, Slider, TextField, TextFieldType } from '@preamp/core';
import {
    DateRange,
    DateRangeInput,
    SLASH_DATE_FORMAT
} from '@preamp/datepicker';
import {
    ActionMeta,
    Filter,
    OptionsType,
    OptionType,
    SelectList,
    ValueType
} from '@preamp/select';

enum DateDimensions {
    START_DATE = 'startDate',
    END_DATE = 'endDate',
    DATE = 'date-1'
}

const VALUE_RANGE: [number, number] = [0, 100];

const drilldownOptions: OptionsType<OptionType> = [
    {
        label: 'Select Options',
        value: 'select-1'
    },
    {
        label: 'Date Range',
        value: 'date-1'
    },
    {
        label: 'Number Range',
        value: 'number-1'
    },
    {
        label: 'Range Slider',
        value: 'slider-1'
    },
    {
        label: 'Text Field',
        value: 'text-1'
    }
];

const defaultSelectOptions: OptionsType<OptionType> = [
    {
        label: 'Option 1',
        value: 'Value 1'
    },
    {
        label: 'Option 2',
        value: 'Value 2'
    },
    {
        label: 'Option 3',
        value: 'Value 3'
    },
    {
        label: 'Option 4',
        value: 'Value 4'
    }
];

interface SimpleFilterState {
    groupOptions: OptionsType<OptionType>;
    isLoading: boolean;
    isSelectLoading: boolean;
    replaceSelectOptionsWithLoader: boolean;
    search: string;
    selectedGroup: string;
    selectedValues: OptionsType<OptionType>;
    selectOptions: OptionsType<OptionType>;
    selectedSliderValues: [number, number];
    sliderValues: [number, number];
    selectedNumberRange: [number, number];
    numberRange: [number, number];
    stagedDateRange: OptionsType<OptionType>;
    stagedOptions: OptionsType<OptionType>;
    stagedDateOptions: OptionsType<OptionType>;
    textFieldValue: string;
}

const reducer = (
    state: SimpleFilterState,
    action: { type: string; payload?: any }
): SimpleFilterState => {
    switch (action.type) {
        case 'IS_LOADING':
            return { ...state, isLoading: action.payload };
        case 'CHANGE_SEARCH':
            return { ...state, search: action.payload };
        case 'OPEN':
            return {
                ...state,
                isLoading: false,
                groupOptions: drilldownOptions
            };
        case 'CLOSE':
            return { ...state, selectOptions: defaultSelectOptions };
        case 'CHANGE_GROUP':
            return { ...state, selectedGroup: action.payload };
        case 'BACK':
            return { ...state, selectOptions: defaultSelectOptions };
        case 'CANCEL':
            return {
                ...state,
                selectOptions: defaultSelectOptions,
                textFieldValue: ''
            };
        case 'APPLY_SELECTED_OPTIONS':
            return {
                ...state,
                ...action.payload
            };
        case 'APPLY_DATE_RANGE':
            return {
                ...state,
                selectedValues: action.payload,
                stagedDateRange: null,
                stagedDateOptions: []
            };
        case 'APPLY_TEXT_FIELD':
            return {
                ...state,
                selectedValues: action.payload
            };
        case 'SELECT_CHANGE':
            return {
                ...state,
                stagedOptions: action.payload
            };
        case 'SELECT_SEARCH_CHANGE_START':
            return {
                ...state,
                isSelectLoading: true,
                replaceSelectOptionsWithLoader: true
            };
        case 'SELECT_SEARCH_CHANGE':
            return {
                ...state,
                isSelectLoading: false,
                replaceSelectOptionsWithLoader: false,
                selectOptions: action.payload
            };
        case 'CONFIRM_DATES':
            return {
                ...state,
                ...action.payload
            };
        case 'CHANGE_TEXT_FIELD':
            return {
                ...state,
                textFieldValue: action.payload
            };
        case 'REMOVE_GROUP':
            return {
                ...state,
                ...action.payload
            };
        case 'REMOVE_All_GROUP':
            return {
                ...state,
                ...action.payload
            };
        case 'CHANGE_SLIDER_VALUES':
            return {
                ...state,
                sliderValues: action.payload
            };
        case 'APPLY_SLIDER_VALUES':
            return {
                ...state,
                selectedSliderValues: state.sliderValues,
                selectedValues: action.payload
            };
        case 'CHANGE_NUMBER_RANGE':
            return {
                ...state,
                numberRange: action.payload
            };
        case 'APPLY_NUMBER_RANGE':
            return {
                ...state,
                selectedNumberRange: state.numberRange,
                selectedValues: action.payload
            };
    }
};

/**
 * Example of the Filter and Search component laid out together
 */
export default function SimpleFilter(): React.ReactElement<any> {
    const [state, dispatch] = React.useReducer(reducer, {
        groupOptions: null,
        isLoading: false,
        isSelectLoading: false,
        replaceSelectOptionsWithLoader: false,
        search: '',
        selectedValues: [],
        selectedGroup: '',
        selectOptions: defaultSelectOptions,
        selectedSliderValues: null,
        sliderValues: VALUE_RANGE,
        selectedNumberRange: null,
        numberRange: null,
        stagedDateRange: null,
        stagedOptions: null,
        stagedDateOptions: [],
        textFieldValue: ''
    });

    const onChangeSearch = ({
        currentTarget: { value: search }
    }: React.FormEvent<HTMLInputElement>): void => {
        console.log('onChangeSearch', search);
        dispatch({ type: 'CHANGE_SEARCH', payload: search });
    };

    const onClearSearch = (): void => {
        console.log('onClearSearch');
        dispatch({ type: 'CHANGE_SEARCH', payload: '' });
    };

    const onOpenHandler = (group: ValueType<OptionType>): void => {
        const { groupOptions } = state;
        console.log('onOpenHandler', group);
        if (groupOptions === null) {
            dispatch({ type: 'IS_LOADING', payload: true });
            // simulate options loading
            setTimeout((): void => {
                dispatch({ type: 'OPEN' });
            }, 1000);
        }
    };

    const onCloseHandler = (group: ValueType<OptionType>): void => {
        console.log('onCloseHandler', group);
        dispatch({ type: 'CLOSE' });
    };

    const onGroupChangeHandler = (
        group: ValueType<OptionType>,
        action: ActionMeta
    ): void => {
        console.log('onGroupChangeHandler', group, action);
        dispatch({ type: 'CHANGE_GROUP', payload: group.value });
    };

    const onBackHandler = (group: ValueType<OptionType>): void => {
        console.log('onBackHandler', group);
        dispatch({ type: 'BACK' });
    };

    const onCancelHandler = (group: ValueType<OptionType>): void => {
        console.log('onCancelHandler', group);
        dispatch({ type: 'CANCEL' });
    };

    const removeSameGroupOptions = (
        group: OptionType
    ): OptionsType<OptionType> => {
        // remove options within the same select group
        const { selectedValues } = state;
        if (selectedValues) {
            // remove any options set within the same group
            return selectedValues.filter(
                (option: OptionType): boolean =>
                    option.group.value !== group.value
            );
        }
        return [];
    };

    const onApplyHandler = (group: ValueType<OptionType>): void => {
        const {
            selectedGroup,
            sliderValues,
            numberRange,
            stagedDateRange,
            stagedDateOptions,
            stagedOptions
        } = state;
        const reducedOptions: OptionsType<OptionType> = removeSameGroupOptions(
            group
        );
        if (
            group.value === drilldownOptions[0].value &&
            stagedOptions[selectedGroup]
        ) {
            // add group data to new selected options
            const optionsToApply: OptionsType<OptionType> = stagedOptions[
                selectedGroup
            ].map(
                (option: OptionType): OptionType => ({
                    ...option,
                    group: group
                })
            );
            dispatch({
                type: 'APPLY_SELECTED_OPTIONS',
                payload: {
                    selectedValues: [...reducedOptions, ...optionsToApply],
                    stagedOptions: {
                        ...state.stagedOptions,
                        [selectedGroup]: optionsToApply
                    }
                }
            });
        } else if (
            group.value === drilldownOptions[1].value &&
            stagedDateRange
        ) {
            // add group data to new selected options
            const dateToApply: OptionsType<OptionType> = {
                ...stagedDateRange,
                group: group
            };
            const reducedOptionsNoStartEndDates: OptionsType<OptionType> = reducedOptions.filter(
                (option: OptionType): boolean =>
                    !['startDate', 'endDate'].includes(option.group.value)
            );
            dispatch({
                type: 'APPLY_DATE_RANGE',
                payload: [
                    ...reducedOptionsNoStartEndDates,
                    dateToApply,
                    ...stagedDateOptions
                ]
            });
        } else if (group.value === drilldownOptions[2].value && numberRange) {
            dispatch({
                type: 'APPLY_NUMBER_RANGE',
                payload: [
                    ...reducedOptions,
                    {
                        group,
                        label: `${numberRange[0]} - ${numberRange[1]}`,
                        value: numberRange
                    }
                ]
            });
        } else if (group.value === drilldownOptions[3].value && sliderValues) {
            dispatch({
                type: 'APPLY_SLIDER_VALUES',
                payload: [
                    ...reducedOptions,
                    {
                        group,
                        label: `${sliderValues[0]} - ${sliderValues[1]}`,
                        value: sliderValues
                    }
                ]
            });
        } else if (
            group.value === drilldownOptions[4].value &&
            state.textFieldValue
        ) {
            dispatch({
                type: 'APPLY_TEXT_FIELD',
                payload: [
                    ...reducedOptions,
                    {
                        label: state.textFieldValue,
                        value: state.textFieldValue,
                        group
                    }
                ]
            });
        }
    };

    const onApplySelectedOptionsHandler = (
        group: ValueType<OptionType>,
        values: OptionsType<OptionType>
    ): void => {
        const reducedOptions: OptionsType<OptionType> = removeSameGroupOptions(
            group
        );
        // add group data to new selected options
        const optionsToApply: OptionsType<OptionType> = values.map(
            (option: OptionType): OptionType => ({
                ...option,
                group
            })
        );
        dispatch({
            type: 'APPLY_SELECTED_OPTIONS',
            payload: {
                selectedValues: [...reducedOptions, ...optionsToApply],
                stagedOptions: {
                    ...state.stagedOptions,
                    [group.value]: optionsToApply
                }
            }
        });
    };

    const onSelectChange = (
        value: ValueType<OptionType>,
        action: ActionMeta
    ): void => {
        console.log('onSelectChange', value, action);
        dispatch({
            type: 'SELECT_CHANGE',
            payload: {
                ...state.stagedOptions,
                [state.selectedGroup]: value
            }
        });
    };

    const onSelectSearchFocus = (
        event: React.FocusEvent<HTMLInputElement>
    ): void => {
        console.log('onSelectSearchFocus', event);
    };

    const onSelectSearchBlur = (
        event: React.FocusEvent<HTMLInputElement>
    ): void => {
        console.log('onSelectSearchBlur', event);
    };

    const onSelectSearchChange = (
        value: string,
        event: React.ChangeEvent<HTMLInputElement>
    ): void => {
        console.log('onSelectSearchChange', value, event);
        dispatch({
            type: 'SELECT_SEARCH_CHANGE_START'
        });
        if (!value || value === '') {
            dispatch({
                type: 'SELECT_SEARCH_CHANGE',
                payload: defaultSelectOptions
            });
        } else if (value.length > 10) {
            // simulate empty results
            setTimeout((): void => {
                dispatch({
                    type: 'SELECT_SEARCH_CHANGE',
                    payload: []
                });
            }, 1000);
        } else {
            // simulate new results
            setTimeout((): void => {
                dispatch({
                    type: 'SELECT_SEARCH_CHANGE',
                    payload: [
                        {
                            label: 'New Option 1',
                            value: 'New Value 1'
                        },
                        {
                            label: 'New Option 2',
                            value: 'New Value 2'
                        }
                    ]
                });
            }, 1000);
        }
    };

    const onConfirmDates = (confirmedDates: DateRange): void => {
        const dateStart = `${dateFnsFormat(
            confirmedDates.from,
            SLASH_DATE_FORMAT
        )}`;
        const dateEnd = `${dateFnsFormat(
            confirmedDates.to,
            SLASH_DATE_FORMAT
        )}`;
        const dateRange = `${dateStart} - ${dateEnd}`;
        const option: OptionType = {
            label: dateRange,
            value: dateRange
        };
        const stagedDateOptions: OptionType[] = [
            {
                group: {
                    value: DateDimensions.START_DATE,
                    label: DateDimensions.START_DATE
                },
                label: dateStart,
                value: dateStart
            },
            {
                group: {
                    value: DateDimensions.END_DATE,
                    label: DateDimensions.END_DATE
                },
                label: dateEnd,
                value: dateEnd
            }
        ];

        dispatch({
            type: 'CONFIRM_DATES',
            payload: { stagedDateRange: option, stagedDateOptions }
        });
    };

    const onTextFieldChange = (
        event: React.ChangeEvent<HTMLInputElement>
    ): void => {
        dispatch({
            type: 'CHANGE_TEXT_FIELD',
            payload: event.currentTarget.value
        });
    };

    const isOptionPartOfDateGroup = (
        selectedGroup: ValueType<OptionType>,
        option: OptionType
    ): boolean => {
        return (
            (selectedGroup.value === DateDimensions.DATE &&
                option.group.value === DateDimensions.START_DATE) ||
            (selectedGroup.value === DateDimensions.DATE &&
                option.group.value === DateDimensions.END_DATE)
        );
    };

    const onRemoveGroupHandler = (group: ValueType<OptionType>): void => {
        const { selectedValues, textFieldValue } = state;
        const updatedValues: OptionsType<OptionType> = selectedValues.filter(
            (option: OptionType): boolean => {
                if (
                    group.value === option.group.value ||
                    isOptionPartOfDateGroup(group, option)
                ) {
                    return false;
                } else {
                    return true;
                }
            }
        );
        dispatch({
            type: 'REMOVE_GROUP',
            payload: {
                selectedValues: updatedValues,
                stagedOptions: { ...state.stagedOptions, [group.value]: null },
                numberRange:
                    group === drilldownOptions[2]
                        ? VALUE_RANGE
                        : state.numberRange,
                selectedNumberRange:
                    group === drilldownOptions[2]
                        ? null
                        : state.selectedNumberRange,
                sliderValues:
                    group === drilldownOptions[3]
                        ? VALUE_RANGE
                        : state.sliderValues,
                selectedSliderValues:
                    group === drilldownOptions[3]
                        ? null
                        : state.selectedSliderValues,
                textFieldValue:
                    group === drilldownOptions[4] ? '' : textFieldValue
            }
        });
    };
    const onRemoveAllGroupsHandler = (): void => {
        dispatch({
            type: 'REMOVE_All_GROUP',
            payload: {
                groupOptions: null,
                isLoading: false,
                isSelectLoading: false,
                replaceSelectOptionsWithLoader: false,
                search: '',
                selectedValues: [],
                selectedGroup: '',
                selectOptions: defaultSelectOptions,
                selectedSliderValues: null,
                sliderValues: VALUE_RANGE,
                selectedNumberRange: null,
                numberRange: null,
                stagedDateRange: null,
                stagedOptions: null,
                stagedDateOptions: [],
                textFieldValue: ''
            }
        });
    };

    const selectComponent = (): React.ReactNode => {
        const {
            isSelectLoading,
            selectedGroup,
            selectedValues,
            selectOptions,
            stagedOptions
        } = state;

        console.log('selectedValues', selectedValues);
        const selectedOptions: OptionsType<OptionType> = stagedOptions
            ? stagedOptions[selectedGroup]
            : selectedValues.filter(
                  (option: OptionType): boolean =>
                      option.group.value === drilldownOptions[0].value
              );

        return (
            <SelectList
                defaultValue={selectedOptions}
                focusOnMount={true}
                isLoading={isSelectLoading}
                isMulti={true}
                isSearchable={true}
                onChange={onSelectChange}
                onSearchBlur={onSelectSearchBlur}
                onSearchChange={onSelectSearchChange}
                onSearchFocus={onSelectSearchFocus}
                options={selectOptions}
            />
        );
    };

    const datepickerComponent = (): React.ReactNode => {
        const { selectedValues, stagedDateOptions } = state;
        let valueFrom: Date = null;
        let valueTo: Date = null;
        let options: OptionsType<OptionType> = [];
        if (selectedValues.length > 0) {
            options = selectedValues;
        }
        if (stagedDateOptions.length > 0) {
            options = stagedDateOptions;
        }

        if (options.length) {
            options.forEach((option: OptionType): void => {
                if (option.group.value === DateDimensions.END_DATE) {
                    valueTo = new Date(
                        `${dateFnsFormat(
                            option.value as string,
                            SLASH_DATE_FORMAT
                        )}`
                    );
                } else if (option.group.value === DateDimensions.START_DATE) {
                    valueFrom = new Date(
                        `${dateFnsFormat(
                            option.value as string,
                            SLASH_DATE_FORMAT
                        )}`
                    );
                }
            });
        }

        return (
            <div className="p-5">
                <DateRangeInput
                    focusOnMount={true}
                    onConfirmDates={onConfirmDates}
                    selectedDates={{
                        from: valueFrom,
                        to: valueTo
                    }}
                />
            </div>
        );
    };

    const textInputComponent = (applyHandler: () => void): React.ReactNode => {
        function onTextInputEnter(event: React.KeyboardEvent): void {
            if (event.key === 'Enter') {
                event.preventDefault();
                applyHandler();
            }
        }

        return (
            <div className="p-5">
                <TextField
                    append="USD"
                    autofocus
                    label="Budget"
                    onChange={onTextFieldChange}
                    onKeyPress={onTextInputEnter}
                    prepend="$"
                    value={state.textFieldValue}
                />
            </div>
        );
    };

    const onNumberRangeChange = (
        position: 0 | 1
    ): ((e: React.ChangeEvent<HTMLInputElement>) => void) => (
        event: React.ChangeEvent<HTMLInputElement>
    ): void => {
        const newRange = [...(state.numberRange || VALUE_RANGE)];
        newRange[position] = Number(event.target.value);
        dispatch({
            type: 'CHANGE_NUMBER_RANGE',
            payload: newRange
        });
    };

    const numberRangeComponent = (): React.ReactNode => {
        const [min, max] =
            state.numberRange || state.selectedNumberRange || VALUE_RANGE;

        return (
            <div className="p-5">
                <div className="flex flex-wrap">
                    <div className="w-1/2 pr-2">
                        <TextField
                            label={`Min (>= ${VALUE_RANGE[0]})`}
                            max={String(max)}
                            min={String(VALUE_RANGE[0])}
                            onChange={onNumberRangeChange(0)}
                            textFieldType={TextFieldType.Number}
                            value={String(min)}
                        />
                    </div>
                    <div className="w-1/2 pl-2">
                        <TextField
                            label={`Max (<= ${VALUE_RANGE[1]})`}
                            max={String(VALUE_RANGE[1])}
                            min={String(min)}
                            onChange={onNumberRangeChange(1)}
                            textFieldType={TextFieldType.Number}
                            value={String(max)}
                        />
                    </div>
                </div>
            </div>
        );
    };

    const onSliderChange = (minValue: number, maxValue?: number): void => {
        dispatch({
            type: 'CHANGE_SLIDER_VALUES',
            payload: [minValue, maxValue]
        });
    };

    const sliderComponent = (): React.ReactNode => (
        <div className="p-5">
            <Slider
                currentValue={state.selectedSliderValues}
                isTextInputBottom
                max={VALUE_RANGE[1]}
                min={VALUE_RANGE[0]}
                onChange={onSliderChange}
                width={215}
            />
        </div>
    );

    const subComponent = (
        group: ValueType<OptionType>,
        applyHandler: () => void
    ): React.ReactNode => {
        let subComp: React.ReactNode;
        switch (group.value) {
            case drilldownOptions[0].value:
                subComp = selectComponent();
                break;
            case drilldownOptions[1].value:
                subComp = datepickerComponent();
                break;
            case drilldownOptions[2].value:
                subComp = numberRangeComponent();
                break;
            case drilldownOptions[3].value:
                subComp = sliderComponent();
                break;
            case drilldownOptions[4].value:
                subComp = textInputComponent(applyHandler);
                break;
            default:
                subComp = null;
        }

        return subComp;
    };

    const getSelectedOptions = (): OptionsType<OptionType> => {
        const { selectedValues, stagedOptions, selectedGroup } = state;
        const selectedOptions: OptionsType<OptionType> =
            stagedOptions && stagedOptions[selectedGroup]
                ? stagedOptions[selectedGroup]
                : selectedValues.filter(
                      (option: OptionType): boolean =>
                          option.group.value === drilldownOptions[0].value
                  );
        return selectedOptions;
    };

    return (
        <section className="my-2">
            <div className="va-search-filter-bar">
                <Search
                    id="search"
                    name="search"
                    onChange={onChangeSearch}
                    onClear={onClearSearch}
                    placeholder="Search"
                    value={state.search}
                />
                <Filter
                    groupOptions={state.groupOptions}
                    isLoading={state.isLoading}
                    onApply={onApplyHandler}
                    onApplySelectedOptions={onApplySelectedOptionsHandler}
                    onBack={onBackHandler}
                    onCancel={onCancelHandler}
                    onChangeGroup={onGroupChangeHandler}
                    onClose={onCloseHandler}
                    onOpen={onOpenHandler}
                    onRemoveAllGroups={onRemoveAllGroupsHandler}
                    onRemoveGroup={onRemoveGroupHandler}
                    selectedValues={state.selectedValues}
                    stagedOptions={getSelectedOptions()}
                    subComponent={subComponent}
                />
            </div>
        </section>
    );
}
