import React from 'react';

import { Toggle } from '@preamp/core';
import {
    ActionMeta,
    OptionsType,
    OptionType,
    SelectList,
    ValueType
} from '@preamp/select';

const defaultSelectOptions: OptionsType<OptionType> = [
    {
        label: 'Option 1',
        value: 'Value 1'
    },
    {
        label: 'Option 2',
        value: 'Value 2'
    },
    {
        label: 'Option 3',
        value: 'Value 3'
    },
    {
        label: 'Option 4',
        value: 'Value 4'
    },
    {
        label: 'Option 5',
        value: 'Value 5'
    }
];

interface SimpleSelectListState {
    isLoading: boolean;
    isSearchable: boolean;
    replaceOptionsWithLoader: boolean;
    scrollCount: number;
    selectOptions: OptionsType<OptionType>;
    selectedValues: OptionsType<OptionType>;
}

/**
 * Example of the SelectList including infinite scroll and optional search
 */
const SimpleSelectList: React.FC = (): React.ReactElement => {
    const [state, setState] = React.useState<SimpleSelectListState>({
        isLoading: false,
        isSearchable: false,
        replaceOptionsWithLoader: true,
        scrollCount: 0,
        selectOptions: defaultSelectOptions,
        selectedValues: [
            {
                label: 'Option 2',
                value: 'Value 2'
            }
        ]
    });

    const onSelectChange = (
        value: ValueType<OptionType>,
        action: ActionMeta
    ): void => {
        // eslint-disable-next-line no-console
        console.log('onSelectChange', value, action);
        setState(
            (prevState): SimpleSelectListState => ({
                ...prevState,
                selectedValues: value
            })
        );
    };

    const onSelectSearchFocus = (
        event: React.FocusEvent<HTMLInputElement>
    ): void => {
        // eslint-disable-next-line no-console
        console.log('onSelectSearchFocus', event);
    };

    const onSelectSearchBlur = (
        event: React.FocusEvent<HTMLInputElement>
    ): void => {
        // eslint-disable-next-line no-console
        console.log('onSelectSearchBlur', event);
    };

    const onSelectSearchChange = (
        value: string,
        event: React.ChangeEvent<HTMLInputElement>
    ): void => {
        // eslint-disable-next-line no-console
        console.log('onSelectSearchChange', value, event);
        setState(
            (prevState): SimpleSelectListState => ({
                ...prevState,
                isLoading: true
            })
        );
        if (!value || value === '') {
            // reset to default options
            setTimeout((): void => {
                setState(
                    (prevState): SimpleSelectListState => ({
                        ...prevState,
                        isLoading: false,
                        selectOptions: defaultSelectOptions
                    })
                );
            }, 500);
        } else if (value.length > 10) {
            // simulate empty results
            setTimeout((): void => {
                setState(
                    (prevState): SimpleSelectListState => ({
                        ...prevState,
                        isLoading: false,
                        selectOptions: []
                    })
                );
            }, 1000);
        } else {
            // simulate new results
            setTimeout((): void => {
                setState(
                    (prevState): SimpleSelectListState => ({
                        ...prevState,
                        isLoading: false,
                        selectOptions: [
                            {
                                label: 'New Option 1',
                                value: 'New Value 1'
                            },
                            {
                                label: 'New Option 2',
                                value: 'New Value 2'
                            }
                        ]
                    })
                );
            }, 1000);
        }
    };

    const infiniteScroll = (): void => {
        setState(
            (prevState): SimpleSelectListState => ({
                ...prevState,
                isLoading: true,
                replaceOptionsWithLoader: false,
                scrollCount: prevState.scrollCount + 1
            })
        );
        // Fake async fetch
        setTimeout((): void => {
            setState(
                (prevState): SimpleSelectListState => ({
                    ...prevState,
                    isLoading: false,
                    replaceOptionsWithLoader: true,
                    selectOptions: prevState.selectOptions.concat(
                        defaultSelectOptions.map(
                            (option: OptionType): OptionsType<OptionType> => ({
                                ...option,
                                label: `${option.label}.${prevState.scrollCount}`
                            })
                        )
                    )
                })
            );
        }, 2000);
    };

    return (
        <section className="max-w-md my-6">
            <Toggle
                checked={state.isSearchable}
                label="Searchable List"
                name="isSearchable"
                onChange={(checked: boolean): void => {
                    setState(
                        (prevState): SimpleSelectListState => ({
                            ...prevState,
                            isSearchable: checked
                        })
                    );
                }}
            />
            <SelectList
                defaultValue={state.selectedValues}
                focusOnMount={true}
                isLoading={state.isLoading}
                isMulti={true}
                isSearchable={state.isSearchable}
                maxMenuHeight={100}
                onChange={onSelectChange}
                onMenuScrollToBottom={infiniteScroll}
                onSearchBlur={onSelectSearchBlur}
                onSearchChange={onSelectSearchChange}
                onSearchFocus={onSelectSearchFocus}
                options={state.selectOptions}
                replaceOptionsWithLoader={state.replaceOptionsWithLoader}
            />
        </section>
    );
};

export default SimpleSelectList;
