import React from 'react';

import cx from 'classnames';

import { Drawer } from '@preamp/core';
import {
    StackSelect,
    StackSelectItem,
    StackSelectOptionType
} from '@preamp/select';
import { Equalizer, Location, Signal } from '@preamp/signal';

const TRANSITION_DURATION = 350;

const data: StackSelectOptionType[] = [
    {
        label: 'Light TV Viewers',
        name: 'color-1',
        value: 'item-1',
        color: '#ff8a80'
    },
    {
        label: 'Medium TV Viewers',
        name: 'color-2',
        value: 'item-2',
        color: '#ff80ab'
    },
    {
        label: 'This is a really long label that should be truncated',
        name: 'color-3',
        value: 'item-3',
        color: '#ea80fc'
    }
];

const dataWithoutLabels: StackSelectOptionType[] = data.map(
    (option: StackSelectOptionType): StackSelectOptionType => ({
        ...option,
        label: ''
    })
);

interface ExampleDrawerState {
    isCollapsed: boolean;
    isHovered: boolean;
    isLockedOpen: boolean;
    selected: Set<React.ReactText>;
}

function reducer(
    state: ExampleDrawerState,
    update: Partial<ExampleDrawerState>
): ExampleDrawerState {
    return { ...state, ...update };
}

/**
 * Basic Drawer containing a collapsible Stack Select.
 */
const ExampleDrawer: React.FC = (): React.ReactElement => {
    const [
        { isCollapsed, isHovered, isLockedOpen, selected },
        setState
    ] = React.useReducer(reducer, {
        isCollapsed: true,
        isHovered: false,
        isLockedOpen: false,
        selected: new Set([data[0].value, data[1].value, data[2].value])
    });

    const isOpening = React.useMemo((): boolean => isCollapsed && isHovered, [
        isCollapsed,
        isHovered
    ]);

    const isStackSelectCollapsed = React.useMemo(
        (): boolean => (isHovered ? false : isCollapsed),
        [isCollapsed, isHovered]
    );

    const handleChange = React.useCallback(
        (_: string, value: string, checked: boolean): void => {
            const newSelectedOptions: Set<React.ReactText> = new Set(selected);

            if (checked) {
                newSelectedOptions.add(value);
            } else {
                newSelectedOptions.delete(value);
            }

            setState({
                selected: newSelectedOptions
            });
        },
        [selected]
    );

    const handleItemChange = (): void => {
        console.log('Item changed');
    };

    const onMouseLeave = React.useCallback((): void => {
        if (!isLockedOpen) {
            setState({
                isCollapsed: true,
                isHovered: false
            });
        }
    }, [isLockedOpen]);

    const onMouseEnter = React.useCallback((): void => {
        if (!isLockedOpen) {
            setState({
                isHovered: true
            });

            setTimeout((): void => {
                if (isHovered) {
                    setState({
                        isCollapsed: false
                    });
                }
            }, TRANSITION_DURATION);
        }
    }, [isHovered, isLockedOpen]);

    const toggleOpen = React.useCallback((): void => {
        setState({
            isCollapsed: false,
            isLockedOpen: !isLockedOpen
        });
    }, [isLockedOpen]);

    return (
        <Drawer
            isCollapsed={!isHovered}
            isLockedOpen={isLockedOpen}
            onClick={toggleOpen}
            onMouseEnter={onMouseEnter}
            onMouseLeave={onMouseLeave}
            title="Filters"
        >
            <div
                className="overflow-hidden"
                style={{
                    visibility: isCollapsed ? 'hidden' : 'visible'
                }}
            >
                <h6 className="h-4 uppercase u-body-3 u-font-medium">
                    Audiences
                </h6>
                <p className="u-body-2 mb-1">
                    Select one or more of the audiences to see how they compare.
                    Select Total Universe to see total content delivery
                    regardless of audience.
                </p>
            </div>
            <div className="mt-8">
                <StackSelectItem
                    color="#FF4500"
                    isCollapsed={isStackSelectCollapsed}
                    isDisabled={false}
                    isMulti={true}
                    isSelected={true}
                    key="single-item"
                    label={isOpening ? '' : 'Total Universe'}
                    name="Total Universe"
                    onChange={handleItemChange}
                    value="item value"
                />
            </div>
            <div className="mt-6">
                <h6 className="mb-2 uppercase u-body-3 u-font-medium">
                    <Signal
                        className={cx('align-text-bottom', {
                            'mr-2': !isCollapsed,
                            'va-drawer-icon': isCollapsed
                        })}
                        icon={Location}
                        size="sm"
                        title="Target Audiences"
                    />
                    {!isCollapsed && 'Target Audiences'}
                </h6>
                <StackSelectItem
                    color="#4B0082"
                    isCollapsed={isStackSelectCollapsed}
                    isDisabled={true}
                    isMulti={true}
                    isSelected={true}
                    key="Target Audience Women"
                    label={isOpening ? '' : 'Women 18-24'}
                    name="Target Audience Women"
                    onChange={handleItemChange}
                    value="item value"
                />
            </div>
            <div className="mt-6 mb-3">
                <h6 className="mb-2 uppercase u-body-3 u-font-medium">
                    <Signal
                        className={cx('align-text-bottom', {
                            'mr-2': !isCollapsed,
                            'va-drawer-icon': isCollapsed
                        })}
                        icon={Equalizer}
                        size="sm"
                        title="Measurement Audiences"
                    />
                    {!isCollapsed && 'Measurement Audiences'}
                </h6>
                <StackSelect
                    isCollapsed={isStackSelectCollapsed}
                    isMulti={true}
                    onChange={handleChange}
                    options={isOpening ? dataWithoutLabels : data}
                    selected={selected}
                />
            </div>
        </Drawer>
    );
};

export default ExampleDrawer;
