import * as React from 'react';

import cx from 'classnames';
import ReactMarkdown from 'react-markdown';

import { TextField, TextFieldType } from '@preamp/core';

import { Signal as SignalIcon } from '../../../signal/';
import * as Icons from '../../../signal/lib/assets/signal-icons';
import { IMarkdownFile } from '../../../types';
import { getMarkdownFileByName } from '../utils/findFile';

import * as styles from './signalDoc.scss';

const iconValues: object = Object.values(Icons);
const iconKeys: string[] = Object.keys(Icons);

export const Signal: React.FC = (): React.ReactElement => {
    const [selectedIcon, setSelectedIcon] = React.useState(iconValues[0]);
    const [selectedIconName, setSelectedIconName] = React.useState(iconKeys[0]);
    const [customIconSize, setCustomSize] = React.useState('3');
    const [iconColorPrimary, setIconColorPrimary] = React.useState(
        'var(--va-teal-300)'
    );
    const [iconColorSecondary, setIconColorSecondary] = React.useState(
        'var(--va-grey-500)'
    );

    const signalDoc: IMarkdownFile = getMarkdownFileByName(
        'signal',
        'signal.md'
    );

    function onIconSizeChange(
        event?: React.ChangeEvent<HTMLInputElement>
    ): void {
        setCustomSize(event.currentTarget.value);
    }
    function onIconPrimaryColorChange(
        event?: React.ChangeEvent<HTMLInputElement>
    ): void {
        setIconColorPrimary(event.currentTarget.value);
    }
    function onIconSecondaryColorChange(
        event?: React.ChangeEvent<HTMLInputElement>
    ): void {
        setIconColorSecondary(event.currentTarget.value);
    }

    function selectIcon(index: number): void {
        setSelectedIcon(iconValues[index]);
        setSelectedIconName(iconKeys[index]);
    }

    return (
        <div
            className={cx(
                'va-grid va-columns-default',
                styles.signalDocumentation
            )}
        >
            <div className="col-span-xl-10 col-span-12 ">
                <section>
                    <ReactMarkdown source={signalDoc.contents} />
                </section>
                <section>
                    <h3>Adding Icons to the Library</h3>
                    <ul>
                        <li>
                            Place raw SVG files in the <code>icon-src</code>{' '}
                            directory
                        </li>
                        <li>
                            Run <code>yarn create:icons</code> from the{' '}
                            <code>signal</code> directory
                        </li>
                        <li>
                            Icon components will be placed in the{' '}
                            <code>assets</code> directory and can be used by the{' '}
                            <code>Signal</code> component.
                        </li>
                    </ul>
                </section>
                <section>
                    <h3>Usage</h3>
                    <p>
                        The <code>Signal</code> component takes an{' '}
                        <code>icon</code> prop that can be any of the generated
                        icon components in the <code>assets</code> directory.
                        Additionally, it can take the following props:
                    </p>
                    <ul>
                        <li>
                            <code>color</code> - sets the primary color of the
                            icon (if this prop isn&apos;t present, the icon will
                            take on the text color value of its direct parent).
                        </li>
                        <li>
                            <code>colorSecondary</code> - if the icon has a
                            secondary color element specified in the SVG, this
                            prop will apply color to it. The secondary color
                            element can be specified in the SVG by applying{' '}
                            <code>id=&quot;secondary&quot;</code> to the path.
                        </li>
                        <li>
                            <code>size</code> - this prop can either use the{' '}
                            <code>SignalSize</code> preset enum values or be a
                            number that will be applied as a <code>rem</code>{' '}
                            value. If no size is specified, the font-size of the
                            parent element will be applied.
                        </li>
                        <li>
                            Additional props can be found on the{' '}
                            <a href="#/signal/icon/Signal/">
                                Signal component page
                            </a>
                        </li>
                    </ul>
                </section>
                <h3>Icons</h3>

                <div className={styles.iconSelection}>
                    <div className={styles.selectedIconContainer}>
                        <SignalIcon
                            color={iconColorPrimary}
                            colorSecondary={iconColorSecondary}
                            icon={selectedIcon}
                            size={parseFloat(customIconSize)}
                        />
                        <div className={styles.codeContainer}>
                            <label>
                                &#60;Signal icon=&#123;{selectedIconName}
                                &#125; size=&#123;
                                {customIconSize}&#125; color=&quot;
                                {iconColorPrimary}
                                &quot; colorSecondary=&quot;{iconColorSecondary}
                                &quot; /&#62;
                            </label>
                        </div>
                    </div>
                    <div className={styles.iconControls}>
                        <div className="va-grid va-columns-default">
                            <div className="col-span-4">
                                <TextField
                                    helpText="Custom icon size from the customeSize prop in rem - if undefined, it takes font-size of parent element."
                                    label="Default size (rem)"
                                    onChange={onIconSizeChange}
                                    textFieldType={TextFieldType.Number}
                                    value={customIconSize}
                                />
                            </div>
                            <div className="col-span-4">
                                <TextField
                                    helpText="Enter CSS Custom Property or hex value"
                                    label="Primary Color"
                                    onChange={onIconPrimaryColorChange}
                                    textFieldType={TextFieldType.Text}
                                    value={iconColorPrimary}
                                />
                            </div>
                            <div className="col-span-4">
                                <TextField
                                    helpText="Enter CSS Custom Property or hex value, or leave blank to use the primary color"
                                    label="Secondary Color"
                                    onChange={onIconSecondaryColorChange}
                                    textFieldType={TextFieldType.Text}
                                    value={iconColorSecondary}
                                />
                            </div>
                        </div>
                    </div>
                </div>
                <h4>Select an Icon:</h4>
                <div className={styles.iconListContainer}>
                    {iconKeys.map(
                        (icon: string, index: number): React.ReactNode => (
                            <div
                                key={index}
                                onClick={(): void => selectIcon(index)}
                                role="button"
                            >
                                <SignalIcon
                                    icon={iconValues[index]}
                                    size={1.5}
                                />
                                <label>{icon}</label>
                            </div>
                        )
                    )}
                </div>
            </div>
        </div>
    );
};
