import React from 'react';

import { IconButton, Search, Toggle } from '@preamp/core';
import {
    Filter,
    OptionsType,
    OptionType,
    SelectList,
    ValueType
} from '@preamp/select';
import { Crop, Download, ViewColumn } from '@preamp/signal';
import { TableActionBar } from '@preamp/tables';

const drilldownOptions: OptionsType<OptionType> = [
    {
        label: 'Colors',
        value: 'colors'
    },
    {
        label: 'Flavors',
        value: 'flavors'
    }
];

const colorOptions: OptionsType<OptionType> = [
    {
        value: 'ocean',
        label: 'Ocean'
    },
    {
        value: 'blue',
        label: 'Blue'
    },
    { value: 'purple', label: 'Purple' },
    { value: 'red', label: 'Red' },
    { value: 'orange', label: 'Orange' }
];

const flavorOptions: OptionsType<OptionType> = [
    { value: 'vanilla', label: 'Vanilla' },
    { value: 'chocolate', label: 'Chocolate' },
    { value: 'strawberry', label: 'Strawberry' },
    {
        value: 'salted-caramel',
        label: 'Salted Caramel'
    }
];

export default function ExampleUtilityBar(): React.ReactElement<{}> {
    const [state, setState] = React.useState({
        filters: [],
        stagedColorOptions: null,
        stagedFlavorOptions: null,
        showBottomBorder: false
    });

    const handleClick = (): void => {
        // eslint-disable-next-line no-console
        console.log('CLICKED');
    };

    const onColorSelectChange = (value: ValueType<OptionType>): void => {
        setState({
            ...state,
            stagedColorOptions: value
        });
    };

    const onFlavorSelectChange = (value: ValueType<OptionType>): void => {
        setState({
            ...state,
            stagedFlavorOptions: value
        });
    };

    const subComponent = (
        selectedGroup: ValueType<OptionType>
    ): React.ReactNode => {
        const { filters } = state;
        let subComp: React.ReactNode;
        switch (selectedGroup.value) {
            case drilldownOptions[0].value:
                const selectedColorOptions: OptionsType<OptionType> =
                    filters.length < 1
                        ? null
                        : filters.filter(
                              (option: OptionType): boolean =>
                                  option.group.value ===
                                  drilldownOptions[0].value
                          )
                        ? true
                        : false;
                subComp = (
                    <SelectList
                        defaultValue={selectedColorOptions}
                        focusOnMount={true}
                        isMulti={true}
                        isSearchable={false}
                        onChange={onColorSelectChange}
                        options={colorOptions}
                    />
                );
                break;
            case drilldownOptions[1].value:
                const selectedFlavorOptions: OptionsType<OptionType> =
                    filters.length < 1
                        ? null
                        : filters.filter((option: OptionType): boolean =>
                              option.group.value === drilldownOptions[1].value
                                  ? true
                                  : false
                          );
                subComp = (
                    <SelectList
                        defaultValue={selectedFlavorOptions}
                        focusOnMount={true}
                        isMulti={true}
                        isSearchable={false}
                        onChange={onFlavorSelectChange}
                        options={flavorOptions}
                    />
                );
                break;
            default:
                subComp = null;
        }

        return subComp;
    };

    const onApplyHandler = (selectedGroup: ValueType<OptionType>): void => {
        const { filters, stagedColorOptions, stagedFlavorOptions } = state;
        let updatedValues: OptionsType<OptionType> = [];
        if (
            selectedGroup.value === drilldownOptions[0].value &&
            stagedColorOptions
        ) {
            // remove options within the same select group
            let reducedOptions: OptionsType<OptionType> = [];
            if (filters) {
                // remove any options set within the same group
                reducedOptions = filters.filter((option: OptionType): boolean =>
                    option.group.value === drilldownOptions[0].value
                        ? false
                        : true
                );
            }
            // add group data to new selected options
            const optionsToApply: OptionsType<OptionType> = stagedColorOptions.map(
                (option: OptionType): any => ({
                    ...option,
                    group: selectedGroup
                })
            );
            updatedValues = [...reducedOptions, ...optionsToApply];
        } else if (
            selectedGroup.value === drilldownOptions[1].value &&
            stagedFlavorOptions
        ) {
            // remove options within the same select group
            let reducedOptions: OptionsType<OptionType> = [];
            if (filters) {
                // remove any options set within the same group
                reducedOptions = filters.filter((option: OptionType): boolean =>
                    option.group.value === drilldownOptions[1].value
                        ? false
                        : true
                );
            }
            // add group data to new selected options
            const optionsToApply: OptionsType<OptionType> = stagedFlavorOptions.map(
                (option: OptionType): any => ({
                    ...option,
                    group: selectedGroup
                })
            );
            updatedValues = [...reducedOptions, ...optionsToApply];
        }
        setState({
            ...state,
            filters: updatedValues
        });
    };

    const onApplySelectedOptionsHandler = (
        group: ValueType<OptionType>,
        values: OptionsType<OptionType>
    ): void => {
        const { filters } = state;
        let reducedOptions: OptionsType<OptionType> = [];
        if (filters) {
            // remove any options set within the same group
            reducedOptions = filters.filter((option: OptionType): boolean =>
                option.group.value === group.value ? false : true
            );
        }
        // add group data to new selected options
        const optionsToApply: OptionsType<OptionType> = values.map(
            (option: OptionType): any => {
                return {
                    ...option,
                    group: group
                };
            }
        );
        const updatedFilters: OptionsType<OptionType> = [
            ...reducedOptions,
            ...optionsToApply
        ];
        setState({
            ...state,
            filters: updatedFilters
        });
    };

    const onRemoveGroupHandler = (
        groupToRemove: ValueType<OptionType>
    ): void => {
        const { filters } = state;
        const updatedFilters: OptionsType<OptionType> = filters.filter(
            (option: OptionType): boolean => {
                if (groupToRemove.value === option.group.value) {
                    return false;
                } else {
                    return true;
                }
            }
        );
        setState({
            ...state,
            filters: updatedFilters
        });
    };

    const onClear = (): void => {
        // eslint-disable-next-line no-console
        console.log('clear');
    };

    const onSearch = (value: string): void => {
        // eslint-disable-next-line no-console
        console.log(`Search: ${value}`);
    };

    const offToggleChanged = (): void => {
        setState({
            ...state,
            showBottomBorder: !state.showBottomBorder
        });
    };

    return (
        <>
            <Toggle
                checked={state.showBottomBorder}
                id="disabled-off-toggle"
                label="Show bottom border"
                name="disabled-off-toggle"
                onChange={offToggleChanged}
            />
            <TableActionBar
                Actions={[
                    <IconButton
                        icon={ViewColumn}
                        key="Search"
                        onClick={handleClick}
                        size="sm"
                    />,
                    <IconButton
                        icon={Crop}
                        key="Help"
                        onClick={handleClick}
                        size="sm"
                    />,
                    <IconButton
                        icon={Download}
                        key="NotificationsNone"
                        onClick={handleClick}
                        size="sm"
                    />
                ]}
                showBottomBorder={state.showBottomBorder}
                showTopBorder
            >
                <Search onClear={onClear} onSearch={onSearch} />
                <Filter
                    groupOptions={drilldownOptions}
                    onApply={onApplyHandler}
                    onApplySelectedOptions={onApplySelectedOptionsHandler}
                    onRemoveGroup={onRemoveGroupHandler}
                    selectedValues={state.filters}
                    subComponent={subComponent}
                />
            </TableActionBar>
        </>
    );
}
