import React from 'react';

import { keyboardKeys } from '@preamp/core';
import { Edit, Trash } from '@preamp/signal';
import {
    DateCell,
    DateCellProps,
    IListTableColumn,
    IRowAction,
    ListTable,
    NameDescriptionCell,
    NameDescriptionCellProps,
    NumberCell,
    NumberCellProps
} from '@preamp/tables';

enum columnEnum {
    Name = 'name',
    Created = 'created',
    LastUpdated = 'lastUpdated',
    Size = 'size'
}

interface SelectableListTableRow {
    identifier: number;
    [columnEnum.Name]: NameDescriptionCellProps;
    [columnEnum.Created]: DateCellProps;
    [columnEnum.LastUpdated]: DateCellProps;
    [columnEnum.Size]: NumberCellProps;
}

interface SelectableListTableState {
    selectedRow: number;
}

function NameDescriptionCellComponent({ value }: any): React.ReactNode {
    return (
        <NameDescriptionCell
            description={value.description}
            descriptionTruncationLines={2}
            name={value.name}
            pending={value.pending}
        />
    );
}
function CreatedDateCellComponent({ value }: any): React.ReactNode {
    return <DateCell date={value.date} description={value.description} />;
}
function UpdatedDateCellComponent({ value }: any): React.ReactNode {
    return <DateCell date={value.date} description={value.description} />;
}
function SizeNumberCellComponent({ value }: any): React.ReactNode {
    return <NumberCell description={value.description} value={value.value} />;
}

const columns: IListTableColumn[] = [
    {
        identifier: columnEnum.Name,
        header: 'Name',
        minWidth: 150,
        CustomCell: NameDescriptionCellComponent
    },
    {
        identifier: columnEnum.Created,
        header: 'Created',
        minWidth: 100,
        CustomCell: CreatedDateCellComponent
    },
    {
        identifier: columnEnum.LastUpdated,
        header: 'Last Updated',
        minWidth: 100,
        CustomCell: UpdatedDateCellComponent
    },
    {
        identifier: columnEnum.Size,
        header: 'Size',
        minWidth: 100,
        isMetric: true,
        CustomCell: SizeNumberCellComponent
    }
];

const rows: SelectableListTableRow[] = [
    {
        identifier: 1,
        name: {
            name:
                'This is a really long name that should truncate and should show an ellipsis after set number of lines',
            description:
                "Is this my espresso machine? Wh-what is-h-how did you get my espresso machine? Must go faster. We gotta burn the rain forest, dump toxic waste, pollute the air, and rip up the OZONE! 'Cause maybe if we screw up this planet enough, they won't want it anymore!"
        },
        [columnEnum.Created]: {
            date: new Date(2017, 1, 20),
            description: 'John Doe'
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2018, 3, 20),
            description: 'John Doe'
        },
        [columnEnum.Size]: { value: 35189 }
    },
    {
        identifier: 2,
        [columnEnum.Name]: {
            name: 'Amy',
            description:
                'Donec id elit non mi porta gravida at eget metus. Sed posuere consectet'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 2, 20),
            description: 'John Doe'
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2019, 4, 20),
            description: 'John Doe'
        },
        [columnEnum.Size]: { value: 14138890 }
    },
    {
        identifier: 3,
        [columnEnum.Name]: {
            name: 'Carl',
            description:
                'Donec id elit non mi porta gravida at eget metus. Sed posuere consectet'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 2, 20),
            description: 'John Doe'
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2016, 5, 20),
            description: 'John Doe'
        },
        [columnEnum.Size]: { value: 145876 }
    },
    {
        identifier: 4,
        [columnEnum.Name]: {
            name: 'Doug',
            description:
                'Donec id elit non mi porta gravida at eget metus. Sed posuere consectet'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 3, 20),
            description: 'John Doe'
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2018, 6, 20),
            description: 'John Doe'
        },
        [columnEnum.Size]: { value: 75463 }
    },
    {
        identifier: 5,
        [columnEnum.Name]: {
            name: 'Emily',
            description:
                'Donec id elit non mi porta gravida at eget metus. Sed posuere consectet'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 4, 20),
            description: 'John Doe'
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2018, 7, 20),
            description: 'John Doe'
        },
        [columnEnum.Size]: { value: 984153 }
    }
];

class SelectableListTable extends React.Component<
    {},
    SelectableListTableState
> {
    state: SelectableListTableState = {
        selectedRow: null
    };

    onRowClicked = (rowIdentifier: number): void => {
        this.setState({ selectedRow: rowIdentifier });
    };

    onRowKeyPressed = (
        rowIdentifier: number,
        event: React.KeyboardEvent
    ): void => {
        const key: number = event.keyCode || event.which;
        if (key === keyboardKeys.Enter || key === keyboardKeys.SpaceBar) {
            event.preventDefault();
            this.setState({ selectedRow: rowIdentifier });
        }
    };

    onClickDelete = (
        rowIdentifier: string | number,
        event: React.MouseEvent
    ): void => {
        event.stopPropagation();
        // eslint-disable-next-line no-console
        console.log('DELETE', rowIdentifier);
    };
    onClickEdit = (
        rowIdentifier: string | number,
        event: React.MouseEvent
    ): void => {
        event.stopPropagation();
        // eslint-disable-next-line no-console
        console.log('EDIT', rowIdentifier);
    };

    render(): React.ReactNode {
        const rowActions: IRowAction[] = [
            {
                icon: Trash,
                onClick: this.onClickDelete,
                dataUI: 'delete-button',
                tooltip: 'Delete this item.'
            },
            {
                icon: Edit,
                onClick: this.onClickEdit,
                dataUI: 'edit-button',
                tooltip: 'Edit this item'
            }
        ];

        return (
            <>
                <div>Selected State</div>
                <ListTable
                    columns={columns}
                    dataUI="basic-list-table"
                    onRowClicked={this.onRowClicked}
                    onRowKeyPress={this.onRowKeyPressed}
                    rowActions={rowActions}
                    rows={rows}
                    selectedRow={this.state.selectedRow}
                />
            </>
        );
    }
}

export default SelectableListTable;
