import React from 'react';

import { sortBy } from 'lodash';

import {
    DateCell,
    DateCellProps,
    IListTableColumn,
    ListTable,
    NameDescriptionCell,
    NameDescriptionCellProps,
    NumberCell,
    NumberCellProps,
    sortDirectionEnum
} from '@preamp/tables';

interface CustomSortingListTableState {
    rows: CustomSortingListTableRow[];
    sortedColumnIdentifier: React.ReactText;
    sortDirection: sortDirectionEnum;
}

enum columnEnum {
    Name = 'name',
    Created = 'created',
    LastUpdated = 'lastUpdated',
    Size = 'size'
}

interface CustomSortingListTableRow {
    identifier: number;
    [columnEnum.Name]: NameDescriptionCellProps;
    [columnEnum.Created]: DateCellProps;
    [columnEnum.LastUpdated]: DateCellProps;
    [columnEnum.Size]: NumberCellProps;
}

const originalRows: CustomSortingListTableRow[] = [
    {
        identifier: 1,
        [columnEnum.Name]: {
            name: 'Bob',
            description:
                'Donec id elit non mi porta gravida at eget metus. Sed posuere consectet'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 1, 20),
            description: 'John Doe'
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2018, 3, 20),
            description: 'John Doe'
        },
        [columnEnum.Size]: { value: 35189 }
    },
    {
        identifier: 2,
        [columnEnum.Name]: {
            name: 'Amy',
            description:
                'Donec id elit non mi porta gravida at eget metus. Sed posuere consectet'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 2, 20),
            description: 'John Doe'
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2019, 4, 20),
            description: 'John Doe'
        },
        [columnEnum.Size]: { value: 14138890 }
    },
    {
        identifier: 3,
        [columnEnum.Name]: {
            name: 'Carl',
            description:
                'Donec id elit non mi porta gravida at eget metus. Sed posuere consectet'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 2, 20),
            description: 'John Doe'
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2016, 5, 20),
            description: 'John Doe'
        },
        [columnEnum.Size]: { value: 145876 }
    },
    {
        identifier: 4,
        [columnEnum.Name]: {
            name: 'Doug',
            description:
                'Donec id elit non mi porta gravida at eget metus. Sed posuere consectet'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 3, 20),
            description: 'John Doe'
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2018, 6, 20),
            description: 'John Doe'
        },
        [columnEnum.Size]: { value: 75463 }
    },
    {
        identifier: 5,
        [columnEnum.Name]: {
            name: 'Emily',
            description:
                'Donec id elit non mi porta gravida at eget metus. Sed posuere consectet'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 4, 20),
            description: 'John Doe'
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2018, 7, 20),
            description: 'John Doe'
        },
        [columnEnum.Size]: { value: 984153 }
    }
];

function NameDescriptionCellComponent({ value }: any): React.ReactNode {
    return (
        <NameDescriptionCell
            description={value.description}
            name={value.name}
            pending={value.pending}
        />
    );
}
function CreatedDateCellComponent({ value }: any): React.ReactNode {
    return <DateCell date={value.date} description={value.description} />;
}
function UpdatedDateCellComponent({ value }: any): React.ReactNode {
    return <DateCell date={value.date} description={value.description} />;
}
function SizeNumberCellComponent({ value }: any): React.ReactNode {
    return <NumberCell description={value.description} value={value.value} />;
}

const originalColumns: IListTableColumn[] = [
    {
        identifier: columnEnum.Name,
        sortable: true,
        header: 'Name',
        minWidth: 150,
        CustomCell: NameDescriptionCellComponent
    },
    {
        identifier: columnEnum.Created,
        header: 'Created',
        minWidth: 100,
        CustomCell: CreatedDateCellComponent
    },
    {
        identifier: columnEnum.LastUpdated,
        header: 'Last Updated',
        minWidth: 100,
        CustomCell: UpdatedDateCellComponent
    },
    {
        identifier: columnEnum.Size,
        sortable: true,
        header: 'Size',
        minWidth: 100,
        isMetric: true,
        CustomCell: SizeNumberCellComponent
    }
];

class CustomSortingListTable extends React.Component<
    CustomSortingListTableState
> {
    state: CustomSortingListTableState = {
        rows: originalRows,
        sortedColumnIdentifier: 'name',
        sortDirection: sortDirectionEnum.Asc
    };
    sortSize = (sortDirection: sortDirectionEnum): void => {
        const { rows } = this.state;
        const newRowsDesc: CustomSortingListTableRow[] = sortBy(
            rows,
            (row: any): React.ReactNode => row[columnEnum.Size].value
        );
        this.setState({
            rows:
                sortDirection === sortDirectionEnum.Asc
                    ? newRowsDesc
                    : newRowsDesc.reverse()
        });
    };
    sortNames = (sortDirection: sortDirectionEnum): void => {
        const { rows } = this.state;
        const newRowsDesc: CustomSortingListTableRow[] = sortBy(
            rows,
            (row: any): React.ReactNode => row[columnEnum.Name].name
        );
        this.setState({
            rows:
                sortDirection === sortDirectionEnum.Asc
                    ? newRowsDesc
                    : newRowsDesc.reverse()
        });
    };

    onColumnSorted = (
        columnIdentifier: string,
        sortDirection: sortDirectionEnum
    ): void => {
        this.setState(
            {
                sortedColumnIdentifier: columnIdentifier,
                sortDirection
            },
            (): void => {
                if (columnIdentifier === columnEnum.Name) {
                    this.sortNames(sortDirection);
                } else if (columnIdentifier === columnEnum.Size) {
                    this.sortSize(sortDirection);
                }
            }
        );
    };

    render(): React.ReactNode {
        const { rows, sortDirection, sortedColumnIdentifier } = this.state;

        return (
            <>
                <div>Custom Sorting</div>
                <ListTable
                    columns={originalColumns}
                    dataUI="custom-sorting"
                    onColumnSorted={this.onColumnSorted}
                    rows={rows}
                    sortDirection={sortDirection}
                    sortedColumnIdentifier={sortedColumnIdentifier}
                />
            </>
        );
    }
}

export default CustomSortingListTable;
