import React from 'react';

import {
    DateCell,
    DateCellProps,
    IListTableColumn,
    ListTable,
    NameDescriptionCell,
    NameDescriptionCellProps,
    NumberCell,
    NumberCellProps
} from '@preamp/tables';

enum columns {
    Name = 'name',
    Created = 'created',
    LastUpdated = 'lastUpdated',
    Size = 'size'
}

interface CheckableListTableRow {
    identifier: number;
    [columns.Name]: NameDescriptionCellProps;
    [columns.Created]: DateCellProps;
    [columns.LastUpdated]: DateCellProps;
    [columns.Size]: NumberCellProps;
}

interface CheckableListTableState {
    checkedRows: Set<number>;
    allRowsChecked: boolean;
}

class CheckableListTable extends React.Component<{}, CheckableListTableState> {
    state: CheckableListTableState = {
        checkedRows: new Set(),
        allRowsChecked: false
    };
    columns: IListTableColumn[] = [
        {
            identifier: columns.Name,
            header: 'Name',
            minWidth: 150,
            CustomCell: ({ value }: any): React.ReactNode => (
                <NameDescriptionCell
                    description={value.description}
                    name={value.name}
                />
            )
        },
        {
            identifier: columns.Created,
            header: 'Created',
            minWidth: 100,
            CustomCell: ({ value }: any): React.ReactNode => (
                <DateCell date={value.date} description={value.description} />
            )
        },
        {
            identifier: columns.LastUpdated,
            header: 'Last Updated',
            minWidth: 100,
            CustomCell: ({ value }: any): React.ReactNode => (
                <DateCell date={value.date} description={value.description} />
            )
        },
        {
            identifier: columns.Size,
            header: 'Size',
            minWidth: 100,
            isMetric: true,
            CustomCell: ({ value }: any): React.ReactNode => (
                <NumberCell
                    description={value.description}
                    value={value.value}
                />
            )
        }
    ];

    rows: CheckableListTableRow[] = [
        {
            identifier: 1,
            name: {
                name: 'Bob',
                description:
                    'Donec id elit non mi porta gravida at eget metus. Sed posuere consectet'
            },
            [columns.Created]: {
                date: new Date(2017, 1, 20),
                description: 'John Doe'
            },
            [columns.LastUpdated]: {
                date: new Date(2018, 3, 20),
                description: 'John Doe'
            },
            [columns.Size]: { value: 35189 }
        },
        {
            identifier: 2,
            [columns.Name]: {
                name: 'Amy',
                description:
                    'Donec id elit non mi porta gravida at eget metus. Sed posuere consectet'
            },
            [columns.Created]: {
                date: new Date(2017, 2, 20),
                description: 'John Doe'
            },
            [columns.LastUpdated]: {
                date: new Date(2019, 4, 20),
                description: 'John Doe'
            },
            [columns.Size]: { value: 14138890 }
        },
        {
            identifier: 3,
            [columns.Name]: {
                name: 'Carl',
                description:
                    'Donec id elit non mi porta gravida at eget metus. Sed posuere consectet'
            },
            [columns.Created]: {
                date: new Date(2017, 2, 20),
                description: 'John Doe'
            },
            [columns.LastUpdated]: {
                date: new Date(2016, 5, 20),
                description: 'John Doe'
            },
            [columns.Size]: { value: 145876 }
        },
        {
            identifier: 4,
            [columns.Name]: {
                name: 'Doug',
                description:
                    'Donec id elit non mi porta gravida at eget metus. Sed posuere consectet'
            },
            [columns.Created]: {
                date: new Date(2017, 3, 20),
                description: 'John Doe'
            },
            [columns.LastUpdated]: {
                date: new Date(2018, 6, 20),
                description: 'John Doe'
            },
            [columns.Size]: { value: 75463 }
        },
        {
            identifier: 5,
            [columns.Name]: {
                name: 'Emily',
                description:
                    'Donec id elit non mi porta gravida at eget metus. Sed posuere consectet'
            },
            [columns.Created]: {
                date: new Date(2017, 4, 20),
                description: 'John Doe'
            },
            [columns.LastUpdated]: {
                date: new Date(2018, 7, 20),
                description: 'John Doe'
            },
            [columns.Size]: { value: 984153 }
        }
    ];

    onRowChecked = (identifier: number, checked: boolean): void => {
        const { checkedRows, allRowsChecked } = this.state;
        const newCheckedRows: Set<number> = new Set(checkedRows);
        let newAllRowsChecked: boolean = allRowsChecked;
        if (checked) {
            newCheckedRows.add(identifier);
            newAllRowsChecked = newCheckedRows.size === this.rows.length;
        } else {
            newCheckedRows.delete(identifier);
            newAllRowsChecked = false;
        }
        this.setState({
            checkedRows: newCheckedRows,
            allRowsChecked: newAllRowsChecked
        });
    };

    onAllChecked = (checked: boolean): void => {
        const newCheckedRows: Set<number> = new Set(this.state.checkedRows);

        if (checked) {
            this.rows.forEach(
                (row: CheckableListTableRow): React.ReactNode =>
                    newCheckedRows.add(row.identifier)
            );
        } else {
            newCheckedRows.clear();
        }

        this.setState({ checkedRows: newCheckedRows, allRowsChecked: checked });
    };

    render(): React.ReactNode {
        return (
            <>
                <div>Checkable</div>
                <ListTable
                    allRowsChecked={this.state.allRowsChecked}
                    checkable
                    checkedRows={this.state.checkedRows}
                    columns={this.columns}
                    dataUI="checkable-list-table"
                    onAllChecked={this.onAllChecked}
                    onRowChecked={this.onRowChecked}
                    rows={this.rows}
                />
            </>
        );
    }
}

export default CheckableListTable;
