import React, { ChangeEvent } from 'react';

import { format } from 'date-fns';
import { get } from 'lodash';

import { TextField, TextFieldType } from '@preamp/core';
import { DateRange, DateRangeInput } from '@preamp/datepicker';
import { IListTableColumn, ListTable, ListTableTheme } from '@preamp/tables';

enum columns {
    Name = 'name',
    DateRange = 'range',
    Max = 'max',
    Min = 'min'
}

interface NestedInputsRow {
    identifier: number;
    [columns.DateRange]: DateRange;
    [columns.Max]: number;
    [columns.Min]: number;
}

interface NestedInputsState {
    editedRows: { [key: number]: NestedInputsRow };
}

class NestedInputs extends React.Component<{}, NestedInputsState> {
    state: NestedInputsState = {
        editedRows: {}
    };
    columns: IListTableColumn[];
    rows: NestedInputsRow[];
    rowCount: number = 0;

    constructor(props: any) {
        super(props);

        this.columns = [
            {
                identifier: columns.DateRange,
                header: 'Date Range',
                minWidth: 60,
                CustomCell: (data: any): React.ReactNode => {
                    this.rowCount += 1;
                    const { original, value } = data;
                    const { onDateChange } = this;
                    const { editedRows } = this.state;
                    const fieldValue: DateRange = get(
                        editedRows,
                        [original.identifier, columns.DateRange],
                        value
                    );

                    function onChange(dates: DateRange): void {
                        onDateChange(original, dates);
                    }

                    return (
                        <DateRangeInput
                            dateRangePickerProps={{ firstDayOfWeek: 1 }}
                            id={`dr-input-${this.rowCount}`}
                            onConfirmDates={onChange}
                            selectedDates={fieldValue}
                        />
                    );
                }
            },
            {
                identifier: columns.Min,
                header: 'Budget Min',
                minWidth: 40,
                isMetric: true,
                CustomCell: this.editableBudgetCell()
            },
            {
                identifier: columns.Max,
                header: (
                    <>
                        Budget Max{' '}
                        <span className="u-label--optional">Optional</span>
                    </>
                ),
                minWidth: 40,
                isMetric: true,
                CustomCell: this.editableBudgetCell()
            }
        ];

        this.rows = [
            {
                identifier: 1,
                [columns.DateRange]: {
                    from: new Date(2019, 0, 0),
                    to: new Date(2019, 1, 0)
                },
                [columns.Min]: 10,
                [columns.Max]: 1000
            },
            {
                identifier: 2,
                [columns.DateRange]: {
                    from: new Date(2019, 2, 0),
                    to: new Date(2019, 3, 0)
                },
                [columns.Min]: 10,
                [columns.Max]: 20000
            },
            {
                identifier: 3,
                [columns.DateRange]: {
                    from: new Date(2019, 4, 0),
                    to: new Date(2019, 5, 0)
                },
                [columns.Min]: 200,
                [columns.Max]: 300000
            },
            {
                identifier: 4,
                [columns.DateRange]: {
                    from: new Date(2019, 6, 0),
                    to: new Date(2019, 7, 0)
                },
                [columns.Min]: 3000,
                [columns.Max]: 40000
            }
        ];
    }

    editableBudgetCell = (): ((data: any) => React.ReactNode) => (
        data: any
    ): React.ReactNode => {
        const { column, original, value } = data;
        const columnIdentifier: string = column.columnIdentifier;
        const { onFieldChange } = this;
        const { editedRows } = this.state;
        const fieldValue: string = get(
            editedRows,
            [original.identifier, columnIdentifier],
            value
        );

        function onChange(event: ChangeEvent<HTMLInputElement>): void {
            onFieldChange(original, columnIdentifier, event);
        }

        return (
            <TextField
                append="USD"
                onChange={onChange}
                prepend="$"
                textFieldType={TextFieldType.Number}
                value={fieldValue}
            />
        );
    };

    onFieldChange = (
        row: NestedInputsRow,
        columnId: string,
        event: React.ChangeEvent<HTMLInputElement>
    ): void => {
        const { identifier } = row;
        const { editedRows } = this.state;
        const newEditedRows: { [key: number]: NestedInputsRow } = {
            ...editedRows
        };

        if (newEditedRows[identifier]) {
            newEditedRows[identifier][columnId] = event.currentTarget.value;
        } else {
            newEditedRows[identifier] = {
                ...row,
                [columnId]: event.currentTarget.value
            };
        }

        this.setState({
            editedRows: newEditedRows
        });
    };

    onDateChange = (row: NestedInputsRow, dates: DateRange): void => {
        const { identifier } = row;
        const { editedRows } = this.state;
        const newEditedRows: { [key: number]: NestedInputsRow } = {
            ...editedRows
        };

        if (newEditedRows[identifier]) {
            newEditedRows[identifier][columns.DateRange] = dates;
        } else {
            newEditedRows[identifier] = {
                ...row,
                [columns.DateRange]: dates
            };
        }

        this.setState({
            editedRows: newEditedRows
        });
    };

    logSavedData = (): React.ReactNode => {
        const { rows } = this;
        const { editedRows } = this.state;

        return rows.map(
            (row: NestedInputsRow, index: number): React.ReactNode => {
                const selectedRow: NestedInputsRow =
                    editedRows[row.identifier] || row;

                return (
                    <div key={index}>
                        <ul>
                            <li>
                                From:{' '}
                                {format(
                                    selectedRow[columns.DateRange].from,
                                    'MM/DD/YYYY'
                                )}
                            </li>
                            <li>
                                To:{' '}
                                {format(
                                    selectedRow[columns.DateRange].to,
                                    'MM/DD/YYYY'
                                )}
                            </li>
                            <li>Min: {selectedRow[columns.Min]}</li>
                            <li>Max: {selectedRow[columns.Max]}</li>
                        </ul>
                    </div>
                );
            }
        );
    };

    render(): React.ReactNode {
        return (
            <>
                <div className="mt-5">
                    <div>Inline Editing</div>
                    <ListTable
                        columns={this.columns}
                        dataUI="editable-list-table"
                        id="editable-list-table"
                        rows={this.rows}
                        theme={ListTableTheme.Simple}
                    />
                    <div className="flex flex-wrap">{this.logSavedData()}</div>
                </div>
                <div className="mt-5">
                    <div>Inline Editing without Hover State</div>
                    <ListTable
                        columns={this.columns}
                        dataUI="editable-list-table-not-hoverable"
                        id="editable-list-table-not-hoverable"
                        isHoverable={false}
                        rows={this.rows}
                        theme={ListTableTheme.Simple}
                    />
                    <div className="flex flex-wrap">{this.logSavedData()}</div>
                </div>
            </>
        );
    }
}

export default NestedInputs;
