import React from 'react';

import { Clear } from '@preamp/signal';
import {
    DateCell,
    DateCellProps,
    IListTableColumn,
    IRowAction,
    ListTable,
    ListTableTheme,
    NameDescriptionCell,
    NameDescriptionCellProps,
    NumberCell,
    NumberCellProps,
    StatusIndicator
} from '@preamp/tables';

enum columnEnum {
    Name = 'name',
    Created = 'created',
    LastUpdated = 'lastUpdated',
    Size = 'size'
}

interface BasicListTableRow {
    identifier: number;
    [columnEnum.Name]: NameDescriptionCellProps;
    [columnEnum.Created]: DateCellProps;
    [columnEnum.LastUpdated]: DateCellProps;
    [columnEnum.Size]: NumberCellProps;
}

const initialRows: BasicListTableRow[] = [
    {
        identifier: 1,
        [columnEnum.Name]: {
            name:
                'This is a really long name that should truncate and should show an ellipsis after set number of lines'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 1, 20)
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2018, 3, 20)
        },
        [columnEnum.Size]: { value: 35189 }
    },
    {
        identifier: 2,
        [columnEnum.Name]: {
            isIndicatorShown: true,
            status: StatusIndicator.Pending,
            name: 'Amy'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 2, 20)
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2019, 4, 20)
        },
        [columnEnum.Size]: { value: 14138890 }
    },
    {
        identifier: 3,
        [columnEnum.Name]: {
            name: 'Carl'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 2, 20)
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2016, 5, 20)
        },
        [columnEnum.Size]: { value: 145876 }
    },
    {
        identifier: 4,
        [columnEnum.Name]: {
            isIndicatorShown: true,
            status: StatusIndicator.Failing,
            name: 'Doug'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 3, 20)
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2018, 6, 20)
        },
        [columnEnum.Size]: { value: 75463 }
    },
    {
        identifier: 5,
        [columnEnum.Name]: {
            name: 'Emily'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 4, 20)
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2018, 7, 20)
        },
        [columnEnum.Size]: { value: 984153 }
    },
    {
        identifier: 6,
        [columnEnum.Name]: {
            isIndicatorShown: true,
            status: StatusIndicator.Pending,
            name: 'Lewis'
        },
        [columnEnum.Created]: {
            date: new Date(2018, 3, 20)
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2019, 6, 20)
        },
        [columnEnum.Size]: { value: 75422 }
    }
];

interface SimpleListTableState {
    rows: BasicListTableRow[];
}

class ListTableSimple extends React.Component<{}, SimpleListTableState> {
    state: SimpleListTableState = {
        rows: initialRows
    };
    columns: IListTableColumn[] = [
        {
            identifier: columnEnum.Name,
            header: 'Name',
            maxWidth: 335,
            CustomCell: ({ value }: any): React.ReactNode => (
                <NameDescriptionCell
                    description={value.description}
                    descriptionTruncationLines={2}
                    indicatorColor={value.indicatorColor}
                    isIndicatorShown={value.isIndicatorShown}
                    name={value.name}
                    nameUrl={`/#/tables/list/ListTable/${value.name}`}
                    onNameClick={this.nameClick}
                    status={value.status}
                />
            )
        },
        {
            identifier: columnEnum.Created,
            header: 'Created',
            maxWidth: 100,
            CustomCell: ({ value }: any): React.ReactNode => (
                <DateCell date={value.date} description={value.description} />
            )
        },
        {
            identifier: columnEnum.LastUpdated,
            header: 'Last Updated',
            minWidth: 100,
            CustomCell: ({ value }: any): React.ReactNode => (
                <DateCell date={value.date} description={value.description} />
            )
        },
        {
            identifier: columnEnum.Size,
            header: 'Size',
            minWidth: 100,
            isMetric: true,
            CustomCell: ({ value }: any): React.ReactNode => (
                <NumberCell
                    description={value.description}
                    value={value.value}
                />
            )
        }
    ];

    nameClick = (event: React.MouseEvent): void => {
        event.stopPropagation();
        alert('Name clicked');
    };

    onClickDelete = (rowIdentifier: number, event: React.MouseEvent): void => {
        event.stopPropagation();
        const { rows } = this.state;
        const newRows: BasicListTableRow[] = rows.filter(
            (row: any): React.ReactNode => row.identifier !== rowIdentifier
        );
        this.setState({ rows: newRows });
        // eslint-disable-next-line no-console
        console.log('DELETE', rowIdentifier);
    };

    rowClicked = (): void => {
        alert('Row clicked');
    };

    render(): React.ReactNode {
        const rowActions: IRowAction[] = [
            { icon: Clear, onClick: this.onClickDelete, size: 'sm' }
        ];

        return (
            <>
                <div className="mt-5">
                    <div>List Table with Simple theme</div>
                    <ListTable
                        columns={this.columns}
                        dataUI="list-table-theme"
                        onRowClicked={this.rowClicked}
                        rowActions={rowActions}
                        rows={this.state.rows}
                        theme={ListTableTheme.Simple}
                    />
                </div>
                <div className="mt-5">
                    <div>List Table with Simple theme without Hover State</div>
                    <ListTable
                        columns={this.columns}
                        dataUI="list-table-theme-without-hover"
                        isHoverable={false}
                        onRowClicked={this.rowClicked}
                        rowActions={rowActions}
                        rows={this.state.rows}
                        theme={ListTableTheme.Simple}
                    />
                </div>
            </>
        );
    }
}

export default ListTableSimple;
