import React, { useState } from 'react';

import {
    DataTable,
    DateCell,
    DateCellProps,
    ExpandedRows,
    IDataTableColumn,
    IListTableColumn,
    ListTable,
    ListTableTheme,
    NameDescriptionCell,
    NameDescriptionCellProps,
    NumberCell,
    NumberCellProps,
    StatusIndicator
} from '@preamp/tables';

enum columnEnum {
    Name = 'name',
    Created = 'created',
    LastUpdated = 'lastUpdated',
    Size = 'size'
}

interface BasicListTableRow {
    identifier: number;
    [columnEnum.Name]: NameDescriptionCellProps;
    [columnEnum.Created]: DateCellProps;
    [columnEnum.LastUpdated]: DateCellProps;
    [columnEnum.Size]: NumberCellProps;
    expandableContent?: React.ReactNode;
}

enum dataTableColumnEnum {
    FirstName = 'firstName',
    LastName = 'lastName',
    Age = 'age',
    Location = 'location',
    Date = 'date'
}

interface IDataTableRow {
    identifier: number;
    [dataTableColumnEnum.FirstName]: string;
    [dataTableColumnEnum.LastName]: string;
    [dataTableColumnEnum.Age]: number;
    [dataTableColumnEnum.Location]: string;
    [dataTableColumnEnum.Date]: string;
}

const dataTableColumns: IDataTableColumn[] = [
    {
        identifier: dataTableColumnEnum.FirstName,
        header: 'First Name',
        sortable: true,
        minWidth: 150
    },
    {
        identifier: dataTableColumnEnum.LastName,
        header: 'Last Name also know as Surname',
        sortable: true
    },
    {
        identifier: dataTableColumnEnum.Age,
        header: 'Age',
        sortable: true,
        minWidth: 80
    },
    {
        identifier: dataTableColumnEnum.Location,
        header: 'Location',
        sortable: true,
        minWidth: 100
    },
    {
        identifier: dataTableColumnEnum.Date,
        header: 'Date',
        sortable: true,
        minWidth: 150
    }
];

const dataTableRows: IDataTableRow[] = [
    {
        identifier: 1,
        [dataTableColumnEnum.FirstName]: 'Bob',
        [dataTableColumnEnum.LastName]: 'Samuelson',
        [dataTableColumnEnum.Age]: 42,
        [dataTableColumnEnum.Location]: 'CA',
        [dataTableColumnEnum.Date]: '11/17/1976'
    },
    {
        identifier: 2,
        [dataTableColumnEnum.FirstName]: 'Amy',
        [dataTableColumnEnum.LastName]: 'Gaudioso',
        [dataTableColumnEnum.Age]: 26,
        [dataTableColumnEnum.Location]: 'MA',
        [dataTableColumnEnum.Date]: '10/20/1992'
    },
    {
        identifier: 3,
        [dataTableColumnEnum.FirstName]: 'Juan',
        [dataTableColumnEnum.LastName]: 'Giuffrida',
        [dataTableColumnEnum.Age]: 32,
        [dataTableColumnEnum.Location]: 'TX',
        [dataTableColumnEnum.Date]: '9/15/1986'
    },
    {
        identifier: 4,
        [dataTableColumnEnum.FirstName]: 'Doug',
        [dataTableColumnEnum.LastName]: 'Ung',
        [dataTableColumnEnum.Age]: 37,
        [dataTableColumnEnum.Location]: 'NY',
        [dataTableColumnEnum.Date]: '12/9/1981'
    },
    {
        identifier: 5,
        [dataTableColumnEnum.FirstName]: 'Ben',
        [dataTableColumnEnum.LastName]: 'Behrens',
        [dataTableColumnEnum.Age]: 35,
        [dataTableColumnEnum.Location]: 'NC',
        [dataTableColumnEnum.Date]: '8/3/1983'
    },
    {
        identifier: 6,
        [dataTableColumnEnum.FirstName]: 'Bill',
        [dataTableColumnEnum.LastName]: 'Samuelson',
        [dataTableColumnEnum.Age]: 42,
        [dataTableColumnEnum.Location]: 'CA',
        [dataTableColumnEnum.Date]: '11/17/1976'
    },
    {
        identifier: 7,
        [dataTableColumnEnum.FirstName]: 'Angie',
        [dataTableColumnEnum.LastName]: 'Gaudioso',
        [dataTableColumnEnum.Age]: 26,
        [dataTableColumnEnum.Location]: 'MA',
        [dataTableColumnEnum.Date]: '10/20/1992'
    },
    {
        identifier: 8,
        [dataTableColumnEnum.FirstName]: 'Wayne',
        [dataTableColumnEnum.LastName]: 'Giuffrida',
        [dataTableColumnEnum.Age]: 32,
        [dataTableColumnEnum.Location]: 'TX',
        [dataTableColumnEnum.Date]: '9/15/1986'
    },
    {
        identifier: 9,
        [dataTableColumnEnum.FirstName]: 'Daryl',
        [dataTableColumnEnum.LastName]: 'Ung',
        [dataTableColumnEnum.Age]: 37,
        [dataTableColumnEnum.Location]: 'NY',
        [dataTableColumnEnum.Date]: '12/9/1981'
    },
    {
        identifier: 10,
        [dataTableColumnEnum.FirstName]: 'Bart',
        [dataTableColumnEnum.LastName]: 'Behrens',
        [dataTableColumnEnum.Age]: 35,
        [dataTableColumnEnum.Location]: 'NC',
        [dataTableColumnEnum.Date]: '8/3/1983'
    }
];

function NameDescriptionCellComponent({ value }: any): React.ReactNode {
    return (
        <NameDescriptionCell
            description={value.description}
            descriptionTruncationLines={2}
            indicatorColor={value.indicatorColor}
            isIndicatorShown={value.isIndicatorShown}
            name={value.name}
            status={value.status}
        />
    );
}
function CreatedDateCellComponent({ value }: any): React.ReactNode {
    return <DateCell date={value.date} description={value.description} />;
}
function UpdatedDateCellComponent({ value }: any): React.ReactNode {
    return <DateCell date={value.date} description={value.description} />;
}
function SizeNumberCellComponent({ value }: any): React.ReactNode {
    return <NumberCell description={value.description} value={value.value} />;
}

const columns: IListTableColumn[] = [
    {
        identifier: columnEnum.Name,
        header: 'Name',
        maxWidth: 335,
        CustomCell: NameDescriptionCellComponent
    },
    {
        identifier: columnEnum.Created,
        header: 'Created',
        maxWidth: 100,
        CustomCell: CreatedDateCellComponent
    },
    {
        identifier: columnEnum.LastUpdated,
        header: 'Last Updated',
        minWidth: 100,
        CustomCell: UpdatedDateCellComponent
    },
    {
        identifier: columnEnum.Size,
        header: 'Size',
        minWidth: 100,
        isMetric: true,
        CustomCell: SizeNumberCellComponent
    }
];

const rows: BasicListTableRow[] = [
    {
        identifier: 1,
        name: {
            name: 'Jeff',
            description: 'Is this my espresso machine?'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 1, 20)
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2018, 3, 20)
        },
        [columnEnum.Size]: { value: 35189 },
        expandableContent: (
            <div style={{ height: '320px', width: '100%' }}>
                <DataTable
                    columns={dataTableColumns}
                    dataUI="basic-data-table"
                    fillHeight
                    rows={dataTableRows}
                    truncateColHeader
                />
            </div>
        )
    },
    {
        identifier: 2,
        [columnEnum.Name]: {
            isIndicatorShown: true,
            status: StatusIndicator.Pending,
            name: 'Amy',
            description: 'Donec id elit non mi'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 2, 20)
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2019, 4, 20)
        },
        [columnEnum.Size]: { value: 14138890 },
        expandableContent: <div>Content for the second row when expanded.</div>
    },
    {
        identifier: 3,
        [columnEnum.Name]: {
            name: 'Carl',
            description: 'porta gravida at eget metus. '
        },
        [columnEnum.Created]: {
            date: new Date(2017, 2, 20)
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2016, 5, 20)
        },
        [columnEnum.Size]: { value: 145876 }
    },
    {
        identifier: 4,
        [columnEnum.Name]: {
            isIndicatorShown: true,
            status: StatusIndicator.Failing,
            name: 'Doug',
            description: 'Sed posuere consectet'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 3, 20)
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2018, 6, 20)
        },
        [columnEnum.Size]: { value: 75463 }
    },
    {
        identifier: 5,
        [columnEnum.Name]: {
            name: 'Emily',
            description: 'Donec id elit '
        },
        [columnEnum.Created]: {
            date: new Date(2017, 4, 20)
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2018, 7, 20)
        },
        [columnEnum.Size]: { value: 984153 }
    },
    {
        identifier: 6,
        [columnEnum.Name]: {
            isIndicatorShown: true,
            indicatorColor: 'blue',
            name: 'Lewis',
            description: 'elit non mi porta gravida at eget metus'
        },
        [columnEnum.Created]: {
            date: new Date(2018, 3, 20)
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2019, 6, 20)
        },
        [columnEnum.Size]: { value: 75422 },
        expandableContent: (
            <div>
                Donec id elit non mi porta gravida at eget metus. Sed posuere
                consectet
            </div>
        )
    }
];

const ExpandableListTableWithState: React.FunctionComponent = (): React.ReactElement => {
    const [expandedRows, setExpandedRows] = useState<ExpandedRows>({ 0: true });
    const [lastExpandedRowIndex, setLastExpandedRowIndex] = useState(0);

    function handleExpansionChange(updatedRows: ExpandedRows): void {
        setExpandedRows(updatedRows);
    }

    function handleOnExpandRow(
        rowindex: number,
        expandedStatus: boolean
    ): void {
        if (expandedStatus) {
            setLastExpandedRowIndex(rowindex);
        }
    }

    return (
        <>
            <div>List Table with Expandable Rows With State</div>
            <ListTable
                columns={columns}
                dataUI="basic-list-table"
                expandable={true}
                expanded={expandedRows}
                onExpandedChange={handleExpansionChange}
                onRowExpand={handleOnExpandRow}
                rows={rows}
                theme={ListTableTheme.Simple}
            />
            <p>Last row index expanded: {lastExpandedRowIndex} </p>
        </>
    );
};

export default ExpandableListTableWithState;
