import React from 'react';

import { noop } from 'lodash';

import { Edit, Trash } from '@preamp/signal';
import {
    DateCell,
    DateCellProps,
    IListTableColumn,
    IRowAction,
    ListTable,
    NameDescriptionCell,
    NameDescriptionCellProps,
    NumberCell,
    NumberCellProps,
    StatusIndicator
} from '@preamp/tables';

enum columnEnum {
    Name = 'name',
    Created = 'created',
    LastUpdated = 'lastUpdated',
    Size = 'size'
}

interface BasicListTableRow {
    identifier: number;
    [columnEnum.Name]: NameDescriptionCellProps;
    [columnEnum.Created]: DateCellProps;
    [columnEnum.LastUpdated]: DateCellProps;
    [columnEnum.Size]: NumberCellProps;
}

class BasicListTable extends React.Component {
    columns: IListTableColumn[] = [
        {
            identifier: columnEnum.Name,
            header: 'Name',
            maxWidth: 335,
            CustomCell: ({ value }: any): React.ReactNode => (
                <NameDescriptionCell
                    description={value.description}
                    descriptionTruncationLines={2}
                    indicatorColor={value.indicatorColor}
                    isIndicatorShown={value.isIndicatorShown}
                    name={value.name}
                    nameUrl={`https://www.videoamp.com/#{value.name}`}
                    onNameClick={this.nameClick}
                    status={value.status}
                />
            )
        },
        {
            identifier: columnEnum.Created,
            header: 'Created',
            maxWidth: 100,
            CustomCell: ({ value }: any): React.ReactNode => (
                <DateCell date={value.date} description={value.description} />
            )
        },
        {
            identifier: columnEnum.LastUpdated,
            header: 'Last Updated',
            minWidth: 100,
            CustomCell: ({ value }: any): React.ReactNode => (
                <DateCell date={value.date} description={value.description} />
            )
        },
        {
            identifier: columnEnum.Size,
            header: 'Size',
            minWidth: 100,
            isMetric: true,
            CustomCell: ({ value }: any): React.ReactNode => (
                <NumberCell
                    description={value.description}
                    value={value.value}
                />
            )
        }
    ];

    rows: BasicListTableRow[] = [
        {
            identifier: 1,
            name: {
                name:
                    'This is a really long name that should truncate and should show an ellipsis after set number of lines',
                description:
                    "Is this my espresso machine? Wh-what is-h-how did you get my espresso machine? Must go faster. We gotta burn the rain forest, dump toxic waste, pollute the air, and rip up the OZONE! 'Cause maybe if we screw up this planet enough, they won't want it anymore!"
            },
            [columnEnum.Created]: {
                date: new Date(2017, 1, 20),
                description: 'John Doe'
            },
            [columnEnum.LastUpdated]: {
                date: new Date(2018, 3, 20),
                description: 'John Doe'
            },
            [columnEnum.Size]: { value: 35189 }
        },
        {
            identifier: 2,
            [columnEnum.Name]: {
                isIndicatorShown: true,
                status: StatusIndicator.Pending,
                name: 'Amy',
                description:
                    'Donec id elit non mi porta gravida at eget metus. Sed posuere consectet'
            },
            [columnEnum.Created]: {
                date: new Date(2017, 2, 20),
                description: 'John Doe'
            },
            [columnEnum.LastUpdated]: {
                date: new Date(2019, 4, 20),
                description: 'John Doe'
            },
            [columnEnum.Size]: { value: 14138890 }
        }
    ];

    nameClick = (event: React.MouseEvent): void => {
        event.stopPropagation();
        alert('Name clicked');
    };

    rowClicked = (): void => {
        alert('Row clicked');
    };

    buildRowActions = (rowIdentifier: React.ReactText): IRowAction[] => {
        if (rowIdentifier === 2) {
            return [{ icon: Trash, onClick: noop }];
        }

        return [
            { icon: Trash, onClick: noop },
            { icon: Edit, onClick: noop }
        ];
    };

    render(): React.ReactNode {
        return (
            <>
                <div>Basic List Table With Custom Build Row Actions</div>
                <ListTable
                    buildRowActions={this.buildRowActions}
                    columns={this.columns}
                    dataUI="basic-list-table-with-custom-build-action"
                    onRowClicked={this.rowClicked}
                    rows={this.rows}
                />
            </>
        );
    }
}

export default BasicListTable;
