import React from 'react';

import { sortBy } from 'lodash';

import { keyboardKeys } from '@preamp/core';
import { Edit, Trash } from '@preamp/signal';
import {
    DateCell,
    DateCellProps,
    IListTableColumn,
    IRowAction,
    ListTable,
    NameDescriptionCell,
    NameDescriptionCellProps,
    NumberCell,
    NumberCellProps,
    sortDirectionEnum
} from '@preamp/tables';

enum columnEnum {
    Name = 'name',
    Created = 'created',
    LastUpdated = 'lastUpdated',
    Size = 'size'
}

interface AllFeatureListTableRow {
    identifier: React.ReactText;
    [columnEnum.Name]: NameDescriptionCellProps;
    [columnEnum.Created]: DateCellProps;
    [columnEnum.LastUpdated]: DateCellProps;
    [columnEnum.Size]: NumberCellProps;
}

interface AllFeatureListTableState {
    selectedRow: React.ReactText;
    rows: AllFeatureListTableRow[];
    isLoading: boolean;
    isLoadingMoreRows: boolean;
    checkedRows: Set<React.ReactText>;
    allRowsChecked: boolean;
    sortedColumnIdentifier: React.ReactText;
    sortDirection: sortDirectionEnum;
}

function NameDescriptionCellComponent({ value }: any): React.ReactNode {
    return (
        <NameDescriptionCell
            description={value.description}
            name={value.name}
            pending={value.pending}
        />
    );
}
function CreatedDateCellComponent({ value }: any): React.ReactNode {
    return <DateCell date={value.date} description={value.description} />;
}
function UpdatedDateCellComponent({ value }: any): React.ReactNode {
    return <DateCell date={value.date} description={value.description} />;
}
function SizeNumberCellComponent({ value }: any): React.ReactNode {
    return <NumberCell description={value.description} value={value.value} />;
}

const originalColumns: IListTableColumn[] = [
    {
        identifier: columnEnum.Name,
        header: 'Name',
        minWidth: 150,
        sortable: true,
        CustomCell: NameDescriptionCellComponent
    },
    {
        identifier: columnEnum.Created,
        header: 'Created',
        minWidth: 100,
        CustomCell: CreatedDateCellComponent
    },
    {
        identifier: columnEnum.LastUpdated,
        header: 'Last Updated',
        minWidth: 100,
        CustomCell: UpdatedDateCellComponent
    },
    {
        identifier: columnEnum.Size,
        header: 'Size',
        minWidth: 100,
        sortable: true,
        isMetric: true,
        CustomCell: SizeNumberCellComponent
    }
];

const originalRows: AllFeatureListTableRow[] = [
    {
        identifier: 1,
        name: {
            name: 'Bob',
            description:
                'Donec id elit non mi porta gravida at eget metus. Sed posuere consectet'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 1, 20),
            description: 'John Doe'
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2018, 3, 20),
            description: 'John Doe'
        },
        [columnEnum.Size]: { value: 35189 }
    },
    {
        identifier: 2,
        [columnEnum.Name]: {
            name: 'Amy',
            description:
                'Donec id elit non mi porta gravida at eget metus. Sed posuere consectet'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 2, 20),
            description: 'John Doe'
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2019, 4, 20),
            description: 'John Doe'
        },
        [columnEnum.Size]: { value: 14138890 }
    },
    {
        identifier: 3,
        [columnEnum.Name]: {
            name: 'Carl',
            description:
                'Donec id elit non mi porta gravida at eget metus. Sed posuere consectet'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 2, 20),
            description: 'John Doe'
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2016, 5, 20),
            description: 'John Doe'
        },
        [columnEnum.Size]: { value: 145876 }
    },
    {
        identifier: 4,
        [columnEnum.Name]: {
            name: 'Doug',
            description:
                'Donec id elit non mi porta gravida at eget metus. Sed posuere consectet'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 3, 20),
            description: 'John Doe'
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2018, 6, 20),
            description: 'John Doe'
        },
        [columnEnum.Size]: { value: 75463 }
    },
    {
        identifier: 5,
        [columnEnum.Name]: {
            name: 'Emily',
            description:
                'Donec id elit non mi porta gravida at eget metus. Sed posuere consectet'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 4, 20),
            description: 'John Doe'
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2018, 7, 20),
            description: 'John Doe'
        },
        [columnEnum.Size]: { value: 984153 }
    },
    {
        identifier: 6,
        [columnEnum.Name]: {
            name: 'Jack',
            description:
                'Donec id elit non mi porta gravida at eget metus. Sed posuere consectet'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 4, 20),
            description: 'John Doe'
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2018, 4, 20),
            description: 'John Doe'
        },
        [columnEnum.Size]: { value: 94123 }
    },
    {
        identifier: 7,
        [columnEnum.Name]: {
            name: 'Helen',
            description:
                'Donec id elit non mi porta gravida at eget metus. Sed posuere consectet'
        },
        [columnEnum.Created]: {
            date: new Date(2019, 2, 24),
            description: 'John Doe'
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2000, 5, 2),
            description: 'John Doe'
        },
        [columnEnum.Size]: { value: 543 }
    },
    {
        identifier: 8,
        [columnEnum.Name]: {
            name: 'Greg',
            description:
                'Donec id elit non mi porta gravida at eget metus. Sed posuere consectet'
        },
        [columnEnum.Created]: {
            date: new Date(1999, 4, 2),
            description: 'John Doe'
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2018, 7, 20),
            description: 'John Doe'
        },
        [columnEnum.Size]: { value: 58764 }
    },
    {
        identifier: 9,
        [columnEnum.Name]: {
            name: 'Fred',
            description:
                'Donec id elit non mi porta gravida at eget metus. Sed posuere consectet'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 4, 20),
            description: 'John Doe'
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2018, 7, 20),
            description: 'John Doe'
        },
        [columnEnum.Size]: { value: 68713 }
    },
    {
        identifier: 10,
        [columnEnum.Name]: {
            name: 'Issac',
            description:
                'Donec id elit non mi porta gravida at eget metus. Sed posuere consectet'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 4, 20),
            description: 'John Doe'
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2018, 7, 20),
            description: 'John Doe'
        },
        [columnEnum.Size]: { value: 6874 }
    }
];

class AllFeatureListTable extends React.Component<
    {},
    AllFeatureListTableState
> {
    state: AllFeatureListTableState = {
        rows: originalRows.slice(0, 3),
        selectedRow: null,
        isLoading: false,
        isLoadingMoreRows: false,
        checkedRows: new Set(),
        allRowsChecked: false,
        sortedColumnIdentifier: 'name',
        sortDirection: sortDirectionEnum.Asc
    };

    onRowClicked: (rowIdentifier: React.ReactText) => void = (
        rowIdentifier: React.ReactText
    ): void => {
        this.setState({ selectedRow: rowIdentifier });
    };

    onRowKeyPressed = (
        rowIdentifier: number,
        event: React.KeyboardEvent
    ): void => {
        const key: number = event.keyCode || event.which;
        if (key === keyboardKeys.Enter || key === keyboardKeys.SpaceBar) {
            event.preventDefault();
            this.setState({ selectedRow: rowIdentifier });
        }
    };

    sortSize: (sortDirection: sortDirectionEnum) => void = (
        sortDirection: sortDirectionEnum
    ): void => {
        const { rows } = this.state;
        const newRowsDesc: AllFeatureListTableRow[] = sortBy(
            rows,
            (row: any): React.ReactNode => row[columnEnum.Size].value
        );
        this.setState({
            rows:
                sortDirection === sortDirectionEnum.Asc
                    ? newRowsDesc
                    : newRowsDesc.reverse()
        });
    };

    sortNames: (sortDirection: sortDirectionEnum) => void = (
        sortDirection: sortDirectionEnum
    ): void => {
        const { rows } = this.state;
        const newRowsDesc: AllFeatureListTableRow[] = sortBy(
            rows,
            (row: any): React.ReactNode => row[columnEnum.Name].name
        );
        this.setState({
            rows:
                sortDirection === sortDirectionEnum.Asc
                    ? newRowsDesc
                    : newRowsDesc.reverse()
        });
    };

    onColumnSorted: (
        columnIdentifier: React.ReactText,
        sortDirection: sortDirectionEnum
    ) => void = (
        columnIdentifier: React.ReactText,
        sortDirection: sortDirectionEnum
    ): void => {
        this.setState(
            {
                sortedColumnIdentifier: columnIdentifier,
                sortDirection
            },
            (): void => {
                if (columnIdentifier === columnEnum.Name) {
                    this.sortNames(sortDirection);
                } else if (columnIdentifier === columnEnum.Size) {
                    this.sortSize(sortDirection);
                }
            }
        );
    };

    appendMoreRows: () => void = (): void => {
        const { allRowsChecked, rows, checkedRows } = this.state;
        const newRows: AllFeatureListTableRow[] = originalRows.slice(
            0,
            rows.length + 1
        );
        const newCheckedRows: Set<React.ReactText> = new Set(checkedRows);
        if (allRowsChecked) {
            newRows.forEach(
                (row: AllFeatureListTableRow): Set<React.ReactText> =>
                    newCheckedRows.add(row.identifier)
            );
        }
        this.setState({
            rows: newRows,
            isLoading: false,
            isLoadingMoreRows: false,
            checkedRows: newCheckedRows
        });
    };

    onClickLoadMore: () => void = (): void => {
        this.setState(
            {
                isLoadingMoreRows: true
            },
            (): void => {
                setTimeout((): void => {
                    this.appendMoreRows();
                    this.onColumnSorted(
                        this.state.sortedColumnIdentifier,
                        this.state.sortDirection
                    );
                }, 500);
            }
        );
    };

    onRowChecked: (identifier: React.ReactText, checked: boolean) => void = (
        identifier: React.ReactText,
        checked: boolean
    ): void => {
        const { checkedRows, allRowsChecked, rows } = this.state;
        const newCheckedRows: Set<React.ReactText> = new Set(checkedRows);
        let newAllRowsChecked: boolean = allRowsChecked;
        if (checked) {
            newCheckedRows.add(identifier);
            newAllRowsChecked = newCheckedRows.size === rows.length;
        } else {
            newCheckedRows.delete(identifier);
            newAllRowsChecked = false;
        }
        this.setState({
            checkedRows: newCheckedRows,
            allRowsChecked: newAllRowsChecked
        });
    };

    onAllChecked = (checked: boolean): void => {
        const newCheckedRows: Set<React.ReactText> = new Set(
            this.state.checkedRows
        );

        if (checked) {
            originalRows.forEach(
                (row: AllFeatureListTableRow): Set<React.ReactText> =>
                    newCheckedRows.add(row.identifier)
            );
        } else {
            newCheckedRows.clear();
        }

        this.setState({ checkedRows: newCheckedRows, allRowsChecked: checked });
    };

    onClickDelete = (
        rowIdentifier: string | number,
        event: React.MouseEvent
    ): void => {
        event.stopPropagation();
        // eslint-disable-next-line no-console
        console.log('DELETE', rowIdentifier);
    };

    onClickEdit = (
        rowIdentifier: string | number,
        event: React.MouseEvent
    ): void => {
        event.stopPropagation();
        // eslint-disable-next-line no-console
        console.log('EDIT', rowIdentifier);
    };

    render(): React.ReactNode {
        const rowActions: IRowAction[] = [
            { icon: Trash, onClick: this.onClickDelete },
            { icon: Edit, onClick: this.onClickEdit }
        ];
        const {
            allRowsChecked,
            checkedRows,
            isLoading,
            isLoadingMoreRows,
            rows,
            selectedRow,
            sortDirection,
            sortedColumnIdentifier
        } = this.state;

        return (
            <>
                <div>All Features</div>
                <ListTable
                    allRowsChecked={allRowsChecked}
                    checkable
                    checkedRows={checkedRows}
                    columns={originalColumns}
                    dataUI="advanced-list-table"
                    isHeaderSticky
                    isLoading={isLoading}
                    isLoadingMoreRows={isLoadingMoreRows}
                    onAllChecked={this.onAllChecked}
                    onClickLoadMore={
                        rows.length !== originalRows.length
                            ? this.onClickLoadMore
                            : null
                    }
                    onColumnSorted={this.onColumnSorted}
                    onRowChecked={this.onRowChecked}
                    onRowClicked={this.onRowClicked}
                    onRowKeyPress={this.onRowKeyPressed}
                    rowActions={rowActions}
                    rows={this.state.rows}
                    selectedRow={selectedRow}
                    sortDirection={sortDirection}
                    sortedColumnIdentifier={sortedColumnIdentifier}
                />
            </>
        );
    }
}

export default AllFeatureListTable;
