import React, { useState } from 'react';

import { Button, Toggle } from '@preamp/core';
import {
    DateCell,
    DateCellProps,
    EmptyState,
    IListTableColumn,
    ListTable,
    NameDescriptionCell,
    NameDescriptionCellProps,
    NumberCell,
    NumberCellProps,
    StatusIndicator
} from '@preamp/tables';

enum columnEnum {
    Name = 'name',
    Created = 'created',
    LastUpdated = 'lastUpdated',
    Size = 'size'
}

interface BasicListTableRow {
    identifier: number;
    [columnEnum.Name]: NameDescriptionCellProps;
    [columnEnum.Created]: DateCellProps;
    [columnEnum.LastUpdated]: DateCellProps;
    [columnEnum.Size]: NumberCellProps;
}

export default function AsyncListTable(): JSX.Element {
    function nameClick(event: React.MouseEvent): void {
        event.stopPropagation();
        alert('Name clicked');
    }
    function NameDescriptionCellComponent({ value }: any): React.ReactNode {
        return (
            <NameDescriptionCell
                description={value.description}
                descriptionTruncationLines={2}
                indicatorColor={value.indicatorColor}
                isIndicatorShown={value.isIndicatorShown}
                name={value.name}
                nameUrl={`https://www.videoamp.com/${value.name}`}
                onNameClick={nameClick}
                status={value.status}
            />
        );
    }
    function CreatedDateCellComponent({ value }: any): React.ReactNode {
        return <DateCell date={value.date} description={value.description} />;
    }
    function UpdatedDateCellComponent({ value }: any): React.ReactNode {
        return <DateCell date={value.date} description={value.description} />;
    }
    function SizeNumberCellComponent({ value }: any): React.ReactNode {
        return (
            <NumberCell description={value.description} value={value.value} />
        );
    }

    const columns: IListTableColumn[] = [
        {
            identifier: columnEnum.Name,
            header: 'Name',
            maxWidth: 335,
            CustomCell: NameDescriptionCellComponent
        },
        {
            identifier: columnEnum.Created,
            header: 'Created',
            maxWidth: 100,
            CustomCell: CreatedDateCellComponent
        },
        {
            identifier: columnEnum.LastUpdated,
            header: 'Last Updated',
            minWidth: 100,
            CustomCell: UpdatedDateCellComponent
        },
        {
            identifier: columnEnum.Size,
            header: 'Size',
            minWidth: 100,
            isMetric: true,
            CustomCell: SizeNumberCellComponent
        }
    ];
    const rows: BasicListTableRow[] = [
        {
            identifier: 1,
            name: {
                name:
                    'This is a really long name that should truncate and should show an ellipsis after set number of lines',
                description:
                    "Is this my espresso machine? Wh-what is-h-how did you get my espresso machine? Must go faster. We gotta burn the rain forest, dump toxic waste, pollute the air, and rip up the OZONE! 'Cause maybe if we screw up this planet enough, they won't want it anymore!"
            },
            [columnEnum.Created]: {
                date: new Date(2017, 1, 20),
                description: 'John Doe'
            },
            [columnEnum.LastUpdated]: {
                date: new Date(2018, 3, 20),
                description: 'John Doe'
            },
            [columnEnum.Size]: { value: 35189 }
        },
        {
            identifier: 2,
            [columnEnum.Name]: {
                isIndicatorShown: true,
                status: StatusIndicator.Pending,
                name: 'Amy',
                description:
                    'Donec id elit non mi porta gravida at eget metus. Sed posuere consectet'
            },
            [columnEnum.Created]: {
                date: new Date(2017, 2, 20),
                description: 'John Doe'
            },
            [columnEnum.LastUpdated]: {
                date: new Date(2019, 4, 20),
                description: 'John Doe'
            },
            [columnEnum.Size]: { value: 14138890 }
        },
        {
            identifier: 3,
            [columnEnum.Name]: {
                name: 'Carl',
                description:
                    'Donec id elit non mi porta gravida at eget metus. Sed posuere consectet'
            },
            [columnEnum.Created]: {
                date: new Date(2017, 2, 20),
                description: 'John Doe'
            },
            [columnEnum.LastUpdated]: {
                date: new Date(2016, 5, 20),
                description: 'John Doe'
            },
            [columnEnum.Size]: { value: 145876 }
        },
        {
            identifier: 4,
            [columnEnum.Name]: {
                isIndicatorShown: true,
                status: StatusIndicator.Failing,
                name: 'Doug',
                description:
                    'Donec id elit non mi porta gravida at eget metus. Sed posuere consectet'
            },
            [columnEnum.Created]: {
                date: new Date(2017, 3, 20),
                description: 'John Doe'
            },
            [columnEnum.LastUpdated]: {
                date: new Date(2018, 6, 20),
                description: 'John Doe'
            },
            [columnEnum.Size]: { value: 75463 }
        },
        {
            identifier: 5,
            [columnEnum.Name]: {
                name: 'Emily',
                description:
                    'Donec id elit non mi porta gravida at eget metus. Sed posuere consectet'
            },
            [columnEnum.Created]: {
                date: new Date(2017, 4, 20),
                description: 'John Doe'
            },
            [columnEnum.LastUpdated]: {
                date: new Date(2018, 7, 20),
                description: 'John Doe'
            },
            [columnEnum.Size]: { value: 984153 }
        },
        {
            identifier: 6,
            [columnEnum.Name]: {
                isIndicatorShown: true,
                indicatorColor: 'blue',
                name: 'Lewis',
                description:
                    'Donec id elit non mi porta gravida at eget metus. Sed posuere consectet'
            },
            [columnEnum.Created]: {
                date: new Date(2018, 3, 20),
                description: 'John Doe'
            },
            [columnEnum.LastUpdated]: {
                date: new Date(2019, 6, 20),
                description: 'John Doe'
            },
            [columnEnum.Size]: { value: 75422 }
        }
    ];
    const [state, setState] = useState({
        loadedRows: [],
        loading: false,
        showEmptyState: false
    });

    function rowClicked(): void {
        alert('Row clicked');
    }

    function toggleEmptyState(): void {
        setState({ ...state, showEmptyState: !state.showEmptyState });
    }

    async function load(): Promise<void> {
        setState({
            ...state,
            loading: true
        });
        setTimeout((): void => {
            setState({
                ...state,
                loadedRows: rows,
                loading: false
            });
        }, 3000);
    }

    async function reset(): Promise<void> {
        setState({
            ...state,
            loading: true
        });
        setTimeout((): void => {
            setState({
                ...state,
                loadedRows: [],
                loading: false
            });
        }, 3000);
    }
    const LetsGetStarted: React.ReactNode = (
        <EmptyState
            buttonChildren="Load Data"
            mainTitle="Let's Get Started!"
            onClick={load}
            subText="Nothing has been created yet, start by loading some data below."
        />
    );

    return (
        <>
            <div>Async Example</div>
            <div className="my-6 flex justify-between">
                <div>
                    <Button onClick={load}>Load Data</Button>
                    <Button onClick={reset}>Clear Data</Button>
                </div>
                <Toggle
                    checked={state.showEmptyState}
                    label="Custom Empty State Component"
                    name="empty-state"
                    onChange={toggleEmptyState}
                />
            </div>
            <ListTable
                columns={columns}
                dataUI="basic-list-table"
                EmptyState={state.showEmptyState ? LetsGetStarted : undefined}
                isLoading={state.loading}
                onRowClicked={rowClicked}
                rows={state.loadedRows}
            />
        </>
    );
}
