import React, { useState } from 'react';

import { Clear } from '@preamp/signal';
import {
    DateCell,
    DateCellProps,
    IListTableColumn,
    IRowAction,
    ListTable,
    ListTableTheme,
    NameDescriptionCell,
    NameDescriptionCellProps,
    NumberCell,
    NumberCellProps,
    StatusIndicator
} from '@preamp/tables';

enum columnEnum {
    Name = 'name',
    Created = 'created',
    LastUpdated = 'lastUpdated',
    Size = 'size'
}

interface BasicListTableRow {
    identifier: number;
    [columnEnum.Name]: NameDescriptionCellProps;
    [columnEnum.Created]: DateCellProps;
    [columnEnum.LastUpdated]: DateCellProps;
    [columnEnum.Size]: NumberCellProps;
    expandableContent?: React.ReactNode;
}

const initialRows: BasicListTableRow[] = [
    {
        identifier: 1,
        [columnEnum.Name]: {
            isIndicatorShown: true,
            status: StatusIndicator.Pending,
            name: 'Amy'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 2, 20)
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2019, 4, 20)
        },
        [columnEnum.Size]: { value: 14138890 }
    },
    {
        identifier: 2,
        [columnEnum.Name]: {
            name: 'Carl'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 2, 20)
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2016, 5, 20)
        },
        [columnEnum.Size]: { value: 145876 }
    },
    {
        identifier: 3,
        [columnEnum.Name]: {
            isIndicatorShown: true,
            status: StatusIndicator.Failing,
            name: 'Doug'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 3, 20)
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2018, 6, 20)
        },
        [columnEnum.Size]: { value: 75463 }
    },
    {
        identifier: 4,
        [columnEnum.Name]: {
            name: 'Emily'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 4, 20)
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2018, 7, 20)
        },
        [columnEnum.Size]: { value: 984153 }
    },
    {
        identifier: 5,
        [columnEnum.Name]: {
            isIndicatorShown: true,
            status: StatusIndicator.Pending,
            name: 'Lewis'
        },
        [columnEnum.Created]: {
            date: new Date(2018, 3, 20)
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2019, 6, 20)
        },
        [columnEnum.Size]: { value: 75422 }
    }
];

const ReadOnlyListTable: React.FC = (): React.ReactElement => {
    const [rows, setRows] = useState<BasicListTableRow[]>(initialRows);
    function nameClick(event: React.MouseEvent): void {
        event.stopPropagation();
        alert('Name clicked');
    }

    function NameDescriptionCellComponent({ value }: any): React.ReactNode {
        return (
            <NameDescriptionCell
                description={value.description}
                descriptionTruncationLines={2}
                indicatorColor={value.indicatorColor}
                isIndicatorShown={value.isIndicatorShown}
                name={value.name}
                nameUrl={`/#/tables/list/ListTable/${value.name}`}
                onNameClick={nameClick}
                status={value.status}
            />
        );
    }
    function CreatedDateCellComponent({ value }: any): React.ReactNode {
        return <DateCell date={value.date} description={value.description} />;
    }
    function UpdatedDateCellComponent({ value }: any): React.ReactNode {
        return <DateCell date={value.date} description={value.description} />;
    }
    function SizeNumberCellComponent({ value }: any): React.ReactNode {
        return (
            <NumberCell description={value.description} value={value.value} />
        );
    }

    const columns: IListTableColumn[] = [
        {
            identifier: columnEnum.Name,
            header: 'Name',
            maxWidth: 335,
            CustomCell: NameDescriptionCellComponent
        },
        {
            identifier: columnEnum.Created,
            header: 'Created',
            maxWidth: 100,
            CustomCell: CreatedDateCellComponent
        },
        {
            identifier: columnEnum.LastUpdated,
            header: 'Last Updated',
            minWidth: 100,
            CustomCell: UpdatedDateCellComponent
        },
        {
            identifier: columnEnum.Size,
            header: 'Size',
            minWidth: 100,
            isMetric: true,
            CustomCell: SizeNumberCellComponent
        }
    ];

    function onClickDelete(
        rowIdentifier: number,
        event: React.MouseEvent
    ): void {
        event.stopPropagation();
        const newRows: BasicListTableRow[] = rows.filter(
            (row: any): React.ReactNode => row.identifier !== rowIdentifier
        );
        setRows(newRows);
        // eslint-disable-next-line no-console
        console.log('DELETE', rowIdentifier);
    }

    function rowClicked(): void {
        alert('Row clicked');
    }

    const rowActions: IRowAction[] = [
        { icon: Clear, onClick: onClickDelete, size: 'sm' }
    ];

    return (
        <>
            <div className="mt-5">
                <div>Read Only List Table</div>
                <ListTable
                    columns={columns}
                    dataUI="list-table-read-only"
                    isReadOnly={true}
                    onRowClicked={rowClicked}
                    rowActions={rowActions}
                    rows={rows}
                    theme={ListTableTheme.Simple}
                />
            </div>
            <div className="mt-5">
                <div>Read Only List Table without Action Icon </div>
                <ListTable
                    columns={columns}
                    dataUI="list-table-read-only"
                    isReadOnly={true}
                    onRowClicked={rowClicked}
                    rows={rows}
                    theme={ListTableTheme.Simple}
                />
            </div>
        </>
    );
};

ReadOnlyListTable.displayName = 'ReadOnlyListTable';

export default ReadOnlyListTable;
