import React from 'react';

import { Toggle } from '@preamp/core';
import {
    DateCellProps,
    DateCell,
    IListTableColumn,
    ListTable,
    ListTableTheme,
    NameDescriptionCell,
    NameDescriptionCellProps,
    NumberCell,
    NumberCellProps,
    StatusIndicator
} from '@preamp/tables';

enum columnEnum {
    Name = 'name',
    Created = 'created',
    LastUpdated = 'lastUpdated',
    Size = 'size'
}

interface BasicListTableRow {
    identifier: number;
    [columnEnum.Name]: NameDescriptionCellProps;
    [columnEnum.Created]: DateCellProps;
    [columnEnum.LastUpdated]: DateCellProps;
    [columnEnum.Size]: NumberCellProps;
}

const DraggableListTable: React.FC = (): React.ReactElement => {
    const [theme, setTheme] = React.useState<ListTableTheme>(
        ListTableTheme.Simple
    );

    function nameCell({ value }: any): React.ReactNode {
        return <NameDescriptionCell name={value.name} status={value.status} />;
    }

    function createdCell({ value }: any): React.ReactNode {
        return <DateCell date={value.date} description={value.description} />;
    }

    function lastUpdatedCell({ value }: any): React.ReactNode {
        return <DateCell date={value.date} description={value.description} />;
    }

    function sizeCell({ value }: any): React.ReactNode {
        return (
            <NumberCell description={value.description} value={value.value} />
        );
    }

    const columns: IListTableColumn[] = [
        {
            identifier: columnEnum.Name,
            header: 'Name',
            maxWidth: 335,
            CustomCell: nameCell
        },
        {
            identifier: columnEnum.Created,
            header: 'Created',
            maxWidth: 100,
            CustomCell: createdCell
        },
        {
            identifier: columnEnum.LastUpdated,
            header: 'Last Updated',
            minWidth: 100,
            CustomCell: lastUpdatedCell
        },
        {
            identifier: columnEnum.Size,
            header: 'Size',
            minWidth: 100,
            isMetric: true,
            CustomCell: sizeCell
        }
    ];

    const rows: BasicListTableRow[] = [
        {
            identifier: 1,
            name: {
                name: 'Jeff'
            },
            [columnEnum.Created]: {
                date: new Date(2017, 1, 20)
            },
            [columnEnum.LastUpdated]: {
                date: new Date(2018, 3, 20)
            },
            [columnEnum.Size]: {
                value: 35189
            }
        },
        {
            identifier: 2,
            [columnEnum.Name]: {
                isIndicatorShown: true,
                status: StatusIndicator.Pending,
                name: 'Amy'
            },
            [columnEnum.Created]: {
                date: new Date(2017, 2, 20)
            },
            [columnEnum.LastUpdated]: {
                date: new Date(2019, 4, 20)
            },
            [columnEnum.Size]: {
                value: 14138890
            }
        },
        {
            identifier: 3,
            [columnEnum.Name]: {
                name: 'Carl'
            },
            [columnEnum.Created]: {
                date: new Date(2017, 2, 20)
            },
            [columnEnum.LastUpdated]: {
                date: new Date(2016, 5, 20)
            },
            [columnEnum.Size]: {
                value: 145876
            }
        },
        {
            identifier: 4,
            [columnEnum.Name]: {
                isIndicatorShown: true,
                status: StatusIndicator.Failing,
                name: 'Doug'
            },
            [columnEnum.Created]: {
                date: new Date(2017, 3, 20)
            },
            [columnEnum.LastUpdated]: {
                date: new Date(2018, 6, 20)
            },
            [columnEnum.Size]: {
                value: 75463
            }
        },
        {
            identifier: 5,
            [columnEnum.Name]: {
                name: 'Emily'
            },
            [columnEnum.Created]: {
                date: new Date(2017, 4, 20)
            },
            [columnEnum.LastUpdated]: {
                date: new Date(2018, 7, 20)
            },
            [columnEnum.Size]: {
                value: 984153
            }
        },
        {
            identifier: 6,
            [columnEnum.Name]: {
                isIndicatorShown: true,
                indicatorColor: 'blue',
                name: 'Lewis'
            },
            [columnEnum.Created]: {
                date: new Date(2018, 3, 20)
            },
            [columnEnum.LastUpdated]: {
                date: new Date(2019, 6, 20)
            },
            [columnEnum.Size]: {
                value: 75422
            }
        }
    ];

    return (
        <>
            <div className="mt-5">
                <div className="mb-2">Draggable List Table</div>
                <Toggle
                    checked={theme === ListTableTheme.Default}
                    label="Enable Default Theme"
                    name="draggable-list-table-theme"
                    onChange={(): void => {
                        setTheme(
                            (prevTheme: ListTableTheme): ListTableTheme => {
                                return prevTheme === ListTableTheme.Default
                                    ? ListTableTheme.Simple
                                    : ListTableTheme.Default;
                            }
                        );
                    }}
                />
                <ListTable
                    columns={columns}
                    dataUI="draggable-list-table"
                    isDraggable
                    onRowMoved={console.log}
                    rows={rows}
                    theme={theme}
                />
            </div>
        </>
    );
};

export default DraggableListTable;
