import React, { useCallback, useMemo, useRef } from 'react';

import { Button } from '@preamp/core';
import { Alert, Signal } from '@preamp/signal';
import {
    DateCell,
    DateCellProps,
    IListTableColumn,
    ListTable,
    NameDescriptionCell,
    NameDescriptionCellProps,
    NumberCell,
    NumberCellProps,
    sortDirectionEnum
} from '@preamp/tables';

enum columnEnum {
    Name = 'name',
    Created = 'created',
    LastUpdated = 'lastUpdated',
    Size = 'size'
}

interface AllFeatureListTableRow {
    identifier: React.ReactText;
    [columnEnum.Name]: NameDescriptionCellProps;
    [columnEnum.Created]: DateCellProps;
    [columnEnum.LastUpdated]: DateCellProps;
    [columnEnum.Size]: NumberCellProps;
}

interface AllFeatureListTableState {
    allRowsChecked: boolean;
    checkedRows: Set<React.ReactText>;
    currentRowId: number;
    identifierSeed: number;
    isLoading: boolean;
    isLoadingMoreRows: boolean;
    rows: AllFeatureListTableRow[];
    selectedRow: React.ReactText;
    sortDirection: sortDirectionEnum;
    sortedColumnIdentifier: React.ReactText;
    onRowChangeWithNoScrollbarCounter: number;
    onScrollToBottomCounter: number;
}

function NameDescriptionCellComponent({ value }: any): React.ReactNode {
    return (
        <NameDescriptionCell
            description={value.description}
            name={value.name}
            pending={value.pending}
        />
    );
}
function CreatedDateCellComponent({ value }: any): React.ReactNode {
    return <DateCell date={value.date} description={value.description} />;
}
function UpdatedDateCellComponent({ value }: any): React.ReactNode {
    return <DateCell date={value.date} description={value.description} />;
}
function SizeNumberCellComponent({ value }: any): React.ReactNode {
    return <NumberCell description={value.description} value={value.value} />;
}

const columns: IListTableColumn[] = [
    {
        identifier: columnEnum.Name,
        header: 'Name',
        minWidth: 150,
        sortable: true,
        CustomCell: NameDescriptionCellComponent
    },
    {
        identifier: columnEnum.Created,
        header: 'Created',
        minWidth: 100,
        CustomCell: CreatedDateCellComponent
    },
    {
        identifier: columnEnum.LastUpdated,
        header: 'Last Updated',
        minWidth: 100,
        CustomCell: UpdatedDateCellComponent
    },
    {
        identifier: columnEnum.Size,
        header: 'Size',
        minWidth: 100,
        sortable: true,
        isMetric: true,
        CustomCell: SizeNumberCellComponent
    }
];

function createRow(id: number): AllFeatureListTableRow {
    return {
        identifier: id,
        name: {
            name: `Row #${id}`,
            description:
                'Donec id elit non mi porta gravida at eget metus. Sed posuere consectet'
        },
        [columnEnum.Created]: {
            date: new Date(2017, 1, 20),
            description: 'John Doe'
        },
        [columnEnum.LastUpdated]: {
            date: new Date(2018, 3, 20),
            description: 'John Doe'
        },
        [columnEnum.Size]: { value: 35189 }
    };
}

const HandlingNoScrollbar: React.FC = (): React.ReactElement => {
    const timeoutRef = useRef(null);
    const [state, setState] = React.useState<AllFeatureListTableState>({
        allRowsChecked: false,
        checkedRows: new Set(),
        currentRowId: 1,
        identifierSeed: 0,
        isLoading: false,
        isLoadingMoreRows: false,
        rows: [],
        selectedRow: null,
        sortDirection: sortDirectionEnum.Asc,
        sortedColumnIdentifier: 'name',
        onRowChangeWithNoScrollbarCounter: 0,
        onScrollToBottomCounter: 0
    });

    const appendMoreRows = useCallback((): void => {
        clearTimeout(timeoutRef.current);

        timeoutRef.current = setTimeout((): void => {
            setState(
                (
                    prevState: AllFeatureListTableState
                ): AllFeatureListTableState => ({
                    ...prevState,
                    currentRowId: prevState.currentRowId + 2,
                    rows: prevState.rows.concat([
                        createRow(prevState.currentRowId),
                        createRow(prevState.currentRowId + 1)
                    ]),
                    isLoading: false,
                    isLoadingMoreRows: false
                })
            );
        }, 1500);
    }, []);

    const onScrollToBottom = useCallback((): void => {
        setState(
            (prev): AllFeatureListTableState => {
                return {
                    ...prev,
                    isLoadingMoreRows: true,
                    onScrollToBottomCounter: prev.onScrollToBottomCounter + 1
                };
            }
        );

        appendMoreRows();
    }, [appendMoreRows]);

    const onRowChangeWithNoScrollbar = useCallback((): void => {
        setState(
            (prev): AllFeatureListTableState => {
                return {
                    ...prev,
                    isLoadingMoreRows: true,
                    onRowChangeWithNoScrollbarCounter:
                        prev.onRowChangeWithNoScrollbarCounter + 1
                };
            }
        );

        appendMoreRows();
    }, [appendMoreRows]);

    const loadNewData = useCallback((): void => {
        setState(
            (prev): AllFeatureListTableState => {
                return {
                    ...prev,
                    rows: [],
                    isLoading: true
                };
            }
        );

        appendMoreRows();
    }, [appendMoreRows]);

    const infiniteScrollEndMessage = useMemo(
        (): React.ReactElement => (
            <>
                <Signal className="mr-2" icon={Alert} size={1.5} />
                You have reached the end
            </>
        ),
        []
    );

    const hasMoreRows = state.rows.length < 10;

    return (
        <>
            <div>Preload data for Infinite Scroll</div>
            <div className="m-4">
                <h3>Times invoked:</h3>
                <p className="my-2">
                    onRowChangeWithNoScrollbar:{' '}
                    {state.onRowChangeWithNoScrollbarCounter}
                </p>
                <p className="my-2">
                    onScrollToBottom: {state.onScrollToBottomCounter}
                </p>
            </div>
            <Button onClick={loadNewData}>Load New Data</Button>
            <div style={{ height: '400px' }}>
                <ListTable
                    columns={columns}
                    dataUI="handling-no-scrollbar"
                    fillHeight
                    infiniteScrollEndMessage={
                        hasMoreRows ? null : infiniteScrollEndMessage
                    }
                    isHeaderSticky
                    isLoading={state.isLoading}
                    isLoadingMoreRows={state.isLoadingMoreRows}
                    onRowChangeWithNoScrollbar={onRowChangeWithNoScrollbar}
                    onScrollToBottom={hasMoreRows && onScrollToBottom}
                    rows={state.rows}
                />
            </div>
        </>
    );
};

export default HandlingNoScrollbar;
