import React from 'react';

import numeral from 'numeral';

import { DataTable, IDataTableColumn } from '@preamp/tables';

import { dataTablePagination } from './data/dataTablePagination';

enum resultsPerPage {
    Ten = 10,
    Twenty = 20,
    Fifty = 50,
    Hundred = 100
}

enum columnEnum {
    Letter = 'letter',
    Number = 'number'
}

interface RowData {
    rows: PaginationDataTableRows[];
    paging: PaginationData;
}

interface PaginationData {
    offset: number;
    limit: number;
    total: number;
}

interface PaginationDataTableRows {
    identifier: number;
    [columnEnum.Letter]: string;
    [columnEnum.Number]: number;
}

const columns: IDataTableColumn[] = [
    {
        identifier: columnEnum.Letter,
        header: 'Random Letter',
        minWidth: 150
    },
    {
        identifier: columnEnum.Number,
        header: 'Random Number',
        minWidth: 150,
        isMetric: true,
        CustomCell: (rowInfo): React.ReactNode =>
            numeral(rowInfo.value).format('0,0')
    }
];

function getRows(offset: number, limit: number): RowData {
    return {
        rows: dataTablePagination.slice(offset, offset + limit),
        paging: {
            offset: offset,
            limit: limit,
            total: dataTablePagination.length
        }
    };
}

interface PaginationDataTableState {
    currentRows: PaginationDataTableRows[];
    limit: number;
    offset: number;
    totalRows: number;
    resultsOnPage: resultsPerPage;
}

class PaginationDataTable extends React.Component<
    {},
    PaginationDataTableState
> {
    state: PaginationDataTableState = {
        currentRows: [],
        limit: 10,
        offset: 0,
        totalRows: 0,
        resultsOnPage: resultsPerPage.Ten
    };

    onResultsPerPageChange = (limit: number, offset: number): void => {
        const rowData: RowData = getRows(offset, limit);
        this.setState({ currentRows: rowData.rows, limit, offset });
    };

    onPageChange = (offset: number): void => {
        const rowData: RowData = getRows(offset, this.state.limit);
        this.setState({ currentRows: rowData.rows, offset });
    };

    componentDidMount(): void {
        const rowData: RowData = getRows(this.state.offset, this.state.limit);
        this.setState({
            currentRows: rowData.rows,
            totalRows: rowData.paging.total
        });
    }

    render(): React.ReactNode {
        const { currentRows, limit, offset, totalRows } = this.state;

        return (
            <>
                <div>Pagination</div>
                <DataTable
                    columns={columns}
                    dataUI="pagination-data-table"
                    limit={limit}
                    offset={offset}
                    onPageChange={this.onPageChange}
                    onResultsPerPageChange={this.onResultsPerPageChange}
                    resultsPerPageOptions={[
                        resultsPerPage.Ten,
                        resultsPerPage.Twenty,
                        resultsPerPage.Fifty,
                        resultsPerPage.Hundred
                    ]}
                    rows={currentRows}
                    showPagination={true}
                    totalRows={totalRows}
                />
            </>
        );
    }
}

export default PaginationDataTable;
