import React from 'react';

import { format } from 'date-fns';
import { sortBy } from 'lodash';
import numeral from 'numeral';

import { DataTable, IDataTableColumn, sortDirectionEnum } from '@preamp/tables';

enum columnEnum {
    Name = 'name',
    Created = 'created',
    LastUpdated = 'lastUpdated',
    Size = 'size'
}

interface CustomSortingDataTableRow {
    identifier: number;
    [columnEnum.Name]: string;
    [columnEnum.Created]: Date;
    [columnEnum.LastUpdated]: Date;
    [columnEnum.Size]: number;
}

const columns: IDataTableColumn[] = [
    {
        sortable: true,
        identifier: columnEnum.Name,
        header: 'Name',
        minWidth: 150
    },
    {
        sortable: true,
        identifier: columnEnum.Created,
        header: 'Created',
        minWidth: 100,
        CustomCell: (rowInfo: any): string =>
            format(rowInfo.value, 'MM/DD/YYYY')
    },
    {
        sortable: true,
        identifier: columnEnum.LastUpdated,
        header: 'Last Updated',
        minWidth: 100,
        CustomCell: (rowInfo: any): string =>
            format(rowInfo.value, 'MM/DD/YYYY')
    },
    {
        sortable: true,
        identifier: columnEnum.Size,
        header: 'Size',
        minWidth: 100,
        isMetric: true,
        CustomCell: (rowInfo: any): React.ReactNode =>
            numeral(rowInfo.value).format('0,0')
    }
];

const originalRows: CustomSortingDataTableRow[] = [
    {
        identifier: 1,
        [columnEnum.Name]: 'Bob',
        [columnEnum.Created]: new Date(1995, 3, 10),
        [columnEnum.LastUpdated]: new Date(1945, 9, 9),
        [columnEnum.Size]: 35189
    },
    {
        identifier: 2,
        [columnEnum.Name]: 'Amy',
        [columnEnum.Created]: new Date(1945, 2, 8),
        [columnEnum.LastUpdated]: new Date(1912, 6, 7),
        [columnEnum.Size]: 14138890
    },
    {
        identifier: 3,
        [columnEnum.Name]: 'Carl',
        [columnEnum.Created]: new Date(1979, 4, 6),
        [columnEnum.LastUpdated]: new Date(2001, 9, 5),
        [columnEnum.Size]: 145876
    },
    {
        identifier: 4,
        [columnEnum.Name]: 'Doug',
        [columnEnum.Created]: new Date(2007, 4, 4),
        [columnEnum.LastUpdated]: new Date(2018, 7, 3),
        [columnEnum.Size]: 75463
    },
    {
        identifier: 5,
        [columnEnum.Name]: 'Emily',
        [columnEnum.Created]: new Date(1998, 9, 2),
        [columnEnum.LastUpdated]: new Date(2005, 7, 1),
        [columnEnum.Size]: 984153
    }
];

interface CustomSortingDataTableState {
    rows: CustomSortingDataTableRow[];
    sortDirection: sortDirectionEnum;
    sortedColumnIdentifier: React.ReactText;
}

class CustomSortingDataTable extends React.Component<
    {},
    CustomSortingDataTableState
> {
    state: CustomSortingDataTableState = {
        rows: originalRows,
        sortDirection: sortDirectionEnum.Asc,
        sortedColumnIdentifier: 'name'
    };

    onColumnSorted = (
        columnIdentifier: string,
        sortDirection: sortDirectionEnum
    ): void => {
        const { rows } = this.state;
        const newRowsDesc: CustomSortingDataTableRow[] = sortBy(
            rows,
            (row: any): React.ReactNode => row[columnIdentifier]
        );
        this.setState({
            sortDirection,
            sortedColumnIdentifier: columnIdentifier,
            rows:
                sortDirection === sortDirectionEnum.Asc
                    ? newRowsDesc
                    : newRowsDesc.reverse()
        });
    };

    render(): React.ReactNode {
        const { sortDirection, sortedColumnIdentifier } = this.state;

        return (
            <>
                <div>Custom Sorting</div>
                <DataTable
                    columns={columns}
                    dataUI="basic-data-table"
                    onColumnSorted={this.onColumnSorted}
                    rows={this.state.rows}
                    sortDirection={sortDirection}
                    sortedColumnIdentifier={sortedColumnIdentifier}
                />
            </>
        );
    }
}

export default CustomSortingDataTable;
