import React from 'react';

import numeral from 'numeral';

import { IconButton, Search, TooltipPosition } from '@preamp/core';
import { Crop, Download, ViewColumn } from '@preamp/signal';
import {
    Filter,
    OptionsType,
    OptionType,
    SelectList,
    ValueType
} from '@preamp/select';
import { DataTable, IDataTableColumn, TableActionBar } from '@preamp/tables';

enum columnEnum {
    Name = 'name',
    Created = 'created',
    LastUpdated = 'lastUpdated',
    Size = 'size'
}

interface FilterSearchDataTableRow {
    identifier: number;
    [columnEnum.Name]: string;
    [columnEnum.Created]: string;
    [columnEnum.LastUpdated]: string;
    [columnEnum.Size]: number;
}

const columns: IDataTableColumn[] = [
    {
        identifier: columnEnum.Name,
        header: 'Name',
        minWidth: 150
    },
    {
        identifier: columnEnum.Created,
        header: 'Created',
        minWidth: 100
    },
    {
        identifier: columnEnum.LastUpdated,
        header: 'Last Updated',
        minWidth: 100
    },
    {
        identifier: columnEnum.Size,
        header: 'Size',
        minWidth: 100,
        isMetric: true,
        CustomCell: (rowInfo: any): React.ReactNode =>
            numeral(rowInfo.value).format('0,0')
    }
];

const rows: FilterSearchDataTableRow[] = [
    {
        identifier: 1,
        [columnEnum.Name]: 'Bob',
        [columnEnum.Created]: '1/20/2017',
        [columnEnum.LastUpdated]: '3/20/2018',
        [columnEnum.Size]: 35189
    },
    {
        identifier: 2,
        [columnEnum.Name]: 'Amy',
        [columnEnum.Created]: '2/20/2017',
        [columnEnum.LastUpdated]: '4/20/2019',
        [columnEnum.Size]: 14138890
    },
    {
        identifier: 3,
        [columnEnum.Name]: 'Carl',
        [columnEnum.Created]: '2/20/2017',
        [columnEnum.LastUpdated]: '5/20/2016',
        [columnEnum.Size]: 145876
    },
    {
        identifier: 4,
        [columnEnum.Name]: 'Doug',
        [columnEnum.Created]: '3/20/2017',
        [columnEnum.LastUpdated]: '6/20/2018',
        [columnEnum.Size]: 75463
    },
    {
        identifier: 5,
        [columnEnum.Name]: 'Emily',
        [columnEnum.Created]: '4/20/2017',
        [columnEnum.LastUpdated]: '7/20/2018',
        [columnEnum.Size]: 984153
    }
];

const drilldownOptions: OptionsType<OptionType> = [
    {
        label: 'Colors',
        value: 'colors'
    },
    {
        label: 'Flavors',
        value: 'flavors'
    }
];

const colorOptions: OptionsType<OptionType> = [
    {
        value: 'ocean',
        label: 'Ocean'
    },
    {
        value: 'blue',
        label: 'Blue'
    },
    { value: 'purple', label: 'Purple' },
    { value: 'red', label: 'Red' },
    { value: 'orange', label: 'Orange' }
];

const flavorOptions: OptionsType<OptionType> = [
    { value: 'vanilla', label: 'Vanilla' },
    { value: 'chocolate', label: 'Chocolate' },
    { value: 'strawberry', label: 'Strawberry' },
    {
        value: 'salted-caramel',
        label: 'Salted Caramel'
    }
];

function TableActionBarDataTable(): React.ReactElement<{}> {
    const [filters, setFilters] = React.useState([]);
    const [stagedColorOptions, setStagedColorOptions] = React.useState(null);
    const [stagedFlavorOptions, setStagedFlavorOptions] = React.useState(null);
    const [tableIsDownloading, setTableIsDownloading] = React.useState(false);
    function onActionClick(): void {
        console.log('you clicked an action!');
    }
    function downloadClick(): void {
        if (!tableIsDownloading) {
            setTableIsDownloading(true);
            setTimeout((): void => {
                setTableIsDownloading(false);
            }, 5000);
        }
    }

    function onColorSelectChange(value: ValueType<OptionType>): void {
        setStagedColorOptions(value);
    }

    function onFlavorSelectChange(value: ValueType<OptionType>): void {
        setStagedFlavorOptions(value);
    }

    function subComponent(
        selectedGroup: ValueType<OptionType>
    ): React.ReactNode {
        let subComp: React.ReactNode;
        switch (selectedGroup.value) {
            case drilldownOptions[0].value:
                const selectedColorOptions: OptionsType<OptionType> =
                    filters.length < 1
                        ? null
                        : filters.filter(
                              (option: OptionType): boolean =>
                                  option.group.value ===
                                  drilldownOptions[0].value
                          );
                subComp = (
                    <SelectList
                        defaultValue={selectedColorOptions}
                        focusOnMount={true}
                        isMulti={true}
                        isSearchable={false}
                        onChange={onColorSelectChange}
                        options={colorOptions}
                    />
                );
                break;
            case drilldownOptions[1].value:
                const selectedFlavorOptions: OptionsType<OptionType> =
                    filters.length < 1
                        ? null
                        : filters.filter(
                              (option: OptionType): boolean =>
                                  option.group.value ===
                                  drilldownOptions[1].value
                          );
                subComp = (
                    <SelectList
                        defaultValue={selectedFlavorOptions}
                        focusOnMount={true}
                        isMulti={true}
                        isSearchable={false}
                        onChange={onFlavorSelectChange}
                        options={flavorOptions}
                    />
                );
                break;
            default:
                subComp = null;
        }

        return subComp;
    }

    function onApplyHandler(selectedGroup: ValueType<OptionType>): void {
        let updatedValues: OptionsType<OptionType> = [];
        if (
            selectedGroup.value === drilldownOptions[0].value &&
            stagedColorOptions
        ) {
            // remove options within the same select group
            let reducedOptions: OptionsType<OptionType> = [];
            if (filters) {
                // remove any options set within the same group
                reducedOptions = filters.filter(
                    (option: OptionType): boolean => {
                        if (option.group.value === drilldownOptions[0].value) {
                            return false;
                        } else {
                            return true;
                        }
                    }
                );
            }
            // add group data to new selected options
            const optionsToApply: OptionsType<OptionType> = stagedColorOptions.map(
                (option: OptionType): React.ReactNode => {
                    return {
                        ...option,
                        group: selectedGroup
                    };
                }
            );
            updatedValues = [...reducedOptions, ...optionsToApply];
        } else if (
            selectedGroup.value === drilldownOptions[1].value &&
            stagedFlavorOptions
        ) {
            // remove options within the same select group
            let reducedOptions: OptionsType<OptionType> = [];
            if (filters) {
                // remove any options set within the same group
                reducedOptions = filters.filter(
                    (option: OptionType): boolean => {
                        if (option.group.value === drilldownOptions[1].value) {
                            return false;
                        } else {
                            return true;
                        }
                    }
                );
            }
            // add group data to new selected options
            const optionsToApply: OptionsType<OptionType> = stagedFlavorOptions.map(
                (option: OptionType): React.ReactNode => {
                    return {
                        ...option,
                        group: selectedGroup
                    };
                }
            );
            updatedValues = [...reducedOptions, ...optionsToApply];
        }
        setFilters(updatedValues);
    }

    function onApplySelectedOptionsHandler(
        group: ValueType<OptionType>,
        values: OptionsType<OptionType>
    ): void {
        let reducedOptions: OptionsType<OptionType> = [];
        if (filters) {
            // remove any options set within the same group
            reducedOptions = filters.filter(
                (option: OptionType): boolean =>
                    option.group.value === group.value
            );
        }
        // add group data to new selected options
        const optionsToApply: OptionsType<OptionType> = values.map(
            (option: OptionType): React.ReactNode => {
                return {
                    ...option,
                    group: group
                };
            }
        );
        const updatedFilters: OptionsType<OptionType> = [
            ...reducedOptions,
            ...optionsToApply
        ];
        setFilters(updatedFilters);
    }

    function onRemoveGroupHandler(groupToRemove: ValueType<OptionType>): void {
        const updatedFilters: OptionsType<OptionType> = filters.filter(
            (option: OptionType): boolean => {
                if (groupToRemove.value === option.group.value) {
                    return false;
                } else {
                    return true;
                }
            }
        );
        setFilters(updatedFilters);
    }

    const onSearch = ({
        currentTarget: { value: search }
    }: React.FormEvent<HTMLInputElement>): void => {
        console.log('searchTerm', search);
    };

    return (
        <>
            <div>Search and Filter</div>
            <TableActionBar
                Actions={[
                    <IconButton
                        icon={ViewColumn}
                        key="VewColumn"
                        onClick={onActionClick}
                        size="sm"
                    />,
                    <IconButton
                        icon={Crop}
                        key="Crop"
                        onClick={onActionClick}
                        size="sm"
                    />,
                    <IconButton
                        disabled={tableIsDownloading}
                        icon={Download}
                        key="GetApp"
                        onClick={downloadClick}
                        size="sm"
                        tooltip={
                            tableIsDownloading ? (
                                <span className="block text-center">
                                    Downloading
                                </span>
                            ) : (
                                <span className="block text-center">
                                    Download csv
                                </span>
                            )
                        }
                        tooltipPosition={TooltipPosition.Bottom}
                    />
                ]}
                showBottomBorder
                showTopBorder
            >
                <Search onChange={onSearch} />
                <Filter
                    groupOptions={drilldownOptions}
                    onApply={onApplyHandler}
                    onApplySelectedOptions={onApplySelectedOptionsHandler}
                    onRemoveGroup={onRemoveGroupHandler}
                    selectedValues={filters}
                    subComponent={subComponent}
                />
            </TableActionBar>
            <DataTable
                columns={columns}
                dataUI="basic-data-table"
                rows={rows}
            />
        </>
    );
}

export default TableActionBarDataTable;
