import React from 'react';

import numeral from 'numeral';

import { Button, ButtonTheme } from '@preamp/core';
import { DropdownButton } from '@preamp/select';
import { DataTable, IDataTableColumn, TableActionBar } from '@preamp/tables';

enum columnEnum {
    Name = 'name',
    Created = 'created',
    LastUpdated = 'lastUpdated',
    Size = 'size'
}

interface BulkActionsDataTableRow {
    identifier: number;
    [columnEnum.Name]: string;
    [columnEnum.Created]: string;
    [columnEnum.LastUpdated]: string;
    [columnEnum.Size]: number;
}

const columns: IDataTableColumn[] = [
    {
        identifier: columnEnum.Name,
        header: 'Name',
        minWidth: 150
    },
    {
        identifier: columnEnum.Created,
        header: 'Created',
        minWidth: 100
    },
    {
        identifier: columnEnum.LastUpdated,
        header: 'Last Updated',
        minWidth: 100
    },
    {
        identifier: columnEnum.Size,
        isMetric: true,
        header: 'Size',
        minWidth: 100,
        CustomCell: (rowInfo: any): React.ReactNode =>
            numeral(rowInfo.value).format('0,0')
    }
];

const rows: BulkActionsDataTableRow[] = [
    {
        identifier: 1,
        [columnEnum.Name]: 'Bob',
        [columnEnum.Created]: '1/20/2017',
        [columnEnum.LastUpdated]: '3/20/2018',
        [columnEnum.Size]: 35189
    },
    {
        identifier: 2,
        [columnEnum.Name]: 'Amy',
        [columnEnum.Created]: '2/20/2017',
        [columnEnum.LastUpdated]: '4/20/2019',
        [columnEnum.Size]: 14138890
    },
    {
        identifier: 3,
        [columnEnum.Name]: 'Carl',
        [columnEnum.Created]: '2/20/2017',
        [columnEnum.LastUpdated]: '5/20/2016',
        [columnEnum.Size]: 145876
    },
    {
        identifier: 4,
        [columnEnum.Name]: 'Doug',
        [columnEnum.Created]: '3/20/2017',
        [columnEnum.LastUpdated]: '6/20/2018',
        [columnEnum.Size]: 75463
    },
    {
        identifier: 5,
        [columnEnum.Name]: 'Emily',
        [columnEnum.Created]: '4/20/2017',
        [columnEnum.LastUpdated]: '7/20/2018',
        [columnEnum.Size]: 984153
    }
];

interface BulkActionsDataTableState {
    checkedRows: Set<number>;
    allRowsChecked: boolean;
}

class BulkActionsDataTable extends React.Component<
    {},
    BulkActionsDataTableState
> {
    state: BulkActionsDataTableState = {
        checkedRows: new Set(),
        allRowsChecked: false
    };

    onRowChecked = (identifier: number, checked: boolean): void => {
        const { checkedRows, allRowsChecked } = this.state;
        const newCheckedRows: Set<number> = new Set(checkedRows);
        let newAllRowsChecked: boolean = allRowsChecked;
        if (checked) {
            newCheckedRows.add(identifier);
            newAllRowsChecked = newCheckedRows.size === rows.length;
        } else {
            newCheckedRows.delete(identifier);
            newAllRowsChecked = false;
        }
        this.setState({
            checkedRows: newCheckedRows,
            allRowsChecked: newAllRowsChecked
        });
    };

    onAllChecked = (checked: boolean): void => {
        const newCheckedRows: Set<number> = new Set(this.state.checkedRows);

        if (checked) {
            rows.forEach(
                (row: BulkActionsDataTableRow): React.ReactNode =>
                    newCheckedRows.add(row.identifier)
            );
        } else {
            newCheckedRows.clear();
        }

        this.setState({ checkedRows: newCheckedRows, allRowsChecked: checked });
    };

    onCreateItemClick = (): void => {
        // eslint-disable-next-line no-console
        console.log('Create Example Clicked');
    };

    render(): React.ReactNode {
        const { allRowsChecked, checkedRows } = this.state;

        return (
            <>
                <div>Bulk Actions</div>
                <TableActionBar
                    Actions={[
                        <div className="va-description" key="selected">
                            {checkedRows.size} item
                            {checkedRows.size === 1 ? '' : 's'} selected
                        </div>,
                        <DropdownButton
                            buttonProps={{ theme: ButtonTheme.Secondary }}
                            key="bulkAction"
                            onChange={this.onCreateItemClick}
                            options={[
                                {
                                    label: 'Delete',
                                    value: 'delete'
                                },
                                { label: 'Clone', value: 'clone' }
                            ]}
                        >
                            Bulk Actions
                        </DropdownButton>,
                        <Button
                            key="createItem"
                            onClick={this.onCreateItemClick}
                        >
                            Create Item
                        </Button>
                    ]}
                    showBottomBorder
                >
                    <h2 className="va-table-action-title">Data Table Name</h2>
                </TableActionBar>
                <DataTable
                    allRowsChecked={allRowsChecked}
                    checkable={true}
                    checkedRows={checkedRows}
                    columns={columns}
                    dataUI="checkable-data-table"
                    onAllChecked={this.onAllChecked}
                    onRowChecked={this.onRowChecked}
                    rows={rows}
                />
            </>
        );
    }
}

export default BulkActionsDataTable;
