import React from 'react';

import { cloneDeep } from 'lodash';

import { SelectType } from '@preamp/select';
import { DataTable, IDataTableColumn, SelectCell } from '@preamp/tables';

interface Option {
    value: string;
    label: string;
}

enum columnEnum {
    ValueOptions = 'valueOption',
    ColorOptions = 'colorOptions',
    FirstName = 'firstName',
    LastName = 'lastName',
    Age = 'age',
    Location = 'location',
    Date = 'date'
}

interface EditableDataTableRow {
    identifier: number;
    [columnEnum.ValueOptions]?: {};
    [columnEnum.ColorOptions]: {
        options: Option[];
        defaultOption: Option;
    };
    [columnEnum.FirstName]: string;
    [columnEnum.LastName]: string;
    [columnEnum.Age]: number;
    [columnEnum.Location]: string;
    [columnEnum.Date]: string;
}

const rows: EditableDataTableRow[] = [
    {
        identifier: 1,
        [columnEnum.ValueOptions]: {},
        [columnEnum.ColorOptions]: {
            options: [
                { value: 'blue', label: 'blue' },
                { value: 'dodgerBlue', label: 'dodgerBlue' },
                { value: 'teal', label: 'teal' },
                { value: 'cyan', label: 'cyan' }
            ],
            defaultOption: { value: 'blue', label: 'blue' }
        },
        [columnEnum.FirstName]: 'Bob',
        [columnEnum.LastName]: 'Samuelson',
        [columnEnum.Age]: 42,
        [columnEnum.Location]: 'CA',
        [columnEnum.Date]: '11/17/1976'
    },
    {
        identifier: 2,
        [columnEnum.ValueOptions]: {},
        [columnEnum.ColorOptions]: {
            options: [
                { value: 'purple', label: 'purple' },
                { value: 'violet', label: 'violet' },
                { value: 'lavender', label: 'lavender' },
                { value: 'plum', label: 'plum' }
            ],
            defaultOption: { value: 'purple', label: 'purple' }
        },
        [columnEnum.FirstName]: 'Amy',
        [columnEnum.LastName]: 'Gaudioso',
        [columnEnum.Age]: 26,
        [columnEnum.Location]: 'MA',
        [columnEnum.Date]: '10/20/1992'
    },
    {
        identifier: 3,
        [columnEnum.ValueOptions]: {},
        [columnEnum.ColorOptions]: {
            options: [
                { value: 'pink', label: 'pink' },
                { value: 'rose', label: 'rose' },
                { value: 'peach', label: 'peach' },
                { value: 'megenta', label: 'megenta' }
            ],
            defaultOption: { value: 'pink', label: 'pink' }
        },
        [columnEnum.FirstName]: 'Juan',
        [columnEnum.LastName]: 'Giuffrida',
        [columnEnum.Age]: 32,
        [columnEnum.Location]: 'TX',
        [columnEnum.Date]: '9/15/1986'
    },
    {
        identifier: 4,
        [columnEnum.ValueOptions]: {},
        [columnEnum.ColorOptions]: {
            options: [
                { value: 'green', label: 'green' },
                { value: 'forest', label: 'forest' },
                { value: 'olive', label: 'olive' },
                { value: 'sage', label: 'sage' }
            ],
            defaultOption: { value: 'green', label: 'green' }
        },
        [columnEnum.FirstName]: 'Doug',
        [columnEnum.LastName]: 'Ung',
        [columnEnum.Age]: 37,
        [columnEnum.Location]: 'NY',
        [columnEnum.Date]: '12/9/1981'
    },
    {
        identifier: 5,
        [columnEnum.ValueOptions]: {},
        [columnEnum.ColorOptions]: {
            options: [
                { value: 'yellow', label: 'yellow' },
                { value: 'orange', label: 'orange' },
                { value: 'gold', label: 'gold' },
                { value: 'corn', label: 'corn' }
            ],
            defaultOption: { value: 'yellow', label: 'yellow' }
        },
        [columnEnum.FirstName]: 'Ben',
        [columnEnum.LastName]: 'Behrens',
        [columnEnum.Age]: 35,
        [columnEnum.Location]: 'NC',
        [columnEnum.Date]: '8/3/1983'
    }
];

interface EditableDataTableState {
    selectedValues: {
        [key: number]: Option;
    };
    selectedColors: {
        [key: number]: Option;
    };
}

class EditableDataTable extends React.Component<{}, EditableDataTableState> {
    state: EditableDataTableState = {
        selectedValues: {},
        selectedColors: {}
    };
    columns: IDataTableColumn[];

    constructor(props: any) {
        super(props);

        this.columns = [
            {
                identifier: columnEnum.ValueOptions,
                header: 'Value',
                onValueChange: this.onValueChange,
                CustomCell: (data: any): React.ReactNode => {
                    const rowIdentifier: number = data.original.identifier;

                    function onChange(selectedOption: Option): void {
                        data.column.onValueChange(
                            rowIdentifier,
                            selectedOption
                        );
                    }

                    return (
                        <SelectCell
                            onChange={onChange}
                            options={[
                                { value: '10', label: '10' },
                                { value: '20', label: '20' },
                                { value: '50', label: '50' },
                                { value: '100', label: '100' }
                            ]}
                            selectType={SelectType.Compact}
                            value={
                                this.state.selectedValues[rowIdentifier] || {
                                    value: '10',
                                    label: '10'
                                }
                            }
                        />
                    );
                }
            },
            {
                identifier: columnEnum.ColorOptions,
                header: 'Color',
                onColorChange: this.onColorChange,
                CustomCell: (data: any): React.ReactNode => {
                    const { value } = data;
                    const { options, defaultOption } = value;
                    const rowIdentifier: number = data.original.identifier;

                    function onChange(selectedOption: Option): void {
                        data.column.onColorChange(
                            rowIdentifier,
                            selectedOption
                        );
                    }

                    return (
                        <SelectCell
                            onChange={onChange}
                            options={options}
                            selectType={SelectType.Compact}
                            value={
                                this.state.selectedColors[rowIdentifier] ||
                                defaultOption
                            }
                        />
                    );
                }
            },
            {
                identifier: columnEnum.FirstName,
                header: 'First Name'
            },
            {
                identifier: columnEnum.LastName,
                header: 'Last Name'
            },
            {
                identifier: columnEnum.Age,
                header: 'Age'
            },
            {
                identifier: columnEnum.Location,
                header: 'Location'
            },
            {
                identifier: columnEnum.Date,
                header: 'Date'
            }
        ];
    }

    onValueChange = (rowIdentifier: number, option: Option): void => {
        const { selectedValues } = this.state;
        const newSelectedValues: { [key: number]: Option } = cloneDeep(
            selectedValues
        );
        newSelectedValues[rowIdentifier] = option;
        this.setState({ selectedValues: newSelectedValues });
    };

    onColorChange = (rowIdentifier: number, option: Option): void => {
        const { selectedColors } = this.state;
        const newSelectedValues: { [key: number]: Option } = cloneDeep(
            selectedColors
        );
        newSelectedValues[rowIdentifier] = option;
        this.setState({ selectedColors: newSelectedValues });
    };

    logSavedData = (): React.ReactNode => {
        const { selectedValues, selectedColors } = this.state;

        return rows.map(
            (row: EditableDataTableRow): React.ReactNode => {
                const selectedValue: Option = selectedValues[
                    row.identifier
                ] || {
                    value: '10',
                    label: '10'
                };
                const selectedColor: Option =
                    selectedColors[row.identifier] ||
                    row[columnEnum.ColorOptions].defaultOption;

                return (
                    <div key={row[columnEnum.FirstName]}>
                        <span>{row[columnEnum.FirstName]}:</span>
                        <ul>
                            <li>value: {selectedValue.label}</li>
                            <li>color: {selectedColor.label}</li>
                        </ul>
                    </div>
                );
            }
        );
    };

    render(): React.ReactNode {
        return (
            <>
                <div>Editable Data Table</div>
                <DataTable
                    columns={this.columns}
                    dataUI="Editable-data-table"
                    rows={rows}
                />
                <div style={{ display: 'flex', flexWrap: 'wrap' }}>
                    {this.logSavedData()}
                </div>
            </>
        );
    }
}

export default EditableDataTable;
