import React from 'react';

import numeral from 'numeral';

import { Button, ButtonTheme, IconButton, Search } from '@preamp/core';
import { Crop, Download, ViewColumn } from '@preamp/signal';
import {
    DropdownButton,
    Filter,
    OptionsType,
    OptionType,
    SelectList,
    ValueType
} from '@preamp/select';
import {
    DataTable,
    EmptyState,
    getRows,
    IDataTableColumn,
    sortDirectionEnum,
    TableActionBar
} from '@preamp/tables';

enum resultsPerPage {
    Five = 5,
    Ten = 10,
    Fifteen = 15,
    Twenty = 20
}

enum columnGroupEnum {
    Letter = 'letter',
    Number = 'number'
}

enum columnsEnum {
    FirstLetter = 'firstLetter',
    SecondLetter = 'secondLetter',
    FirstNumber = 'firstNumber',
    SecondNumber = 'secondNumber'
}

interface RowData {
    rows: AllFeaturesDataTableRow[];
    paging: PaginationData;
}

interface PaginationData {
    offset: number;
    limit: number;
    total: number;
}

interface AllFeaturesDataTableRow {
    identifier: number;
    [columnsEnum.FirstLetter]: string;
    [columnsEnum.SecondLetter]: string;
    [columnsEnum.FirstNumber]: number;
    [columnsEnum.SecondNumber]: number;
}

const drilldownOptions: OptionsType<OptionType> = [
    {
        label: 'Letters',
        value: 'letters'
    },
    {
        label: 'Numbers',
        value: 'numbers'
    }
];

const letterOptions: OptionsType<OptionType> = [
    { label: 'a-e', value: 'a-e', range: { first: 'a', last: 'e' } },
    { label: 'f-l', value: 'f-l', range: { first: 'f', last: 'l' } },
    { label: 'm-s', value: 'm-s', range: { first: 'm', last: 's' } },
    { label: 't-z', value: 't-z', range: { first: 't', last: 'z' } }
];

const numberOptions: OptionsType<OptionType> = [
    { label: '1-25', value: '1-25', range: { first: '1', last: '25' } },
    { label: '25-50', value: '25-50', range: { first: '25', last: '50' } },
    { label: '51-75', value: '51-75', range: { first: '51', last: '75' } },
    { label: '76-100', value: '76-100', range: { first: '76', last: '100' } }
];

const columns: IDataTableColumn[] = [
    {
        identifier: columnGroupEnum.Letter,
        header: 'Random Letters',
        columns: [
            {
                sortable: true,
                identifier: columnsEnum.FirstLetter,
                header: 'First Letter',
                minWidth: 150
            },
            {
                sortable: true,
                identifier: columnsEnum.SecondLetter,
                header: 'Second Letter',
                minWidth: 150
            }
        ]
    },
    {
        identifier: columnGroupEnum.Number,
        header: 'Random Number',
        columns: [
            {
                sortable: true,
                identifier: columnsEnum.FirstNumber,
                header: 'First Number',
                minWidth: 150,
                isMetric: true,
                CustomCell: (rowInfo: any): React.ReactNode =>
                    numeral(rowInfo.value).format('0,0')
            },
            {
                sortable: true,
                identifier: columnsEnum.SecondNumber,
                header: 'Second Number',
                minWidth: 150,
                isMetric: true,
                CustomCell: (rowInfo: any): React.ReactNode =>
                    numeral(rowInfo.value).format('0,0')
            }
        ]
    }
];

interface AllFeaturesDataTableState {
    currentRows: AllFeaturesDataTableRow[];
    limit: number;
    offset: number;
    totalRows: number;
    sortedColumnIdentifier: React.ReactText;
    sortDirection: sortDirectionEnum;
    searchText: string;
    checkedRows: Set<number>;
    allRowsChecked: boolean;
    filters: OptionsType<OptionType>;
    stagedLetterOptions: OptionsType<OptionType>;
    stagedNumberOptions: OptionsType<OptionType>;
}

class AllFeaturesDataTable extends React.Component<
    {},
    AllFeaturesDataTableState
> {
    state: AllFeaturesDataTableState = {
        currentRows: [],
        limit: 10,
        offset: 0,
        totalRows: 0,
        sortedColumnIdentifier: null,
        sortDirection: sortDirectionEnum.Unsorted,
        searchText: '',
        filters: [],
        checkedRows: new Set(),
        allRowsChecked: false,
        stagedLetterOptions: null,
        stagedNumberOptions: null
    };

    NoResults: React.ReactNode = (
        <EmptyState mainTitle="This doesn’t seem to be the results you’re looking for." />
    );

    onResultsPerPageChange: (limit: number) => void = (limit: number): void => {
        const {
            offset,
            sortedColumnIdentifier,
            sortDirection,
            searchText,
            filters
        } = this.state;
        const rowData: RowData = getRows({
            offset,
            limit,
            sortByColumnIdentifier: sortedColumnIdentifier,
            sortDirection,
            searchText,
            filters
        });
        this.setState({ currentRows: rowData.rows, limit });
    };

    onPageChange: (offset: number) => void = (offset: number): void => {
        const {
            limit,
            sortedColumnIdentifier,
            sortDirection,
            searchText,
            filters
        } = this.state;
        const rowData: RowData = getRows({
            offset,
            limit,
            sortByColumnIdentifier: sortedColumnIdentifier,
            sortDirection,
            searchText,
            filters
        });
        this.setState({
            currentRows: rowData.rows,
            offset,
            sortedColumnIdentifier,
            sortDirection
        });
    };

    onColumnSorted: (
        columnIdentifier: React.ReactText,
        newSortDirection: sortDirectionEnum
    ) => void = (
        columnIdentifier: React.ReactText,
        newSortDirection: sortDirectionEnum
    ): void => {
        const { offset, limit, searchText, filters } = this.state;
        const rowData: RowData = getRows({
            offset,
            limit,
            sortByColumnIdentifier: columnIdentifier,
            sortDirection: newSortDirection,
            searchText,
            filters
        });
        this.setState({
            currentRows: rowData.rows,
            sortedColumnIdentifier: columnIdentifier,
            sortDirection: newSortDirection
        });
    };

    clearSearch = (): void => {
        const {
            limit,
            sortedColumnIdentifier,
            sortDirection,
            filters
        } = this.state;
        const rowData: RowData = getRows({
            offset: 0,
            limit,
            sortByColumnIdentifier: sortedColumnIdentifier,
            sortDirection,
            searchText: '',
            filters
        });
        this.setState({
            currentRows: rowData.rows,
            searchText: '',
            totalRows: rowData.paging.total
        });
    };

    onSearch = ({
        currentTarget: { value: search }
    }: React.FormEvent<HTMLInputElement>): void => {
        const {
            limit,
            sortedColumnIdentifier,
            sortDirection,
            filters
        } = this.state;
        const rowData: RowData = getRows({
            offset: 0,
            limit,
            sortByColumnIdentifier: sortedColumnIdentifier,
            sortDirection,
            searchText: search,
            filters
        });
        this.setState({
            currentRows: rowData.rows,
            searchText: search,
            offset: 0,
            totalRows: rowData.paging.total
        });
    };

    onActionClick = (): void => {
        // eslint-disable-next-line no-console
        console.log('you clicked an action!');
    };

    onRowClicked: (rowIdentifier: React.ReactText) => void = (
        rowIdentifier: React.ReactText
    ): void => {
        // eslint-disable-next-line no-console
        console.log('onRowClicked', rowIdentifier);
    };

    onRowKeyPressed = (
        rowIdentifier: number,
        event: React.KeyboardEvent
    ): void => {
        // eslint-disable-next-line no-console
        console.log('onRowKeyPressed', rowIdentifier, event);
    };

    onRowChecked = (identifier: number, checked: boolean): void => {
        const { checkedRows, allRowsChecked, totalRows } = this.state;
        const newCheckedRows: Set<number> = new Set(checkedRows);
        let newAllRowsChecked: boolean = allRowsChecked;
        if (checked) {
            newCheckedRows.add(identifier);
            newAllRowsChecked = newCheckedRows.size === totalRows;
        } else {
            newCheckedRows.delete(identifier);
            newAllRowsChecked = false;
        }
        this.setState({
            checkedRows: newCheckedRows,
            allRowsChecked: newAllRowsChecked
        });
    };

    onAllChecked = (checked: boolean): void => {
        const { checkedRows, totalRows, searchText, filters } = this.state;

        const newCheckedRows: Set<number> = new Set(checkedRows);

        const allFilteredRows: RowData = getRows({
            limit: totalRows,
            offset: 0,
            searchText,
            filters
        });

        if (checked) {
            allFilteredRows.rows.forEach(
                (row: AllFeaturesDataTableRow): React.ReactNode =>
                    newCheckedRows.add(row.identifier)
            );
        } else {
            allFilteredRows.rows.forEach(
                (row: AllFeaturesDataTableRow): React.ReactNode =>
                    newCheckedRows.delete(row.identifier)
            );
        }

        this.setState({ checkedRows: newCheckedRows, allRowsChecked: checked });
    };

    componentDidMount(): void {
        const rowData: RowData = getRows({
            offset: this.state.offset,
            limit: this.state.limit,
            searchText: '',
            filters: []
        });
        this.setState({
            currentRows: rowData.rows,
            totalRows: rowData.paging.total
        });
    }

    onBulkActionsClick = (): void => {
        // eslint-disable-next-line no-console
        console.log('Clicked Bulk Actions');
    };

    onCreateItemClick = (): void => {
        // eslint-disable-next-line no-console
        console.log('Clicked Bulk Actions');
    };

    onRemoveGroupHandler = (groupToRemove: ValueType<OptionType>): void => {
        const {
            limit,
            searchText,
            filters,
            sortDirection,
            sortedColumnIdentifier
        } = this.state;
        const updatedFilters: OptionsType<OptionType> = filters.filter(
            (option: OptionType): boolean => {
                if (groupToRemove.value === option.group.value) {
                    return false;
                } else {
                    return true;
                }
            }
        );
        const rowData: RowData = getRows({
            offset: 0,
            limit,
            sortByColumnIdentifier: sortedColumnIdentifier,
            sortDirection,
            searchText,
            filters: updatedFilters
        });
        this.setState({
            currentRows: rowData.rows,
            filters: updatedFilters,
            offset: 0,
            totalRows: rowData.paging.total
        });
    };

    onApplyHandler = (selectedGroup: ValueType<OptionType>): void => {
        const {
            limit,
            searchText,
            filters,
            sortDirection,
            sortedColumnIdentifier,
            stagedLetterOptions,
            stagedNumberOptions
        } = this.state;
        let updatedValues: OptionsType<OptionType> = [];
        if (
            selectedGroup.value === drilldownOptions[0].value &&
            stagedLetterOptions
        ) {
            // remove options within the same select group
            let reducedOptions: OptionsType<OptionType> = [];
            if (filters) {
                // remove any options set within the same group
                reducedOptions = filters.filter(
                    (option: OptionType): boolean => {
                        if (option.group.value === drilldownOptions[0].value) {
                            return false;
                        } else {
                            return true;
                        }
                    }
                );
            }
            // add group data to new selected options
            const optionsToApply: OptionsType<OptionType> = stagedLetterOptions.map(
                (option: OptionType): React.ReactNode => {
                    return {
                        ...option,
                        group: selectedGroup
                    };
                }
            );
            updatedValues = [...reducedOptions, ...optionsToApply];
        } else if (
            selectedGroup.value === drilldownOptions[1].value &&
            stagedNumberOptions
        ) {
            // remove options within the same select group
            let reducedOptions: OptionsType<OptionType> = [];
            if (filters) {
                // remove any options set within the same group
                reducedOptions = filters.filter(
                    (option: OptionType): boolean => {
                        if (option.group.value === drilldownOptions[1].value) {
                            return false;
                        } else {
                            return true;
                        }
                    }
                );
            }
            // add group data to new selected options
            const optionsToApply: OptionsType<OptionType> = stagedNumberOptions.map(
                (option: OptionType): React.ReactNode => {
                    return {
                        ...option,
                        group: selectedGroup
                    };
                }
            );
            updatedValues = [...reducedOptions, ...optionsToApply];
        }
        const rowData: RowData = getRows({
            offset: 0,
            limit,
            sortByColumnIdentifier: sortedColumnIdentifier,
            sortDirection,
            searchText,
            filters: updatedValues
        });
        this.setState({
            currentRows: rowData.rows,
            filters: updatedValues,
            offset: 0,
            totalRows: rowData.paging.total
        });
    };

    onApplySelectedOptionsHandler = (
        group: ValueType<OptionType>,
        values: OptionsType<OptionType>
    ): void => {
        const {
            limit,
            searchText,
            filters,
            sortDirection,
            sortedColumnIdentifier
        } = this.state;
        let reducedOptions: OptionsType<OptionType> = [];
        if (filters) {
            // remove any options set within the same group
            reducedOptions = filters.filter((option: OptionType): boolean => {
                if (option.group.value === group.value) {
                    return false;
                } else {
                    return true;
                }
            });
        }
        // add group data to new selected options
        const optionsToApply: OptionsType<OptionType> = values.map(
            (option: OptionType): React.ReactNode => {
                return {
                    ...option,
                    group: group
                };
            }
        );
        const updatedFilters: OptionsType<OptionType> = [
            ...reducedOptions,
            ...optionsToApply
        ];
        const rowData: RowData = getRows({
            offset: 0,
            limit,
            sortByColumnIdentifier: sortedColumnIdentifier,
            sortDirection,
            searchText,
            filters: updatedFilters
        });
        this.setState({
            currentRows: rowData.rows,
            filters: updatedFilters,
            offset: 0,
            totalRows: rowData.paging.total
        });
    };

    onLetterSelectChange = (value: ValueType<OptionType>): void => {
        this.setState({
            stagedLetterOptions: value
        });
    };

    onNumberSelectChange = (value: ValueType<OptionType>): void => {
        this.setState({
            stagedNumberOptions: value
        });
    };

    subComponent = (selectedGroup: ValueType<OptionType>): React.ReactNode => {
        const { filters } = this.state;
        let subComp: React.ReactNode;
        switch (selectedGroup.value) {
            case drilldownOptions[0].value:
                const selectedLetterOptions: OptionsType<OptionType> =
                    filters.length < 1
                        ? null
                        : filters.filter((option: OptionType): boolean => {
                              if (
                                  option.group.value ===
                                  drilldownOptions[0].value
                              ) {
                                  return true;
                              } else {
                                  return false;
                              }
                          });
                subComp = (
                    <SelectList
                        defaultValue={selectedLetterOptions}
                        focusOnMount={true}
                        isMulti={true}
                        isSearchable={false}
                        onChange={this.onLetterSelectChange}
                        options={letterOptions}
                    />
                );
                break;
            case drilldownOptions[1].value:
                const selectedNumberOptions: OptionsType<OptionType> =
                    filters.length < 1
                        ? null
                        : filters.filter((option: OptionType): boolean => {
                              if (
                                  option.group.value ===
                                  drilldownOptions[1].value
                              ) {
                                  return true;
                              } else {
                                  return false;
                              }
                          });
                subComp = (
                    <SelectList
                        defaultValue={selectedNumberOptions}
                        focusOnMount={true}
                        isMulti={true}
                        isSearchable={false}
                        onChange={this.onNumberSelectChange}
                        options={numberOptions}
                    />
                );
                break;
            default:
                subComp = null;
        }

        return subComp;
    };

    render(): React.ReactNode {
        const {
            currentRows,
            limit,
            offset,
            totalRows,
            searchText,
            allRowsChecked,
            checkedRows,
            sortDirection,
            sortedColumnIdentifier,
            filters
        } = this.state;

        return (
            <>
                <div>All Features</div>
                <TableActionBar
                    Actions={[
                        <div className="va-description" key="selected">
                            {checkedRows.size} item
                            {checkedRows.size === 1 ? '' : 's'} selected
                        </div>,
                        <DropdownButton
                            buttonProps={{ theme: ButtonTheme.Secondary }}
                            key="bulkAction"
                            onChange={this.onCreateItemClick}
                            options={[
                                {
                                    label: 'Delete',
                                    value: 'delete'
                                },
                                { label: 'Clone', value: 'clone' }
                            ]}
                        >
                            Bulk Actions
                        </DropdownButton>,
                        <Button
                            key="createItem"
                            onClick={this.onCreateItemClick}
                        >
                            Create Item
                        </Button>
                    ]}
                    showBottomBorder
                >
                    <h2 className="va-table-action-title">Data Table Name</h2>
                </TableActionBar>
                <TableActionBar
                    Actions={[
                        <IconButton
                            icon={ViewColumn}
                            key="Search"
                            onClick={this.onActionClick}
                            size="sm"
                        />,
                        <IconButton
                            icon={Crop}
                            key="Help"
                            onClick={this.onActionClick}
                            size="sm"
                        />,
                        <IconButton
                            icon={Download}
                            key="NotificationsNone"
                            onClick={this.onActionClick}
                            size="sm"
                        />
                    ]}
                    showBottomBorder
                >
                    <Search
                        onChange={this.onSearch}
                        onClear={this.clearSearch}
                        value={searchText}
                    />
                    <Filter
                        groupOptions={drilldownOptions}
                        onApply={this.onApplyHandler}
                        onApplySelectedOptions={
                            this.onApplySelectedOptionsHandler
                        }
                        onRemoveGroup={this.onRemoveGroupHandler}
                        selectedValues={filters}
                        subComponent={this.subComponent}
                    />
                </TableActionBar>
                <div style={{ height: 400 }}>
                    <DataTable
                        allRowsChecked={allRowsChecked}
                        checkable={true}
                        checkedRows={checkedRows}
                        columns={columns}
                        dataUI="all-features-data-table"
                        EmptyState={this.NoResults}
                        fillHeight
                        limit={limit}
                        numberOfPinnedColumns={1}
                        offset={offset}
                        onAllChecked={this.onAllChecked}
                        onColumnSorted={this.onColumnSorted}
                        onPageChange={this.onPageChange}
                        onResultsPerPageChange={this.onResultsPerPageChange}
                        onRowChecked={this.onRowChecked}
                        onRowClicked={this.onRowClicked}
                        onRowKeyPress={this.onRowKeyPressed}
                        resultsPerPageOptions={[
                            resultsPerPage.Five,
                            resultsPerPage.Ten,
                            resultsPerPage.Fifteen,
                            resultsPerPage.Twenty
                        ]}
                        rows={currentRows}
                        showPagination={true}
                        sortDirection={sortDirection}
                        sortedColumnIdentifier={sortedColumnIdentifier}
                        totalRows={totalRows}
                        truncateColHeader
                    />
                </div>
            </>
        );
    }
}

export default AllFeaturesDataTable;
