import React, { FC, ReactElement, ReactNode, useMemo, useState } from 'react';

import cx from 'classnames';

import { RadioButton, RadioButtonGroup } from '@preamp/core';
import { OptionType, OptionsType } from '@preamp/select';
import { Zap } from '@preamp/signal';
import {
    AxisCell,
    BarCell,
    CustomMessageCell,
    DataTable,
    DEFAULT_NUMBER_FORMAT,
    DotPrefix,
    HeaderSelectCell,
    HeaderSelectCellSortMode,
    IDataTableColumn,
    ITableRow,
    NumberFormatter
} from '@preamp/tables';

type DataItem<T = string | number> = Record<string, T>;

interface ChartTableRow<T = string | number> {
    identifier: T;
    rawItem: DataItem<T>;
    [_: string]: T | DataItem<T>;
}

type ColumnSettings = {
    className?: string;
    limit?: number;
    maxValue?: number;
    onChange(_: OptionsType<OptionType>): void;
    onClick?: (_: ITableRow) => void;
    onSortClick?(): void;
    options: OptionsType<string>[];
    selected: string;
    sortMode?: HeaderSelectCellSortMode;
};

const METRIC_COLUMN_COLOR = 'var(--waveform-default-500)';
const COMPARED_METRIC_COLUMN_COLOR = 'var(--waveform-default-900)';

const FORMATTERS: Record<string, NumberFormatter> = {
    metric1: {
        noDecimals: '0,0',
        withDecimals: '0,0.00'
    },
    metric2: {
        noDecimals: '$0,0a',
        withDecimals: '$0,0.00'
    },
    metric3: {
        noDecimals: '%0,0',
        withDecimals: '%0,0.00'
    }
};

const PAGE_SIZES = [10, 20, 50];

const ITEMS = [
    'Alabama',
    'Alaska',
    'Arizona',
    'Arkansas',
    'California',
    'Colorado',
    'Connecticut',
    'Delaware',
    'Florida',
    'Georgia',
    'Hawaii',
    'Idaho',
    'Illinois',
    'Indiana',
    'Iowa',
    'Kansas',
    'Kentucky',
    'Louisiana',
    'Maine',
    'Maryland',
    'Massachusetts',
    'Michigan',
    'Minnesota',
    'Mississippi',
    'Missouri',
    'Montana',
    'Nebraska',
    'Nevada',
    'New Hampshire',
    'New Jersey',
    'New Mexico',
    'New York',
    'North Carolina',
    'North Dakota',
    'Ohio',
    'Oklahoma',
    'Oregon',
    'Pennsylvania',
    'Rhode Island',
    'South Carolina',
    'South Dakota',
    'Tennessee',
    'Texas',
    'Utah',
    'Vermont',
    'Virginia',
    'Washington',
    'West Virginia',
    'Wisconsin',
    'Wyoming'
];

const DATA_ITEMS: DataItem[] = ITEMS.map(
    (name: string, i: number): DataItem => ({
        name,
        metric1: (ITEMS.length - i) * 50,
        metric2: Math.random() * 1000,
        metric3: Math.random(),
        metric4: Math.random() * 500000
    })
);

const METRIC_OPTIONS: OptionsType<OptionType> = [
    { value: 'metric1', label: 'Metric 1' },
    { value: 'metric2', label: 'Metric 2' },
    { value: 'metric3', label: 'Metric 3' },
    { value: 'metric4', label: 'Metric 4' }
];

const ROWS = DATA_ITEMS.map(
    (dataItem: DataItem, idx: number): ChartTableRow => ({
        identifier: `row-${idx}`,
        rawItem: dataItem,
        ...dataItem
    })
);

function getMaxColumnValue(rows: ChartTableRow[], metric: string): number {
    return Math.max(
        ...rows.map((row: ChartTableRow): number => row[metric] as number)
    );
}

function buildColumns(
    metric: ColumnSettings,
    comparedMetric: ColumnSettings,
    hasMoreRows: boolean,
    isEmpty: boolean,
    hasErrors: boolean
): IDataTableColumn[] {
    const metricFormatter =
        FORMATTERS[metric.selected] || DEFAULT_NUMBER_FORMAT;
    const comparedMetricFormatter =
        FORMATTERS[comparedMetric.selected] || DEFAULT_NUMBER_FORMAT;

    return [
        { header: 'State', identifier: 'name' },
        {
            // eslint-disable-next-line react/display-name
            CustomCell: ({ value }: { value: number }): ReactNode => (
                <BarCell
                    color={METRIC_COLUMN_COLOR}
                    limit={metric.limit}
                    maxValue={metric.maxValue}
                    numberFormat={metricFormatter.withDecimals}
                    value={value}
                />
            ),
            Footer:
                !hasErrors && !isEmpty ? (
                    <>
                        <AxisCell
                            limit={metric.limit}
                            maxValue={metric.maxValue}
                            numberFormat={metricFormatter.noDecimals}
                        />
                        {!hasMoreRows && (
                            <CustomMessageCell icon={<Zap />}>
                                That&apos;s all.
                            </CustomMessageCell>
                        )}
                    </>
                ) : (
                    undefined
                ),
            header: (
                <HeaderSelectCell
                    className={metric.className}
                    label="metric"
                    onChange={metric.onChange}
                    onSortClick={metric.onSortClick}
                    options={metric.options}
                    optionsToDisable={[comparedMetric.selected]}
                    sortMode={metric.sortMode}
                    value={metric.selected}
                    valuePrefix={<DotPrefix color={METRIC_COLUMN_COLOR} />}
                />
            ),
            identifier: metric.selected
        },
        {
            // eslint-disable-next-line react/display-name
            CustomCell: ({ value }: { value: number }): ReactNode => (
                <BarCell
                    color={COMPARED_METRIC_COLUMN_COLOR}
                    limit={comparedMetric.limit}
                    maxValue={comparedMetric.maxValue}
                    numberFormat={comparedMetricFormatter.withDecimals}
                    value={value}
                />
            ),
            Footer:
                !hasErrors && !isEmpty ? (
                    <AxisCell
                        limit={comparedMetric.limit}
                        maxValue={comparedMetric.maxValue}
                        numberFormat={comparedMetricFormatter.noDecimals}
                    />
                ) : (
                    undefined
                ),
            header: (
                <HeaderSelectCell
                    label="compared to"
                    onChange={comparedMetric.onChange}
                    options={comparedMetric.options}
                    optionsToDisable={[metric.selected]}
                    value={comparedMetric.selected}
                    valuePrefix={
                        <DotPrefix color={COMPARED_METRIC_COLUMN_COLOR} />
                    }
                />
            ),
            identifier: comparedMetric.selected
        }
    ];
}

enum SimulateActions {
    Empty = 'empty',
    Error = 'error',
    Loading = 'loading',
    None = 'none'
}

const ChartTableExample: FC = (): ReactElement<HTMLElement> => {
    const [metric, setMetric] = useState<OptionType>(METRIC_OPTIONS[0]);
    const [comparedMetric, setComparedMetric] = useState<OptionType>(
        METRIC_OPTIONS[1]
    );
    const [sortMode, setSortMode] = useState<HeaderSelectCellSortMode>(
        HeaderSelectCellSortMode.Desc
    );
    const [offset, setOffset] = useState<number>(0);
    const [limit, setLimit] = useState<number>(PAGE_SIZES[0]);
    const [simulation, setSimulation] = useState<SimulateActions>(
        SimulateActions.None
    );

    const sortedRows = useMemo(
        (): ChartTableRow[] =>
            [...ROWS].sort(
                (row1, row2): number =>
                    ((row1[metric.value as string] as number) -
                        (row2[metric.value as string] as number)) *
                    (sortMode === HeaderSelectCellSortMode.Desc ? -1 : 1)
            ),
        [metric, sortMode]
    );
    const isEmpty = useMemo(
        (): boolean => simulation === SimulateActions.Empty,
        [simulation]
    );
    const isLoading = useMemo(
        (): boolean => simulation === SimulateActions.Loading,
        [simulation]
    );
    const hasErrors = useMemo(
        (): boolean => simulation === SimulateActions.Error,
        [simulation]
    );
    const pageRows = useMemo(
        (): ChartTableRow[] =>
            isEmpty || hasErrors
                ? []
                : sortedRows.slice(offset, offset + limit),
        [hasErrors, isEmpty, limit, offset, sortedRows]
    );
    const hasMoreRows = useMemo(
        (): boolean => sortedRows.slice(offset + limit).length > 0,
        [limit, offset, sortedRows]
    );

    function onSortClick(): void {
        setSortMode(
            (current: HeaderSelectCellSortMode): HeaderSelectCellSortMode =>
                current === HeaderSelectCellSortMode.Asc
                    ? HeaderSelectCellSortMode.Desc
                    : HeaderSelectCellSortMode.Asc
        );
    }

    const columns: IDataTableColumn[] = useMemo((): IDataTableColumn[] => {
        const maxValueMetric = getMaxColumnValue(ROWS, metric.value as string);
        const maxValueComparedMetric = getMaxColumnValue(
            ROWS,
            comparedMetric.value as string
        );

        return buildColumns(
            // metric
            {
                maxValue: maxValueMetric,
                onChange: setMetric,
                onSortClick,
                options: METRIC_OPTIONS,
                selected: metric.value as string,
                sortMode
            },
            // compared metric
            {
                maxValue: maxValueComparedMetric,
                onChange: setComparedMetric,
                options: METRIC_OPTIONS,
                selected: comparedMetric.value as string
            },
            hasMoreRows,
            isEmpty,
            hasErrors
        );
    }, [
        comparedMetric.value,
        hasErrors,
        hasMoreRows,
        isEmpty,
        metric.value,
        sortMode
    ]);

    return (
        <section className="mt-5">
            <h4>Chart Table</h4>

            <div className="mt-2 flex justify-end">
                <div className="va-form-label mt-1">Simulate action:</div>
                <RadioButtonGroup
                    isInline
                    name="simulate-actions"
                    onChange={(value): void =>
                        setSimulation(value as SimulateActions)
                    }
                    required={false}
                    selectedValue={simulation}
                >
                    {Object.entries(SimulateActions).map(
                        ([label, key]: [string, string]): ReactElement => (
                            <RadioButton
                                key={key}
                                label={label}
                                name="simulation"
                                value={key}
                            />
                        )
                    )}
                </RadioButtonGroup>
            </div>
            <div>
                <DataTable
                    className={cx('va-chart-table', {
                        'va-chart-table--empty': isEmpty,
                        'va-chart-table--error': hasErrors,
                        'va-chart-table--loading': isLoading
                    })}
                    columns={columns}
                    dataUI="va-chart-table"
                    fillHeight
                    height={300}
                    isLoading={isLoading}
                    limit={limit}
                    offset={offset}
                    onPageChange={setOffset}
                    onResultsPerPageChange={setLimit}
                    resultsPerPageOptions={PAGE_SIZES}
                    rows={pageRows}
                    showPagination={
                        !hasErrors &&
                        !isEmpty &&
                        sortedRows.length > PAGE_SIZES[0]
                    }
                    totalRows={sortedRows.length}
                />
            </div>
        </section>
    );
};

export default ChartTableExample;
