import React from 'react';

import { BarChart, PlotData, LegendData, PlotValue } from '@preamp/waveform';
import { Button } from '@preamp/core';

const basicData: PlotData[] = [
    {
        name: 'Group 1',
        data: [['Item 1.1', 152343]]
    },
    {
        name: 'Group 2',
        data: [
            ['Item 2.1', 155838],
            ['Item 2.2', 142747]
        ]
    },
    {
        name: 'Group 3',
        data: [
            ['Item 3.1', 120301],
            ['Item 3.2', 143255]
        ]
    },
    {
        name: 'Group 4',
        data: [['Item 4', 131998]]
    },
    {
        name: 'Group 5',
        data: [
            ['Item 5.1', 153558],
            ['Item 5.2', 123020],
            ['Item 5.3', 152321]
        ]
    }
];

const ToggableLegendFilterExample: React.FC = (): React.ReactElement => {
    const [data, setData] = React.useState<PlotData[]>(basicData);

    const filterByLegend = (
        currentSelection: LegendData[]
    ): LegendData[] | void => {
        const selection = currentSelection
            .filter((item: LegendData): boolean => item.selected)
            .map((item: LegendData): string => item.name as string);

        const filteredData = !selection.length
            ? basicData
            : basicData.map(
                  ({ name, data }: PlotData): PlotData => ({
                      name,
                      data: !selection.includes(name)
                          ? data.map(
                                ([name]: PlotValue): PlotValue => [name, 0]
                            )
                          : data
                  })
              );
        setData([...filteredData]);
    };

    return (
        <>
            <p className="mb-5">Filtering data when clicking on legend</p>
            <BarChart data={data} onLegendClick={filterByLegend} />
        </>
    );
};

const ToggableLegendKeepLegendSelectionExample: React.FC = (): React.ReactElement => {
    const [data, setData] = React.useState<PlotData[]>(basicData);
    const [legendData, setLegendData] = React.useState<LegendData[]>(null);
    const [loading, setLoading] = React.useState<boolean>(false);

    const filterByLegend = (
        currentSelection: LegendData[]
    ): LegendData[] | void => {
        const selection = currentSelection
            .filter((item: LegendData): boolean => item.selected)
            .map((item: LegendData): string => item.name as string);

        const filteredData = !selection.length
            ? basicData
            : basicData.map(
                  ({ name, data }: PlotData): PlotData => ({
                      name,
                      data: !selection.includes(name)
                          ? data.map(
                                ([name]: PlotValue): PlotValue => [name, 0]
                            )
                          : data
                  })
              );
        setData([...filteredData]);
        setLegendData(currentSelection);
    };

    const simulateServerCall = (): void => {
        setTimeout((): void => {
            setLoading(false);
            filterByLegend(legendData || basicData);
        }, 1000);
    };

    const refreshChart = (): void => {
        setLoading(true);
        simulateServerCall();
    };

    return (
        <>
            <p className="mb-5">
                Filtering data when clicking on legend and keeping legend
                selection after refreshing
            </p>
            <Button className="mb-5" onClick={refreshChart}>
                Refresh chart
            </Button>
            <BarChart
                data={data}
                isLoading={loading}
                legendData={legendData}
                onLegendClick={filterByLegend}
            />
        </>
    );
};

export const BarChartToggableLegendExample: React.FC = (): React.ReactElement => (
    <section className="mt-5">
        <h4>Legend Interaction</h4>
        <p className="mt-2 mb-5">
            Callback to perform an action after clicking on a legend. The
            legendData property could be used to set the initial selection
            state.
        </p>

        <h5 className="mt-5 mb-2">* Example 1:</h5>
        <ToggableLegendFilterExample />

        <h5 className="mt-5 mb-2">* Example 2:</h5>
        <ToggableLegendKeepLegendSelectionExample />
    </section>
);

export default BarChartToggableLegendExample;
