import React from 'react';

import { cloneDeep, get, max, min } from 'lodash';
import {
    BarChart,
    BARCHART_DEFAULT_DOMAIN,
    DataDomain,
    PlotData,
    PlotValue
} from '@preamp/waveform';
import { TextField, TextFieldType, Toggle } from '@preamp/core';

const basicData: PlotData[] = [
    {
        name: 'Added',
        data: [
            ['Adam Molano', 15],
            ['Amy Gaudioso', 20],
            ['Ben Behrens', 0],
            ['Bob Samuelson', 1],
            ['Daniel Raynes', -7], // bad value: it will be internally filtered out
            ['Doug Ung', 25],
            ['Helen Kim', -1], // bad value: it will be internally filtered out
            ['Juan Giuffrida', 20],
            ['Vera Carrillo-Marquez', -5], // bad value: it will be internally filtered out
            ['Matt Rodriguez', null]
        ]
    }
];

const DATA_RANGE: [number, number] = ((): [number, number] => {
    const values = basicData[0].data.map(
        ([, value]: PlotValue): number => value
    );
    return [min(values), max(values)];
})();

export const DataDomainsBarChartExample: React.FC = (): React.ReactElement => {
    const [isCustomDomain, setIsCustomDomain] = React.useState<boolean>();
    const [domain, setDomain] = React.useState<DataDomain>(
        cloneDeep(BARCHART_DEFAULT_DOMAIN)
    );

    function changeDomainProperty(prop: string, value: number | boolean): void {
        setDomain(
            (prevDomain: DataDomain): DataDomain => {
                const newDomain = cloneDeep(prevDomain);
                newDomain.y[0][prop] = value;
                return newDomain;
            }
        );
    }

    function onChange(
        prop: string
    ): (_: React.ChangeEvent<HTMLInputElement>) => void {
        return (e: React.ChangeEvent<HTMLInputElement>): void => {
            changeDomainProperty(prop, Number(e.target.value));
        };
    }

    function onChangeNulls(value: boolean): void {
        changeDomainProperty('allowsNulls', value);
    }

    function onToggleCustomDomain(value: boolean): void {
        setDomain(cloneDeep(BARCHART_DEFAULT_DOMAIN));
        setIsCustomDomain(value);
    }

    return (
        <section className="mt-5">
            <h4>Data Domains</h4>
            <p className="mt-2 mb-5">
                Values out of range or not allowed by the chart definition will
                be filtered out from data.
            </p>

            <Toggle
                checked={isCustomDomain}
                label="Set a custom domain"
                name="custom-domain"
                onChange={onToggleCustomDomain}
            />

            <div className="flex items-end mb-4">
                <div className="flex-1 text-right pr-4 pb-3">Y:</div>
                <div className="flex-1 pr-4">
                    <TextField
                        disabled={!isCustomDomain}
                        label="min"
                        onChange={onChange('min')}
                        textFieldType={TextFieldType.Number}
                        value={`${get(domain, ['y', 0, 'min'], DATA_RANGE[0])}`}
                    />
                </div>
                <div className="flex-1 pr-4">
                    <TextField
                        disabled={!isCustomDomain}
                        label="max"
                        onChange={onChange('max')}
                        textFieldType={TextFieldType.Number}
                        value={`${get(domain, ['y', 0, 'max'], DATA_RANGE[1])}`}
                    />
                </div>
                <div className="flex-1">
                    <Toggle
                        checked={domain.y[0].allowsNulls}
                        disabled={!isCustomDomain}
                        label="Allows nulls"
                        name="allows-nulls"
                        onChange={onChangeNulls}
                    />
                </div>
            </div>

            <BarChart data={basicData} dataDomain={domain} />
        </section>
    );
};

export default DataDomainsBarChartExample;
