import React from 'react';

import {
    BarLineChart,
    PlotData,
    LegendData,
    PlotValue
} from '@preamp/waveform';
import { Button } from '@preamp/core';

const preStyles = {
    height: 320,
    overflow: 'auto',
    fontSize: '.7rem'
};

const basicData: PlotData[] = [
    {
        name: 'Added',
        data: [
            ['Adam Molano', [152343, 102]],
            ['Amy Gaudioso', [155838, 120]],
            ['Ben Behrens', [142747, 245]],
            ['Bob Samuelson', [120301, 100]],
            ['Daniel Raynes', [143255, 77]],
            ['Doug Ung', [131998, 89]],
            ['Helen Kim', [153558, 300]],
            ['Juan Giuffrida', [123020, 59]],
            ['Vera Carrillo-Marquez', [152321, 93]],
            ['Matt Rodriguez', [102321, 63]]
        ]
    }
];

const ToggableLegendSelectItemExample: React.FC = (): React.ReactElement => {
    const [selectedItem, setSelectedItem] = React.useState(null);

    const listSelection = (
        currentSelection: LegendData[],
        clicked: number,
        event: React.MouseEvent<HTMLLIElement>
    ): LegendData[] | void => {
        event && event.preventDefault();

        const clickedLegend = currentSelection[clicked];
        if (clickedLegend) {
            setSelectedItem(
                clickedLegend.selected
                    ? {
                          [clickedLegend.name as string]: (clickedLegend as PlotData).data.map(
                              ([key, value]: PlotValue): string =>
                                  `${key}: ${value}`
                          )
                      }
                    : null
            );
        }

        const newSelectionState = currentSelection.map(
            ({ selected, ...rest }: LegendData, index: number): LegendData => ({
                ...rest,
                selected: index === clicked && selected
            })
        );

        return newSelectionState;
    };

    return (
        <>
            <p className="mb-5">
                Listing selected data when clicking on legend
            </p>
            <div className="va-grid va-columns-default">
                <div className="col-span-sm-12 col-span-md-8">
                    <BarLineChart
                        autoWidth
                        data={basicData}
                        onLegendClick={listSelection}
                        xAxisTitle="Team members"
                        y2AxisTitle="Commits"
                        yAxisTitle="Lines of code"
                    />
                </div>
                <div className="col-span-sm-12 col-span-md-4">
                    {selectedItem && (
                        <pre style={preStyles}>
                            {JSON.stringify(selectedItem, null, '  ')}
                        </pre>
                    )}
                </div>
            </div>
        </>
    );
};

const ToggableLegendKeepLegendSelectionExample: React.FC = (): React.ReactElement => {
    const [selectedItem, setSelectedItem] = React.useState(null);
    const [legendData, setLegendData] = React.useState<LegendData[]>(null);
    const [loading, setLoading] = React.useState<boolean>(false);

    const listSelection = (
        currentSelection: LegendData[],
        clicked: number,
        event?: React.MouseEvent<HTMLLIElement>
    ): LegendData[] | void => {
        event && event.preventDefault();

        const clickedLegend = currentSelection
            ? currentSelection[clicked]
            : null;
        if (clickedLegend) {
            setSelectedItem(
                clickedLegend.selected
                    ? {
                          [clickedLegend.name as string]: (clickedLegend as PlotData).data.map(
                              ([key, value]: PlotValue): string =>
                                  `${key}: ${value}`
                          )
                      }
                    : null
            );
        }

        const newSelectionState = currentSelection
            ? currentSelection.map(
                  (
                      { selected, ...rest }: LegendData,
                      index: number
                  ): LegendData => ({
                      ...rest,
                      selected: index === clicked && selected
                  })
              )
            : null;

        setLegendData(newSelectionState);

        return newSelectionState;
    };

    const simulateServerCall = (): void => {
        setTimeout((): void => {
            setLoading(false);
            const selectedIndex = (legendData || basicData).findIndex(
                (l: LegendData): boolean => l.selected
            );
            listSelection(legendData, selectedIndex);
        }, 1000);
    };

    const refreshChart = (): void => {
        setLoading(true);
        simulateServerCall();
    };

    return (
        <>
            <p className="mb-5">
                Listing selected data when clicking on legend and keeping legend
                selection after refreshing
            </p>

            <Button className="mb-5" onClick={refreshChart}>
                Refresh chart
            </Button>

            <div className="va-grid va-columns-default">
                <div className="col-span-sm-12 col-span-md-8">
                    <BarLineChart
                        autoWidth
                        data={basicData}
                        isLoading={loading}
                        legendData={legendData}
                        onLegendClick={listSelection}
                        xAxisTitle="Team members"
                        y2AxisTitle="Commits"
                        yAxisTitle="Lines of code"
                    />
                </div>
                <div className="col-span-sm-12 col-span-md-4">
                    {selectedItem && !loading && (
                        <pre style={preStyles}>
                            {JSON.stringify(selectedItem, null, '  ')}
                        </pre>
                    )}
                </div>
            </div>
        </>
    );
};

export const BarLineChartToggableLegendExample: React.FC = (): React.ReactElement => (
    <section className="mt-5">
        <h4>Legend Interaction</h4>
        <p className="mt-2">
            Callback to perform an action after clicking on a legend.
        </p>

        <h5 className="mt-5 mb-2">* Example 1:</h5>
        <ToggableLegendSelectItemExample />

        <h5 className="mt-5 mb-2">* Example 2:</h5>
        <ToggableLegendKeepLegendSelectionExample />
    </section>
);

export default BarLineChartToggableLegendExample;
