import React from 'react';

import { cloneDeep, get, min, max } from 'lodash';
import {
    BarLineChart,
    BARLINECHART_DEFAULT_DOMAIN,
    PlotData,
    DataDomain,
    PlotValue
} from '@preamp/waveform';
import { TextField, TextFieldType, Toggle } from '@preamp/core';

const basicData: PlotData[] = [
    {
        name: 'Added',
        data: [
            // [key, [BAR value, LINE value]]
            ['Adam Molano', [15, 1]],
            ['Amy Gaudioso', [20, 5]],
            ['Ben Behrens', [0, 10]],
            ['Bob Samuelson', [1, 9]],
            ['Daniel Raynes', [-10, 8]], // bad value: it will be internally filtered out
            ['Doug Ung', [25, -1]], // bad value: it will be internally filtered out
            ['Helen Kim', [-1, 2]], // bad value: it will be internally filtered out
            ['Juan Giuffrida', [20, null]], // bad value: it will be internally filtered out
            ['Vera Carrillo-Marquez', [-25, 4]], // bad value: it will be internally filtered out
            ['Matt Rodriguez', [null, 4]]
        ]
    }
];

const BARS = 0;
const LINES = 1;

const DATA_RANGES: [number, number][] = [BARS, LINES].map((dataIndex: number): [
    number,
    number
] => {
    const values = basicData[0].data.map(
        ([, value]: PlotValue): number => value[dataIndex]
    );
    return [min(values), max(values)];
});

export const DataDomainsBarChartExample: React.FC = (): React.ReactElement => {
    const [isCustomDomain, setIsCustomDomain] = React.useState<boolean>();
    const [domain, setDomain] = React.useState<DataDomain>(
        cloneDeep(BARLINECHART_DEFAULT_DOMAIN)
    );

    function changeDomainProperty(
        dataIndex: number,
        prop: string,
        value: number | boolean
    ): void {
        setDomain(
            (prevDomain: DataDomain): DataDomain => {
                const newDomain = cloneDeep(prevDomain);
                newDomain.y[dataIndex][prop] = value;
                return newDomain;
            }
        );
    }

    function onChange(
        dataIndex: number, // 0: bars, 1: lines
        prop: string
    ): (_: React.ChangeEvent<HTMLInputElement>) => void {
        return (e: React.ChangeEvent<HTMLInputElement>): void => {
            changeDomainProperty(dataIndex, prop, Number(e.target.value));
        };
    }

    function onChangeNulls(value: boolean): void {
        changeDomainProperty(BARS, 'allowsNulls', value);
    }

    function onToggleCustomDomain(value: boolean): void {
        setDomain(cloneDeep(BARLINECHART_DEFAULT_DOMAIN));
        setIsCustomDomain(value);
    }

    return (
        <section className="mt-5">
            <h4>Plot Data Domains</h4>
            <p className="mt-2 mb-5">
                Values out of range or not allowed by the chart definition will
                be filtered out from data.
            </p>

            <Toggle
                checked={isCustomDomain}
                label="Set a custom domain"
                name="custom-domain"
                onChange={onToggleCustomDomain}
            />

            <div className="flex items-end mb-4">
                <div className="flex-1 text-right pr-4 pb-3">Bars:</div>
                <div className="flex-1 pr-4">
                    <TextField
                        disabled={!isCustomDomain}
                        label="min"
                        onChange={onChange(BARS, 'min')}
                        textFieldType={TextFieldType.Number}
                        value={`${get(
                            domain,
                            ['y', BARS, 'min'],
                            DATA_RANGES[BARS][0]
                        )}`}
                    />
                </div>
                <div className="flex-1 pr-4">
                    <TextField
                        disabled={!isCustomDomain}
                        label="max"
                        onChange={onChange(BARS, 'max')}
                        textFieldType={TextFieldType.Number}
                        value={`${get(
                            domain,
                            ['y', BARS, 'max'],
                            DATA_RANGES[BARS][1]
                        )}`}
                    />
                </div>
                <div className="flex-1">
                    <Toggle
                        checked={domain.y[BARS].allowsNulls}
                        disabled={!isCustomDomain}
                        label="Allows nulls"
                        name="allows-nulls"
                        onChange={onChangeNulls}
                    />
                </div>
            </div>

            <div className="flex items-end mb-4">
                <div className="flex-1 text-right pr-4 pb-3">Lines:</div>
                <div className="flex-1 pr-4">
                    <TextField
                        disabled={!isCustomDomain}
                        label="min"
                        onChange={onChange(LINES, 'min')}
                        textFieldType={TextFieldType.Number}
                        value={`${get(
                            domain,
                            ['y', LINES, 'min'],
                            DATA_RANGES[LINES][0]
                        )}`}
                    />
                </div>
                <div className="flex-1 pr-4">
                    <TextField
                        disabled={!isCustomDomain}
                        label="max"
                        onChange={onChange(LINES, 'max')}
                        textFieldType={TextFieldType.Number}
                        value={`${get(
                            domain,
                            ['y', LINES, 'max'],
                            DATA_RANGES[LINES][1]
                        )}`}
                    />
                </div>
                <div className="flex-1 u-body-2 mb-3">Nulls not allowed</div>
            </div>

            <BarLineChart data={basicData} dataDomain={domain} />
        </section>
    );
};

export default DataDomainsBarChartExample;
