import React, { useState, useMemo, useCallback } from 'react';

import { Search } from '@preamp/core';
import { Filter, SelectList, OptionType, ValueType } from '@preamp/select';
import { TableActionBar } from '@preamp/tables';
import { BubbleChart, NumericValue } from '@preamp/waveform';
import { smallBubbleData } from './data/smallBubble';

type Datum = NumericValue;
type Data = Datum[];

interface Dataset {
    name: string;
    data: Data;
}

interface FilteredChartState {
    dataset: Dataset[];
    isLoading: boolean;
    selectedGroup: string;
    selectedFilters: Record<string, OptionType[]>;
    stagedFilters: OptionType[];
}

const defaultState: FilteredChartState = {
    dataset: [].concat(smallBubbleData[0], smallBubbleData[2]),
    isLoading: false,
    selectedGroup: null,
    selectedFilters: {},
    stagedFilters: []
};

const buildFilterGroups = (dataset: Dataset[]): OptionType[] => {
    return dataset.map(
        (data): OptionType => ({
            label: data.name,
            value: data.name
        })
    );
};

const datumToOption = (group: string, datum: Datum): OptionType => ({
    group: { label: group, value: group },
    label: datum[2].key,
    value: datum[2].key
});

const filterGroupData = (data: Data, filters: OptionType[]): Data => {
    const filteredKeys: string[] = filters.map(
        (filter: OptionType): string => filter.value as string
    );
    return data.reduce((acc: Data, datum: Datum): Data => {
        if (filteredKeys.includes(datum[2].key)) {
            acc.push(datum);
        }
        return acc;
    }, []);
};

/**
 * A BubbleChart show-casing filtering
 */
export const FilteredBubbleChart: React.FC = (): React.ReactElement => {
    const [state, setState] = useState<FilteredChartState>(defaultState);

    const filterGroups = useMemo((): OptionType[] => {
        return buildFilterGroups(state.dataset);
    }, [state.dataset]);

    const groupOptions = useMemo((): OptionType[] => {
        switch (state.selectedGroup) {
            case 'Network':
                return state.dataset[0].data.map(
                    (datum: Datum): OptionType =>
                        datumToOption('Network', datum)
                );
            case 'Daypart':
                return state.dataset[1].data.map(
                    (datum: Datum): OptionType =>
                        datumToOption('Daypart', datum)
                );
            default:
                return [];
        }
    }, [state.dataset, state.selectedGroup]);

    const flattenedFilters = useMemo((): OptionType[] => {
        return Object.values(state.selectedFilters).reduce(
            (acc: OptionType[], filters: OptionType[]): OptionType[] =>
                acc.concat(filters),
            []
        );
    }, [state.selectedFilters]);

    const filteredDataset = useMemo((): Dataset[] => {
        return state.dataset.reduce(
            (acc: Dataset[], dataset: Dataset): Dataset[] => {
                const groupFilters = state.selectedFilters[dataset.name] || [];
                if (groupFilters.length > 0) {
                    acc.push({
                        name: dataset.name,
                        data: filterGroupData(dataset.data, groupFilters)
                    });
                } else {
                    acc.push(dataset);
                }
                return acc;
            },
            []
        );
    }, [state.selectedFilters, state.dataset]);

    const onSelectChange = useCallback(
        (stagedFilters: ValueType<OptionType>): void => {
            setState(
                (prevState: FilteredChartState): FilteredChartState => ({
                    ...prevState,
                    stagedFilters
                })
            );
        },
        [setState]
    );

    const onFilterGroupSelect = useCallback(
        (option: OptionType): void => {
            setState(
                (prevState: FilteredChartState): FilteredChartState => ({
                    ...prevState,
                    selectedGroup: option.value as string
                })
            );
        },
        [setState]
    );

    const clearSelectedGroup = useCallback((): void => {
        setState(
            (prevState: FilteredChartState): FilteredChartState => ({
                ...prevState,
                selectedGroup: null
            })
        );
    }, [setState]);

    const onApply = useCallback(
        (group: OptionType): void => {
            setState(
                (prevState: FilteredChartState): FilteredChartState => {
                    const { selectedFilters, stagedFilters } = prevState;
                    return {
                        ...prevState,
                        selectedFilters: {
                            ...selectedFilters,
                            [group.value as string]: stagedFilters
                        },
                        selectedGroup: null,
                        stagedFilters: []
                    };
                }
            );
        },
        [setState]
    );

    const onApplySelectedOptions = useCallback(
        (group: OptionType, values: OptionType[]): void => {
            setState(
                (prevState: FilteredChartState): FilteredChartState => {
                    const { selectedFilters } = prevState;
                    return {
                        ...prevState,
                        selectedFilters: {
                            ...selectedFilters,
                            [group.value as string]: values
                        }
                    };
                }
            );
        },
        [setState]
    );

    const onRemoveGroup = useCallback(
        (group: OptionType): void => {
            setState(
                (prevState: FilteredChartState): FilteredChartState => {
                    const { selectedFilters } = prevState;
                    return {
                        ...prevState,
                        selectedFilters: {
                            ...selectedFilters,
                            [group.value as string]: []
                        }
                    };
                }
            );
        },
        [setState]
    );

    const groupSelectList = useCallback((): React.ReactElement => {
        return (
            <SelectList
                defaultValue={state.stagedFilters}
                focusOnMount={true}
                isLoading={false}
                isMulti={true}
                isSearchable={false}
                onChange={onSelectChange}
                options={groupOptions}
            />
        );
    }, [onSelectChange, groupOptions, state.stagedFilters]);

    function setQuery(value: string): void {
        console.log('Search query', value);
    }

    return (
        <section className="mt-5">
            <h3>Simulate Filtering Data</h3>
            <section className="mb-6"></section>
            <h4>Bubble Chart with Filtering</h4>
            <p className="mt-2 mb-3">
                Selecting different filters will re-render the bubble chart
            </p>
            <TableActionBar>
                <Search defaultValue={''} onClear={null} onSearch={setQuery} />
                <Filter
                    groupOptions={filterGroups}
                    isLoading={state.isLoading}
                    onApply={onApply}
                    onApplySelectedOptions={onApplySelectedOptions}
                    onBack={clearSelectedGroup}
                    onCancel={clearSelectedGroup}
                    onChangeGroup={onFilterGroupSelect}
                    onRemoveGroup={onRemoveGroup}
                    selectedValues={flattenedFilters}
                    subComponent={groupSelectList}
                />
            </TableActionBar>
            <BubbleChart
                autoWidth
                colors={[
                    'var(--waveform-default-500)',
                    'var(--waveform-violet-500)',
                    'var(--waveform-magenta-500)',
                    'var(--waveform-success-500)',
                    'var(--waveform-sky-500)'
                ]}
                data={filteredDataset}
                dataUI="data-change_bubble-chart"
                isLoading={state.isLoading}
                xAxisTitle="Comparison Index"
                yAxisTitle="Base Index"
            />
        </section>
    );
};

export default FilteredBubbleChart;
