import React from 'react';

import { BubbleChart, LegendData, PlotData } from '@preamp/waveform';
import { smallBubbleData } from './data/smallBubble';
import { Button } from '@preamp/core';

const bubbleData1 = smallBubbleData.slice(0, 2);
const bubbleData2 = smallBubbleData.slice(2, 4);

const preStyles = {
    height: 320,
    overflow: 'auto',
    fontSize: '.7rem'
};

const ToggableLegendBlurBubblesExample: React.FC = (): React.ReactElement => {
    const [filteredData, setFilteredData] = React.useState<PlotData[]>([
        ...bubbleData1
    ]);

    const blurBubbles = (
        currentSelection: LegendData[],
        _clicked: number,
        event?: React.MouseEvent<HTMLLIElement>
    ): LegendData[] | void => {
        event && event.preventDefault();

        const selection = currentSelection
            .filter((item: LegendData): boolean => item.selected)
            .map((item: LegendData): string => item.name as string);

        const mappedData = !selection.length
            ? bubbleData1
            : bubbleData1.map(
                  (datum: PlotData): PlotData => ({
                      ...datum,
                      color: !selection.includes(datum.name)
                          ? 'var(--va-grey-20)'
                          : datum.color
                  })
              );
        setFilteredData([...mappedData]);
    };

    return (
        <>
            <p className="mb-5">Blurring bubbles when clicking on legend</p>
            <BubbleChart
                autoWidth
                data={filteredData}
                dataUI="bubble-chart-1"
                onLegendClick={blurBubbles}
                xAxisTitle="Comparison Index"
                xTickAlignToData={false}
                yAxisTitle="Base Index"
            />
        </>
    );
};

const ToggableLegendKeepLegendSelectionExample: React.FC = (): React.ReactElement => {
    const [filteredData, setFilteredData] = React.useState<PlotData[]>(
        bubbleData1
    );
    const [legendData, setLegendData] = React.useState<LegendData[]>(null);
    const [loading, setLoading] = React.useState<boolean>(false);

    const blurBubbles = (
        currentSelection: LegendData[],
        _clicked?: number,
        event?: React.MouseEvent<HTMLLIElement>
    ): LegendData[] | void => {
        event && event.preventDefault();

        const selection = currentSelection
            .filter((item: LegendData): boolean => item.selected)
            .map((item: LegendData): string => item.name as string);

        const mappedData = !selection.length
            ? bubbleData1
            : bubbleData1.map(
                  (datum: PlotData): PlotData => ({
                      ...datum,
                      color: !selection.includes(datum.name)
                          ? 'var(--va-grey-20)'
                          : datum.color
                  })
              );
        setFilteredData([...mappedData]);
        setLegendData(currentSelection);
    };

    const simulateServerCall = (): void => {
        setTimeout((): void => {
            setLoading(false);
            blurBubbles(legendData || bubbleData1);
        }, 1000);
    };

    const refreshChart = (): void => {
        setLoading(true);
        simulateServerCall();
    };

    return (
        <>
            <p className="mt-2 mb-5">
                Blurring bubbles when clicking on legend and keeping legend
                selection after refreshing
            </p>
            <Button onClick={refreshChart}>Refresh chart</Button>
            <BubbleChart
                autoWidth
                data={filteredData}
                dataUI="bubble-chart-1"
                isLoading={loading}
                legendData={legendData}
                onLegendClick={blurBubbles}
                xAxisTitle="Comparison Index"
                xTickAlignToData={false}
                yAxisTitle="Base Index"
            />
        </>
    );
};

const ToggableLegendListSelectionExample: React.FC = (): React.ReactElement => {
    const [selectedItem, setSelectedItem] = React.useState<PlotData>(null);

    const listSelection = (
        currentSelection: LegendData[],
        clicked: number,
        event: React.MouseEvent<HTMLLIElement>
    ): LegendData[] | void => {
        event && event.preventDefault();

        const newSelectionState = currentSelection.map(
            (item: LegendData, index: number): LegendData => ({
                ...item,
                selected: index === clicked ? item.selected : false
            })
        );

        setSelectedItem(
            newSelectionState[clicked].selected
                ? (newSelectionState[clicked] as PlotData)
                : null
        );

        return newSelectionState;
    };

    return (
        <>
            <p className="mb-5">
                Listing selected data when clicking on legend
            </p>
            <div className="va-grid va-columns-default">
                <div className="col-span-sm-12 col-span-md-8">
                    <BubbleChart
                        autoWidth
                        circleRange={[10, 20]}
                        data={bubbleData2}
                        dataUI="bubble-chart-3"
                        onLegendClick={listSelection}
                        xAxisTitle="Comparison Index"
                        xTickAlignToData={false}
                        yAxisTitle="Base Index"
                    />
                </div>
                <div className="col-span-sm-12 col-span-md-4">
                    {selectedItem && (
                        <pre style={preStyles}>
                            {JSON.stringify(selectedItem, null, '  ')}
                        </pre>
                    )}
                </div>
            </div>
        </>
    );
};

export const BubbleChartClickableLegendsExample: React.FC = (): React.ReactElement => (
    <section className="mt-5">
        <h4>Legend Interaction</h4>
        <p className="mt-2">
            Callback to perform an action after clicking on a legend. The
            legendData property could be used to set the initial selection
            state.
        </p>

        <h5 className="mt-5 mb-2">* Example 1:</h5>
        <ToggableLegendBlurBubblesExample />

        <h5 className="mt-5 mb-2">* Example 2:</h5>
        <ToggableLegendKeepLegendSelectionExample />

        <h5 className="mt-5 mb-2">* Example 3:</h5>
        <ToggableLegendListSelectionExample />
    </section>
);

export default BubbleChartClickableLegendsExample;
