import React from 'react';

import { PropertyName, cloneDeep, get, set } from 'lodash';
import {
    BubbleChart,
    BUBBLECHART_DEFAULT_DOMAIN,
    PlotData,
    DataDomain
} from '@preamp/waveform';
import { TextField, TextFieldType, Toggle } from '@preamp/core';

export const smallBubbleData: PlotData[] = [
    {
        name: 'Network',
        data: [
            [
                1,
                5,
                {
                    radiusSize: 1,
                    key: 'NFL_Network',
                    audienceName: 'Audience A',
                    value: 'NFL Network'
                }
            ],
            [
                3,
                5,
                {
                    radiusSize: 2,
                    key: 'Cooking_Channel',
                    audienceName: 'Audience A',
                    value: 'Cooking Channel'
                }
            ],
            [
                4,
                null, // bad value: it will be internally filtered out,
                {
                    radiusSize: 2,
                    key: 'Cooking_Channel',
                    audienceName: 'Audience A',
                    value: 'Cooking Channel'
                }
            ],
            [
                4,
                4,
                {
                    radiusSize: 0, // bad value: it will be internally filtered out
                    key: 'Cooking_Channel',
                    audienceName: 'Audience A',
                    value: 'Cooking Channel'
                }
            ]
        ]
    },
    {
        name: 'Genre',
        data: [
            [
                7,
                8,
                {
                    radiusSize: 5,
                    key: 'Sports_Event',
                    audienceName: 'Audience A',
                    value: 'Sports Event'
                }
            ],
            [
                9,
                6,
                {
                    radiusSize: 10,
                    key: 'Music',
                    audienceName: 'Audience A',
                    value: 'Music'
                }
            ],
            [
                -1, // bad value: it will be internally filtered out
                1,
                {
                    radiusSize: 10,
                    key: 'Music B',
                    audienceName: 'Audience A',
                    value: 'Music B'
                }
            ]
        ]
    }
];

const CONFIG_PROP: { [_: string]: PropertyName[] } = {
    x: ['x'],
    y: ['y', 0],
    radiusSize: ['meta', 'radiusSize']
};

export const DataDomainsBubbleChartExample: React.FC = (): React.ReactElement => {
    const [isCustomDomain, setIsCustomDomain] = React.useState<boolean>();
    const [domain, setDomain] = React.useState<DataDomain>(
        cloneDeep(BUBBLECHART_DEFAULT_DOMAIN)
    );

    function changeDomainProperty(
        config: 'x' | 'y' | 'radiusSize',
        prop: string,
        value: number
    ): void {
        setDomain(
            (prevDomain: DataDomain): DataDomain => {
                const newDomain = cloneDeep(prevDomain);
                set(newDomain, [...CONFIG_PROP[config], prop], value);
                return newDomain;
            }
        );
    }

    function onChange(
        config: 'x' | 'y' | 'radiusSize',
        prop: string
    ): (_: React.ChangeEvent<HTMLInputElement>) => void {
        return (e: React.ChangeEvent<HTMLInputElement>): void => {
            changeDomainProperty(config, prop, Number(e.target.value));
        };
    }

    function onToggleCustomDomain(value: boolean): void {
        setDomain(cloneDeep(BUBBLECHART_DEFAULT_DOMAIN));
        setIsCustomDomain(value);
    }

    return (
        <section className="mt-5">
            <h4>Data Domains / Data Scales</h4>
            <p className="mt-2 mb-5">
                Domains will be mapped to chart scales and those values out of
                range or not allowed by the chart definition will be filtered
                out from data.
            </p>

            <Toggle
                checked={isCustomDomain}
                label="Set custom domains/scales"
                name="custom-domain"
                onChange={onToggleCustomDomain}
            />

            <div className="flex items-end mb-4">
                <div className="flex-1 text-right pr-4 pb-3">X:</div>
                <div className="flex-1 pr-4">
                    <TextField
                        disabled={!isCustomDomain}
                        label="min"
                        onChange={onChange('x', 'min')}
                        textFieldType={TextFieldType.Number}
                        value={`${get(domain, ['x', 'min'], -1)}`}
                    />
                </div>
                <div className="flex-1 pr-4">
                    <TextField
                        disabled={!isCustomDomain}
                        label="max"
                        onChange={onChange('x', 'max')}
                        textFieldType={TextFieldType.Number}
                        value={`${get(domain, ['x', 'max'], 9)}`}
                    />
                </div>
                <div className="flex-1 u-body-2 mb-3">Nulls not allowed</div>
            </div>

            <div className="flex items-end mb-4">
                <div className="flex-1 text-right pr-4 pb-3">Y:</div>
                <div className="flex-1 pr-4">
                    <TextField
                        disabled={!isCustomDomain}
                        label="min"
                        onChange={onChange('y', 'min')}
                        textFieldType={TextFieldType.Number}
                        value={`${get(domain, ['y', 0, 'min'], 1)}`}
                    />
                </div>
                <div className="flex-1 pr-4">
                    <TextField
                        disabled={!isCustomDomain}
                        label="max"
                        onChange={onChange('y', 'max')}
                        textFieldType={TextFieldType.Number}
                        value={`${get(domain, ['y', 0, 'max'], 8)}`}
                    />
                </div>
                <div className="flex-1 u-body-2 mb-3">Nulls not allowed</div>
            </div>

            <div className="flex items-end mb-4">
                <div className="flex-1 text-right pr-4 pb-3">Radius Size:</div>
                <div className="flex-1 pr-4">
                    <TextField
                        disabled={!isCustomDomain}
                        label="min"
                        onChange={onChange('radiusSize', 'min')}
                        textFieldType={TextFieldType.Number}
                        value={`${get(
                            domain,
                            ['meta', 'radiusSize', 'min'],
                            0
                        )}`}
                    />
                </div>
                <div className="flex-1 pr-4">
                    <TextField
                        disabled={!isCustomDomain}
                        label="max"
                        onChange={onChange('radiusSize', 'max')}
                        textFieldType={TextFieldType.Number}
                        value={`${get(
                            domain,
                            ['meta', 'radiusSize', 'max'],
                            10
                        )}`}
                    />
                </div>
                <div className="flex-1 u-body-2 mb-3">Nulls not allowed</div>
            </div>

            <BubbleChart
                autoWidth
                data={smallBubbleData}
                dataDomain={domain}
                dataUI="bubble-chart-1"
                xAxisTitle="Comparison Index"
                xTickAlignToData={false}
                yAxisTitle="Base Index"
            />
        </section>
    );
};

export default DataDomainsBubbleChartExample;
