import React from 'react';

import { LineChart, PlotData, LegendData, PlotValue } from '@preamp/waveform';

import { smallData } from './data/small';
import { Button } from '@preamp/core';

const ToggableLegendFilterExample: React.FC = (): React.ReactElement => {
    const [data, setData] = React.useState<PlotData[]>(smallData);

    const filterByLegend = (
        currentSelection: LegendData[],
        _clicked: number,
        event: React.MouseEvent<HTMLLIElement>
    ): void => {
        event && event.preventDefault();

        const selection = currentSelection
            .filter((item: LegendData): boolean => item.selected)
            .map((item: LegendData): string => item.name as string);

        const filteredData = !selection.length
            ? smallData
            : smallData.map(
                  ({ name, data }: PlotData): PlotData => ({
                      name,
                      data: !selection.includes(name)
                          ? data.map(
                                ([name]: PlotValue): PlotValue => [name, null]
                            )
                          : data
                  })
              );
        setData([...filteredData]);
    };

    return (
        <>
            <p className="mb-5">Filtering data when clicking on legend</p>
            <LineChart data={data} onLegendClick={filterByLegend} />
        </>
    );
};

const ToggableLegendKeepLegendSelectionExample: React.FC = (): React.ReactElement => {
    const [data, setData] = React.useState<PlotData[]>(smallData);
    const [legendData, setLegendData] = React.useState<LegendData[]>(null);
    const [loading, setLoading] = React.useState<boolean>(false);

    const filterByLegend = (
        currentSelection: LegendData[],
        _clicked?: number,
        event?: React.MouseEvent<HTMLLIElement>
    ): void => {
        event && event.preventDefault();

        const selection = currentSelection
            .filter((item: LegendData): boolean => item.selected)
            .map((item: LegendData): string => item.name as string);

        const filteredData = !selection.length
            ? smallData
            : smallData.map(
                  ({ name, data }: PlotData): PlotData => ({
                      name,
                      data: !selection.includes(name)
                          ? data.map(
                                ([name]: PlotValue): PlotValue => [name, null]
                            )
                          : data
                  })
              );
        setData([...filteredData]);
        setLegendData(currentSelection);
    };

    const simulateServerCall = (): void => {
        setTimeout((): void => {
            setLoading(false);
            filterByLegend(legendData || smallData);
        }, 1000);
    };

    const refreshChart = (): void => {
        setLoading(true);
        simulateServerCall();
    };

    return (
        <>
            <p className="mb-5">
                Filtering data when clicking on legend and keeping legend
                selection after refreshing
            </p>
            <Button className="mb-5" onClick={refreshChart}>
                Refresh chart
            </Button>
            <LineChart
                data={data}
                isLoading={loading}
                legendData={legendData}
                onLegendClick={filterByLegend}
            />
        </>
    );
};

export const LineChartToggableLegendExample: React.FC = (): React.ReactElement => (
    <section className="mt-5">
        <h4>Legend Interaction</h4>
        <p className="mt-2 mb-5">
            Callback to perform an action after clicking on a legend. The
            legendData property could be used to set the initial selection
            state.
        </p>

        <h5 className="mt-5 mb-2">* Example 1:</h5>
        <ToggableLegendFilterExample />

        <h5 className="mt-5 mb-2">* Example 2:</h5>
        <ToggableLegendKeepLegendSelectionExample />
    </section>
);

export default LineChartToggableLegendExample;
