import React, { FunctionComponent, ReactElement } from 'react';
import { flatten, max } from 'lodash';

import { Toggle } from '@preamp/core';
import {
    CurveFunction,
    LineChart,
    PlotData,
    DataDomain,
    PlotValue
} from '@preamp/waveform';

import { smallData } from './data/small';

const data: PlotData<PlotValue>[] = smallData.slice(0, 2).map(
    (plotData: PlotData<PlotValue>): PlotData<PlotValue> => ({
        ...plotData,
        data: plotData.data.map(
            ([date, value]: PlotValue, i: number): PlotValue => {
                switch (i) {
                    case 0:
                    case 3:
                    case 4:
                    case 10:
                        return [date, null];
                    default:
                        return [date, value];
                }
            }
        )
    })
);
const secondaryData: PlotData<PlotValue>[] = data.map(
    ({ data, name }): PlotData<PlotValue> => ({
        // get running totals
        data: data
            .reduce(
                (acc, [date, value]): PlotValue[] => [
                    ...acc,
                    [date, acc.length ? acc[acc.length - 1][1] + value : value]
                ],
                []
            )
            .map(
                ([date, value]: PlotValue, i: number): PlotValue => {
                    switch (i) {
                        case 0:
                        case 6:
                            return [date, null];
                        default:
                            return [date, value];
                    }
                }
            ),
        name: `Total ${name}`
    })
);

function getAllValues(data: PlotData<PlotValue>[]): number[] {
    return flatten(
        data.map(({ data }): number[] => data.map(([, value]): number => value))
    );
}

const maxValue: number = max([
    ...getAllValues(data),
    ...getAllValues(secondaryData)
]);

export const SecondaryDataWithNullDataPointsLineChart: FunctionComponent = (): ReactElement => {
    const [showSecondaryData, setShowSecondaryData] = React.useState<boolean>(
        true
    );
    const [showSameDomain, setShowSameDomain] = React.useState<boolean>(false);

    const customDomain: DataDomain =
        showSecondaryData && showSameDomain ? { y: [{ max: maxValue }] } : {};

    return (
        <section className="mt-5">
            <h4>Secondary Data With Null Data Points</h4>
            <Toggle
                checked={showSecondaryData}
                label="Secondary Data"
                name="secondary-data-with-null-data-points-toggle"
                onChange={(checked: boolean): void => {
                    setShowSecondaryData(checked);
                }}
            />
            {showSecondaryData && (
                <Toggle
                    checked={showSameDomain}
                    label="Same Domain"
                    name="same-domain-with-null-data-points-toggle"
                    onChange={(checked: boolean): void => {
                        setShowSameDomain(checked);
                    }}
                />
            )}
            <LineChart
                colors={[
                    'var(--waveform-sky-300)',
                    'var(--waveform-magenta-300)'
                ]}
                data={data}
                dataDomain={customDomain}
                secondaryColors={
                    showSecondaryData
                        ? [
                              'var(--waveform-sky-700)',
                              'var(--waveform-magenta-700)'
                          ]
                        : undefined
                }
                secondaryCurve={
                    showSecondaryData ? CurveFunction.MonotoneX : undefined
                }
                secondaryData={showSecondaryData ? secondaryData : undefined}
                secondaryDataDomain={customDomain}
                xAxisTitle="Time"
                y2AxisTitle={showSecondaryData ? 'Totals' : undefined}
                y2LabelFormat={showSecondaryData ? ',.0f' : undefined}
                yAxisTitle="Pets"
            />
        </section>
    );
};

export default SecondaryDataWithNullDataPointsLineChart;
