import React, { FunctionComponent, ReactElement } from 'react';

import { cloneDeep, get, set } from 'lodash';

import {
    LINECHART_DEFAULT_DOMAIN,
    LineChart,
    LinePlotData,
    DataDomain
} from '@preamp/waveform';
import { TextField, TextFieldType, Toggle } from '@preamp/core';

const data: LinePlotData[] = [
    {
        name: 'Cats',
        data: [
            [new Date(2021, 0, 1), 10],
            [new Date(2021, 0, 2), -20], // bad value: it will be internally filtered out
            [new Date(2021, 0, 3), 6],
            [new Date(2021, 0, 4), 4],
            [null, 5], // bad value: it will be internally filtered out
            [new Date(2021, 0, 6), 8],
            [new Date(2021, 0, 7), 12]
        ]
    },
    {
        name: 'Dogs',
        data: [
            [new Date(2021, 0, 1), 8],
            [new Date(2021, 0, 2), 12],
            [new Date(2021, 0, 3), 12],
            [new Date(2021, 0, 4), 9],
            [new Date(2021, 0, 5), 7],
            [new Date(2021, 0, 6), 5],
            [new Date(2021, 0, 7), null] // bad value: it will be internally filtered out
        ]
    }
];

/**
 * Plot Data Domains: Values out of range or not allowed by the chart definition will be filtered out from data.
 */
export const DataDomainsLineChartExample: FunctionComponent = (): ReactElement => {
    const [isCustomDomain, setIsCustomDomain] = React.useState<boolean>();
    const [domain, setDomain] = React.useState<DataDomain>(
        cloneDeep(LINECHART_DEFAULT_DOMAIN)
    );

    function changeDomainProperty(prop: string, value: number | boolean): void {
        setDomain(
            (prevDomain: DataDomain): DataDomain => {
                const newDomain = cloneDeep(prevDomain);
                set(newDomain, ['y', 0, prop], value);
                return newDomain;
            }
        );
    }

    function onChange(
        prop: string
    ): (_: React.ChangeEvent<HTMLInputElement>) => void {
        return (e: React.ChangeEvent<HTMLInputElement>): void => {
            changeDomainProperty(prop, Number(e.target.value));
        };
    }

    function onToggleCustomDomain(value: boolean): void {
        setDomain(cloneDeep(LINECHART_DEFAULT_DOMAIN));
        setIsCustomDomain(value);
    }

    return (
        <>
            <Toggle
                checked={isCustomDomain}
                label="Set a custom domain"
                name="custom-domain"
                onChange={onToggleCustomDomain}
            />

            <div className="flex items-end mb-4">
                <div className="flex-1 text-right pr-4 pb-3">Y:</div>
                <div className="flex-1 pr-4">
                    <TextField
                        disabled={!isCustomDomain}
                        label="Min"
                        onChange={onChange('min')}
                        textFieldType={TextFieldType.Number}
                        value={`${get(domain, ['y', 0, 'min'], 0)}`}
                    />
                </div>
                <div className="flex-1 pr-4">
                    <TextField
                        disabled={!isCustomDomain}
                        label="Max"
                        onChange={onChange('max')}
                        textFieldType={TextFieldType.Number}
                        value={`${get(domain, ['y', 0, 'max'], 12)}`}
                    />
                </div>
                <div className="flex-1 u-body-2 mb-3">Nulls not allowed</div>
            </div>

            <LineChart autoWidth data={data} dataDomain={domain} />
        </>
    );
};

export default DataDomainsLineChartExample;
