import React from 'react';

import {
    LollipopChart,
    PlotData,
    CategoryValue,
    LegendData
} from '@preamp/waveform';

import { featureData } from './data/feature';
import { Button } from '@preamp/core';

export const basicData: PlotData<CategoryValue> = {
    name: 'Added',
    data: [
        ['Adam Molano', 152343],
        ['Amy Gaudioso', 155838],
        ['Ben Behrens', 142747],
        ['Bob Samuelson', 120301],
        ['Daniel Raynes', 143255],
        ['Doug Ung', 131998],
        ['Helen Kim', 153558],
        ['Juan Giuffrida', 123020],
        ['Vera Carrillo-Marquez', 152321]
    ]
};

const ToggableLegendFilterExample: React.FC = (): React.ReactElement => {
    const [data, setData] = React.useState<PlotData>(basicData);
    const [feature, setFeature] = React.useState<PlotData>(featureData);

    const filterByLegend = (
        currentSelection: LegendData[],
        _clicked?: number,
        event?: React.MouseEvent<HTMLLIElement>
    ): LegendData[] | void => {
        event && event.preventDefault();

        const selection = currentSelection
            .filter((item: LegendData): boolean => item.selected)
            .map((item: LegendData): string => item.name as string);

        setData({
            ...basicData,
            color:
                selection.length && !selection.includes(basicData.name)
                    ? 'var(--va-grey-20)'
                    : null
        });

        setFeature({
            ...feature,
            color:
                selection.length && !selection.includes(featureData.name)
                    ? 'var(--va-grey-20)'
                    : null
        });
    };

    return (
        <>
            <p className="mb-5">Filtering data when clicking on legend</p>
            <LollipopChart
                autoWidth
                data={data}
                feature={feature}
                onLegendClick={filterByLegend}
                xAxisTitle="Team Members"
                yAxisTitle="Lines of Code"
            />
        </>
    );
};

const ToggableLegendKeepLegendSelectionExample: React.FC = (): React.ReactElement => {
    const [data, setData] = React.useState<PlotData>(basicData);
    const [feature, setFeature] = React.useState<PlotData>(featureData);
    const [legendData, setLegendData] = React.useState<LegendData[]>(null);
    const [loading, setLoading] = React.useState<boolean>(false);

    const filterByLegend = (
        currentSelection: LegendData[],
        _clicked?: number,
        event?: React.MouseEvent<HTMLLIElement>
    ): LegendData[] | void => {
        event && event.preventDefault();

        const selection = currentSelection
            .filter((item: LegendData): boolean => item.selected)
            .map((item: LegendData): string => item.name as string);

        setData({
            ...basicData,
            color:
                selection.length && !selection.includes(basicData.name)
                    ? 'var(--va-grey-20)'
                    : null
        });

        setFeature({
            ...featureData,
            color:
                selection.length && !selection.includes(featureData.name)
                    ? 'var(--va-grey-20)'
                    : null
        });

        setLegendData(currentSelection);
    };

    const simulateServerCall = (): void => {
        setTimeout((): void => {
            setLoading(false);
            filterByLegend(legendData || [basicData, featureData]);
        }, 1000);
    };

    const refreshChart = (): void => {
        setLoading(true);
        simulateServerCall();
    };

    return (
        <>
            <p className="mb-5">
                Filtering data when clicking on legend and keeping legend
                selection after refreshing
            </p>
            <Button className="mb-5" onClick={refreshChart}>
                Refresh chart
            </Button>

            <LollipopChart
                autoWidth
                data={data}
                feature={feature}
                isLoading={loading}
                legendData={legendData}
                onLegendClick={filterByLegend}
                xAxisTitle="Team Members"
                yAxisTitle="Lines of Code"
            />
        </>
    );
};

export const LollipopChartToggableLegendExample: React.FC = (): React.ReactElement => (
    <section className="mt-5">
        <h4>Legend Interaction</h4>
        <p className="mt-2">
            Callback to perform an action after clicking on a legend. The
            legendData property could be used to set the initial selection
            state.
        </p>

        <h5 className="mt-5 mb-2">* Example 1:</h5>
        <ToggableLegendFilterExample />

        <h5 className="mt-5 mb-2">* Example 2:</h5>
        <ToggableLegendKeepLegendSelectionExample />
    </section>
);

export default LollipopChartToggableLegendExample;
