import React from 'react';

import { cloneDeep, get, set } from 'lodash';

import {
    CategoryValue,
    LOLLIPOPCHART_DEFAULT_DOMAIN,
    LollipopChart,
    PlotData,
    DataDomain
} from '@preamp/waveform';

import { TextField, TextFieldType, Toggle } from '@preamp/core';

export const basicData: PlotData<CategoryValue> = {
    name: 'Added',
    data: [
        ['Adam Molano', 5],
        ['Amy Gaudioso', 6],
        ['Ben Behrens', 6],
        ['Bob Samuelson', 7],
        ['Daniel Raynes', 6],
        ['Doug Ung', 8],
        ['Helen Kim', 9],
        ['Juan Giuffrida', 10],
        ['Vera Carrillo-Marquez', 10]
    ]
};

export const featureData: PlotData<CategoryValue> = {
    name: 'Removed',
    data: [
        ['Adam Molano', 10],
        ['Amy Gaudioso', 11],
        ['Ben Behrens', 11],
        ['Bob Samuelson', 12],
        ['Daniel Raynes', 11],
        ['Doug Ung', 13],
        ['Helen Kim', 14],
        ['Juan Giuffrida', 15],
        ['Vera Carrillo-Marquez', 15]
    ]
};

export const DataDomainsLollipopChartExample: React.FC = (): React.ReactElement => {
    const [isCustomDomain, setIsCustomDomain] = React.useState<boolean>();
    const [domain, setDomain] = React.useState<DataDomain>(
        cloneDeep(LOLLIPOPCHART_DEFAULT_DOMAIN)
    );
    const [featureDomain, setFeatureDomain] = React.useState<DataDomain>(
        cloneDeep(LOLLIPOPCHART_DEFAULT_DOMAIN)
    );

    function onChange(
        domainName: string, // 'data' or 'feature
        prop: string
    ): (_: React.ChangeEvent<HTMLInputElement>) => void {
        return (e: React.ChangeEvent<HTMLInputElement>): void => {
            (domainName === 'feature' ? setFeatureDomain : setDomain)(
                (prevDomain: DataDomain): DataDomain => {
                    const newDomain = cloneDeep(prevDomain);
                    set(newDomain, ['y', 0, prop], Number(e.target.value));
                    return newDomain;
                }
            );
        };
    }

    function onToggleCustomDomain(value: boolean): void {
        setDomain(cloneDeep(LOLLIPOPCHART_DEFAULT_DOMAIN));
        setFeatureDomain(cloneDeep(LOLLIPOPCHART_DEFAULT_DOMAIN));
        setIsCustomDomain(value);
    }

    return (
        <section className="mt-5">
            <h4>Plot Data Domains</h4>
            <p className="mt-2 mb-5">
                Values out of range or not allowed by the chart definition will
                be filtered out from data.
            </p>

            <Toggle
                checked={isCustomDomain}
                label="Set custom domains"
                name="custom-domain"
                onChange={onToggleCustomDomain}
            />

            <div className="flex items-end mb-4">
                <div className="flex-1 text-right pr-4 pb-3">Data:</div>
                <div className="flex-1 pr-4">
                    <TextField
                        disabled={!isCustomDomain}
                        label="min"
                        onChange={onChange('data', 'min')}
                        textFieldType={TextFieldType.Number}
                        value={`${get(domain, ['y', 0, 'min'], 5)}`}
                    />
                </div>
                <div className="flex-1 pr-4">
                    <TextField
                        disabled={!isCustomDomain}
                        label="max"
                        onChange={onChange('data', 'max')}
                        textFieldType={TextFieldType.Number}
                        value={`${get(domain, ['y', 0, 'max'], 10)}`}
                    />
                </div>
                <div className="flex-1 u-body-2 mb-3">Nulls not allowed</div>
            </div>

            <div className="flex items-end mb-4">
                <div className="flex-1 text-right pr-4 pb-3">Feature Data:</div>
                <div className="flex-1 pr-4">
                    <TextField
                        disabled={!isCustomDomain}
                        label="min"
                        onChange={onChange('feature', 'min')}
                        textFieldType={TextFieldType.Number}
                        value={`${get(featureDomain, ['y', 0, 'min'], 10)}`}
                    />
                </div>
                <div className="flex-1 pr-4">
                    <TextField
                        disabled={!isCustomDomain}
                        label="max"
                        onChange={onChange('feature', 'max')}
                        textFieldType={TextFieldType.Number}
                        value={`${get(featureDomain, ['y', 0, 'max'], 15)}`}
                    />
                </div>
                <div className="flex-1 u-body-2 mb-3">Nulls not allowed</div>
            </div>

            <LollipopChart
                data={basicData}
                dataDomain={domain}
                feature={featureData}
                featureDomain={featureDomain}
            />
        </section>
    );
};

export default DataDomainsLollipopChartExample;
