import React from 'react';

import { cloneDeep, get, set } from 'lodash';
import {
    MekkoChart,
    MEKKOCHART_DEFAULT_DOMAIN,
    PlotData,
    DataDomain
} from '@preamp/waveform';
import { TextField, TextFieldType, Toggle } from '@preamp/core';

const basicData: PlotData[] = [
    {
        name: 'Mariokart Winners',
        data: [
            ['Baby Mario', 20],
            ['Peach', -14], // bad value: it will be internally filtered out
            ['Yoshi', 24],
            ['Toad', 10],
            ['Wario', 12]
        ]
    },
    {
        name: 'Mariokart Losers',
        data: [
            ['Luigi', 5],
            ['Bowser', 8],
            ['Birdo', null],
            ['Waluigi', 8]
        ]
    }
];

export const DataDomainsMekkoChartExample: React.FC = (): React.ReactElement => {
    const [isCustomDomain, setIsCustomDomain] = React.useState<boolean>();
    const [domain, setDomain] = React.useState<DataDomain>(
        cloneDeep(MEKKOCHART_DEFAULT_DOMAIN)
    );

    function changeDomainProperty(prop: string, value: number | boolean): void {
        setDomain(
            (prevDomain: DataDomain): DataDomain => {
                const newDomain = cloneDeep(prevDomain);
                set(newDomain, ['y', 0, prop], value);
                return newDomain;
            }
        );
    }

    function onChange(
        prop: string
    ): (_: React.ChangeEvent<HTMLInputElement>) => void {
        return (e: React.ChangeEvent<HTMLInputElement>): void => {
            changeDomainProperty(prop, Number(e.target.value));
        };
    }

    function onChangeNulls(value: boolean): void {
        changeDomainProperty('allowsNulls', value);
    }

    function onToggleCustomDomain(value: boolean): void {
        setDomain(cloneDeep(MEKKOCHART_DEFAULT_DOMAIN));
        setIsCustomDomain(value);
    }

    return (
        <section className="mt-5">
            <h4>Plot Data Domains</h4>
            <p className="mt-2 mb-5">
                Values out of range or not allowed by the chart definition will
                be filtered out from data.
            </p>

            <Toggle
                checked={isCustomDomain}
                label="Set a custom domain"
                name="custom-domain"
                onChange={onToggleCustomDomain}
            />

            <div className="flex items-end mb-4">
                <div className="flex-1 text-right pr-4 pb-3">Values:</div>
                <div className="flex-1 pr-4">
                    <TextField
                        disabled={!isCustomDomain}
                        label="min"
                        onChange={onChange('min')}
                        textFieldType={TextFieldType.Number}
                        value={`${get(domain, ['y', 0, 'min'], -14)}`}
                    />
                </div>
                <div className="flex-1 pr-4">
                    <TextField
                        disabled={!isCustomDomain}
                        label="max"
                        onChange={onChange('max')}
                        textFieldType={TextFieldType.Number}
                        value={`${get(domain, ['y', 0, 'max'], 24)}`}
                    />
                </div>
                <div className="flex-1">
                    <Toggle
                        checked={domain.y[0].allowsNulls}
                        disabled={!isCustomDomain}
                        label="Allows nulls"
                        name="allows-nulls"
                        onChange={onChangeNulls}
                    />
                </div>
            </div>

            <MekkoChart data={basicData} dataDomain={domain} />
        </section>
    );
};

export default DataDomainsMekkoChartExample;
