import React from 'react';

import {
    PlotData,
    LegendData,
    PlotValue,
    MultiBarChart
} from '@preamp/waveform';
import { Button } from '@preamp/core';

const multiData: PlotData[] = [
    {
        name: 'Cats',
        data: [
            ['Adam Molano', 152343, { id: 0 }],
            ['Amy Gaudioso', 155838, { id: 1 }],
            ['Ben Behrens', 142747, { id: 2 }],
            ['Bob Samuelson', 120301, { id: 3 }],
            ['Daniel Raynes', 143255, { id: 4 }],
            ['Doug Ung', 131998, { id: 5 }]
        ]
    },
    {
        name: 'Dogs',
        data: [
            ['Adam Molano', 155838, { id: 0 }],
            ['Amy Gaudioso', 142747, { id: 1 }],
            ['Ben Behrens', 120301, { id: 2 }],
            ['Bob Samuelson', 143255, { id: 3 }],
            ['Daniel Raynes', 131998, { id: 4 }],
            ['Doug Ung', 153558, { id: 5 }]
        ]
    },
    {
        name: 'Birds',
        data: [
            ['Adam Molano', 142747, { id: 0 }],
            ['Amy Gaudioso', 155838, { id: 1 }],
            ['Ben Behrens', 142747, { id: 2 }],
            ['Bob Samuelson', 120301, { id: 3 }],
            ['Daniel Raynes', 143255, { id: 4 }],
            ['Doug Ung', 131998, { id: 5 }]
        ]
    }
];

const ToggableLegendFilterExample: React.FC = (): React.ReactElement => {
    const [data, setData] = React.useState<PlotData[]>(multiData);

    const filterByLegend = (
        currentSelection: LegendData[],
        _clicked: number,
        event: React.MouseEvent<HTMLLIElement>
    ): void => {
        event && event.preventDefault();

        const selection = currentSelection
            .filter((item: LegendData): boolean => item.selected)
            .map((item: LegendData): string => item.name as string);

        const filteredData = !selection.length
            ? multiData
            : multiData.map(
                  ({ name, data }: PlotData): PlotData => {
                      return {
                          name,
                          data: !selection.includes(name)
                              ? data.map(
                                    (item: PlotValue): PlotValue => [
                                        item[0],
                                        0,
                                        item[2]
                                    ]
                                )
                              : data
                      };
                  }
              );
        setData([...filteredData]);
    };

    return (
        <>
            <p className="mb-5">Filtering data when clicking on legend</p>
            <MultiBarChart data={data} onLegendClick={filterByLegend} />
        </>
    );
};

const ToggableLegendKeepLegendSelectionExample: React.FC = (): React.ReactElement => {
    const [data, setData] = React.useState<PlotData[]>(multiData);
    const [legendData, setLegendData] = React.useState<LegendData[]>(null);
    const [loading, setLoading] = React.useState<boolean>(false);

    const filterByLegend = (
        currentSelection: LegendData[],
        _clicked?: number,
        event?: React.MouseEvent<HTMLLIElement>
    ): void => {
        event && event.preventDefault();

        const selection = currentSelection
            .filter((item: LegendData): boolean => item.selected)
            .map((item: LegendData): string => item.name as string);

        const filteredData = !selection.length
            ? multiData
            : multiData.map(
                  ({ name, data }: PlotData): PlotData => {
                      return {
                          name,
                          data: !selection.includes(name)
                              ? data.map(
                                    (item: PlotValue): PlotValue => [
                                        item[0],
                                        0,
                                        item[2]
                                    ]
                                )
                              : data
                      };
                  }
              );
        setData([...filteredData]);
        setLegendData(currentSelection);
    };

    const simulateServerCall = (): void => {
        setTimeout((): void => {
            setLoading(false);
            filterByLegend(legendData || multiData);
        }, 1000);
    };

    const refreshChart = (): void => {
        setLoading(true);
        simulateServerCall();
    };

    return (
        <>
            <p className="mb-5">
                Filtering data when clicking on legend and keeping legend
                selection after refreshing
            </p>

            <Button className="mb-5" onClick={refreshChart}>
                Refresh chart
            </Button>

            <MultiBarChart
                data={data}
                isLoading={loading}
                legendData={legendData}
                onLegendClick={filterByLegend}
            />
        </>
    );
};

export const MultiBarChartToggableLegendExample: React.FC = (): React.ReactElement => (
    <section className="mt-5">
        <h4>Legend Interaction</h4>
        <p className="mt-2 mb-5">
            Callback to perform an action after clicking on a legend. The
            legendData property could be used to set the initial selection
            state.
        </p>

        <h5 className="mt-5 mb-2">* Example 1:</h5>
        <ToggableLegendFilterExample />

        <h5 className="mt-5 mb-2">* Example 2:</h5>
        <ToggableLegendKeepLegendSelectionExample />
    </section>
);

export default MultiBarChartToggableLegendExample;
