import React from 'react';

import {
    BarChartData,
    MultiBarChart,
    PlotData,
    PlotValue,
    YAxisColumn
} from '@preamp/waveform';

import { format } from 'd3';

const chartData: PlotData[] = [
    {
        name: 'JS',
        data: [
            [
                'Adam Molano',
                150000,
                { id: 0, coverage: 0.7, lastCommit: '2020, May 20th' }
            ],
            [
                'Amy Gaudioso',
                100000,
                { id: 1, coverage: 0.9, lastCommit: '2019, Sep 1st' }
            ],
            [
                'Ben Behrens',
                140000,
                { id: 2, coverage: 0.65, lastCommit: '2018, Jun 15th' }
            ],
            [
                'Matt Rodriguez',
                90000,
                { id: 3, coverage: 0.95, lastCommit: '2020, Sep 29th' }
            ]
        ]
    },
    {
        name: 'HTML',
        data: [
            [
                'Adam Molano',
                125000,
                { id: 0, coverage: 0.46, lastCommit: '2020, Mar 11th' }
            ],
            ['Amy Gaudioso', 0, { id: 1 }],
            [
                'Ben Behrens',
                25000,
                { id: 2, coverage: 0.87, lastCommit: '2019, Oct 31th' }
            ],
            [
                'Matt Rodriguez',
                120000,
                { id: 3, coverage: 0.85, lastCommit: '2020, Aug 1st' }
            ]
        ]
    },
    {
        name: 'CSS',
        data: [
            [
                'Adam Molano',
                145000,
                { id: 0, coverage: 0.92, lastCommit: '2018, Aug 12th' }
            ],
            [
                'Amy Gaudioso',
                130000,
                { id: 1, coverage: 0.9, lastCommit: '2020, Feb 29th' }
            ],
            ['Ben Behrens', 0, { id: 2 }],
            [
                'Matt Rodriguez',
                135000,
                { id: 3, coverage: 0.69, lastCommit: '2020, Sep 14th' }
            ]
        ]
    }
];

const getPlotValue = (d: BarChartData): PlotValue => {
    const group = chartData.find(
        ({ name }: PlotData): boolean => name === d.key
    );
    const dev = group
        ? group.data.find(([name]: PlotValue): boolean => name === d.name)
        : null;
    // adding group name to meta data
    dev[2].lang = group.name;
    return dev;
};

const getMetaValue = (d: BarChartData, key: string): string | number => {
    const [, , meta]: PlotValue = getPlotValue(d);
    return meta ? meta[key] : null;
};

export const MultiBarChartYAxisColumnsExample: React.FC = (): React.ReactElement => {
    const [output, setOutput] = React.useState<string>('...');

    const displayMoreInfo = (d: BarChartData): void => {
        const [teamMember, , { lastCommit, lang }]: PlotValue = getPlotValue(d);
        setOutput(
            lastCommit
                ? `${teamMember} committed ${lang} changes on ${lastCommit}`
                : `${teamMember} never committed ${lang} changes`
        );
    };

    const yAxisColumn: YAxisColumn[] = [
        {
            name: 'Last commit date',
            width: 100,
            value: (d: BarChartData): string | number =>
                getMetaValue(d, 'lastCommit') || '-',
            textAlign: 'left',
            onClick: displayMoreInfo
        },
        {
            name: '% of Coverage',
            width: 70,
            value: (d: BarChartData): string | number => {
                const coverage = getMetaValue(d, 'coverage') as number;
                return isNaN(coverage) ? '-' : `${Math.round(coverage * 100)}%`;
            },
            textAlign: 'center'
        },
        {
            name: 'Lines of Code',
            width: 80,
            value: ({ value }: BarChartData): string | number =>
                format('~s')(value)
        }
    ];

    return (
        <section className="mt-5">
            <h4>Y-axis columns</h4>
            <p className="mt-2">
                MultiBarChart can display the y-axis as columns.
            </p>

            <MultiBarChart
                data={chartData}
                isHorizontal
                showMetricLine
                xAxisTitle="Index"
                yAxisColumn={yAxisColumn}
                yAxisTitle="Team Member"
            />

            <p className="u-font-bold u-body-1 m-2">
                Click on the <span className="italic">Last Commit Date</span>{' '}
                cell to see more info:
            </p>
            <p className="u-font-bold u-body-2 u-well m-2">{output}</p>
        </section>
    );
};

export default MultiBarChartYAxisColumnsExample;
