import React from 'react';

import { PieChart, LegendData, CategoryValue } from '@preamp/waveform';
import { Button } from '@preamp/core';

const basicData: CategoryValue[] = [
    ['Dachshund', 68],
    ['Chihuahua', 48],
    ['Beagle', 38],
    ['Pomeranian', 26]
];

const toLegendData = (data: CategoryValue[]): LegendData[] =>
    data.map(
        ([key]: CategoryValue): LegendData => ({
            name: key
        })
    );

const ToggableLegendFilterExample: React.FC = (): React.ReactElement => {
    const [filteredData, setFilteredData] = React.useState<CategoryValue[]>([
        ...basicData
    ]);

    const filterByLegend = (
        currentSelection: LegendData[],
        _clicked: number,
        event: React.MouseEvent<HTMLLIElement>
    ): void => {
        event && event.preventDefault();

        const selection = currentSelection
            .filter(({ selected }: LegendData): boolean => selected)
            .map(({ name }: LegendData): string => name as string);

        const data = !selection.length
            ? basicData
            : basicData.map(
                  ([key, value, meta]: CategoryValue): CategoryValue => {
                      return selection.includes(key)
                          ? [key, value, meta]
                          : [
                                key,
                                value,
                                {
                                    ...meta,
                                    color:
                                        'var(--chart-indicator-unselect-color)'
                                }
                            ];
                  }
              );
        setFilteredData([...data]);
    };

    return (
        <>
            <p className="mb-5">Filtering data when clicking on legend</p>
            <PieChart
                autoWidth
                data={filteredData}
                onLegendClick={filterByLegend}
            />
        </>
    );
};

const ToggableLegendKeepLegendSelectionExample: React.FC = (): React.ReactElement => {
    const [data, setData] = React.useState<CategoryValue[]>(basicData);
    const [legendData, setLegendData] = React.useState<LegendData[]>(null);
    const [loading, setLoading] = React.useState<boolean>(false);

    const filterByLegend = (
        currentSelection: LegendData[],
        _clicked?: number,
        event?: React.MouseEvent<HTMLLIElement>
    ): void => {
        event && event.preventDefault();

        const selection = currentSelection
            .filter((item: LegendData): boolean => item.selected)
            .map((item: LegendData): string => item.name as string);

        const filteredData = !selection.length
            ? basicData
            : basicData.map(
                  (item: CategoryValue): CategoryValue => {
                      return selection.includes(item[0])
                          ? item
                          : [
                                item[0],
                                item[1],
                                {
                                    color:
                                        'var(--chart-indicator-unselect-color)'
                                }
                            ];
                  }
              );
        setData([...filteredData]);
        setLegendData(currentSelection);
    };

    const simulateServerCall = (): void => {
        setTimeout((): void => {
            setLoading(false);
            filterByLegend(legendData || toLegendData(basicData));
        }, 1000);
    };

    const refreshChart = (): void => {
        setLoading(true);
        simulateServerCall();
    };

    return (
        <>
            <p className="mb-5">
                Filtering data when clicking on legend and keeping legend
                selection after refreshing
            </p>
            <Button onClick={refreshChart}>Refresh chart</Button>
            <PieChart
                autoWidth
                data={data}
                isLoading={loading}
                legendData={legendData}
                onLegendClick={filterByLegend}
            />
        </>
    );
};

export const PieChartToggableLegendExample: React.FC = (): React.ReactElement => (
    <section className="mt-5">
        <h4>Legend Interaction</h4>
        <p className="mt-2 mb-5">
            Callback to perform an action after clicking on a legend.
        </p>

        <h5 className="mt-5 mb-2">* Example 1:</h5>
        <ToggableLegendFilterExample />

        <h5 className="mt-5 mb-2">* Example 2:</h5>
        <ToggableLegendKeepLegendSelectionExample />
    </section>
);

export default PieChartToggableLegendExample;
