import React from 'react';

import { ScatterChart, PlotData, LegendData } from '@preamp/waveform';
import { Button } from '@preamp/core';

export const smallData: PlotData[] = [
    {
        name: 'Ice Cream',
        data: [
            [57.56, 215],
            [61.52, 325],
            [53.42, 85],
            [59.36, 332],
            [65.3, 406],
            [71.78, 522],
            [66.92, 412],
            [77.18, 614],
            [74.12, 544],
            [64.58, 421],
            [72.68, 145],
            [62.96, 408]
        ]
    },
    {
        name: 'Hot Chocolate',
        data: [
            [57.56, 502],
            [61.52, 480],
            [53.42, 717],
            [59.36, 499],
            [65.3, 442],
            [71.78, 427],
            [66.92, 433],
            [77.18, 501],
            [74.12, 418],
            [64.58, 449],
            [72.68, 420],
            [62.96, 491]
        ]
    }
];

const ToggableLegendFilterExample: React.FC = (): React.ReactElement => {
    const [data, setData] = React.useState<PlotData[]>(smallData);

    const filterByLegend = (
        currentSelection: LegendData[],
        _clicked: number,
        event: React.MouseEvent<HTMLLIElement>
    ): void => {
        event && event.preventDefault();

        const selection = currentSelection
            .filter((item: LegendData): boolean => item.selected)
            .map((item: LegendData): string => item.name as string);

        const mappedData = !selection.length
            ? smallData
            : smallData.map(
                  (point: PlotData): PlotData => ({
                      ...point,
                      color: !selection.includes(point.name)
                          ? 'var(--va-grey-300)'
                          : (point.color = null)
                  })
              );
        setData([...mappedData]);
    };

    return (
        <>
            <p className="mb-5">Filtering data when clicking on legend</p>
            <ScatterChart
                autoWidth
                data={data}
                onLegendClick={filterByLegend}
            />
        </>
    );
};

const ToggableLegendKeepLegendSelectionExample: React.FC = (): React.ReactElement => {
    const [filteredData, setFilteredData] = React.useState<PlotData[]>(
        smallData
    );
    const [legendData, setLegendData] = React.useState<LegendData[]>(null);
    const [loading, setLoading] = React.useState<boolean>(false);

    const filterByLegend = (
        currentSelection: LegendData[],
        _clicked?: number,
        event?: React.MouseEvent<HTMLLIElement>
    ): void => {
        event && event.preventDefault();

        const selection = currentSelection
            .filter((item: LegendData): boolean => item.selected)
            .map((item: LegendData): string => item.name as string);

        const mappedData = !selection.length
            ? smallData
            : smallData.map(
                  (point: PlotData): PlotData => ({
                      ...point,
                      color: !selection.includes(point.name)
                          ? 'var(--va-grey-300)'
                          : (point.color = null)
                  })
              );
        setFilteredData([...mappedData]);
        setLegendData(currentSelection);
    };

    const simulateServerCall = (): void => {
        setTimeout((): void => {
            setLoading(false);
            filterByLegend(legendData || smallData);
        }, 1000);
    };

    const refreshChart = (): void => {
        setLoading(true);
        simulateServerCall();
    };

    return (
        <>
            <p className="mb-5">
                Filtering data when clicking on legend and keeping legend
                selection after refreshing
            </p>
            <Button className="mb-5" onClick={refreshChart}>
                Refresh chart
            </Button>
            <ScatterChart
                autoWidth
                data={filteredData}
                isLoading={loading}
                legendData={legendData}
                onLegendClick={filterByLegend}
            />
        </>
    );
};

export const ScatterChartToggableLegendExample: React.FC = (): React.ReactElement => (
    <section className="mt-5">
        <h4>Legend Interaction</h4>
        <p className="mt-2 mb-5">
            Callback to perform an action after clicking on a legend. The
            legendData property could be used to set the initial selection
            state.
        </p>

        <h5 className="mt-5 mb-2">* Example 1:</h5>
        <ToggableLegendFilterExample />

        <h5 className="mt-5 mb-2">* Example 2:</h5>
        <ToggableLegendKeepLegendSelectionExample />
    </section>
);

export default ScatterChartToggableLegendExample;
