import React from 'react';

import { cloneDeep, get, set } from 'lodash';
import {
    SCATTERCHART_DEFAULT_DOMAIN,
    NumericValue,
    PlotData,
    DataDomain,
    ScatterChart
} from '@preamp/waveform';

import { TextField, TextFieldType, Toggle } from '@preamp/core';

const basicData: PlotData<NumericValue>[] = [
    {
        name: 'Ice Cream',
        data: [
            [5, 2],
            [6, 3],
            [5, 0], // bad value: it will be internally filtered out
            [null, 3], // bad value: it will be internally filtered out
            [6, null], // bad value: it will be internally filtered out
            [7, 5]
        ]
    },
    {
        name: 'Hot Chocolate',
        data: [
            [6, 2],
            [0, 7], // bad value: it will be internally filtered out
            [-4, 4], // bad value: it will be internally filtered out
            [4, 1],
            [2, 5],
            [2, -4] // bad value: it will be internally filtered out
        ]
    }
];

export const DataDomainsScatterChartExample: React.FC = (): React.ReactElement => {
    const [isCustomDomain, setIsCustomDomain] = React.useState<boolean>();
    const [domain, setDomain] = React.useState<DataDomain>(
        cloneDeep(SCATTERCHART_DEFAULT_DOMAIN)
    );

    function changeDomainProperty(
        axis: 'x' | 'y',
        prop: string,
        value: number | boolean
    ): void {
        setDomain(
            (prevDomain: DataDomain): DataDomain => {
                const newDomain = cloneDeep(prevDomain);
                const path = axis === 'x' ? [axis] : [axis, 0];
                set(newDomain, [...path, prop], value);
                return newDomain;
            }
        );
    }

    function onChange(
        axis: 'x' | 'y',
        prop: string
    ): (_: React.ChangeEvent<HTMLInputElement>) => void {
        return (e: React.ChangeEvent<HTMLInputElement>): void => {
            changeDomainProperty(axis, prop, Number(e.target.value));
        };
    }

    function onToggleCustomDomain(value: boolean): void {
        setDomain(cloneDeep(SCATTERCHART_DEFAULT_DOMAIN));
        setIsCustomDomain(value);
    }

    return (
        <section className="mt-5">
            <h4>Plot Data Domains</h4>
            <p className="mt-2 mb-5">
                Values out of range or not allowed by the chart definition will
                be filtered out from data.
            </p>

            <Toggle
                checked={isCustomDomain}
                label="Set custom domains"
                name="custom-domain"
                onChange={onToggleCustomDomain}
            />

            <div className="flex items-end mb-4">
                <div className="flex-1 text-right pr-4 pb-3">X:</div>
                <div className="flex-1 pr-4">
                    <TextField
                        disabled={!isCustomDomain}
                        label="min"
                        onChange={onChange('x', 'min')}
                        textFieldType={TextFieldType.Number}
                        value={`${get(domain, ['x', 'min'], -4)}`}
                    />
                </div>
                <div className="flex-1 pr-4">
                    <TextField
                        disabled={!isCustomDomain}
                        label="max"
                        onChange={onChange('x', 'max')}
                        textFieldType={TextFieldType.Number}
                        value={`${get(domain, ['x', 'max'], 7)}`}
                    />
                </div>
                <div className="flex-1 u-body-2 mb-3">Nulls not allowed</div>
            </div>

            <div className="flex items-end mb-4">
                <div className="flex-1 text-right pr-4 pb-3">Y:</div>
                <div className="flex-1 pr-4">
                    <TextField
                        disabled={!isCustomDomain}
                        label="min"
                        onChange={onChange('y', 'min')}
                        textFieldType={TextFieldType.Number}
                        value={`${get(domain, ['y', 0, 'min'], -4)}`}
                    />
                </div>
                <div className="flex-1 pr-4">
                    <TextField
                        disabled={!isCustomDomain}
                        label="max"
                        onChange={onChange('y', 'max')}
                        textFieldType={TextFieldType.Number}
                        value={`${get(domain, ['y', 0, 'max'], 7)}`}
                    />
                </div>
                <div className="flex-1 u-body-2 mb-3">Nulls not allowed</div>
            </div>

            <ScatterChart autoWidth data={basicData} dataDomain={domain} />
        </section>
    );
};

export default DataDomainsScatterChartExample;
