import React, { useState } from 'react';
import { v4 as uuidv4 } from 'uuid';

import { Button, ButtonTheme } from '@preamp/core';
import { TornadoChart, PlotData, defaultColors } from '@preamp/waveform';
import { useEffect } from 'react';

const MAX = 20;
const HEIGHT = 500;

const randomNumber = (): number => {
    return Math.floor(Math.random() * MAX);
};

const generateData = (): [number, number][] => {
    const response = [];

    for (let i = 0; i < MAX; i += 1) {
        response.push([`${i + 1}x`, randomNumber()]);
    }

    return response;
};

const basicData: PlotData[][] = [
    [
        {
            name: 'Cats',
            data: generateData()
        },
        {
            name: 'Dogs',
            data: generateData()
        },
        {
            name: 'Bears',
            data: generateData()
        },
        {
            name: 'Frogs',
            data: generateData()
        }
    ],
    [
        {
            name: 'Cats',
            data: generateData()
        },
        {
            name: 'Dogs',
            data: generateData()
        },
        {
            name: 'Bears',
            data: generateData()
        },
        {
            name: 'Frogs',
            data: generateData()
        }
    ]
];

interface ChartItemButton {
    isActive: boolean;
    color: string;
}

interface ChartItemButtons {
    [key: string]: ChartItemButton;
}

/**
 * A TornadoChart with dynamic line items
 */
const chartItemButtons = basicData[0].reduce(
    (acc, data, i): ChartItemButtons => {
        acc[data.name] = { isActive: true, color: defaultColors[i] };

        return acc;
    },
    {}
);

const chartColors = Object.values(chartItemButtons).map(
    (lineItemBtn: ChartItemButton): string => {
        return lineItemBtn.color;
    }
);

const TornadoChartDynamicItems: React.FC = (): React.ReactElement => {
    const [data, setData] = useState<PlotData[][]>(basicData);
    const [btns, setBtns] = useState<ChartItemButtons>(chartItemButtons);
    const [isLoading, setIsLoading] = useState(false);
    const [isMaxHeightActive, setIsMaxHeightActive] = useState(false);

    useEffect((): void => {
        const data = basicData.map((data): PlotData[] => {
            return data.filter((item): boolean => btns[item.name].isActive);
        });

        setData(data);
        setIsLoading(false);
    }, [btns]);

    const handleClick = (name: string): void => {
        setBtns(
            (prevState: ChartItemButtons): ChartItemButtons => {
                return {
                    ...prevState,
                    [name]: {
                        ...prevState[name],
                        isActive: !prevState[name].isActive
                    }
                };
            }
        );
        setIsLoading(true);
    };

    return (
        <section className="mt-8">
            <h2>Tornado Chart with dynamic line items</h2>
            <p className="max-w-sm mt-2">
                This is an example where you can toggle the visiblity of
                different line items
            </p>
            <p className="mt-2">
                {Object.keys(btns).map(
                    (name): React.ReactElement => (
                        <Button
                            key={`${uuidv4()},`}
                            onClick={(): void => {
                                handleClick(name);
                            }}
                            theme={
                                btns[name].isActive
                                    ? ButtonTheme.Primary
                                    : ButtonTheme.Secondary
                            }
                        >
                            {name}
                        </Button>
                    )
                )}
            </p>
            <div>
                <p className="max-w-sm my-4">
                    Set a max height to add an overflow to the y-axis
                </p>
                <Button
                    onClick={(): void => {
                        setIsMaxHeightActive(true);
                    }}
                    theme={ButtonTheme.Secondary}
                >
                    Set Max Height {HEIGHT}px
                </Button>
                <Button
                    disabled={!isMaxHeightActive}
                    onClick={(): void => {
                        setIsMaxHeightActive(false);
                    }}
                    theme={ButtonTheme.Secondary}
                >
                    Reset
                </Button>
            </div>
            <TornadoChart
                autoWidth={true}
                className={'px-4'}
                colors={chartColors}
                data={data}
                isLoading={isLoading}
                maxHeight={isMaxHeightActive ? HEIGHT : null}
                minHeight={data[0].length > 2 ? 300 * data[0].length : HEIGHT}
                showMetricLine={true}
            />
        </section>
    );
};

export default TornadoChartDynamicItems;
