import React from 'react';

import {
    PlotData,
    LegendData,
    PlotValue,
    TornadoChart
} from '@preamp/waveform';
import { Button } from '@preamp/core';

const basicData: PlotData[][] = [
    [
        {
            name: 'Cats',
            data: [
                ['Adam Molano', 152343, { id: 0 }],
                ['Amy Gaudioso', 155838, { id: 1 }],
                ['Ben Behrens', 142747, { id: 2 }],
                ['Bob Samuelson', 120301, { id: 3 }],
                ['Daniel Raynes', 143255, { id: 4 }],
                ['Doug Ung', 131998, { id: 5 }],
                ['Helen Kim', 153558, { id: 6 }],
                ['Juan Giuffrida', 123020, { id: 7 }],
                ['Vera Carrillo-Marquez', 152321, { id: 8 }]
            ]
        },
        {
            name: 'Dogs',
            data: [
                ['Adam Molano', 155838, { id: 0 }],
                ['Amy Gaudioso', 142747, { id: 1 }],
                ['Ben Behrens', 120301, { id: 2 }],
                ['Bob Samuelson', 143255, { id: 3 }],
                ['Daniel Raynes', 131998, { id: 4 }],
                ['Doug Ung', 153558, { id: 5 }],
                ['Helen Kim', 123020, { id: 6 }],
                ['Juan Giuffrida', 152321, { id: 7 }],
                ['Vera Carrillo-Marquez', 155843, { id: 8 }]
            ]
        }
    ],
    [
        {
            name: 'Cats',
            data: [
                ['Adam Molano', 142747, { id: 0 }],
                ['Amy Gaudioso', 155838, { id: 1 }],
                ['Ben Behrens', 142747, { id: 2 }],
                ['Bob Samuelson', 120301, { id: 3 }],
                ['Daniel Raynes', 143255, { id: 4 }],
                ['Doug Ung', 131998, { id: 5 }],
                ['Helen Kim', 153558, { id: 6 }],
                ['Juan Giuffrida', 123020, { id: 7 }],
                ['Vera Carrillo-Marquez', 152221, { id: 8 }]
            ]
        },
        {
            name: 'Dogs',
            data: [
                ['Adam Molano', 154343, { id: 0 }],
                ['Amy Gaudioso', 165838, { id: 1 }],
                ['Ben Behrens', 142947, { id: 2 }],
                ['Bob Samuelson', 125301, { id: 3 }],
                ['Daniel Raynes', 143155, { id: 4 }],
                ['Doug Ung', 131798, { id: 5 }],
                ['Helen Kim', 153358, { id: 6 }],
                ['Juan Giuffrida', 123920, { id: 7 }],
                ['Vera Carrillo-Marquez', 152329, { id: 8 }]
            ]
        }
    ]
];

const ToggableLegendFilterExample: React.FC = (): React.ReactElement => {
    const [data, setData] = React.useState<PlotData[][]>(basicData);

    const filterByLegend = (
        currentSelection: LegendData[],
        _clicked: number,
        event: React.MouseEvent<HTMLLIElement>
    ): void => {
        event && event.preventDefault();

        const selection = currentSelection
            .filter((item: LegendData): boolean => item.selected)
            .map((item: LegendData): string => item.name as string);

        const filteredData = !selection.length
            ? basicData
            : basicData.map((groups): PlotData[] =>
                  groups.map(
                      ({ name, data }): PlotData => {
                          return {
                              name,
                              data: !selection.includes(name)
                                  ? data.map(
                                        (item: PlotValue): PlotValue => [
                                            item[0],
                                            0,
                                            item[2]
                                        ]
                                    )
                                  : data
                          };
                      }
                  )
              );
        setData([...filteredData]);
    };

    return (
        <>
            <p className="mb-5">Filtering data when clicking on legend</p>
            <TornadoChart data={data} onLegendClick={filterByLegend} />
        </>
    );
};

const ToggableLegendKeepLegendSelectionExample: React.FC = (): React.ReactElement => {
    const [data, setData] = React.useState<PlotData[][]>(basicData);
    const [legendData, setLegendData] = React.useState<LegendData[]>(null);
    const [loading, setLoading] = React.useState<boolean>(false);

    const filterByLegend = (
        currentSelection: LegendData[],
        _clicked?: number,
        event?: React.MouseEvent<HTMLLIElement>
    ): void => {
        event && event.preventDefault();

        const selection = currentSelection
            .filter((item: LegendData): boolean => item.selected)
            .map((item: LegendData): string => item.name as string);

        const filteredData = !selection.length
            ? basicData
            : basicData.map((groups): PlotData[] =>
                  groups.map(
                      ({ name, data }): PlotData => ({
                          name,
                          data: !selection.includes(name)
                              ? data.map(
                                    ([key, , meta]: PlotValue): PlotValue => [
                                        key,
                                        0,
                                        meta
                                    ]
                                )
                              : data
                      })
                  )
              );
        setData([...filteredData]);
        setLegendData(currentSelection);
    };

    const simulateServerCall = (): void => {
        setTimeout((): void => {
            setLoading(false);
            filterByLegend(legendData || basicData[1]);
        }, 1000);
    };

    const refreshChart = (): void => {
        setLoading(true);
        simulateServerCall();
    };

    return (
        <>
            <p className="mb-5">
                Filtering data when clicking on legend and keeping legend
                selection after refreshing
            </p>
            <Button className="mb-5" onClick={refreshChart}>
                Refresh chart
            </Button>
            <TornadoChart
                data={data}
                isLoading={loading}
                legendData={legendData}
                onLegendClick={filterByLegend}
            />
        </>
    );
};

export const TornadoChartToggableLegendExample: React.FC = (): React.ReactElement => (
    <section className="mt-5">
        <h4>Legend Interaction</h4>
        <p className="mt-2 mb-5">
            Callback to perform an action after clicking on a legend.
        </p>

        <h5 className="mt-5 mb-2">* Example 1:</h5>
        <ToggableLegendFilterExample />

        <h5 className="mt-5 mb-2">* Example 2:</h5>
        <ToggableLegendKeepLegendSelectionExample />
    </section>
);

export default TornadoChartToggableLegendExample;
