import React from 'react';

import { set } from 'lodash';

import {
    ChartEvent,
    ChartEventType,
    sortByDate,
    startOf,
    TimeValue,
    TrendLineChart,
    TrendLineChartPlotData
} from '@preamp/waveform';
import { DividerTitle } from '@preamp/core';
import { ControlsProps, ExampleControls } from './controls/ExampleControls';
import { generateAxisData, TODAY } from './controls/utils';
import { Audiences, Buy, CheckCircle } from '@preamp/signal';

const EVENT_TYPES: ChartEventType[] = [
    {
        id: 'event-type-1',
        label: 'Event Type 1',
        icon: Audiences,
        iconScale: 0.5
    },
    {
        id: 'event-type-2',
        label: 'Event Type 2',
        icon: Buy,
        iconScale: 0.6
    },
    {
        id: 'event-type-3',
        label: 'Event Type 3',
        icon: CheckCircle
    }
];

const DEFAULT_EVENTS: ChartEvent[] = [
    {
        name: 'Very Long Name for Event 1',
        date: new Date(TODAY.getFullYear(), TODAY.getMonth(), 3),
        description: 'Description for event 1',
        type: EVENT_TYPES[0]
    },
    {
        name: 'Very Long Name for this Event 2',
        date: new Date(TODAY.getFullYear(), TODAY.getMonth(), 3),
        description: 'Description for event 2',
        type: EVENT_TYPES[1]
    },
    {
        name: 'Event 3',
        date: new Date(TODAY.getFullYear(), TODAY.getMonth(), 3),
        description: 'Description for event 3',
        type: EVENT_TYPES[2]
    },
    {
        name: 'Event 4',
        date: new Date(TODAY.getFullYear(), TODAY.getMonth(), 3),
        description: 'Description for event 4'
    },
    {
        name: 'Event 5',
        date: new Date(TODAY.getFullYear(), TODAY.getMonth(), 10),
        description: 'Description for event 5'
    }
];

const DEFAULT_SETTINGS: ControlsProps = {
    dateRange: {
        from: new Date(TODAY.getFullYear(), TODAY.getMonth(), 1),
        to: new Date(TODAY.getFullYear(), TODAY.getMonth(), 10)
    },
    eventLabelWidth: 50,
    leftYAxisNr: 1,
    forecastPoints: 4,
    forecastStartDate: new Date(TODAY.getFullYear(), TODAY.getMonth(), 6),
    maxEventsByDate: 3,
    points: 5,
    shouldShowEvents: true,
    shouldShowTooltips: true,
    xTickCount: 5,
    yAxisNr: 3,
    yAxisUniqueDomain: false,
    yAxisWidth: 90,
    yTickCount: 0,
    zoomHeight: 30
};

export const TrendLineChartConfigurationExample: React.FC = (): React.ReactElement => {
    const [controls, setControls] = React.useState<ControlsProps>({
        ...DEFAULT_SETTINGS
    });
    const [events, setEvents] = React.useState<ChartEvent[]>([]);

    const data = React.useMemo(
        (): TrendLineChartPlotData[] =>
            // create random data
            generateAxisData(
                controls.yAxisNr,
                controls.points,
                controls.dateRange.from,
                controls.forecastStartDate,
                controls.yAxisMaxValues
            ),
        [
            controls.dateRange.from,
            controls.forecastStartDate,
            controls.points,
            controls.yAxisMaxValues,
            controls.yAxisNr
        ]
    );

    const forecastData = React.useMemo(
        (): TrendLineChartPlotData[] =>
            // create random data
            generateAxisData(
                controls.yAxisNr,
                controls.forecastPoints,
                controls.forecastStartDate,
                controls.dateRange.to,
                controls.yAxisMaxValues,
                true
            ),
        [
            controls.dateRange.to,
            controls.forecastPoints,
            controls.forecastStartDate,
            controls.yAxisMaxValues,
            controls.yAxisNr
        ]
    );

    React.useEffect((): void => {
        setEvents(DEFAULT_EVENTS);
    }, [data, forecastData]);

    function setControl(
        property: string,
        value: string | number | boolean | Date
    ): void {
        setControls(set({ ...controls }, property, value));
    }

    function addEvent(): void {
        if (controls.event && controls.event.date && controls.event.name) {
            const { date } = controls.event;
            const newEvent = {
                ...controls.event,
                date: startOf(date),
                description: controls.event.description || 'No description',
                type: controls.event.type
            };
            setEvents((prevEvents: ChartEvent[]): ChartEvent[] =>
                sortByDate([...prevEvents, newEvent])
            );
            setControl('event', null);
        }
    }

    function getCustomTitle(): string {
        return 'Custom Tooltip';
    }

    function customTooltipLabel(
        name: string,
        color: string,
        formattedValue: string,
        _rawValue: TimeValue,
        _active: boolean,
        _index: number
    ): React.ReactNode {
        return (
            <div className="u-body-2">
                <span style={{ color }}>{name}</span>: {formattedValue}
            </div>
        );
    }

    return (
        <section className="mt-5">
            <h4>Configuration Example</h4>
            <p className="mt-2 mb-5">
                Modify any TrendLineChart property as desired.
            </p>

            <ExampleControls
                addEvent={addEvent}
                controls={controls}
                eventTypes={EVENT_TYPES}
                setControl={setControl}
            />

            <DividerTitle className="my-6" text="Result" />

            <TrendLineChart
                autoWidth
                data={data}
                eventLabelWidth={controls.eventLabelWidth}
                events={events}
                eventTypes={EVENT_TYPES}
                forecastData={forecastData}
                hideZoom={controls.hideZoom}
                maxEventsByDate={controls.maxEventsByDate}
                shouldShowEvents={controls.shouldShowEvents}
                shouldShowTooltips={controls.shouldShowTooltips}
                tooltipCustomLabelDescription={
                    controls.customTooltip ? customTooltipLabel : null
                }
                tooltipCustomTitle={
                    controls.customTooltip ? getCustomTitle : null
                }
                xAxisTitle="Dates"
                xLabelFormat={'%m/%d/%Y'}
                xTickAlignToData={controls.xTicksAlignToData}
                xTickCount={
                    controls.isCustomXTickCount
                        ? controls.xTickCount
                        : undefined
                }
                yAxisLeft={controls.leftYAxisNr}
                yAxisUniqueDomain={controls.yAxisUniqueDomain}
                yAxisWidth={controls.yAxisWidth}
                yLabelFormats={['.8', '$.2f']} // formatters for the two first y-axis
                yTickCount={
                    controls.isCustomYTickCount
                        ? controls.yTickCount
                        : undefined
                }
                zoomHeight={controls.zoomHeight}
            />
        </section>
    );
};

export default TrendLineChartConfigurationExample;
