import React from 'react';

import { cloneDeep, flatMap, get, min, max, set } from 'lodash';

import {
    PlotData,
    DataDomain,
    TRENDLINECHART_DEFAULT_DOMAIN,
    TRENDLINECHART_DEFAULT_Y_DOMAIN,
    TrendLineChart,
    TimeValue
} from '@preamp/waveform';

import { TextField, TextFieldType, Toggle } from '@preamp/core';

const basicData: PlotData[] = [
    {
        name: 'y-axis #1',
        data: [
            [new Date(2018, 0, 1), 12],
            [new Date(2018, 1, 1), 13],
            [new Date(2018, 2, 1), 13],
            [new Date(2018, 3, 1), 12],
            [new Date(2018, 4, 1), 12],
            [new Date(2018, 5, 1), 15],
            [new Date(2018, 6, 1), 14],
            [new Date(2018, 7, 1), 14],
            [new Date(2018, 8, 1), null],
            [new Date(2018, 9, 1), 11],
            [new Date(2018, 10, 1), 10],
            [new Date(2018, 11, 1), 10]
        ]
    },
    {
        name: 'y-axis #2',
        data: [
            [new Date(2018, 0, 1), 11],
            [new Date(2018, 1, 1), 11],
            [new Date(2018, 2, 1), 12],
            [new Date(2018, 3, 1), 14],
            [new Date(2018, 4, 1), null],
            [new Date(2018, 5, 1), null],
            [new Date(2018, 6, 1), 15],
            [new Date(2018, 7, 1), 15],
            [new Date(2018, 8, 1), 14],
            [new Date(2018, 9, 1), 13],
            [new Date(2018, 10, 1), 13],
            [new Date(2018, 11, 1), 12]
        ]
    },
    {
        name: 'y-axis #3',
        data: [
            [new Date(2018, 0, 1), 5],
            [new Date(2018, 1, 1), 7],
            [new Date(2018, 2, 1), 7],
            [new Date(2018, 3, 1), 8],
            [new Date(2018, 4, 1), 9],
            [new Date(2018, 5, 1), 9],
            [new Date(2018, 6, 1), 8],
            [new Date(2018, 7, 1), 10],
            [new Date(2018, 8, 1), 12],
            [new Date(2018, 9, 1), 14],
            [new Date(2018, 10, 1), 14],
            [new Date(2018, 11, 1), 16]
        ]
    }
];

const forecastData: PlotData[] = [
    {
        name: 'y-axis #1',
        data: [
            [new Date(2019, 0, 1), 10],
            [new Date(2019, 1, 1), 9],
            [new Date(2019, 2, 1), 9],
            [new Date(2019, 3, 1), 10]
        ]
    },
    {
        name: 'y-axis #2',
        data: [
            [new Date(2019, 0, 1), 11],
            [new Date(2019, 1, 1), 11],
            [new Date(2019, 2, 1), 12],
            [new Date(2019, 3, 1), -20]
        ]
    },
    {
        name: 'y-axis #3',
        data: [
            [new Date(2019, 0, 1), 15],
            [new Date(2019, 1, 1), 15],
            [new Date(2019, 2, 1), 14],
            [new Date(2019, 3, 1), 14]
        ]
    }
];

const DEFAULT_DOMAINS: DataDomain[] = [...new Array(basicData.length)].map(
    (_): DataDomain => ({
        ...TRENDLINECHART_DEFAULT_DOMAIN,
        y: [{ ...TRENDLINECHART_DEFAULT_Y_DOMAIN }]
    })
);

const DATA_RANGES: [number, number][] = [...new Array(basicData.length)].map(
    (_, i: number): [number, number] => {
        const values = flatMap(
            [...basicData[i].data, ...forecastData[i].data].map(
                ([, value]: TimeValue): number => value
            )
        );
        return [min(values), max(values)];
    }
);

export const DataDomainsTrendLineChartExample: React.FC = (): React.ReactElement => {
    const [isCustomDomain, setIsCustomDomain] = React.useState<boolean>();
    const [domains, setDomains] = React.useState<DataDomain[]>(
        cloneDeep(DEFAULT_DOMAINS)
    );

    function onChange(
        yAxis: number,
        prop: string
    ): (_: React.ChangeEvent<HTMLInputElement>) => void {
        return (e: React.ChangeEvent<HTMLInputElement>): void => {
            const value = Number(e.target.value);
            setDomains((prevDomains: DataDomain[]): DataDomain[] => {
                const newDomains = [...prevDomains];
                set(newDomains, [yAxis, 'y', 0, prop], value);
                return newDomains;
            });
        };
    }

    function onToggleCustomDomain(value: boolean): void {
        setIsCustomDomain(value);
        setDomains(cloneDeep(DEFAULT_DOMAINS));
    }

    return (
        <section className="mt-5">
            <h4>Plot Data Domains</h4>
            <p className="mt-2 mb-5">
                Values out of range or not allowed by the chart definition will
                be filtered out from data.
            </p>

            <Toggle
                checked={isCustomDomain}
                label="Set custom domains"
                name="custom-domain"
                onChange={onToggleCustomDomain}
            />

            {[...new Array(basicData.length)].map(
                (_, yAxis: number): React.ReactElement => (
                    <div
                        className="flex items-end mb-4"
                        key={'y-axis-' + yAxis}
                    >
                        <div className="flex-1 text-right pr-4 pb-3">
                            {basicData[yAxis].name}:
                        </div>
                        <div className="flex-1 pr-4">
                            <TextField
                                disabled={!isCustomDomain}
                                label="min"
                                onChange={onChange(yAxis, 'min')}
                                textFieldType={TextFieldType.Number}
                                value={`${get(
                                    domains,
                                    [yAxis, 'y', 0, 'min'],
                                    DATA_RANGES[yAxis][0]
                                )}`}
                            />
                        </div>
                        <div className="flex-1 pr-4">
                            <TextField
                                disabled={!isCustomDomain}
                                label="max"
                                onChange={onChange(yAxis, 'max')}
                                textFieldType={TextFieldType.Number}
                                value={`${get(
                                    domains,
                                    [yAxis, 'y', 0, 'max'],
                                    DATA_RANGES[yAxis][1]
                                )}`}
                            />
                        </div>
                        <div className="flex-1 u-body-2 mb-3">
                            Nulls not allowed
                        </div>
                    </div>
                )
            )}

            <TrendLineChart
                autoWidth
                data={basicData}
                dataDomain={domains}
                dataUI="data-domain-example"
                forecastData={forecastData}
            />
        </section>
    );
};

export default DataDomainsTrendLineChartExample;
