import React from 'react';

import { get } from 'lodash';

import {
    Button,
    ButtonTheme,
    DividerTitle,
    TextField,
    TextFieldType,
    Toggle
} from '@preamp/core';
import { DatePickerInput, DateRange, DateRangeInput } from '@preamp/datepicker';
import { getDefaultMaxYValue, MaximumYAxisValues, TODAY } from './utils';
import {
    ChartEvent,
    ChartEventType,
    MIN_ZOOM_HEIGHT,
    MAX_ZOOM_HEIGHT
} from '@preamp/waveform';
import { OptionType, Select } from '@preamp/select';

export interface ControlsProps {
    customTooltip?: boolean;
    dateRange: DateRange;
    event?: ChartEvent;
    eventLabelWidth?: number;
    forecastPoints: number;
    forecastStartDate: Date;
    hideZoom?: boolean;
    isCustomXTickCount?: boolean;
    isCustomYTickCount?: boolean;
    leftYAxisNr: number;
    maxEventsByDate: number;
    points: number;
    shouldShowEvents: boolean;
    shouldShowTooltips: boolean;
    xTickCount?: number;
    xTicksAlignToData?: boolean;
    yAxisNr: number;
    yAxisMaxValues?: MaximumYAxisValues;
    yAxisUniqueDomain: boolean;
    yAxisWidth: number;
    yTickCount: number;
    zoomHeight: number;
}

interface ExampleControlsProps {
    controls: ControlsProps;
    setControl: (
        property: string,
        value: string | number | boolean | object
    ) => void;
    addEvent: () => void;
    eventTypes: ChartEventType[];
}

export const ExampleControls = ({
    controls,
    setControl,
    addEvent,
    eventTypes
}: ExampleControlsProps): React.ReactElement => {
    React.useEffect((): void => {
        if (controls.yAxisNr < controls.leftYAxisNr) {
            setControl('leftYAxisNr', controls.yAxisNr);
        }
    }, [controls.leftYAxisNr, controls.yAxisNr, setControl]);

    React.useEffect((): void => {
        if (
            controls.forecastStartDate.getTime() <
                controls.dateRange.from.getTime() ||
            controls.forecastStartDate.getTime() >
                controls.dateRange.to.getTime()
        ) {
            setControl('forecastStartDate', controls.dateRange.to);
        }
    }, [controls.dateRange, controls.forecastStartDate, setControl]);

    return (
        <>
            <DividerTitle className="my-6" text="Y-Axis" />

            <div className="flex pl-32">
                <div className="flex-1 pr-3">
                    <TextField
                        helpText="(Between 1 and 8)"
                        label="Number of Y-Axis"
                        max="8"
                        min="1"
                        onChange={(
                            event: React.ChangeEvent<HTMLInputElement>
                        ): void => {
                            setControl('yAxisNr', Number(event.target.value));
                        }}
                        step={1}
                        textFieldType={TextFieldType.Number}
                        value={`${controls.yAxisNr}`}
                    />
                </div>
                <div className="flex-1 pr-3">
                    <TextField
                        helpText={`(Between 0 and ${controls.yAxisNr})`}
                        label="Y-Axis on the left side"
                        max={`${controls.yAxisNr}`}
                        min="0"
                        onChange={(
                            event: React.ChangeEvent<HTMLInputElement>
                        ): void => {
                            setControl(
                                'leftYAxisNr',
                                Number(event.target.value)
                            );
                        }}
                        step="1"
                        textFieldType={TextFieldType.Number}
                        value={`${controls.leftYAxisNr}`}
                    />
                </div>
                <div className="flex-1 pr-3">
                    <TextField
                        label="Y-Axis width (px)"
                        min="0"
                        onChange={(
                            event: React.ChangeEvent<HTMLInputElement>
                        ): void => {
                            setControl(
                                'yAxisWidth',
                                Number(event.target.value)
                            );
                        }}
                        step="1"
                        textFieldType={TextFieldType.Number}
                        value={`${controls.yAxisWidth}`}
                    />
                </div>
            </div>

            <div className="flex pt-6 pl-32">
                <div className="flex-1 pr-3">
                    <Toggle
                        checked={controls.yAxisUniqueDomain}
                        label="Keep same domain for all y-axis"
                        name="same-domain"
                        onChange={(checked: boolean): void => {
                            setControl('yAxisUniqueDomain', checked);
                        }}
                    />
                </div>
                <div className="flex-1 pr-3">
                    <Toggle
                        checked={controls.isCustomYTickCount}
                        label="Custom number of Y-Axis labels"
                        name="custom-ytick-count"
                        onChange={(checked: boolean): void => {
                            setControl('isCustomYTickCount', checked);
                        }}
                    />
                </div>
                <div className="flex-1">
                    <TextField
                        disabled={!controls.isCustomYTickCount}
                        label="Number of Y-Axis labels"
                        onChange={(
                            event: React.ChangeEvent<HTMLInputElement>
                        ): void => {
                            setControl(
                                'yTickCount',
                                Number(event.target.value)
                            );
                        }}
                        step="1"
                        textFieldType={TextFieldType.Number}
                        value={`${controls.yTickCount}`}
                    />
                </div>
            </div>

            <div className="pt-6 pl-32 va-form-label">
                Maximum Y-axis values
            </div>
            <div className="flex flex-row pl-32">
                {[...new Array(controls.yAxisNr)].map(
                    (_, idx: number): React.ReactElement => (
                        <div className="pr-3" key={`maxY-${idx}`}>
                            <TextField
                                helpText={`y-axis #${idx + 1}`}
                                min="0"
                                onChange={(
                                    event: React.ChangeEvent<HTMLInputElement>
                                ): void => {
                                    setControl('yAxisMaxValues', {
                                        ...(controls.yAxisMaxValues || {}),
                                        [idx]: Number(event.target.value)
                                    });
                                }}
                                step="1"
                                textFieldType={TextFieldType.Number}
                                value={`${(controls.yAxisMaxValues || {})[
                                    idx
                                ] || getDefaultMaxYValue(idx)}`}
                            />
                        </div>
                    )
                )}
                <Button
                    className="mt-1"
                    disabled={!controls.yAxisMaxValues}
                    onClick={(): void => {
                        setControl('yAxisMaxValues', null);
                    }}
                    theme={ButtonTheme.Primary}
                >
                    Reset
                </Button>
            </div>

            <DividerTitle className="my-6" text="X-Axis" />

            <div className="flex pl-32">
                <div className="flex-1 pr-3">
                    <DateRangeInput
                        dateRangePickerProps={{
                            firstDayOfWeek: 1,
                            initialMonth: controls.dateRange.from || TODAY
                        }}
                        isFullWidth
                        label="Date Range"
                        onConfirmDates={(dates: DateRange): void => {
                            setControl('dateRange', dates);
                        }}
                        selectedDates={controls.dateRange}
                    />
                </div>
                <div className="flex-1 pr-3">
                    <TextField
                        label="Data Points"
                        min="1"
                        onChange={(
                            event: React.ChangeEvent<HTMLInputElement>
                        ): void =>
                            setControl('points', Number(event.target.value))
                        }
                        step={1}
                        textFieldType={TextFieldType.Number}
                        value={`${controls.points}`}
                    />
                </div>
                <div className="flex-1 pr-3">
                    <DatePickerInput
                        datePickerProps={{
                            initialMonth: controls.dateRange.from,
                            disabledDays: {
                                before: controls.dateRange.from,
                                after: controls.dateRange.to
                            }
                        }}
                        label="Forecast Start Date"
                        onHandleDayClick={(d: Date): void => {
                            setControl('forecastStartDate', d);
                        }}
                        selectedDay={controls.forecastStartDate}
                    />
                </div>
                <div className="flex-1">
                    <TextField
                        label="Forecast Data Points"
                        min="1"
                        onChange={(
                            event: React.ChangeEvent<HTMLInputElement>
                        ): void =>
                            setControl(
                                'forecastPoints',
                                Number(event.target.value)
                            )
                        }
                        step={1}
                        textFieldType={TextFieldType.Number}
                        value={`${controls.forecastPoints}`}
                    />
                </div>
            </div>
            <div className="flex pt-6 pl-32">
                <div className="flex-1 pr-3">
                    <Toggle
                        checked={controls.xTicksAlignToData}
                        label="Align X-Axis labels to datapoints"
                        name="align-xticks-to-data"
                        onChange={(checked: boolean): void => {
                            setControl('xTicksAlignToData', checked);
                        }}
                    />
                </div>
                <div className="flex-1 pr-3">
                    <Toggle
                        checked={controls.isCustomXTickCount}
                        label="Custom number of X-Axis labels"
                        name="custom-xtick-count"
                        onChange={(checked: boolean): void => {
                            setControl('isCustomXTickCount', checked);
                        }}
                    />
                </div>
                <div className="flex-1">
                    <TextField
                        disabled={!controls.isCustomXTickCount}
                        helpText="The specified amount is only a hint"
                        label="Number of X-Axis labels"
                        min="1"
                        onChange={(
                            event: React.ChangeEvent<HTMLInputElement>
                        ): void =>
                            setControl('xTickCount', Number(event.target.value))
                        }
                        step={1}
                        textFieldType={TextFieldType.Number}
                        value={`${controls.xTickCount}`}
                    />
                </div>
            </div>

            <DividerTitle className="my-6" text="Event Generator" />

            <div className="flex pl-32">
                <div className="flex-1 pr-3">
                    <Toggle
                        checked={controls.shouldShowEvents}
                        label="Show event markers"
                        name="show-events-toggle"
                        onChange={(checked: boolean): void => {
                            setControl('shouldShowEvents', checked);
                        }}
                    />
                </div>
                <div className="flex-1 pr-3">
                    <TextField
                        disabled={!controls.shouldShowEvents}
                        helpText="0 for only 'More events' labels"
                        label="Max. Event Markers displayed for a date"
                        min="0"
                        onChange={(
                            event: React.ChangeEvent<HTMLInputElement>
                        ): void => {
                            setControl(
                                'maxEventsByDate',
                                Number(event.target.value)
                            );
                        }}
                        step="1"
                        textFieldType={TextFieldType.Number}
                        value={`${controls.maxEventsByDate}`}
                    />
                </div>
                <div className="flex-1">
                    <TextField
                        disabled={!controls.shouldShowEvents}
                        helpText="0 for only icons. Blank for no truncation."
                        label="Event Label Width (px)"
                        min="0"
                        onChange={(
                            event: React.ChangeEvent<HTMLInputElement>
                        ): void => {
                            setControl(
                                'eventLabelWidth',
                                event.target.value
                                    ? Number(event.target.value)
                                    : undefined
                            );
                        }}
                        step="1"
                        textFieldType={TextFieldType.Number}
                        value={`${
                            controls.eventLabelWidth >= 0
                                ? controls.eventLabelWidth
                                : ''
                        }`}
                    />
                </div>
            </div>

            <div className="flex pt-6 pl-32">
                <div className="flex-1 pr-3">
                    <DatePickerInput
                        datePickerProps={{
                            initialMonth: controls.dateRange.from,
                            disabledDays: {
                                before: controls.dateRange.from,
                                after: controls.dateRange.to
                            }
                        }}
                        inputProps={{ disabled: !controls.shouldShowEvents }}
                        label="Event Date"
                        onHandleDayClick={(d: Date): void => {
                            setControl('event.date', d);
                        }}
                        selectedDay={get(controls, ['event', 'date'])}
                    />
                </div>
                <div className="flex-1 pr-3">
                    <TextField
                        disabled={!controls.shouldShowEvents}
                        label="Event Name"
                        onChange={(
                            event: React.ChangeEvent<HTMLInputElement>
                        ): void => {
                            setControl('event.name', event.target.value);
                        }}
                        value={`${get(controls, ['event', 'name'], '')}`}
                    />
                </div>
                <div className="flex-1 pr-3">
                    <Select
                        label="Event Type"
                        onChange={(eventType: OptionType): void =>
                            setControl('event.type', eventType)
                        }
                        options={eventTypes}
                        value={get(controls, ['event', 'type'])}
                    />
                </div>
                <div className="flex-1 pr-3">
                    <TextField
                        disabled={!controls.shouldShowEvents}
                        label="Event Description"
                        onChange={(
                            event: React.ChangeEvent<HTMLInputElement>
                        ): void => {
                            setControl('event.description', event.target.value);
                        }}
                        value={`${get(controls, ['event', 'description'], '')}`}
                    />
                </div>
                <Button
                    className="self-end mb-1"
                    disabled={
                        !controls.event ||
                        !controls.event.date ||
                        !controls.event.name
                    }
                    onClick={addEvent}
                >
                    Add event
                </Button>
            </div>

            <DividerTitle className="my-6" text="Tooltips" />

            <div className="flex pl-32">
                <div className="flex-1 pr-3">
                    <Toggle
                        checked={controls.shouldShowTooltips}
                        label="Show tooltips"
                        name="show-tooltips-toggle"
                        onChange={(checked: boolean): void => {
                            setControl('shouldShowTooltips', checked);
                        }}
                    />
                </div>
                <div className="flex-1">
                    <Toggle
                        checked={controls.customTooltip}
                        disabled={!controls.shouldShowTooltips}
                        label="Custom tooltip"
                        name="custom-tooltip-toggle"
                        onChange={(checked: boolean): void => {
                            setControl('customTooltip', checked);
                        }}
                    />
                </div>
            </div>

            <DividerTitle className="my-6" text="Zoom Settings" />

            <div className="flex pl-32">
                <div className="flex-1">
                    <Toggle
                        checked={controls.hideZoom}
                        label="Hide zoom glider"
                        name="hide-zoom-glider"
                        onChange={(checked: boolean): void => {
                            setControl('hideZoom', checked);
                        }}
                    />
                </div>

                <div className="flex-1">
                    <TextField
                        disabled={controls.hideZoom}
                        helpText={`(Between ${MIN_ZOOM_HEIGHT} and ${MAX_ZOOM_HEIGHT})`}
                        label="Zoom height (px)"
                        max={`${MAX_ZOOM_HEIGHT}`}
                        min={`${MIN_ZOOM_HEIGHT}`}
                        onChange={(
                            event: React.ChangeEvent<HTMLInputElement>
                        ): void =>
                            setControl('zoomHeight', Number(event.target.value))
                        }
                        step="1"
                        textFieldType={TextFieldType.Number}
                        value={`${controls.zoomHeight}`}
                    />
                </div>
            </div>
        </>
    );
};

export default ExampleControls;
