import React from 'react';

import { LegendData, SetValue, VennDiagram } from '@preamp/waveform';
import { Button } from '@preamp/core';

const basicData: SetValue[] = [
    [['A'], 28],
    [['B'], 35],
    [['C'], 48],
    [['A', 'B'], 10],
    [['A', 'C'], 10],
    [['B', 'C'], 6],
    [['A', 'B', 'C'], 20]
];

const toLegendData = (data: SetValue[]): LegendData[] =>
    data.map(
        ([ids, _, meta]: SetValue): LegendData => ({
            color: meta && meta.color,
            name: ids.join(' ∩ ')
        })
    );

const ToggableLegendFilterExample: React.FC = (): React.ReactElement => {
    const [data, setData] = React.useState<SetValue[]>(basicData);

    function filterByLegend(
        currentSelection: LegendData[],
        _clicked: number,
        event: React.MouseEvent<HTMLLIElement>
    ): void {
        event && event.preventDefault();

        const selection = currentSelection
            .filter((item: LegendData): boolean => item.selected)
            .map((item: LegendData): string => item.name as string);

        const filteredData = !selection.length
            ? basicData
            : basicData.map(
                  ([key, value, meta]: SetValue): SetValue => [
                      key,
                      value,
                      selection.includes(key.join(' ∩ '))
                          ? meta
                          : { ...(meta || {}), color: 'rgb(231, 231, 231)' }
                  ]
              );
        setData([...filteredData]);
    }

    return (
        <>
            <p className="mb-5">Filtering data when clicking on legend</p>
            <VennDiagram
                data={data}
                onLegendClick={filterByLegend}
                width={340}
            />
        </>
    );
};

const ToggableLegendKeepLegendSelectionExample: React.FC = (): React.ReactElement => {
    const [data, setData] = React.useState<SetValue[]>(basicData);
    const [legendData, setLegendData] = React.useState<LegendData[]>(null);
    const [loading, setLoading] = React.useState<boolean>(false);

    function filterByLegend(
        currentSelection: LegendData[],
        _clicked?: number,
        event?: React.MouseEvent<HTMLLIElement>
    ): void {
        event && event.preventDefault();

        const selection = currentSelection
            .filter((item: LegendData): boolean => item.selected)
            .map((item: LegendData): string => item.name as string);

        const filteredData = !selection.length
            ? basicData
            : basicData.map(
                  ([key, value, meta]: SetValue): SetValue => [
                      key,
                      value,
                      selection.includes(key.join(' ∩ '))
                          ? meta
                          : { ...(meta || {}), color: 'rgb(231, 231, 231)' }
                  ]
              );
        setData([...filteredData]);
        setLegendData(currentSelection);
    }

    const simulateServerCall = (): void => {
        setTimeout((): void => {
            setLoading(false);
            filterByLegend(legendData || toLegendData(basicData));
        }, 1000);
    };

    const refreshChart = (): void => {
        setLoading(true);
        simulateServerCall();
    };

    return (
        <>
            <p className="mb-5">
                Filtering data when clicking on legend and keeping legend
                selection after refreshing
            </p>
            <Button className="mb-5" onClick={refreshChart}>
                Refresh chart
            </Button>
            <VennDiagram
                data={data}
                isLoading={loading}
                legendData={legendData}
                onLegendClick={filterByLegend}
                width={340}
            />
        </>
    );
};

/**
 * Legend Interaction
 */
export const VennDiagramToggableLegendInteractionExample: React.FC = (): React.ReactElement => (
    <section>
        <p className="mt-5">
            Callback to perform an action after clicking on a legend. The
            legendData property could be used to set the initial selection
            state.
        </p>

        <h5 className="mt-5 mb-2">* Example 1:</h5>
        <ToggableLegendFilterExample />

        <h5 className="mt-5 mb-2">* Example 2:</h5>
        <ToggableLegendKeepLegendSelectionExample />
    </section>
);

export default VennDiagramToggableLegendInteractionExample;
